/*$selected_borders_color: if($variant == 'light', darken($selected_bg_color, 30%), darken($selected_bg_color, 20%));*/
/*$borders_color: if($variant == 'light', shade($bg_color, .85), shade($bg_color, .88));*/
/*$dark_shadow: #000;*/
/*$light_shadow: #fff;*/
/*$button_border_strength: if(lightness($bg) > 50, 0, .1);*/
/*$button_shadow_strength: if(lightness($bg) > 50, 0, .1);*/
/*$selected_borders_color: if($variant == 'light', darken($selected_bg_color, 30%), darken($selected_bg_color, 20%));*/
/*$borders_color: if($variant == 'light', shade($bg_color, .85), shade($bg_color, .88));*/
/*$dark_shadow: #000;*/
/*$light_shadow: #fff;*/
/*$button_border_strength: if(lightness($bg) > 50, 0, .1);*/
/*$button_shadow_strength: if(lightness($bg) > 50, 0, .1);*/
/* dark color scheme */
@define-color dark_bg_color #0f0f0f;
@define-color dark_fg_color #a5a5a5;
/* colormap actually used by the theme, to be overridden in other css files */
@define-color theme_bg_color #a5a5a5;
@define-color theme_fg_color #000000;
@define-color theme_base_color #d0d0d0;
@define-color theme_text_color #000000;
@define-color theme_selected_bg_color #B36A87;
@define-color theme_selected_fg_color #000000;
@define-color theme_tooltip_bg_color #a5a5a5;
@define-color theme_tooltip_fg_color #000000;
/* shadow effects */
@define-color light_shadow #a6a6a6;
@define-color dark_shadow black;
/* misc colors used by gtk+ */
@define-color info_fg_color #fff;
@define-color info_bg_color #03a9f4;
@define-color warning_fg_color #fff;
@define-color warning_bg_color #ef6c00;
@define-color question_fg_color #fff;
@define-color question_bg_color #673ab7;
@define-color error_fg_color #fff;
@define-color error_bg_color #f44336;
@define-color link_color #3f51b5;
@define-color success_color #4caf50;
@define-color warning_color #ef6c00;
@define-color error_color #f44336;
/* widget colors */
@define-color titlebar_bg_color @dark_bg_color;
@define-color titlebar_fg_color @dark_fg_color;
@define-color menubar_bg_color @dark_bg_color;
@define-color menubar_fg_color @dark_fg_color;
@define-color toolbar_bg_color @theme_bg_color;
@define-color toolbar_fg_color @theme_fg_color;
@define-color menu_bg_color @dark_bg_color;
@define-color menu_fg_color @dark_fg_color;
@define-color panel_bg_color @dark_bg_color;
@define-color panel_fg_color @dark_fg_color;
@define-color borders mix(#a5a5a5,#000000,0.08);
@define-color unfocused_borders mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9);
@define-color button_bg_color #8b639b;
@define-color button_fg_color #000000;
@define-color header_button_bg_color #8b639b;
@define-color header_button_fg_color #000000;
@define-color insensitive_bg_color mix(#a5a5a5,#d0d0d0,0.6);
@define-color insensitive_fg_color mix(#000000,#a5a5a5,0.5);
/* osd */
@define-color osd_base #0f0f0f;
@define-color osd_bg rgba(15, 15, 15, 0.8);
@define-color osd_fg #a5a5a5;
@define-color osd_insensitive_bg_color mix(#a5a5a5,rgba(15, 15, 15, 0.8),0.5);
@define-color osd_insensitive_fg_color mix(#a5a5a5,#0f0f0f,0.6);
@define-color osd_borders_color rgba(13, 13, 13, 0.8);
/* lightdm greeter colors */
@define-color lightdm_bg_color #0f0f0f;
@define-color lightdm_fg_color #a5a5a5;
/* widget text/foreground color on backdrop windows */
@define-color theme_unfocused_fg_color mix(#000000,#a5a5a5,0.5);
/* text color for entries, views and content in general on backdrop windows */
@define-color theme_unfocused_text_color #000000;
/* widget base background color on backdrop windows */
@define-color theme_unfocused_bg_color #a5a5a5;
/* text widgets and the like base background color on backdrop windows */
@define-color theme_unfocused_base_color #d0d0d0;
/* base background color of selections on backdrop windows */
@define-color theme_unfocused_selected_bg_color #B36A87;
/* text/foreground color of selections on backdrop windows */
@define-color theme_unfocused_selected_fg_color #000000;
/* insensitive color on backdrop windows*/
@define-color unfocused_insensitive_color #7f7f7f;
/* window manager colors */
@define-color wm_bg #0f0f0f;
@define-color wm_border_focused #B36A87;
@define-color wm_border_unfocused #0f0f0f;
@define-color wm_title_focused mix(#a5a5a5,#0f0f0f,0.1);
@define-color wm_title_unfocused mix(#a5a5a5,#0f0f0f,0.4);
@define-color wm_icons_focused mix(#a5a5a5,#0f0f0f,0.1);
@define-color wm_icons_focused_prelight #B36A87;
@define-color wm_icons_focused_pressed #974d6a;
@define-color wm_icons_unfocused mix(#a5a5a5,#0f0f0f,0.4);
@define-color wm_icons_unfocused_prelight #B36A87;
@define-color wm_icons_unfocused_pressed #974d6a;
/**************
 ! GTK settings
***************/
* {
  -GtkWindow-resize-grip-height: 0;
  -GtkWindow-resize-grip-width: 0;
  -WnckTasklist-fade-overlay-rect: 0;
  -GtkWidget-cursor-aspect-ratio: 0.04;
  outline-color: rgba(179, 106, 135, 0.5);
  outline-style: dashed;
  outline-width: 1px;
  outline-offset: -1px;
  -gtk-outline-radius: 3px; }

/*************
 ! Base states
 *************/
*:selected, .gtkstyle-fallback:selected, GucharmapChartable:focus, GucharmapChartable:hover, GucharmapChartable:active, GucharmapChartable:selected, .gedit-document-panel-document-row:selected, .gedit-document-panel-document-row:selected:hover, GeditViewFrame .gedit-search-slider .not-found:selected, :focus:selected, GucharmapChartable:focus, .gedit-document-panel-document-row:focus:selected:hover, GeditViewFrame .gedit-search-slider .not-found:focus:selected {
  background-color: #B36A87;
  color: #000000; }

* {
  /* hyperlinks */
  -GtkIMHtml-hyperlink-color: #3f51b5; }
  *:disabled, *:disabled:disabled {
    color: mix(#000000,#a5a5a5,0.5); }
  *:disabled, *:disabled {
    -gtk-icon-effect: dim; }
  *:hover {
    -gtk-icon-effect: highlight; }
  *:link, *:visited {
    color: #3f51b5; }

.background {
  background-color: #a5a5a5;
  color: #000000; }
  .background:backdrop {
    text-shadow: none;
    -gtk-icon-shadow: none; }
  .background.csd {
    background-color: #a5a5a5; }

.gtkstyle-fallback {
  background-color: rgba(165, 165, 165, 0.5);
  color: #000000; }
  .gtkstyle-fallback:hover {
    background-color: #b6b6b6;
    color: #000000; }
  .gtkstyle-fallback:active {
    background-color: #959595;
    color: #000000; }
  .gtkstyle-fallback:disabled {
    background-color: #a5a5a5;
    color: mix(#000000,#a5a5a5,0.5); }

image, image:disabled, label, label:disabled, box, box:disabled, grid, grid:disabled {
  background-color: transparent; }

label.separator {
  color: #000000; }
  label.separator:backdrop {
    color: mix(#000000,#a5a5a5,0.5); }

label selection {
  background-color: #B36A87;
  color: #000000; }

label:disabled {
  color: mix(#000000,#a5a5a5,0.5); }
  label:disabled:backdrop {
    color: #7f7f7f; }

label:backdrop {
  color: mix(#000000,#a5a5a5,0.5); }

assistant .sidebar {
  background-color: #d0d0d0;
  border-top: 1px solid mix(#a5a5a5,#000000,0.08); }
  assistant .sidebar:backdrop {
    background-color: #d0d0d0;
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9); }

assistant.csd .sidebar {
  border-top-style: none; }

assistant .sidebar label {
  padding: 3px 6px; }

assistant .sidebar label.highlight {
  background-color: mix(#000000,#a5a5a5,0.8); }

/*********
 ! Buttons
**********/
@keyframes needs_attention {
  from {
    background-image: -gtk-gradient(radial, center center, 0, center center, 0.01, to(#B36A87), to(transparent)); }
  to {
    background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(#B36A87), to(transparent)); } }

stacksidebar row.needs-attention > label {
  animation: needs_attention 150ms ease-in;
  background-image: -gtk-gradient(radial, center center, 0, center center, 0.5, to(#B36A87), to(transparent)), -gtk-gradient(radial, center center, 0, center center, 0.5, to(rgba(255, 255, 255, 0.769231)), to(transparent));
  background-size: 6px 6px, 6px 6px;
  background-repeat: no-repeat;
  background-position: right 3px, right 4px; }
  stacksidebar row.needs-attention > label:backdrop {
    background-size: 6px 6px, 0 0; }
  stacksidebar row.needs-attention > label:dir(rtl) {
    background-position: left 3px, left 4px; }

button, headerbar button, .titlebar:not(headerbar) button, toolbar button, toolbar.inline-toolbar button, .suggested-action, headerbar.selection-mode button.suggested-action,
.titlebar:not(headerbar).selection-mode button.suggested-action, .destructive-action, headerbar.selection-mode button,
.titlebar:not(headerbar).selection-mode button, infobar.info button, infobar.warning button, infobar.question button, infobar.error button, popover.background button, frame.app-notification button, button.osd,
#XfceNotifyWindow button, .osd button, .osd spinbutton:not(.vertical) button,
#XfceNotifyWindow spinbutton:not(.vertical) button, .osd spinbutton.vertical button:first-child,
#XfceNotifyWindow spinbutton.vertical button:first-child, .nemo-window toolbar button, .nemo-window .sidebar + separator + box .primary-toolbar button.flat, #login_window button, #shutdown_button button, #restart_button button {
  min-height: 20px;
  min-width: 20px;
  padding: 3px 5px;
  border-width: 1px;
  border-style: solid;
  border-radius: 3px;
  transition: 150ms ease;
  outline-color: transparent; }

calendar.view, calendar.view:backdrop, calendar.button, calendar.button:hover, calendar.button:backdrop, calendar.button:disabled, menu menuitem calendar.button,
.menu menuitem calendar.button,
.context-menu menuitem calendar.button, menu menuitem calendar.button:hover,
.menu menuitem calendar.button:hover,
.context-menu menuitem calendar.button:hover, modelbutton.flat,
menuitem.button.flat, notebook > header > tabs > tab button.flat:hover, notebook > header > tabs > tab button.flat:active, notebook > header > tabs > tab button.flat:active:hover {
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: none; }

spinbutton:not(.vertical) button, .linked:not(.vertical) > combobox > box > button.combo:dir(ltr), toolbar.inline-toolbar toolbutton > button.flat,
toolbar.inline-toolbar toolbutton:backdrop > button.flat {
  border-radius: 0;
  border-left-style: none;
  border-right-style: solid; }
  spinbutton:not(.vertical) button:dir(rtl), .linked:not(.vertical) > combobox > box > button.combo:dir(rtl), toolbar.inline-toolbar toolbutton > button.flat:dir(rtl),
  toolbar.inline-toolbar toolbutton:backdrop > button.flat:dir(rtl) {
    border-radius: 0;
    border-right-style: none;
    border-left-style: solid; }

.linked.vertical > combobox > box > button.combo {
  border-radius: 0;
  border-top-style: none;
  border-bottom-style: solid; }
  .linked.vertical > combobox > box > button.combo:dir(rtl) {
    border-radius: 0;
    border-top-style: none;
    border-bottom-style: solid; }

.inline-toolbar button,
.linked > button, combobox box button, combobox box entry, headerbar .linked > button, .titlebar:not(headerbar) .linked > button, toolbar .linked > button, headerbar.selection-mode .linked > button,
.titlebar:not(headerbar).selection-mode .linked > button, popover.background .linked > button, .nemo-window toolbar .linked > button, .nemo-window toolbar toolitem stack widget button {
  border-width: 1px;
  border-style: solid;
  border-radius: 0;
  border-right-style: none;
  border-left-style: none; }
  .inline-toolbar button:first-child,
  .linked > button:first-child, combobox box button:first-child, combobox box entry:first-child, .linked:not(.vertical) > combobox:first-child > box > button.combo, headerbar .linked > button:first-child, .titlebar:not(headerbar) .linked > button:first-child, toolbar .linked > button:first-child, toolbar.inline-toolbar toolbutton:first-child > button.flat,
  toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat, popover.background .linked > button:first-child, .nemo-window toolbar toolitem stack widget button:first-child {
    border-width: 1px;
    border-radius: 3px;
    border-left-style: solid;
    border-right-style: none;
    border-top-right-radius: 0;
    border-bottom-right-radius: 0; }
    .inline-toolbar button:first-child:dir(rtl),
    .linked > button:first-child:dir(rtl), combobox box button:first-child:dir(rtl), combobox box entry:first-child:dir(rtl), .linked:not(.vertical) > combobox:first-child > box > button.combo:dir(rtl), headerbar .linked > button:first-child:dir(rtl), .titlebar:not(headerbar) .linked > button:first-child:dir(rtl), toolbar .linked > button:first-child:dir(rtl), toolbar.inline-toolbar toolbutton:first-child > button.flat:dir(rtl),
    toolbar.inline-toolbar toolbutton:backdrop:first-child > button.flat:dir(rtl), popover.background .linked > button:first-child:dir(rtl), .nemo-window toolbar toolitem stack widget button:first-child:dir(rtl) {
      border-left-style: none;
      border-right-style: solid; }
  .inline-toolbar button:last-child,
  .linked > button:last-child, combobox box button:last-child, combobox box entry:last-child, .linked:not(.vertical) > combobox:last-child > box > button.combo, headerbar .linked > button:last-child, .titlebar:not(headerbar) .linked > button:last-child, toolbar .linked > button:last-child, toolbar.inline-toolbar toolbutton:last-child > button.flat,
  toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat, popover.background .linked > button:last-child, .nemo-window toolbar toolitem stack widget button:last-child {
    border-width: 1px;
    border-radius: 3px;
    border-left-style: none;
    border-right-style: solid;
    border-top-left-radius: 0;
    border-bottom-left-radius: 0; }
    .inline-toolbar button:last-child:dir(rtl),
    .linked > button:last-child:dir(rtl), combobox box button:last-child:dir(rtl), combobox box entry:last-child:dir(rtl), .linked:not(.vertical) > combobox:last-child > box > button.combo:dir(rtl), headerbar .linked > button:last-child:dir(rtl), .titlebar:not(headerbar) .linked > button:last-child:dir(rtl), toolbar .linked > button:last-child:dir(rtl), toolbar.inline-toolbar toolbutton:last-child > button.flat:dir(rtl),
    toolbar.inline-toolbar toolbutton:backdrop:last-child > button.flat:dir(rtl), popover.background .linked > button:last-child:dir(rtl), .nemo-window toolbar toolitem stack widget button:last-child:dir(rtl) {
      border-left-style: solid;
      border-right-style: none; }
  .inline-toolbar button:only-child,
  .linked > button:only-child, combobox box button:only-child, combobox box entry:only-child, .linked:not(.vertical) > combobox:only-child > box > button.combo, headerbar .linked > button:only-child, .titlebar:not(headerbar) .linked > button:only-child, toolbar .linked > button:only-child, toolbar.inline-toolbar toolbutton:only-child > button.flat,
  toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat, popover.background .linked > button:only-child, .nemo-window toolbar toolitem stack widget button:only-child, .inline-toolbar button:first-child:only-child,
  .linked > button:first-child:only-child, combobox box button:first-child:only-child, combobox box entry:first-child:only-child, .linked:not(.vertical) > combobox:first-child > box > button.combo:only-child, .linked:not(.vertical) > combobox:only-child > box > button.combo:first-child, headerbar .linked > button:first-child:only-child, .titlebar:not(headerbar) .linked > button:first-child:only-child, toolbar .linked > button:first-child:only-child, toolbar.inline-toolbar toolbutton:first-child > button.flat:only-child, toolbar.inline-toolbar toolbutton:only-child > button.flat:first-child, popover.background .linked > button:first-child:only-child, .nemo-window toolbar toolitem stack widget button:first-child:only-child {
    border-width: 1px;
    border-style: solid; }
  .inline-toolbar button:only-child,
  .linked > button:only-child, combobox box button:only-child, combobox box entry:only-child, .linked:not(.vertical) > combobox:only-child > box > button.combo, headerbar .linked > button:only-child, .titlebar:not(headerbar) .linked > button:only-child, toolbar .linked > button:only-child, toolbar.inline-toolbar toolbutton:only-child > button.flat,
  toolbar.inline-toolbar toolbutton:backdrop:only-child > button.flat, popover.background .linked > button:only-child, .nemo-window toolbar toolitem stack widget button:only-child {
    border-radius: 3px; }

.linked.vertical > button {
  border-width: 1px;
  border-style: solid;
  border-radius: 0;
  border-top-style: none;
  border-bottom-style: none; }
  .linked.vertical > button:first-child, .linked.vertical > combobox:first-child > box > button.combo {
    border-width: 1px;
    border-radius: 3px;
    border-top-style: solid;
    border-bottom-style: none;
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0; }
    .linked.vertical > button:first-child:dir(rtl), .linked.vertical > combobox:first-child > box > button.combo:dir(rtl) {
      border-top-style: none;
      border-bottom-style: solid; }
  .linked.vertical > button:last-child, .linked.vertical > combobox:last-child > box > button.combo {
    border-width: 1px;
    border-radius: 3px;
    border-top-style: none;
    border-bottom-style: solid;
    border-top-left-radius: 0;
    border-top-right-radius: 0; }
    .linked.vertical > button:last-child:dir(rtl), .linked.vertical > combobox:last-child > box > button.combo:dir(rtl) {
      border-top-style: solid;
      border-bottom-style: none; }
  .linked.vertical > button:only-child, .linked.vertical > combobox:only-child > box > button.combo, .linked.vertical > button:first-child:only-child, .linked.vertical > combobox:first-child > box > button.combo:only-child, .linked.vertical > combobox:only-child > box > button.combo:first-child {
    border-width: 1px;
    border-style: solid; }
  .linked.vertical > button:only-child, .linked.vertical > combobox:only-child > box > button.combo {
    border-radius: 3px; }

infobar.info button.close, infobar.warning button.close, infobar.question button.close, infobar.error button.close, notebook > header > tabs > arrow {
  border: 1px solid transparent;
  background-color: transparent;
  background-image: none;
  box-shadow: none; }
  infobar.info button.close:focus, infobar.warning button.close:focus, infobar.question button.close:focus, infobar.error button.close:focus, notebook > header > tabs > arrow:focus, infobar.info button.close:hover, infobar.warning button.close:hover, infobar.question button.close:hover, infobar.error button.close:hover, notebook > header > tabs > arrow:hover {
    border: 1px solid rgba(165, 165, 165, 0.3);
    background-color: rgba(0, 0, 0, 0.2);
    background-image: none;
    box-shadow: none; }
  infobar.info button.close:active, infobar.warning button.close:active, infobar.question button.close:active, infobar.error button.close:active, notebook > header > tabs > arrow:active, infobar.info button.close:checked, infobar.warning button.close:checked, infobar.question button.close:checked, infobar.error button.close:checked, notebook > header > tabs > arrow:checked, infobar.info button.close:active:hover, infobar.warning button.close:active:hover, infobar.question button.close:active:hover, infobar.error button.close:active:hover, notebook > header > tabs > arrow:active:hover, infobar.info button.close:checked:hover, infobar.warning button.close:checked:hover, infobar.question button.close:checked:hover, infobar.error button.close:checked:hover, notebook > header > tabs > arrow:checked:hover {
    border: 1px solid rgba(179, 106, 135, 0.3);
    background-color: rgba(0, 0, 0, 0.1);
    background-image: none;
    box-shadow: none; }

button {
  background-color: #8b639b;
  background-image: none;
  border-color: rgba(0, 0, 0, 0.32);
  color: #000000;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
  button:focus, button:hover {
    border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
  button:active, button:active:hover, button:active:focus, button:active:hover:focus, button:checked, button:checked:hover, button:checked:focus, button:checked:hover:focus {
    border-color: rgba(0, 0, 0, 0.32); }
  button:disabled {
    border-color: rgba(0, 0, 0, 0.32); }
  button:active:disabled, button:checked:disabled {
    border-color: rgba(0, 0, 0, 0.32); }
  button.flat {
    color: #000000;
    border-color: rgba(139, 99, 155, 0);
    background-color: rgba(139, 99, 155, 0);
    background-image: none;
    box-shadow: none; }
  button:hover, button.flat:hover {
    background-color: #916ba0;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.4);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    button:hover:focus, button:hover:hover, button.flat:hover:focus, button.flat:hover:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
    button:hover:active, button:hover:active:hover, button:hover:active:focus, button:hover:active:hover:focus, button:hover:checked, button:hover:checked:hover, button:hover:checked:focus, button:hover:checked:hover:focus, button.flat:hover:active, button.flat:hover:active:hover, button.flat:hover:active:focus, button.flat:hover:active:hover:focus, button.flat:hover:checked, button.flat:hover:checked:hover, button.flat:hover:checked:focus, button.flat:hover:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.4); }
    button:hover:disabled, button.flat:hover:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    button:hover:active:disabled, button:hover:checked:disabled, button.flat:hover:active:disabled, button.flat:hover:checked:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
  button:focus, button.flat:focus {
    background-color: #916ba0;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.4);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    button:focus:focus, button:focus:hover, button.flat:focus:focus, button.flat:focus:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
    button:focus:active, button:focus:active:hover, button:focus:active:focus, button:focus:active:hover:focus, button:focus:checked, button:focus:checked:hover, button:focus:checked:focus, button:focus:checked:hover:focus, button.flat:focus:active, button.flat:focus:active:hover, button.flat:focus:active:focus, button.flat:focus:active:hover:focus, button.flat:focus:checked, button.flat:focus:checked:hover, button.flat:focus:checked:focus, button.flat:focus:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.4); }
    button:focus:disabled, button.flat:focus:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    button:focus:active:disabled, button:focus:checked:disabled, button.flat:focus:active:disabled, button.flat:focus:checked:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    button:focus:hover, button.flat:focus:hover {
      background-color: #9772a5;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
      button:focus:hover:focus, button:focus:hover:hover, button.flat:focus:hover:focus, button.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      button:focus:hover:active, button:focus:hover:active:hover, button:focus:hover:active:focus, button:focus:hover:active:hover:focus, button:focus:hover:checked, button:focus:hover:checked:hover, button:focus:hover:checked:focus, button:focus:hover:checked:hover:focus, button.flat:focus:hover:active, button.flat:focus:hover:active:hover, button.flat:focus:hover:active:focus, button.flat:focus:hover:active:hover:focus, button.flat:focus:hover:checked, button.flat:focus:hover:checked:hover, button.flat:focus:hover:checked:focus, button.flat:focus:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      button:focus:hover:disabled, button.flat:focus:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      button:focus:hover:active:disabled, button:focus:hover:checked:disabled, button.flat:focus:hover:active:disabled, button.flat:focus:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
  button:checked, button:active, button.flat:checked, button.flat:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
    border-color: rgba(0, 0, 0, 0.32); }
    button:checked:focus, button:checked:hover, button:active:focus, button:active:hover, button.flat:checked:focus, button.flat:checked:hover, button.flat:active:focus, button.flat:active:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    button:checked:active, button:checked:active:hover, button:checked:active:focus, button:checked:active:hover:focus, button:checked:checked, button:checked:checked:hover, button:checked:checked:focus, button:checked:checked:hover:focus, button:active:active, button:active:active:hover, button:active:active:focus, button:active:active:hover:focus, button:active:checked, button:active:checked:hover, button:active:checked:focus, button:active:checked:hover:focus, button.flat:checked:active, button.flat:checked:active:hover, button.flat:checked:active:focus, button.flat:checked:active:hover:focus, button.flat:checked:checked, button.flat:checked:checked:hover, button.flat:checked:checked:focus, button.flat:checked:checked:hover:focus, button.flat:active:active, button.flat:active:active:hover, button.flat:active:active:focus, button.flat:active:active:hover:focus, button.flat:active:checked, button.flat:active:checked:hover, button.flat:active:checked:focus, button.flat:active:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    button:checked:disabled, button:active:disabled, button.flat:checked:disabled, button.flat:active:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    button:checked:active:disabled, button:checked:checked:disabled, button:active:active:disabled, button:active:checked:disabled, button.flat:checked:active:disabled, button.flat:checked:checked:disabled, button.flat:active:active:disabled, button.flat:active:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    button:checked:focus, button:checked:hover, button:active:focus, button:active:hover, button.flat:checked:focus, button.flat:checked:hover, button.flat:active:focus, button.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  button:focus, button:hover, button.flat:focus, button.flat:hover {
    color: #000000; }
  button:disabled:disabled, button.flat:disabled:disabled {
    background-color: alpha(mix(#8b639b,#000000,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(#8b639b,#000000,0.5);
    box-shadow: none; }
  button:active:disabled, button:checked:disabled, button.flat:active:disabled, button.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  button.separator, button .separator {
    border: 1px solid currentColor;
    color: rgba(139, 99, 155, 0.9); }
    button.separator:disabled, button .separator:disabled {
      color: rgba(139, 99, 155, 0.85); }
  .inline-toolbar button,
  .linked > button {
    box-shadow: inset -1px 0 rgba(0, 0, 0, 0.22), 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .inline-toolbar button:focus, .inline-toolbar button:hover,
    .linked > button:focus,
    .linked > button:hover {
      box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.22),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    .inline-toolbar button:active, .inline-toolbar button:active:hover, .inline-toolbar button:active:focus, .inline-toolbar button:active:hover:focus, .inline-toolbar button:checked, .inline-toolbar button:checked:hover, .inline-toolbar button:checked:focus, .inline-toolbar button:checked:hover:focus,
    .linked > button:active,
    .linked > button:active:hover,
    .linked > button:active:focus,
    .linked > button:active:hover:focus,
    .linked > button:checked,
    .linked > button:checked:hover,
    .linked > button:checked:focus,
    .linked > button:checked:hover:focus {
      box-shadow: inset -1px 0 rgba(0, 0, 0, 0.22), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
    .inline-toolbar button:disabled,
    .linked > button:disabled {
      box-shadow: inset -1px 0 #6f4f7c; }
    .inline-toolbar button:last-child, .inline-toolbar button:only-child,
    .linked > button:last-child,
    .linked > button:only-child {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .inline-toolbar button:last-child:hover, .inline-toolbar button:only-child:hover,
    .linked > button:last-child:hover,
    .linked > button:only-child:hover {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    .inline-toolbar button:disabled:last-child, .inline-toolbar button:disabled:only-child, .inline-toolbar button:active:disabled:last-child, .inline-toolbar button:active:disabled:only-child, .inline-toolbar button:checked:disabled:last-child, .inline-toolbar button:checked:disabled:only-child,
    .linked > button:disabled:last-child,
    .linked > button:disabled:only-child,
    .linked > button:active:disabled:last-child,
    .linked > button:active:disabled:only-child,
    .linked > button:checked:disabled:last-child,
    .linked > button:checked:disabled:only-child {
      box-shadow: none; }
    .inline-toolbar button:active:last-child, .inline-toolbar button:active:last-child:focus, .inline-toolbar button:active:last-child:hover, .inline-toolbar button:active:last-child:hover:focus, .inline-toolbar button:checked:last-child, .inline-toolbar button:checked:last-child:focus, .inline-toolbar button:checked:last-child:hover, .inline-toolbar button:checked:last-child:hover:focus,
    .linked > button:active:last-child,
    .linked > button:active:last-child:focus,
    .linked > button:active:last-child:hover,
    .linked > button:active:last-child:hover:focus,
    .linked > button:checked:last-child,
    .linked > button:checked:last-child:focus,
    .linked > button:checked:last-child:hover,
    .linked > button:checked:last-child:hover:focus {
      box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    .inline-toolbar button:active:only-child, .inline-toolbar button:active:only-child:focus, .inline-toolbar button:active:only-child:hover, .inline-toolbar button:active:only-child:hover:focus, .inline-toolbar button:checked:only-child, .inline-toolbar button:checked:only-child:focus, .inline-toolbar button:checked:only-child:hover, .inline-toolbar button:checked:only-child:hover:focus,
    .linked > button:active:only-child,
    .linked > button:active:only-child:focus,
    .linked > button:active:only-child:hover,
    .linked > button:active:only-child:hover:focus,
    .linked > button:checked:only-child,
    .linked > button:checked:only-child:focus,
    .linked > button:checked:only-child:hover,
    .linked > button:checked:only-child:hover:focus {
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  .linked.vertical > button {
    box-shadow: inset 0 -1px rgba(0, 0, 0, 0.22), 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .linked.vertical > button:focus, .linked.vertical > button:hover {
      box-shadow: inset 0 -1px mix(#B36A87,rgba(0, 0, 0, 0.22),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    .linked.vertical > button:active, .linked.vertical > button:active:hover, .linked.vertical > button:active:focus, .linked.vertical > button:active:hover:focus, .linked.vertical > button:checked, .linked.vertical > button:checked:hover, .linked.vertical > button:checked:focus, .linked.vertical > button:checked:hover:focus {
      box-shadow: inset 0 -1px rgba(0, 0, 0, 0.22), inset 1px 0 rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.05); }
    .linked.vertical > button:disabled {
      box-shadow: inset 0 -1px #6f4f7c; }
    .linked.vertical > button:last-child, .linked.vertical > button:only-child {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .linked.vertical > button:last-child:hover, .linked.vertical > button:only-child:hover {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    .linked.vertical > button:disabled:last-child, .linked.vertical > button:disabled:only-child, .linked.vertical > button:active:disabled:last-child, .linked.vertical > button:active:disabled:only-child, .linked.vertical > button:checked:disabled:last-child, .linked.vertical > button:checked:disabled:only-child {
      box-shadow: none; }
    .linked.vertical > button:active:last-child, .linked.vertical > button:active:last-child:focus, .linked.vertical > button:active:last-child:hover, .linked.vertical > button:active:last-child:hover:focus, .linked.vertical > button:checked:last-child, .linked.vertical > button:checked:last-child:focus, .linked.vertical > button:checked:last-child:hover, .linked.vertical > button:checked:last-child:hover:focus {
      box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    .linked.vertical > button:active:only-child, .linked.vertical > button:active:only-child:focus, .linked.vertical > button:active:only-child:hover, .linked.vertical > button:active:only-child:hover:focus, .linked.vertical > button:checked:only-child, .linked.vertical > button:checked:only-child:focus, .linked.vertical > button:checked:only-child:hover, .linked.vertical > button:checked:only-child:hover:focus {
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  button.circular, button.nautilus-circular-button.image-button, button.circular-button {
    padding: 0;
    min-width: 28px;
    min-height: 28px;
    border-radius: 9999px;
    -gtk-outline-radius: 9999px; }
    button.circular label, button.nautilus-circular-button.image-button label, button.circular-button label {
      padding: 0; }

spinbutton:disabled {
  opacity: .4; }

spinbutton button {
  color: #000000; }
  spinbutton button:active, spinbutton button:checked, spinbutton button:hover {
    background-color: #a282af;
    background-image: none; }
  spinbutton button:disabled {
    color: mix(#000000,#a5a5a5,0.7); }
  spinbutton button:backdrop {
    color: mix(#d0d0d0,mix(#000000,#a5a5a5,0.5),0.9); }
  spinbutton button:backdrop:disabled {
    color: rgba(127, 127, 127, 0.8); }

spinbutton:not(.vertical) {
  /*@extend %entry;*/
  background-color: #d0d0d0;
  background-image: none;
  /*@include border($base_color);*/
  padding: 0;
  border-radius: 3px;
  color: #000000;
  caret-color: #000000;
  /*&:focus, &:active { border-color: border_focus($borders_color); }*/ }
  spinbutton:not(.vertical):disabled {
    background-color: #bbbbbb;
    background-image: none;
    color: mix(#d0d0d0,#000000,0.5); }
  spinbutton:not(.vertical) entry {
    border-radius: 3px 0 0 3px;
    border-right-width: 0;
    box-shadow: none; }
  spinbutton:not(.vertical) button {
    border-radius: 0;
    /*border-color: alpha($borders_color, .8);*/
    /*border-style: none none none solid;*/
    background-image: none;
    box-shadow: none;
    /*
                @if (lightness($bg_color) > 50) {
                    background-color: shade($bg_color, 1.08);
                }

                &:hover { 
                    @if (lightness($bg_color) > 50) {
                        background-color: shade($bg_color, 1.11);
                    } 
                }
		*/ }
    spinbutton:not(.vertical) button:dir(rtl) {
      border-style: none solid none none; }
    spinbutton:not(.vertical) button:active {
      box-shadow: inset 0 2px 3px -1px rgba(0, 0, 0, 0.2); }
    spinbutton:not(.vertical) button:backdrop {
      border-color: alpha(mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9),0.8); }
    spinbutton:not(.vertical) button:backdrop:disabled {
      border-style: none none none solid; }
      spinbutton:not(.vertical) button:backdrop:disabled:dir(rtl) {
        border-style: none solid none none; }
    spinbutton:not(.vertical) button:dir(rtl):first-child {
      border-radius: 3px 0 0 3px; }
    spinbutton:not(.vertical) button:dir(ltr):last-child {
      border-radius: 0 3px 3px 0; }

spinbutton.vertical button, spinbutton.vertical entry {
  min-width: 0;
  padding-left: 1px;
  padding-right: 1px; }

spinbutton.vertical entry {
  border-radius: 0;
  border-top-width: 0;
  border-bottom-width: 0; }

spinbutton.vertical button.up {
  border-style: solid solid none solid;
  border-radius: 3px 3px 0 0; }

spinbutton.vertical button.down {
  border-style: none solid solid solid;
  border-radius: 0 0 3px 3px; }

/******************
! ComboBoxes *
*******************/
combobox button.combo {
  min-width: 0;
  padding-left: 5px;
  padding-right: 5px; }

combobox arrow {
  -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
  min-height: 16px;
  min-width: 16px; }

combobox box button, combobox box entry {
  padding: 3px 5px; }

/*********
 ! Entry *
**********/
.linked:not(.vertical) > entry {
  border-width: 1px;
  border-radius: 0;
  border-right-width: 0;
  border-left-width: 0; }
  .linked:not(.vertical) > entry:first-child {
    border-width: 1px;
    border-radius: 3px;
    border-right-width: 0;
    border-bottom-right-radius: 0;
    border-top-right-radius: 0; }
    .linked:not(.vertical) > entry:first-child:dir(rtl) {
      border-left-width: 0;
      border-right-width: 1px; }
  .linked:not(.vertical) > entry:last-child {
    border-width: 1px;
    border-radius: 3px;
    border-left-width: 0;
    border-bottom-left-radius: 0;
    border-top-left-radius: 0; }
    .linked:not(.vertical) > entry:last-child:dir(rtl) {
      border-left-width: 1px;
      border-right-width: 0; }
  .linked:not(.vertical) > entry:only-child, .linked:not(.vertical) > entry:first-child:only-child {
    border-width: 1px; }
  .linked:not(.vertical) > entry:only-child {
    border-radius: 3px; }

.linked.vertical > entry {
  border-width: 1px;
  border-radius: 0;
  border-top-width: 0;
  border-bottom-width: 0; }
  .linked.vertical > entry:first-child {
    border-width: 1px;
    border-radius: 3px;
    border-top-width: 1px;
    border-bottom-width: 0;
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0; }
    .linked.vertical > entry:first-child:dir(rtl) {
      border-top-width: 0;
      border-bottom-width: 1px; }
  .linked.vertical > entry:last-child {
    border-width: 1px;
    border-radius: 3px;
    border-top-width: 0;
    border-bottom-width: 1px;
    border-top-left-radius: 0;
    border-top-right-radius: 0; }
    .linked.vertical > entry:last-child:dir(rtl) {
      border-top-width: 1px;
      border-bottom-width: 0; }
  .linked.vertical > entry:only-child, .linked.vertical > entry:first-child:only-child {
    border-width: 1px; }
  .linked.vertical > entry:only-child {
    border-radius: 3px; }

entry, menuitem entry, popover.background entry, .osd entry,
#XfceNotifyWindow entry, #login_window entry {
  border-width: 1px;
  border-style: solid;
  border-radius: 3px;
  border-color: #848484;
  transition: border 100ms ease-out;
  box-shadow: inset 1px 0 rgba(0, 0, 0, 0.1), inset 0 1px rgba(0, 0, 0, 0.12), inset -1px 0 rgba(0, 0, 0, 0.1), inset 0 -1px rgba(0, 0, 0, 0.05); }
  entry:focus, popover.background entry:focus,
  #XfceNotifyWindow entry:focus, #login_window entry:focus, entry:hover, popover.background entry:hover,
  #XfceNotifyWindow entry:hover, #login_window entry:hover, entry:active, popover.background entry:active,
  #XfceNotifyWindow entry:active, #login_window entry:active {
    transition: border 100ms ease-in; }
  entry:selected, popover.background entry:selected,
  #XfceNotifyWindow entry:selected, #login_window entry:selected, entry:selected:selected:focus,
  #XfceNotifyWindow entry:selected:selected:focus, #login_window entry:selected:selected:focus {
    background-color: #B36A87;
    color: #000000; }
  entry:disabled, popover.background entry:disabled,
  #XfceNotifyWindow entry:disabled, #login_window entry:disabled {
    box-shadow: none; }
  entry progress, popover.background entry progress, .osd entry progress,
  #XfceNotifyWindow entry progress, #login_window entry progress {
    background-color: #B36A87;
    background-image: none;
    border-width: 0;
    border-radius: 3px;
    color: #000000; }
  entry image.left,
  #XfceNotifyWindow entry image.left, #login_window entry image.left {
    padding-right: 3px; }
  entry image.right,
  #XfceNotifyWindow entry image.right, #login_window entry image.right {
    padding-left: 3px; }
  entry.warning, popover.background entry.warning,
  #XfceNotifyWindow entry.warning, #login_window entry.warning {
    color: #fff;
    border-color: #bf5600;
    background-color: mix(#d0d0d0,#ef6c00,0.6); }
    entry.warning image,
    #XfceNotifyWindow entry.warning image, #login_window entry.warning image {
      color: #fff; }
    entry.warning:focus,
    #XfceNotifyWindow entry.warning:focus, #login_window entry.warning:focus {
      color: #fff;
      border-color: mix(#B36A87,#ef6c00,0.3);
      background-color: #ef6c00;
      box-shadow: none; }
    entry.warning selection,
    #XfceNotifyWindow entry.warning selection, #login_window entry.warning selection {
      background-color: #fff;
      color: #ef6c00; }
  entry.error, popover.background entry.error,
  #XfceNotifyWindow entry.error, #login_window entry.error {
    color: #fff;
    border-color: #e21b0c;
    background-color: mix(#d0d0d0,#f44336,0.6); }
    entry.error image,
    #XfceNotifyWindow entry.error image, #login_window entry.error image {
      color: #fff; }
    entry.error:focus,
    #XfceNotifyWindow entry.error:focus, #login_window entry.error:focus {
      color: #fff;
      border-color: mix(#B36A87,#f44336,0.3);
      background-color: #f44336;
      box-shadow: none; }
    entry.error selection,
    #XfceNotifyWindow entry.error selection, #login_window entry.error selection {
      background-color: #fff;
      color: #f44336; }
  entry.search-missing, popover.background entry.search-missing,
  #XfceNotifyWindow entry.search-missing, #login_window entry.search-missing {
    color: #fff;
    border-color: #e21b0c;
    background-color: mix(#d0d0d0,#f44336,0.6); }
    entry.search-missing image,
    #XfceNotifyWindow entry.search-missing image, #login_window entry.search-missing image {
      color: #fff; }
    entry.search-missing:focus,
    #XfceNotifyWindow entry.search-missing:focus, #login_window entry.search-missing:focus {
      color: #fff;
      border-color: mix(#B36A87,#f44336,0.3);
      background-color: #f44336;
      box-shadow: none; }
    entry.search-missing selection,
    #XfceNotifyWindow entry.search-missing selection, #login_window entry.search-missing selection {
      background-color: #fff;
      color: #f44336; }

entry {
  background-color: #d0d0d0;
  background-image: none;
  border-color: shade(mix(#d0d0d0,#000000,0.2),0.8);
  padding: 3px;
  color: #000000;
  caret-color: #000000;
  -gtk-secondary-caret-color: #000000; }
  entry:focus, entry:hover {
    border-color: mix(#B36A87,mix(#d0d0d0,#000000,0.2),0.3); }
  entry:active, entry:active:hover, entry:active:focus, entry:active:hover:focus, entry:checked, entry:checked:hover, entry:checked:focus, entry:checked:hover:focus {
    border-color: shade(mix(#d0d0d0,#000000,0.2),0.7); }
  entry:disabled {
    border-color: shade(mix(#d0d0d0,#000000,0.2),0.85); }
  entry:active:disabled, entry:checked:disabled {
    border-color: shade(mix(#d0d0d0,#000000,0.2),0.8); }
  entry:focus, entry:active {
    border-color: mix(#B36A87,mix(#a5a5a5,#000000,0.08),0.3); }
  entry:disabled {
    background-color: #bbbbbb;
    background-image: none;
    border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.8);
    /*@include border(mix($bg, $fg, .20));*/
    /*border-color: alpha(mix($bg, $fg, .15), .8);*/
    /*opacity: .8;*/
    color: mix(#d0d0d0,#000000,0.5); }
    entry:disabled:focus, entry:disabled:hover {
      border-color: mix(#B36A87,alpha(mix(#d0d0d0,#000000,0.2),0.4),0.3); }
    entry:disabled:active, entry:disabled:active:hover, entry:disabled:active:focus, entry:disabled:active:hover:focus, entry:disabled:checked, entry:disabled:checked:hover, entry:disabled:checked:focus, entry:disabled:checked:hover:focus {
      border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.7); }
    entry:disabled:disabled {
      border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.85); }
    entry:disabled:active:disabled, entry:disabled:checked:disabled {
      border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.8); }

/*********
 ! Buttons
**********/
/******************
! ComboBoxes *
*******************/
/*********
 ! Buttons
**********/
/******************
! ComboBoxes *
*******************/
/*********
 ! Toolbar
**********/
toolbar {
  padding: 2px;
  border-style: none; }
  toolbar.horizontal separator {
    margin: 0 5px 1px; }
  toolbar.vertical separator {
    margin: 5px 1px 5px 0; }

headerbar {
  border-width: 0 0 1px;
  border-style: solid; }
  headerbar entry, headerbar spinbutton, headerbar separator, headerbar button {
    margin-top: 6px;
    margin-bottom: 6px; }
  headerbar switch {
    margin-top: 4px;
    margin-bottom: 4px; }
  window:not(.tiled):not(.maximized) separator:first-child + headerbar:backdrop, window:not(.tiled):not(.maximized) separator:first-child + headerbar,
  window:not(.tiled):not(.maximized) headerbar:first-child:backdrop,
  window:not(.tiled):not(.maximized) headerbar:first-child {
    border-top-left-radius: 3px; }
  window:not(.tiled):not(.maximized) headerbar:last-child:backdrop, window:not(.tiled):not(.maximized) headerbar:last-child {
    border-top-right-radius: 3px; }

headerbar, .titlebar:not(headerbar) {
  background-color: #0f0f0f;
  background-image: none;
  border-color: #0c0c0c;
  color: #a5a5a5;
  background-color: #0f0f0f;
  background-image: none;
  border-radius: 3px 3px 0 0;
  color: mix(#a5a5a5,#0f0f0f,0.1);
  padding: 0 6px;
  min-height: 42px; }
  headerbar:focus, .titlebar:focus:not(headerbar), headerbar:hover, .titlebar:hover:not(headerbar) {
    border-color: mix(#B36A87,#0f0f0f,0.3); }
  headerbar:active, .titlebar:active:not(headerbar), headerbar:active:hover, .titlebar:active:hover:not(headerbar), headerbar:active:focus, .titlebar:active:focus:not(headerbar), headerbar:active:hover:focus, .titlebar:active:hover:focus:not(headerbar), headerbar:checked, .titlebar:checked:not(headerbar), headerbar:checked:hover, .titlebar:checked:hover:not(headerbar), headerbar:checked:focus, .titlebar:checked:focus:not(headerbar), headerbar:checked:hover:focus, .titlebar:checked:hover:focus:not(headerbar) {
    border-color: #0b0b0b; }
  headerbar:disabled, .titlebar:disabled:not(headerbar) {
    border-color: #0d0d0d; }
  headerbar:active:disabled, .titlebar:active:disabled:not(headerbar), headerbar:checked:disabled, .titlebar:checked:disabled:not(headerbar) {
    border-color: #0c0c0c; }
  headerbar:disabled, .titlebar:disabled:not(headerbar) {
    background-color: #0e0e0e;
    background-image: none;
    color: mix(#a5a5a5,#0f0f0f,0.5); }
  headerbar .title, .titlebar:not(headerbar) .title {
    font-weight: bold;
    padding: 0 6px; }
  headerbar .subtitle, .titlebar:not(headerbar) .subtitle {
    font-size: smaller;
    padding: 0 6px; }
  headerbar button, .titlebar:not(headerbar) button {
    background-color: #0f0f0f;
    background-image: none;
    border-color: rgba(132, 132, 132, 0.22);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    headerbar button:focus, .titlebar:not(headerbar) button:focus, headerbar button:hover, .titlebar:not(headerbar) button:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
    headerbar button:active, .titlebar:not(headerbar) button:active, headerbar button:active:hover, .titlebar:not(headerbar) button:active:hover, headerbar button:active:focus, .titlebar:not(headerbar) button:active:focus, headerbar button:active:hover:focus, .titlebar:not(headerbar) button:active:hover:focus, headerbar button:checked, .titlebar:not(headerbar) button:checked, headerbar button:checked:hover, .titlebar:not(headerbar) button:checked:hover, headerbar button:checked:focus, .titlebar:not(headerbar) button:checked:focus, headerbar button:checked:hover:focus, .titlebar:not(headerbar) button:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.22); }
    headerbar button:disabled, .titlebar:not(headerbar) button:disabled {
      border-color: rgba(140, 140, 140, 0.22); }
    headerbar button:active:disabled, .titlebar:not(headerbar) button:active:disabled, headerbar button:checked:disabled, .titlebar:not(headerbar) button:checked:disabled {
      border-color: rgba(132, 132, 132, 0.22); }
    headerbar button.flat, .titlebar:not(headerbar) button.flat {
      color: #a5a5a5;
      border-color: rgba(15, 15, 15, 0);
      background-color: rgba(15, 15, 15, 0);
      background-image: none;
      box-shadow: none; }
    headerbar button:hover, .titlebar:not(headerbar) button:hover, headerbar button.flat:hover, .titlebar:not(headerbar) button.flat:hover {
      background-color: #101010;
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      headerbar button:hover:focus, .titlebar:not(headerbar) button:hover:focus, headerbar button:hover:hover, .titlebar:not(headerbar) button:hover:hover, headerbar button.flat:hover:focus, .titlebar:not(headerbar) button.flat:hover:focus, headerbar button.flat:hover:hover, .titlebar:not(headerbar) button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      headerbar button:hover:active, .titlebar:not(headerbar) button:hover:active, headerbar button:hover:active:hover, .titlebar:not(headerbar) button:hover:active:hover, headerbar button:hover:active:focus, .titlebar:not(headerbar) button:hover:active:focus, headerbar button:hover:active:hover:focus, .titlebar:not(headerbar) button:hover:active:hover:focus, headerbar button:hover:checked, .titlebar:not(headerbar) button:hover:checked, headerbar button:hover:checked:hover, .titlebar:not(headerbar) button:hover:checked:hover, headerbar button:hover:checked:focus, .titlebar:not(headerbar) button:hover:checked:focus, headerbar button:hover:checked:hover:focus, .titlebar:not(headerbar) button:hover:checked:hover:focus, headerbar button.flat:hover:active, .titlebar:not(headerbar) button.flat:hover:active, headerbar button.flat:hover:active:hover, .titlebar:not(headerbar) button.flat:hover:active:hover, headerbar button.flat:hover:active:focus, .titlebar:not(headerbar) button.flat:hover:active:focus, headerbar button.flat:hover:active:hover:focus, .titlebar:not(headerbar) button.flat:hover:active:hover:focus, headerbar button.flat:hover:checked, .titlebar:not(headerbar) button.flat:hover:checked, headerbar button.flat:hover:checked:hover, .titlebar:not(headerbar) button.flat:hover:checked:hover, headerbar button.flat:hover:checked:focus, .titlebar:not(headerbar) button.flat:hover:checked:focus, headerbar button.flat:hover:checked:hover:focus, .titlebar:not(headerbar) button.flat:hover:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      headerbar button:hover:disabled, .titlebar:not(headerbar) button:hover:disabled, headerbar button.flat:hover:disabled, .titlebar:not(headerbar) button.flat:hover:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      headerbar button:hover:active:disabled, .titlebar:not(headerbar) button:hover:active:disabled, headerbar button:hover:checked:disabled, .titlebar:not(headerbar) button:hover:checked:disabled, headerbar button.flat:hover:active:disabled, .titlebar:not(headerbar) button.flat:hover:active:disabled, headerbar button.flat:hover:checked:disabled, .titlebar:not(headerbar) button.flat:hover:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
    headerbar button:focus, .titlebar:not(headerbar) button:focus, headerbar button.flat:focus, .titlebar:not(headerbar) button.flat:focus {
      background-color: #101010;
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      headerbar button:focus:focus, .titlebar:not(headerbar) button:focus:focus, headerbar button:focus:hover, .titlebar:not(headerbar) button:focus:hover, headerbar button.flat:focus:focus, .titlebar:not(headerbar) button.flat:focus:focus, headerbar button.flat:focus:hover, .titlebar:not(headerbar) button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      headerbar button:focus:active, .titlebar:not(headerbar) button:focus:active, headerbar button:focus:active:hover, .titlebar:not(headerbar) button:focus:active:hover, headerbar button:focus:active:focus, .titlebar:not(headerbar) button:focus:active:focus, headerbar button:focus:active:hover:focus, .titlebar:not(headerbar) button:focus:active:hover:focus, headerbar button:focus:checked, .titlebar:not(headerbar) button:focus:checked, headerbar button:focus:checked:hover, .titlebar:not(headerbar) button:focus:checked:hover, headerbar button:focus:checked:focus, .titlebar:not(headerbar) button:focus:checked:focus, headerbar button:focus:checked:hover:focus, .titlebar:not(headerbar) button:focus:checked:hover:focus, headerbar button.flat:focus:active, .titlebar:not(headerbar) button.flat:focus:active, headerbar button.flat:focus:active:hover, .titlebar:not(headerbar) button.flat:focus:active:hover, headerbar button.flat:focus:active:focus, .titlebar:not(headerbar) button.flat:focus:active:focus, headerbar button.flat:focus:active:hover:focus, .titlebar:not(headerbar) button.flat:focus:active:hover:focus, headerbar button.flat:focus:checked, .titlebar:not(headerbar) button.flat:focus:checked, headerbar button.flat:focus:checked:hover, .titlebar:not(headerbar) button.flat:focus:checked:hover, headerbar button.flat:focus:checked:focus, .titlebar:not(headerbar) button.flat:focus:checked:focus, headerbar button.flat:focus:checked:hover:focus, .titlebar:not(headerbar) button.flat:focus:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      headerbar button:focus:disabled, .titlebar:not(headerbar) button:focus:disabled, headerbar button.flat:focus:disabled, .titlebar:not(headerbar) button.flat:focus:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      headerbar button:focus:active:disabled, .titlebar:not(headerbar) button:focus:active:disabled, headerbar button:focus:checked:disabled, .titlebar:not(headerbar) button:focus:checked:disabled, headerbar button.flat:focus:active:disabled, .titlebar:not(headerbar) button.flat:focus:active:disabled, headerbar button.flat:focus:checked:disabled, .titlebar:not(headerbar) button.flat:focus:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
      headerbar button:focus:hover, .titlebar:not(headerbar) button:focus:hover, headerbar button.flat:focus:hover, .titlebar:not(headerbar) button.flat:focus:hover {
        background-color: #111111;
        background-image: none;
        border-color: rgba(132, 132, 132, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        headerbar button:focus:hover:focus, .titlebar:not(headerbar) button:focus:hover:focus, headerbar button:focus:hover:hover, .titlebar:not(headerbar) button:focus:hover:hover, headerbar button.flat:focus:hover:focus, .titlebar:not(headerbar) button.flat:focus:hover:focus, headerbar button.flat:focus:hover:hover, .titlebar:not(headerbar) button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
        headerbar button:focus:hover:active, .titlebar:not(headerbar) button:focus:hover:active, headerbar button:focus:hover:active:hover, .titlebar:not(headerbar) button:focus:hover:active:hover, headerbar button:focus:hover:active:focus, .titlebar:not(headerbar) button:focus:hover:active:focus, headerbar button:focus:hover:active:hover:focus, .titlebar:not(headerbar) button:focus:hover:active:hover:focus, headerbar button:focus:hover:checked, .titlebar:not(headerbar) button:focus:hover:checked, headerbar button:focus:hover:checked:hover, .titlebar:not(headerbar) button:focus:hover:checked:hover, headerbar button:focus:hover:checked:focus, .titlebar:not(headerbar) button:focus:hover:checked:focus, headerbar button:focus:hover:checked:hover:focus, .titlebar:not(headerbar) button:focus:hover:checked:hover:focus, headerbar button.flat:focus:hover:active, .titlebar:not(headerbar) button.flat:focus:hover:active, headerbar button.flat:focus:hover:active:hover, .titlebar:not(headerbar) button.flat:focus:hover:active:hover, headerbar button.flat:focus:hover:active:focus, .titlebar:not(headerbar) button.flat:focus:hover:active:focus, headerbar button.flat:focus:hover:active:hover:focus, .titlebar:not(headerbar) button.flat:focus:hover:active:hover:focus, headerbar button.flat:focus:hover:checked, .titlebar:not(headerbar) button.flat:focus:hover:checked, headerbar button.flat:focus:hover:checked:hover, .titlebar:not(headerbar) button.flat:focus:hover:checked:hover, headerbar button.flat:focus:hover:checked:focus, .titlebar:not(headerbar) button.flat:focus:hover:checked:focus, headerbar button.flat:focus:hover:checked:hover:focus, .titlebar:not(headerbar) button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.3); }
        headerbar button:focus:hover:disabled, .titlebar:not(headerbar) button:focus:hover:disabled, headerbar button.flat:focus:hover:disabled, .titlebar:not(headerbar) button.flat:focus:hover:disabled {
          border-color: rgba(140, 140, 140, 0.3); }
        headerbar button:focus:hover:active:disabled, .titlebar:not(headerbar) button:focus:hover:active:disabled, headerbar button:focus:hover:checked:disabled, .titlebar:not(headerbar) button:focus:hover:checked:disabled, headerbar button.flat:focus:hover:active:disabled, .titlebar:not(headerbar) button.flat:focus:hover:active:disabled, headerbar button.flat:focus:hover:checked:disabled, .titlebar:not(headerbar) button.flat:focus:hover:checked:disabled {
          border-color: rgba(132, 132, 132, 0.3); }
    headerbar button:checked, .titlebar:not(headerbar) button:checked, headerbar button:active, .titlebar:not(headerbar) button:active, headerbar button.flat:checked, .titlebar:not(headerbar) button.flat:checked, headerbar button.flat:active, .titlebar:not(headerbar) button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
      border-color: rgba(132, 132, 132, 0.22); }
      headerbar button:checked:focus, .titlebar:not(headerbar) button:checked:focus, headerbar button:checked:hover, .titlebar:not(headerbar) button:checked:hover, headerbar button:active:focus, .titlebar:not(headerbar) button:active:focus, headerbar button:active:hover, .titlebar:not(headerbar) button:active:hover, headerbar button.flat:checked:focus, .titlebar:not(headerbar) button.flat:checked:focus, headerbar button.flat:checked:hover, .titlebar:not(headerbar) button.flat:checked:hover, headerbar button.flat:active:focus, .titlebar:not(headerbar) button.flat:active:focus, headerbar button.flat:active:hover, .titlebar:not(headerbar) button.flat:active:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
      headerbar button:checked:active, .titlebar:not(headerbar) button:checked:active, headerbar button:checked:active:hover, .titlebar:not(headerbar) button:checked:active:hover, headerbar button:checked:active:focus, .titlebar:not(headerbar) button:checked:active:focus, headerbar button:checked:active:hover:focus, .titlebar:not(headerbar) button:checked:active:hover:focus, headerbar button:checked:checked, .titlebar:not(headerbar) button:checked:checked, headerbar button:checked:checked:hover, .titlebar:not(headerbar) button:checked:checked:hover, headerbar button:checked:checked:focus, .titlebar:not(headerbar) button:checked:checked:focus, headerbar button:checked:checked:hover:focus, .titlebar:not(headerbar) button:checked:checked:hover:focus, headerbar button:active:active, .titlebar:not(headerbar) button:active:active, headerbar button:active:active:hover, .titlebar:not(headerbar) button:active:active:hover, headerbar button:active:active:focus, .titlebar:not(headerbar) button:active:active:focus, headerbar button:active:active:hover:focus, .titlebar:not(headerbar) button:active:active:hover:focus, headerbar button:active:checked, .titlebar:not(headerbar) button:active:checked, headerbar button:active:checked:hover, .titlebar:not(headerbar) button:active:checked:hover, headerbar button:active:checked:focus, .titlebar:not(headerbar) button:active:checked:focus, headerbar button:active:checked:hover:focus, .titlebar:not(headerbar) button:active:checked:hover:focus, headerbar button.flat:checked:active, .titlebar:not(headerbar) button.flat:checked:active, headerbar button.flat:checked:active:hover, .titlebar:not(headerbar) button.flat:checked:active:hover, headerbar button.flat:checked:active:focus, .titlebar:not(headerbar) button.flat:checked:active:focus, headerbar button.flat:checked:active:hover:focus, .titlebar:not(headerbar) button.flat:checked:active:hover:focus, headerbar button.flat:checked:checked, .titlebar:not(headerbar) button.flat:checked:checked, headerbar button.flat:checked:checked:hover, .titlebar:not(headerbar) button.flat:checked:checked:hover, headerbar button.flat:checked:checked:focus, .titlebar:not(headerbar) button.flat:checked:checked:focus, headerbar button.flat:checked:checked:hover:focus, .titlebar:not(headerbar) button.flat:checked:checked:hover:focus, headerbar button.flat:active:active, .titlebar:not(headerbar) button.flat:active:active, headerbar button.flat:active:active:hover, .titlebar:not(headerbar) button.flat:active:active:hover, headerbar button.flat:active:active:focus, .titlebar:not(headerbar) button.flat:active:active:focus, headerbar button.flat:active:active:hover:focus, .titlebar:not(headerbar) button.flat:active:active:hover:focus, headerbar button.flat:active:checked, .titlebar:not(headerbar) button.flat:active:checked, headerbar button.flat:active:checked:hover, .titlebar:not(headerbar) button.flat:active:checked:hover, headerbar button.flat:active:checked:focus, .titlebar:not(headerbar) button.flat:active:checked:focus, headerbar button.flat:active:checked:hover:focus, .titlebar:not(headerbar) button.flat:active:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.22); }
      headerbar button:checked:disabled, .titlebar:not(headerbar) button:checked:disabled, headerbar button:active:disabled, .titlebar:not(headerbar) button:active:disabled, headerbar button.flat:checked:disabled, .titlebar:not(headerbar) button.flat:checked:disabled, headerbar button.flat:active:disabled, .titlebar:not(headerbar) button.flat:active:disabled {
        border-color: rgba(140, 140, 140, 0.22); }
      headerbar button:checked:active:disabled, .titlebar:not(headerbar) button:checked:active:disabled, headerbar button:checked:checked:disabled, .titlebar:not(headerbar) button:checked:checked:disabled, headerbar button:active:active:disabled, .titlebar:not(headerbar) button:active:active:disabled, headerbar button:active:checked:disabled, .titlebar:not(headerbar) button:active:checked:disabled, headerbar button.flat:checked:active:disabled, .titlebar:not(headerbar) button.flat:checked:active:disabled, headerbar button.flat:checked:checked:disabled, .titlebar:not(headerbar) button.flat:checked:checked:disabled, headerbar button.flat:active:active:disabled, .titlebar:not(headerbar) button.flat:active:active:disabled, headerbar button.flat:active:checked:disabled, .titlebar:not(headerbar) button.flat:active:checked:disabled {
        border-color: rgba(132, 132, 132, 0.22); }
      headerbar button:checked:focus, .titlebar:not(headerbar) button:checked:focus, headerbar button:checked:hover, .titlebar:not(headerbar) button:checked:hover, headerbar button:active:focus, .titlebar:not(headerbar) button:active:focus, headerbar button:active:hover, .titlebar:not(headerbar) button:active:hover, headerbar button.flat:checked:focus, .titlebar:not(headerbar) button.flat:checked:focus, headerbar button.flat:checked:hover, .titlebar:not(headerbar) button.flat:checked:hover, headerbar button.flat:active:focus, .titlebar:not(headerbar) button.flat:active:focus, headerbar button.flat:active:hover, .titlebar:not(headerbar) button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    headerbar button:focus, .titlebar:not(headerbar) button:focus, headerbar button:hover, .titlebar:not(headerbar) button:hover, headerbar button.flat:focus, .titlebar:not(headerbar) button.flat:focus, headerbar button.flat:hover, .titlebar:not(headerbar) button.flat:hover {
      color: #a5a5a5; }
    headerbar button:disabled:disabled, .titlebar:not(headerbar) button:disabled:disabled, headerbar button.flat:disabled:disabled, .titlebar:not(headerbar) button.flat:disabled:disabled {
      background-color: alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#0f0f0f,#a5a5a5,0.5);
      box-shadow: none; }
    headerbar button:active:disabled, .titlebar:not(headerbar) button:active:disabled, headerbar button:checked:disabled, .titlebar:not(headerbar) button:checked:disabled, headerbar button.flat:active:disabled, .titlebar:not(headerbar) button.flat:active:disabled, headerbar button.flat:checked:disabled, .titlebar:not(headerbar) button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    headerbar button.separator, .titlebar:not(headerbar) button.separator, headerbar button .separator, .titlebar:not(headerbar) button .separator {
      border: 1px solid currentColor;
      color: rgba(15, 15, 15, 0.9); }
      headerbar button.separator:disabled, .titlebar:not(headerbar) button.separator:disabled, headerbar button .separator:disabled, .titlebar:not(headerbar) button .separator:disabled {
        color: rgba(15, 15, 15, 0.85); }
  headerbar .linked > button, .titlebar:not(headerbar) .linked > button {
    box-shadow: inset -1px 0 rgba(0, 0, 0, 0.22), 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    headerbar .linked > button:focus, .titlebar:not(headerbar) .linked > button:focus, headerbar .linked > button:hover, .titlebar:not(headerbar) .linked > button:hover {
      box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.22),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    headerbar .linked > button:active, .titlebar:not(headerbar) .linked > button:active, headerbar .linked > button:active:hover, .titlebar:not(headerbar) .linked > button:active:hover, headerbar .linked > button:active:focus, .titlebar:not(headerbar) .linked > button:active:focus, headerbar .linked > button:active:hover:focus, .titlebar:not(headerbar) .linked > button:active:hover:focus, headerbar .linked > button:checked, .titlebar:not(headerbar) .linked > button:checked, headerbar .linked > button:checked:hover, .titlebar:not(headerbar) .linked > button:checked:hover, headerbar .linked > button:checked:focus, .titlebar:not(headerbar) .linked > button:checked:focus, headerbar .linked > button:checked:hover:focus, .titlebar:not(headerbar) .linked > button:checked:hover:focus {
      box-shadow: inset -1px 0 rgba(0, 0, 0, 0.22), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
    headerbar .linked > button:disabled, .titlebar:not(headerbar) .linked > button:disabled {
      box-shadow: inset -1px 0 #0c0c0c; }
    headerbar .linked > button:last-child, .titlebar:not(headerbar) .linked > button:last-child, headerbar .linked > button:only-child, .titlebar:not(headerbar) .linked > button:only-child {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    headerbar .linked > button:last-child:hover, .titlebar:not(headerbar) .linked > button:last-child:hover, headerbar .linked > button:only-child:hover, .titlebar:not(headerbar) .linked > button:only-child:hover {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    headerbar .linked > button:disabled:last-child, .titlebar:not(headerbar) .linked > button:disabled:last-child, headerbar .linked > button:disabled:only-child, .titlebar:not(headerbar) .linked > button:disabled:only-child, headerbar .linked > button:active:disabled:last-child, .titlebar:not(headerbar) .linked > button:active:disabled:last-child, headerbar .linked > button:active:disabled:only-child, .titlebar:not(headerbar) .linked > button:active:disabled:only-child, headerbar .linked > button:checked:disabled:last-child, .titlebar:not(headerbar) .linked > button:checked:disabled:last-child, headerbar .linked > button:checked:disabled:only-child, .titlebar:not(headerbar) .linked > button:checked:disabled:only-child {
      box-shadow: none; }
    headerbar .linked > button:active:last-child, .titlebar:not(headerbar) .linked > button:active:last-child, headerbar .linked > button:active:last-child:focus, .titlebar:not(headerbar) .linked > button:active:last-child:focus, headerbar .linked > button:active:last-child:hover, .titlebar:not(headerbar) .linked > button:active:last-child:hover, headerbar .linked > button:active:last-child:hover:focus, .titlebar:not(headerbar) .linked > button:active:last-child:hover:focus, headerbar .linked > button:checked:last-child, .titlebar:not(headerbar) .linked > button:checked:last-child, headerbar .linked > button:checked:last-child:focus, .titlebar:not(headerbar) .linked > button:checked:last-child:focus, headerbar .linked > button:checked:last-child:hover, .titlebar:not(headerbar) .linked > button:checked:last-child:hover, headerbar .linked > button:checked:last-child:hover:focus, .titlebar:not(headerbar) .linked > button:checked:last-child:hover:focus {
      box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    headerbar .linked > button:active:only-child, .titlebar:not(headerbar) .linked > button:active:only-child, headerbar .linked > button:active:only-child:focus, .titlebar:not(headerbar) .linked > button:active:only-child:focus, headerbar .linked > button:active:only-child:hover, .titlebar:not(headerbar) .linked > button:active:only-child:hover, headerbar .linked > button:active:only-child:hover:focus, .titlebar:not(headerbar) .linked > button:active:only-child:hover:focus, headerbar .linked > button:checked:only-child, .titlebar:not(headerbar) .linked > button:checked:only-child, headerbar .linked > button:checked:only-child:focus, .titlebar:not(headerbar) .linked > button:checked:only-child:focus, headerbar .linked > button:checked:only-child:hover, .titlebar:not(headerbar) .linked > button:checked:only-child:hover, headerbar .linked > button:checked:only-child:hover:focus, .titlebar:not(headerbar) .linked > button:checked:only-child:hover:focus {
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  headerbar combobox, .titlebar:not(headerbar) combobox, headerbar button, .titlebar:not(headerbar) button {
    padding: 3px; }
    headerbar combobox.text-button, .titlebar:not(headerbar) combobox.text-button, headerbar button.text-button, .titlebar:not(headerbar) button.text-button {
      padding: 3px; }
    headerbar combobox.image-button, .titlebar:not(headerbar) combobox.image-button, headerbar button.image-button, .titlebar:not(headerbar) button.image-button {
      padding: 3px; }
  headerbar separator, .titlebar:not(headerbar) separator, headerbar separator:disabled, .titlebar:not(headerbar) separator:disabled {
    color: #090909;
    border-color: currentColor;
    -GtkWidget-window-dragging: true; }
  headerbar:backdrop, .titlebar:backdrop:not(headerbar) {
    background-color: #0f0f0f;
    background-image: none;
    color: mix(#a5a5a5,#0f0f0f,0.6);
    text-shadow: none; }
  headerbar.default-decoration, .default-decoration.titlebar:not(headerbar) {
    min-height: 24px;
    box-shadow: none;
    border: 0; }
    headerbar.default-decoration button.titlebutton, .default-decoration.titlebar:not(headerbar) button.titlebutton {
      min-height: 16px;
      min-width: 16px;
      margin: 0;
      padding: 0; }
  .tiled headerbar:backdrop, .tiled .titlebar:backdrop:not(headerbar), .tiled headerbar, .tiled .titlebar:not(headerbar),
  .maximized headerbar:backdrop,
  .maximized .titlebar:backdrop:not(headerbar),
  .maximized headerbar,
  .maximized .titlebar:not(headerbar) {
    border-radius: 0; }
  headerbar .title, .titlebar:not(headerbar) .title {
    font-weight: bold; }
  headerbar separator.titlebutton, .titlebar:not(headerbar) separator.titlebutton {
    margin-left: 3px; }
  headerbar button, .titlebar:not(headerbar) button {
    background-color: #8b639b;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.32);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    headerbar button:focus, .titlebar:not(headerbar) button:focus, headerbar button:hover, .titlebar:not(headerbar) button:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    headerbar button:active, .titlebar:not(headerbar) button:active, headerbar button:active:hover, .titlebar:not(headerbar) button:active:hover, headerbar button:active:focus, .titlebar:not(headerbar) button:active:focus, headerbar button:active:hover:focus, .titlebar:not(headerbar) button:active:hover:focus, headerbar button:checked, .titlebar:not(headerbar) button:checked, headerbar button:checked:hover, .titlebar:not(headerbar) button:checked:hover, headerbar button:checked:focus, .titlebar:not(headerbar) button:checked:focus, headerbar button:checked:hover:focus, .titlebar:not(headerbar) button:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar button:disabled, .titlebar:not(headerbar) button:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar button:active:disabled, .titlebar:not(headerbar) button:active:disabled, headerbar button:checked:disabled, .titlebar:not(headerbar) button:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar button.flat, .titlebar:not(headerbar) button.flat {
      color: #000000;
      border-color: rgba(139, 99, 155, 0);
      background-color: rgba(139, 99, 155, 0);
      background-image: none;
      box-shadow: none; }
    headerbar button:hover, .titlebar:not(headerbar) button:hover, headerbar button.flat:hover, .titlebar:not(headerbar) button.flat:hover {
      background-color: #916ba0;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      headerbar button:hover:focus, .titlebar:not(headerbar) button:hover:focus, headerbar button:hover:hover, .titlebar:not(headerbar) button:hover:hover, headerbar button.flat:hover:focus, .titlebar:not(headerbar) button.flat:hover:focus, headerbar button.flat:hover:hover, .titlebar:not(headerbar) button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      headerbar button:hover:active, .titlebar:not(headerbar) button:hover:active, headerbar button:hover:active:hover, .titlebar:not(headerbar) button:hover:active:hover, headerbar button:hover:active:focus, .titlebar:not(headerbar) button:hover:active:focus, headerbar button:hover:active:hover:focus, .titlebar:not(headerbar) button:hover:active:hover:focus, headerbar button:hover:checked, .titlebar:not(headerbar) button:hover:checked, headerbar button:hover:checked:hover, .titlebar:not(headerbar) button:hover:checked:hover, headerbar button:hover:checked:focus, .titlebar:not(headerbar) button:hover:checked:focus, headerbar button:hover:checked:hover:focus, .titlebar:not(headerbar) button:hover:checked:hover:focus, headerbar button.flat:hover:active, .titlebar:not(headerbar) button.flat:hover:active, headerbar button.flat:hover:active:hover, .titlebar:not(headerbar) button.flat:hover:active:hover, headerbar button.flat:hover:active:focus, .titlebar:not(headerbar) button.flat:hover:active:focus, headerbar button.flat:hover:active:hover:focus, .titlebar:not(headerbar) button.flat:hover:active:hover:focus, headerbar button.flat:hover:checked, .titlebar:not(headerbar) button.flat:hover:checked, headerbar button.flat:hover:checked:hover, .titlebar:not(headerbar) button.flat:hover:checked:hover, headerbar button.flat:hover:checked:focus, .titlebar:not(headerbar) button.flat:hover:checked:focus, headerbar button.flat:hover:checked:hover:focus, .titlebar:not(headerbar) button.flat:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar button:hover:disabled, .titlebar:not(headerbar) button:hover:disabled, headerbar button.flat:hover:disabled, .titlebar:not(headerbar) button.flat:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar button:hover:active:disabled, .titlebar:not(headerbar) button:hover:active:disabled, headerbar button:hover:checked:disabled, .titlebar:not(headerbar) button:hover:checked:disabled, headerbar button.flat:hover:active:disabled, .titlebar:not(headerbar) button.flat:hover:active:disabled, headerbar button.flat:hover:checked:disabled, .titlebar:not(headerbar) button.flat:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
    headerbar button:focus, .titlebar:not(headerbar) button:focus, headerbar button.flat:focus, .titlebar:not(headerbar) button.flat:focus {
      background-color: #916ba0;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      headerbar button:focus:focus, .titlebar:not(headerbar) button:focus:focus, headerbar button:focus:hover, .titlebar:not(headerbar) button:focus:hover, headerbar button.flat:focus:focus, .titlebar:not(headerbar) button.flat:focus:focus, headerbar button.flat:focus:hover, .titlebar:not(headerbar) button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      headerbar button:focus:active, .titlebar:not(headerbar) button:focus:active, headerbar button:focus:active:hover, .titlebar:not(headerbar) button:focus:active:hover, headerbar button:focus:active:focus, .titlebar:not(headerbar) button:focus:active:focus, headerbar button:focus:active:hover:focus, .titlebar:not(headerbar) button:focus:active:hover:focus, headerbar button:focus:checked, .titlebar:not(headerbar) button:focus:checked, headerbar button:focus:checked:hover, .titlebar:not(headerbar) button:focus:checked:hover, headerbar button:focus:checked:focus, .titlebar:not(headerbar) button:focus:checked:focus, headerbar button:focus:checked:hover:focus, .titlebar:not(headerbar) button:focus:checked:hover:focus, headerbar button.flat:focus:active, .titlebar:not(headerbar) button.flat:focus:active, headerbar button.flat:focus:active:hover, .titlebar:not(headerbar) button.flat:focus:active:hover, headerbar button.flat:focus:active:focus, .titlebar:not(headerbar) button.flat:focus:active:focus, headerbar button.flat:focus:active:hover:focus, .titlebar:not(headerbar) button.flat:focus:active:hover:focus, headerbar button.flat:focus:checked, .titlebar:not(headerbar) button.flat:focus:checked, headerbar button.flat:focus:checked:hover, .titlebar:not(headerbar) button.flat:focus:checked:hover, headerbar button.flat:focus:checked:focus, .titlebar:not(headerbar) button.flat:focus:checked:focus, headerbar button.flat:focus:checked:hover:focus, .titlebar:not(headerbar) button.flat:focus:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar button:focus:disabled, .titlebar:not(headerbar) button:focus:disabled, headerbar button.flat:focus:disabled, .titlebar:not(headerbar) button.flat:focus:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar button:focus:active:disabled, .titlebar:not(headerbar) button:focus:active:disabled, headerbar button:focus:checked:disabled, .titlebar:not(headerbar) button:focus:checked:disabled, headerbar button.flat:focus:active:disabled, .titlebar:not(headerbar) button.flat:focus:active:disabled, headerbar button.flat:focus:checked:disabled, .titlebar:not(headerbar) button.flat:focus:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar button:focus:hover, .titlebar:not(headerbar) button:focus:hover, headerbar button.flat:focus:hover, .titlebar:not(headerbar) button.flat:focus:hover {
        background-color: #9772a5;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        headerbar button:focus:hover:focus, .titlebar:not(headerbar) button:focus:hover:focus, headerbar button:focus:hover:hover, .titlebar:not(headerbar) button:focus:hover:hover, headerbar button.flat:focus:hover:focus, .titlebar:not(headerbar) button.flat:focus:hover:focus, headerbar button.flat:focus:hover:hover, .titlebar:not(headerbar) button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        headerbar button:focus:hover:active, .titlebar:not(headerbar) button:focus:hover:active, headerbar button:focus:hover:active:hover, .titlebar:not(headerbar) button:focus:hover:active:hover, headerbar button:focus:hover:active:focus, .titlebar:not(headerbar) button:focus:hover:active:focus, headerbar button:focus:hover:active:hover:focus, .titlebar:not(headerbar) button:focus:hover:active:hover:focus, headerbar button:focus:hover:checked, .titlebar:not(headerbar) button:focus:hover:checked, headerbar button:focus:hover:checked:hover, .titlebar:not(headerbar) button:focus:hover:checked:hover, headerbar button:focus:hover:checked:focus, .titlebar:not(headerbar) button:focus:hover:checked:focus, headerbar button:focus:hover:checked:hover:focus, .titlebar:not(headerbar) button:focus:hover:checked:hover:focus, headerbar button.flat:focus:hover:active, .titlebar:not(headerbar) button.flat:focus:hover:active, headerbar button.flat:focus:hover:active:hover, .titlebar:not(headerbar) button.flat:focus:hover:active:hover, headerbar button.flat:focus:hover:active:focus, .titlebar:not(headerbar) button.flat:focus:hover:active:focus, headerbar button.flat:focus:hover:active:hover:focus, .titlebar:not(headerbar) button.flat:focus:hover:active:hover:focus, headerbar button.flat:focus:hover:checked, .titlebar:not(headerbar) button.flat:focus:hover:checked, headerbar button.flat:focus:hover:checked:hover, .titlebar:not(headerbar) button.flat:focus:hover:checked:hover, headerbar button.flat:focus:hover:checked:focus, .titlebar:not(headerbar) button.flat:focus:hover:checked:focus, headerbar button.flat:focus:hover:checked:hover:focus, .titlebar:not(headerbar) button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        headerbar button:focus:hover:disabled, .titlebar:not(headerbar) button:focus:hover:disabled, headerbar button.flat:focus:hover:disabled, .titlebar:not(headerbar) button.flat:focus:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        headerbar button:focus:hover:active:disabled, .titlebar:not(headerbar) button:focus:hover:active:disabled, headerbar button:focus:hover:checked:disabled, .titlebar:not(headerbar) button:focus:hover:checked:disabled, headerbar button.flat:focus:hover:active:disabled, .titlebar:not(headerbar) button.flat:focus:hover:active:disabled, headerbar button.flat:focus:hover:checked:disabled, .titlebar:not(headerbar) button.flat:focus:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
    headerbar button:checked, .titlebar:not(headerbar) button:checked, headerbar button:active, .titlebar:not(headerbar) button:active, headerbar button.flat:checked, .titlebar:not(headerbar) button.flat:checked, headerbar button.flat:active, .titlebar:not(headerbar) button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
      border-color: rgba(0, 0, 0, 0.32); }
      headerbar button:checked:focus, .titlebar:not(headerbar) button:checked:focus, headerbar button:checked:hover, .titlebar:not(headerbar) button:checked:hover, headerbar button:active:focus, .titlebar:not(headerbar) button:active:focus, headerbar button:active:hover, .titlebar:not(headerbar) button:active:hover, headerbar button.flat:checked:focus, .titlebar:not(headerbar) button.flat:checked:focus, headerbar button.flat:checked:hover, .titlebar:not(headerbar) button.flat:checked:hover, headerbar button.flat:active:focus, .titlebar:not(headerbar) button.flat:active:focus, headerbar button.flat:active:hover, .titlebar:not(headerbar) button.flat:active:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      headerbar button:checked:active, .titlebar:not(headerbar) button:checked:active, headerbar button:checked:active:hover, .titlebar:not(headerbar) button:checked:active:hover, headerbar button:checked:active:focus, .titlebar:not(headerbar) button:checked:active:focus, headerbar button:checked:active:hover:focus, .titlebar:not(headerbar) button:checked:active:hover:focus, headerbar button:checked:checked, .titlebar:not(headerbar) button:checked:checked, headerbar button:checked:checked:hover, .titlebar:not(headerbar) button:checked:checked:hover, headerbar button:checked:checked:focus, .titlebar:not(headerbar) button:checked:checked:focus, headerbar button:checked:checked:hover:focus, .titlebar:not(headerbar) button:checked:checked:hover:focus, headerbar button:active:active, .titlebar:not(headerbar) button:active:active, headerbar button:active:active:hover, .titlebar:not(headerbar) button:active:active:hover, headerbar button:active:active:focus, .titlebar:not(headerbar) button:active:active:focus, headerbar button:active:active:hover:focus, .titlebar:not(headerbar) button:active:active:hover:focus, headerbar button:active:checked, .titlebar:not(headerbar) button:active:checked, headerbar button:active:checked:hover, .titlebar:not(headerbar) button:active:checked:hover, headerbar button:active:checked:focus, .titlebar:not(headerbar) button:active:checked:focus, headerbar button:active:checked:hover:focus, .titlebar:not(headerbar) button:active:checked:hover:focus, headerbar button.flat:checked:active, .titlebar:not(headerbar) button.flat:checked:active, headerbar button.flat:checked:active:hover, .titlebar:not(headerbar) button.flat:checked:active:hover, headerbar button.flat:checked:active:focus, .titlebar:not(headerbar) button.flat:checked:active:focus, headerbar button.flat:checked:active:hover:focus, .titlebar:not(headerbar) button.flat:checked:active:hover:focus, headerbar button.flat:checked:checked, .titlebar:not(headerbar) button.flat:checked:checked, headerbar button.flat:checked:checked:hover, .titlebar:not(headerbar) button.flat:checked:checked:hover, headerbar button.flat:checked:checked:focus, .titlebar:not(headerbar) button.flat:checked:checked:focus, headerbar button.flat:checked:checked:hover:focus, .titlebar:not(headerbar) button.flat:checked:checked:hover:focus, headerbar button.flat:active:active, .titlebar:not(headerbar) button.flat:active:active, headerbar button.flat:active:active:hover, .titlebar:not(headerbar) button.flat:active:active:hover, headerbar button.flat:active:active:focus, .titlebar:not(headerbar) button.flat:active:active:focus, headerbar button.flat:active:active:hover:focus, .titlebar:not(headerbar) button.flat:active:active:hover:focus, headerbar button.flat:active:checked, .titlebar:not(headerbar) button.flat:active:checked, headerbar button.flat:active:checked:hover, .titlebar:not(headerbar) button.flat:active:checked:hover, headerbar button.flat:active:checked:focus, .titlebar:not(headerbar) button.flat:active:checked:focus, headerbar button.flat:active:checked:hover:focus, .titlebar:not(headerbar) button.flat:active:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar button:checked:disabled, .titlebar:not(headerbar) button:checked:disabled, headerbar button:active:disabled, .titlebar:not(headerbar) button:active:disabled, headerbar button.flat:checked:disabled, .titlebar:not(headerbar) button.flat:checked:disabled, headerbar button.flat:active:disabled, .titlebar:not(headerbar) button.flat:active:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar button:checked:active:disabled, .titlebar:not(headerbar) button:checked:active:disabled, headerbar button:checked:checked:disabled, .titlebar:not(headerbar) button:checked:checked:disabled, headerbar button:active:active:disabled, .titlebar:not(headerbar) button:active:active:disabled, headerbar button:active:checked:disabled, .titlebar:not(headerbar) button:active:checked:disabled, headerbar button.flat:checked:active:disabled, .titlebar:not(headerbar) button.flat:checked:active:disabled, headerbar button.flat:checked:checked:disabled, .titlebar:not(headerbar) button.flat:checked:checked:disabled, headerbar button.flat:active:active:disabled, .titlebar:not(headerbar) button.flat:active:active:disabled, headerbar button.flat:active:checked:disabled, .titlebar:not(headerbar) button.flat:active:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar button:checked:focus, .titlebar:not(headerbar) button:checked:focus, headerbar button:checked:hover, .titlebar:not(headerbar) button:checked:hover, headerbar button:active:focus, .titlebar:not(headerbar) button:active:focus, headerbar button:active:hover, .titlebar:not(headerbar) button:active:hover, headerbar button.flat:checked:focus, .titlebar:not(headerbar) button.flat:checked:focus, headerbar button.flat:checked:hover, .titlebar:not(headerbar) button.flat:checked:hover, headerbar button.flat:active:focus, .titlebar:not(headerbar) button.flat:active:focus, headerbar button.flat:active:hover, .titlebar:not(headerbar) button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    headerbar button:focus, .titlebar:not(headerbar) button:focus, headerbar button:hover, .titlebar:not(headerbar) button:hover, headerbar button.flat:focus, .titlebar:not(headerbar) button.flat:focus, headerbar button.flat:hover, .titlebar:not(headerbar) button.flat:hover {
      color: #000000; }
    headerbar button:disabled:disabled, .titlebar:not(headerbar) button:disabled:disabled, headerbar button.flat:disabled:disabled, .titlebar:not(headerbar) button.flat:disabled:disabled {
      background-color: alpha(mix(#8b639b,#000000,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#8b639b,#000000,0.5);
      box-shadow: none; }
    headerbar button:active:disabled, .titlebar:not(headerbar) button:active:disabled, headerbar button:checked:disabled, .titlebar:not(headerbar) button:checked:disabled, headerbar button.flat:active:disabled, .titlebar:not(headerbar) button.flat:active:disabled, headerbar button.flat:checked:disabled, .titlebar:not(headerbar) button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    headerbar button.separator, .titlebar:not(headerbar) button.separator, headerbar button .separator, .titlebar:not(headerbar) button .separator {
      border: 1px solid currentColor;
      color: rgba(139, 99, 155, 0.9); }
      headerbar button.separator:disabled, .titlebar:not(headerbar) button.separator:disabled, headerbar button .separator:disabled, .titlebar:not(headerbar) button .separator:disabled {
        color: rgba(139, 99, 155, 0.85); }
  headerbar button.titlebutton + separator.titlebutton, .titlebar:not(headerbar) button.titlebutton + separator.titlebutton {
    margin-left: 0;
    margin-right: 3px; }
  headerbar button.titlebutton, .titlebar:not(headerbar) button.titlebutton {
    border: 0;
    background-image: none;
    background-color: transparent;
    color: mix(#a5a5a5,#0f0f0f,0.1);
    box-shadow: none; }
    headerbar button.titlebutton:hover, .titlebar:not(headerbar) button.titlebutton:hover, headerbar button.titlebutton:hover:focus, .titlebar:not(headerbar) button.titlebutton:hover:focus {
      background-image: none;
      background-color: transparent;
      color: #B36A87;
      box-shadow: none; }
    headerbar button.titlebutton:active, .titlebar:not(headerbar) button.titlebutton:active, headerbar button.titlebutton:active:hover, .titlebar:not(headerbar) button.titlebutton:active:hover {
      background-image: none;
      background-color: transparent;
      color: #a95778;
      box-shadow: none; }
    headerbar button.titlebutton:backdrop, .titlebar:not(headerbar) button.titlebutton:backdrop {
      background: none;
      color: mix(#a5a5a5,#0f0f0f,0.6);
      -gtk-icon-shadow: none; }

toolbar {
  background-color: #a5a5a5;
  background-image: none;
  border-color: #848484;
  color: #000000; }
  toolbar:focus, toolbar:hover {
    border-color: mix(#B36A87,#a5a5a5,0.3); }
  toolbar:active, toolbar:active:hover, toolbar:active:focus, toolbar:active:hover:focus, toolbar:checked, toolbar:checked:hover, toolbar:checked:focus, toolbar:checked:hover:focus {
    border-color: #747474; }
  toolbar:disabled {
    border-color: #8c8c8c; }
  toolbar:active:disabled, toolbar:checked:disabled {
    border-color: #848484; }
  toolbar:disabled {
    background-color: #959595;
    background-image: none;
    color: mix(#000000,#a5a5a5,0.5); }
  toolbar .title {
    font-weight: bold;
    padding: 0 6px; }
  toolbar .subtitle {
    font-size: smaller;
    padding: 0 6px; }
  toolbar button {
    background-color: #a5a5a5;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.32);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    toolbar button:focus, toolbar button:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    toolbar button:active, toolbar button:active:hover, toolbar button:active:focus, toolbar button:active:hover:focus, toolbar button:checked, toolbar button:checked:hover, toolbar button:checked:focus, toolbar button:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    toolbar button:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    toolbar button:active:disabled, toolbar button:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    toolbar button.flat {
      color: #000000;
      border-color: rgba(165, 165, 165, 0);
      background-color: rgba(165, 165, 165, 0);
      background-image: none;
      box-shadow: none; }
    toolbar button:hover, toolbar button.flat:hover {
      background-color: #adadad;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
      toolbar button:hover:focus, toolbar button:hover:hover, toolbar button.flat:hover:focus, toolbar button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      toolbar button:hover:active, toolbar button:hover:active:hover, toolbar button:hover:active:focus, toolbar button:hover:active:hover:focus, toolbar button:hover:checked, toolbar button:hover:checked:hover, toolbar button:hover:checked:focus, toolbar button:hover:checked:hover:focus, toolbar button.flat:hover:active, toolbar button.flat:hover:active:hover, toolbar button.flat:hover:active:focus, toolbar button.flat:hover:active:hover:focus, toolbar button.flat:hover:checked, toolbar button.flat:hover:checked:hover, toolbar button.flat:hover:checked:focus, toolbar button.flat:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      toolbar button:hover:disabled, toolbar button.flat:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      toolbar button:hover:active:disabled, toolbar button:hover:checked:disabled, toolbar button.flat:hover:active:disabled, toolbar button.flat:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
    toolbar button:focus, toolbar button.flat:focus {
      background-color: #adadad;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      toolbar button:focus:focus, toolbar button:focus:hover, toolbar button.flat:focus:focus, toolbar button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      toolbar button:focus:active, toolbar button:focus:active:hover, toolbar button:focus:active:focus, toolbar button:focus:active:hover:focus, toolbar button:focus:checked, toolbar button:focus:checked:hover, toolbar button:focus:checked:focus, toolbar button:focus:checked:hover:focus, toolbar button.flat:focus:active, toolbar button.flat:focus:active:hover, toolbar button.flat:focus:active:focus, toolbar button.flat:focus:active:hover:focus, toolbar button.flat:focus:checked, toolbar button.flat:focus:checked:hover, toolbar button.flat:focus:checked:focus, toolbar button.flat:focus:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      toolbar button:focus:disabled, toolbar button.flat:focus:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      toolbar button:focus:active:disabled, toolbar button:focus:checked:disabled, toolbar button.flat:focus:active:disabled, toolbar button.flat:focus:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      toolbar button:focus:hover, toolbar button.flat:focus:hover {
        background-color: #b6b6b6;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
        toolbar button:focus:hover:focus, toolbar button:focus:hover:hover, toolbar button.flat:focus:hover:focus, toolbar button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        toolbar button:focus:hover:active, toolbar button:focus:hover:active:hover, toolbar button:focus:hover:active:focus, toolbar button:focus:hover:active:hover:focus, toolbar button:focus:hover:checked, toolbar button:focus:hover:checked:hover, toolbar button:focus:hover:checked:focus, toolbar button:focus:hover:checked:hover:focus, toolbar button.flat:focus:hover:active, toolbar button.flat:focus:hover:active:hover, toolbar button.flat:focus:hover:active:focus, toolbar button.flat:focus:hover:active:hover:focus, toolbar button.flat:focus:hover:checked, toolbar button.flat:focus:hover:checked:hover, toolbar button.flat:focus:hover:checked:focus, toolbar button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar button:focus:hover:disabled, toolbar button.flat:focus:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar button:focus:hover:active:disabled, toolbar button:focus:hover:checked:disabled, toolbar button.flat:focus:hover:active:disabled, toolbar button.flat:focus:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
    toolbar button:checked, toolbar button:active, toolbar button.flat:checked, toolbar button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
      border-color: rgba(0, 0, 0, 0.32); }
      toolbar button:checked:focus, toolbar button:checked:hover, toolbar button:active:focus, toolbar button:active:hover, toolbar button.flat:checked:focus, toolbar button.flat:checked:hover, toolbar button.flat:active:focus, toolbar button.flat:active:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      toolbar button:checked:active, toolbar button:checked:active:hover, toolbar button:checked:active:focus, toolbar button:checked:active:hover:focus, toolbar button:checked:checked, toolbar button:checked:checked:hover, toolbar button:checked:checked:focus, toolbar button:checked:checked:hover:focus, toolbar button:active:active, toolbar button:active:active:hover, toolbar button:active:active:focus, toolbar button:active:active:hover:focus, toolbar button:active:checked, toolbar button:active:checked:hover, toolbar button:active:checked:focus, toolbar button:active:checked:hover:focus, toolbar button.flat:checked:active, toolbar button.flat:checked:active:hover, toolbar button.flat:checked:active:focus, toolbar button.flat:checked:active:hover:focus, toolbar button.flat:checked:checked, toolbar button.flat:checked:checked:hover, toolbar button.flat:checked:checked:focus, toolbar button.flat:checked:checked:hover:focus, toolbar button.flat:active:active, toolbar button.flat:active:active:hover, toolbar button.flat:active:active:focus, toolbar button.flat:active:active:hover:focus, toolbar button.flat:active:checked, toolbar button.flat:active:checked:hover, toolbar button.flat:active:checked:focus, toolbar button.flat:active:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      toolbar button:checked:disabled, toolbar button:active:disabled, toolbar button.flat:checked:disabled, toolbar button.flat:active:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      toolbar button:checked:active:disabled, toolbar button:checked:checked:disabled, toolbar button:active:active:disabled, toolbar button:active:checked:disabled, toolbar button.flat:checked:active:disabled, toolbar button.flat:checked:checked:disabled, toolbar button.flat:active:active:disabled, toolbar button.flat:active:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      toolbar button:checked:focus, toolbar button:checked:hover, toolbar button:active:focus, toolbar button:active:hover, toolbar button.flat:checked:focus, toolbar button.flat:checked:hover, toolbar button.flat:active:focus, toolbar button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    toolbar button:focus, toolbar button:hover, toolbar button.flat:focus, toolbar button.flat:hover {
      color: #000000; }
    toolbar button:disabled:disabled, toolbar button.flat:disabled:disabled {
      background-color: alpha(mix(#a5a5a5,#000000,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#a5a5a5,#000000,0.5);
      box-shadow: none; }
    toolbar button:active:disabled, toolbar button:checked:disabled, toolbar button.flat:active:disabled, toolbar button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    toolbar button.separator, toolbar button .separator {
      border: 1px solid currentColor;
      color: rgba(165, 165, 165, 0.9); }
      toolbar button.separator:disabled, toolbar button .separator:disabled {
        color: rgba(165, 165, 165, 0.85); }
  toolbar .linked > button {
    box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    toolbar .linked > button:focus, toolbar .linked > button:hover {
      box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.12),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    toolbar .linked > button:active, toolbar .linked > button:active:hover, toolbar .linked > button:active:focus, toolbar .linked > button:active:hover:focus, toolbar .linked > button:checked, toolbar .linked > button:checked:hover, toolbar .linked > button:checked:focus, toolbar .linked > button:checked:hover:focus {
      box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
    toolbar .linked > button:disabled {
      box-shadow: inset -1px 0 #848484; }
    toolbar .linked > button:last-child, toolbar .linked > button:only-child {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    toolbar .linked > button:last-child:hover, toolbar .linked > button:only-child:hover {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    toolbar .linked > button:disabled:last-child, toolbar .linked > button:disabled:only-child, toolbar .linked > button:active:disabled:last-child, toolbar .linked > button:active:disabled:only-child, toolbar .linked > button:checked:disabled:last-child, toolbar .linked > button:checked:disabled:only-child {
      box-shadow: none; }
    toolbar .linked > button:active:last-child, toolbar .linked > button:active:last-child:focus, toolbar .linked > button:active:last-child:hover, toolbar .linked > button:active:last-child:hover:focus, toolbar .linked > button:checked:last-child, toolbar .linked > button:checked:last-child:focus, toolbar .linked > button:checked:last-child:hover, toolbar .linked > button:checked:last-child:hover:focus {
      box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    toolbar .linked > button:active:only-child, toolbar .linked > button:active:only-child:focus, toolbar .linked > button:active:only-child:hover, toolbar .linked > button:active:only-child:hover:focus, toolbar .linked > button:checked:only-child, toolbar .linked > button:checked:only-child:focus, toolbar .linked > button:checked:only-child:hover, toolbar .linked > button:checked:only-child:hover:focus {
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  toolbar combobox, toolbar button {
    padding: 3px; }
    toolbar combobox.text-button, toolbar button.text-button {
      padding: 3px; }
    toolbar combobox.image-button, toolbar button.image-button {
      padding: 3px; }
  toolbar separator, toolbar separator:disabled {
    color: #636363;
    border-color: currentColor;
    -GtkWidget-window-dragging: true; }
  toolbar.inline-toolbar {
    padding: 1px;
    border-width: 0 1px 1px;
    border-style: solid;
    border-color: mix(#a5a5a5,#000000,0.08);
    background-color: mix(mix(#a5a5a5,#000000,0.08),#a5a5a5,0.7);
    background-image: none; }
    toolbar.inline-toolbar:backdrop {
      border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9);
      background-color: mix(#a5a5a5,mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9),0.35);
      transition: 200ms ease-out; }
    toolbar.inline-toolbar button {
      background-color: #a5a5a5;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.32);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
      toolbar.inline-toolbar button:focus, toolbar.inline-toolbar button:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      toolbar.inline-toolbar button:active, toolbar.inline-toolbar button:active:hover, toolbar.inline-toolbar button:active:focus, toolbar.inline-toolbar button:active:hover:focus, toolbar.inline-toolbar button:checked, toolbar.inline-toolbar button:checked:hover, toolbar.inline-toolbar button:checked:focus, toolbar.inline-toolbar button:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      toolbar.inline-toolbar button:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      toolbar.inline-toolbar button:active:disabled, toolbar.inline-toolbar button:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      toolbar.inline-toolbar button.flat {
        color: #000000;
        border-color: rgba(165, 165, 165, 0);
        background-color: rgba(165, 165, 165, 0);
        background-image: none;
        box-shadow: none; }
      toolbar.inline-toolbar button:hover, toolbar.inline-toolbar button.flat:hover {
        background-color: #adadad;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        color: #000000;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
        toolbar.inline-toolbar button:hover:focus, toolbar.inline-toolbar button:hover:hover, toolbar.inline-toolbar button.flat:hover:focus, toolbar.inline-toolbar button.flat:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        toolbar.inline-toolbar button:hover:active, toolbar.inline-toolbar button:hover:active:hover, toolbar.inline-toolbar button:hover:active:focus, toolbar.inline-toolbar button:hover:active:hover:focus, toolbar.inline-toolbar button:hover:checked, toolbar.inline-toolbar button:hover:checked:hover, toolbar.inline-toolbar button:hover:checked:focus, toolbar.inline-toolbar button:hover:checked:hover:focus, toolbar.inline-toolbar button.flat:hover:active, toolbar.inline-toolbar button.flat:hover:active:hover, toolbar.inline-toolbar button.flat:hover:active:focus, toolbar.inline-toolbar button.flat:hover:active:hover:focus, toolbar.inline-toolbar button.flat:hover:checked, toolbar.inline-toolbar button.flat:hover:checked:hover, toolbar.inline-toolbar button.flat:hover:checked:focus, toolbar.inline-toolbar button.flat:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar.inline-toolbar button:hover:disabled, toolbar.inline-toolbar button.flat:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar.inline-toolbar button:hover:active:disabled, toolbar.inline-toolbar button:hover:checked:disabled, toolbar.inline-toolbar button.flat:hover:active:disabled, toolbar.inline-toolbar button.flat:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
      toolbar.inline-toolbar button:focus, toolbar.inline-toolbar button.flat:focus {
        background-color: #adadad;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        color: #000000;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
        toolbar.inline-toolbar button:focus:focus, toolbar.inline-toolbar button:focus:hover, toolbar.inline-toolbar button.flat:focus:focus, toolbar.inline-toolbar button.flat:focus:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        toolbar.inline-toolbar button:focus:active, toolbar.inline-toolbar button:focus:active:hover, toolbar.inline-toolbar button:focus:active:focus, toolbar.inline-toolbar button:focus:active:hover:focus, toolbar.inline-toolbar button:focus:checked, toolbar.inline-toolbar button:focus:checked:hover, toolbar.inline-toolbar button:focus:checked:focus, toolbar.inline-toolbar button:focus:checked:hover:focus, toolbar.inline-toolbar button.flat:focus:active, toolbar.inline-toolbar button.flat:focus:active:hover, toolbar.inline-toolbar button.flat:focus:active:focus, toolbar.inline-toolbar button.flat:focus:active:hover:focus, toolbar.inline-toolbar button.flat:focus:checked, toolbar.inline-toolbar button.flat:focus:checked:hover, toolbar.inline-toolbar button.flat:focus:checked:focus, toolbar.inline-toolbar button.flat:focus:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar.inline-toolbar button:focus:disabled, toolbar.inline-toolbar button.flat:focus:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar.inline-toolbar button:focus:active:disabled, toolbar.inline-toolbar button:focus:checked:disabled, toolbar.inline-toolbar button.flat:focus:active:disabled, toolbar.inline-toolbar button.flat:focus:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        toolbar.inline-toolbar button:focus:hover, toolbar.inline-toolbar button.flat:focus:hover {
          background-color: #b6b6b6;
          background-image: none;
          border-color: rgba(0, 0, 0, 0.4);
          box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
          toolbar.inline-toolbar button:focus:hover:focus, toolbar.inline-toolbar button:focus:hover:hover, toolbar.inline-toolbar button.flat:focus:hover:focus, toolbar.inline-toolbar button.flat:focus:hover:hover {
            border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
          toolbar.inline-toolbar button:focus:hover:active, toolbar.inline-toolbar button:focus:hover:active:hover, toolbar.inline-toolbar button:focus:hover:active:focus, toolbar.inline-toolbar button:focus:hover:active:hover:focus, toolbar.inline-toolbar button:focus:hover:checked, toolbar.inline-toolbar button:focus:hover:checked:hover, toolbar.inline-toolbar button:focus:hover:checked:focus, toolbar.inline-toolbar button:focus:hover:checked:hover:focus, toolbar.inline-toolbar button.flat:focus:hover:active, toolbar.inline-toolbar button.flat:focus:hover:active:hover, toolbar.inline-toolbar button.flat:focus:hover:active:focus, toolbar.inline-toolbar button.flat:focus:hover:active:hover:focus, toolbar.inline-toolbar button.flat:focus:hover:checked, toolbar.inline-toolbar button.flat:focus:hover:checked:hover, toolbar.inline-toolbar button.flat:focus:hover:checked:focus, toolbar.inline-toolbar button.flat:focus:hover:checked:hover:focus {
            border-color: rgba(0, 0, 0, 0.4); }
          toolbar.inline-toolbar button:focus:hover:disabled, toolbar.inline-toolbar button.flat:focus:hover:disabled {
            border-color: rgba(0, 0, 0, 0.4); }
          toolbar.inline-toolbar button:focus:hover:active:disabled, toolbar.inline-toolbar button:focus:hover:checked:disabled, toolbar.inline-toolbar button.flat:focus:hover:active:disabled, toolbar.inline-toolbar button.flat:focus:hover:checked:disabled {
            border-color: rgba(0, 0, 0, 0.4); }
      toolbar.inline-toolbar button:checked, toolbar.inline-toolbar button:active, toolbar.inline-toolbar button.flat:checked, toolbar.inline-toolbar button.flat:active {
        /*@include linear-gradient(shade($bg, .7), to top);*/
        background-color: #B36A87;
        background-image: none;
        color: #000000;
        box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
        border-color: rgba(0, 0, 0, 0.32); }
        toolbar.inline-toolbar button:checked:focus, toolbar.inline-toolbar button:checked:hover, toolbar.inline-toolbar button:active:focus, toolbar.inline-toolbar button:active:hover, toolbar.inline-toolbar button.flat:checked:focus, toolbar.inline-toolbar button.flat:checked:hover, toolbar.inline-toolbar button.flat:active:focus, toolbar.inline-toolbar button.flat:active:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
        toolbar.inline-toolbar button:checked:active, toolbar.inline-toolbar button:checked:active:hover, toolbar.inline-toolbar button:checked:active:focus, toolbar.inline-toolbar button:checked:active:hover:focus, toolbar.inline-toolbar button:checked:checked, toolbar.inline-toolbar button:checked:checked:hover, toolbar.inline-toolbar button:checked:checked:focus, toolbar.inline-toolbar button:checked:checked:hover:focus, toolbar.inline-toolbar button:active:active, toolbar.inline-toolbar button:active:active:hover, toolbar.inline-toolbar button:active:active:focus, toolbar.inline-toolbar button:active:active:hover:focus, toolbar.inline-toolbar button:active:checked, toolbar.inline-toolbar button:active:checked:hover, toolbar.inline-toolbar button:active:checked:focus, toolbar.inline-toolbar button:active:checked:hover:focus, toolbar.inline-toolbar button.flat:checked:active, toolbar.inline-toolbar button.flat:checked:active:hover, toolbar.inline-toolbar button.flat:checked:active:focus, toolbar.inline-toolbar button.flat:checked:active:hover:focus, toolbar.inline-toolbar button.flat:checked:checked, toolbar.inline-toolbar button.flat:checked:checked:hover, toolbar.inline-toolbar button.flat:checked:checked:focus, toolbar.inline-toolbar button.flat:checked:checked:hover:focus, toolbar.inline-toolbar button.flat:active:active, toolbar.inline-toolbar button.flat:active:active:hover, toolbar.inline-toolbar button.flat:active:active:focus, toolbar.inline-toolbar button.flat:active:active:hover:focus, toolbar.inline-toolbar button.flat:active:checked, toolbar.inline-toolbar button.flat:active:checked:hover, toolbar.inline-toolbar button.flat:active:checked:focus, toolbar.inline-toolbar button.flat:active:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.32); }
        toolbar.inline-toolbar button:checked:disabled, toolbar.inline-toolbar button:active:disabled, toolbar.inline-toolbar button.flat:checked:disabled, toolbar.inline-toolbar button.flat:active:disabled {
          border-color: rgba(0, 0, 0, 0.32); }
        toolbar.inline-toolbar button:checked:active:disabled, toolbar.inline-toolbar button:checked:checked:disabled, toolbar.inline-toolbar button:active:active:disabled, toolbar.inline-toolbar button:active:checked:disabled, toolbar.inline-toolbar button.flat:checked:active:disabled, toolbar.inline-toolbar button.flat:checked:checked:disabled, toolbar.inline-toolbar button.flat:active:active:disabled, toolbar.inline-toolbar button.flat:active:checked:disabled {
          border-color: rgba(0, 0, 0, 0.32); }
        toolbar.inline-toolbar button:checked:focus, toolbar.inline-toolbar button:checked:hover, toolbar.inline-toolbar button:active:focus, toolbar.inline-toolbar button:active:hover, toolbar.inline-toolbar button.flat:checked:focus, toolbar.inline-toolbar button.flat:checked:hover, toolbar.inline-toolbar button.flat:active:focus, toolbar.inline-toolbar button.flat:active:hover {
          /*@include linear-gradient(shade($bg, .65), to top);*/
          background-color: #b8738f;
          background-image: none;
          color: #000000; }
      toolbar.inline-toolbar button:focus, toolbar.inline-toolbar button:hover, toolbar.inline-toolbar button.flat:focus, toolbar.inline-toolbar button.flat:hover {
        color: #000000; }
      toolbar.inline-toolbar button:disabled:disabled, toolbar.inline-toolbar button.flat:disabled:disabled {
        background-color: alpha(mix(#a5a5a5,#000000,0.2),0.4);
        background-image: none;
        /*border: 1px solid alpha($bg, .2);*/
        opacity: .4;
        color: mix(#a5a5a5,#000000,0.5);
        box-shadow: none; }
      toolbar.inline-toolbar button:active:disabled, toolbar.inline-toolbar button:checked:disabled, toolbar.inline-toolbar button.flat:active:disabled, toolbar.inline-toolbar button.flat:checked:disabled {
        background-color: rgba(179, 106, 135, 0.6);
        background-image: none;
        color: #000000;
        box-shadow: none; }
      toolbar.inline-toolbar button.separator, toolbar.inline-toolbar button .separator {
        border: 1px solid currentColor;
        color: rgba(165, 165, 165, 0.9); }
        toolbar.inline-toolbar button.separator:disabled, toolbar.inline-toolbar button .separator:disabled {
          color: rgba(165, 165, 165, 0.85); }

window.csd > .titlebar:not(headerbar) {
  padding: 0;
  background-color: transparent;
  background-image: none;
  border-style: none;
  border-color: transparent;
  box-shadow: none; }

.titlebar:not(headerbar) > separator {
  background-color: #0d0d0d; }

.background:not(.tiled):not(.maximized) .titlebar:backdrop, .background:not(.tiled):not(.maximized) .titlebar {
  border-top-left-radius: 3px;
  border-top-right-radius: 3px; }

.background:not(.csd):not(.ssd):not(.solid-csd) headerbar, .background:not(.csd):not(.ssd):not(.solid-csd) headerbar:not(:last-child), .background:not(.csd):not(.ssd):not(.solid-csd) headerbar:backdrop, .background:not(.csd):not(.ssd):not(.solid-csd) headerbar:backdrop:not(:last-child) {
  border-radius: 0;
  border-top-color: transparent; }

/**************
 ! Action-bar *
***************/
actionbar > revealer > box {
  padding: 3px;
  border-top: 1px solid mix(#a5a5a5,#000000,0.08); }
  actionbar > revealer > box:backdrop {
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9); }

/****************************
 ! Search and Location bars *
*****************************/
searchbar,
.location-bar {
  background-color: #a2a2a2;
  background-image: none;
  border-width: 0 0 1px;
  border-style: solid;
  border-color: #848484;
  color: #000000; }

/******************
 ! Action buttons *
*******************/
.suggested-action, headerbar.selection-mode button.suggested-action,
.titlebar:not(headerbar).selection-mode button.suggested-action {
  background-color: #4caf50;
  background-image: none;
  border-color: rgba(0, 0, 0, 0.32);
  color: #000000;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
  .suggested-action:focus, headerbar.selection-mode button.suggested-action:focus,
  .titlebar:not(headerbar).selection-mode button.suggested-action:focus, .suggested-action:hover, headerbar.selection-mode button.suggested-action:hover,
  .titlebar:not(headerbar).selection-mode button.suggested-action:hover {
    border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
  .suggested-action:active, headerbar.selection-mode button.suggested-action:active,
  .titlebar:not(headerbar).selection-mode button.suggested-action:active, .suggested-action:active:hover,
  .titlebar:not(headerbar).selection-mode button.suggested-action:active:hover, .suggested-action:active:focus,
  .titlebar:not(headerbar).selection-mode button.suggested-action:active:focus, .suggested-action:active:hover:focus, .suggested-action:checked, headerbar.selection-mode button.suggested-action:checked,
  .titlebar:not(headerbar).selection-mode button.suggested-action:checked, .suggested-action:checked:hover,
  .titlebar:not(headerbar).selection-mode button.suggested-action:checked:hover, .suggested-action:checked:focus,
  .titlebar:not(headerbar).selection-mode button.suggested-action:checked:focus, .suggested-action:checked:hover:focus {
    border-color: rgba(0, 0, 0, 0.32); }
  .suggested-action:disabled, headerbar.selection-mode button.suggested-action:disabled,
  .titlebar:not(headerbar).selection-mode button.suggested-action:disabled {
    border-color: rgba(0, 0, 0, 0.32); }
  .suggested-action:active:disabled,
  .titlebar:not(headerbar).selection-mode button.suggested-action:active:disabled, .suggested-action:checked:disabled,
  .titlebar:not(headerbar).selection-mode button.suggested-action:checked:disabled {
    border-color: rgba(0, 0, 0, 0.32); }
  .suggested-action.flat, headerbar.selection-mode button.flat.suggested-action,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action {
    color: #000000;
    border-color: rgba(76, 175, 80, 0);
    background-color: rgba(76, 175, 80, 0);
    background-image: none;
    box-shadow: none; }
  .suggested-action:hover, headerbar.selection-mode button.suggested-action:hover,
  .titlebar:not(headerbar).selection-mode button.suggested-action:hover, .suggested-action.flat:hover,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action:hover {
    background-color: #53b457;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.4);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    .suggested-action:hover:focus,
    .titlebar:not(headerbar).selection-mode button.suggested-action:hover:focus, .suggested-action:hover:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:hover:hover, .suggested-action.flat:hover:focus, .suggested-action.flat:hover:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
    .suggested-action:hover:active,
    .titlebar:not(headerbar).selection-mode button.suggested-action:hover:active, .suggested-action:hover:active:hover, .suggested-action:hover:active:focus, .suggested-action:hover:active:hover:focus, .suggested-action:hover:checked,
    .titlebar:not(headerbar).selection-mode button.suggested-action:hover:checked, .suggested-action:hover:checked:hover, .suggested-action:hover:checked:focus, .suggested-action:hover:checked:hover:focus, .suggested-action.flat:hover:active, .suggested-action.flat:hover:active:hover, .suggested-action.flat:hover:active:focus, .suggested-action.flat:hover:active:hover:focus, .suggested-action.flat:hover:checked, .suggested-action.flat:hover:checked:hover, .suggested-action.flat:hover:checked:focus, .suggested-action.flat:hover:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.4); }
    .suggested-action:hover:disabled,
    .titlebar:not(headerbar).selection-mode button.suggested-action:hover:disabled, .suggested-action.flat:hover:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    .suggested-action:hover:active:disabled, .suggested-action:hover:checked:disabled, .suggested-action.flat:hover:active:disabled, .suggested-action.flat:hover:checked:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
  .suggested-action:focus, headerbar.selection-mode button.suggested-action:focus,
  .titlebar:not(headerbar).selection-mode button.suggested-action:focus, .suggested-action.flat:focus,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action:focus {
    background-color: #53b457;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.4);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    .suggested-action:focus:focus,
    .titlebar:not(headerbar).selection-mode button.suggested-action:focus:focus, .suggested-action:focus:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:focus:hover, .suggested-action.flat:focus:focus, .suggested-action.flat:focus:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
    .suggested-action:focus:active,
    .titlebar:not(headerbar).selection-mode button.suggested-action:focus:active, .suggested-action:focus:active:hover, .suggested-action:focus:active:focus, .suggested-action:focus:active:hover:focus, .suggested-action:focus:checked,
    .titlebar:not(headerbar).selection-mode button.suggested-action:focus:checked, .suggested-action:focus:checked:hover, .suggested-action:focus:checked:focus, .suggested-action:focus:checked:hover:focus, .suggested-action.flat:focus:active, .suggested-action.flat:focus:active:hover, .suggested-action.flat:focus:active:focus, .suggested-action.flat:focus:active:hover:focus, .suggested-action.flat:focus:checked, .suggested-action.flat:focus:checked:hover, .suggested-action.flat:focus:checked:focus, .suggested-action.flat:focus:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.4); }
    .suggested-action:focus:disabled,
    .titlebar:not(headerbar).selection-mode button.suggested-action:focus:disabled, .suggested-action.flat:focus:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    .suggested-action:focus:active:disabled, .suggested-action:focus:checked:disabled, .suggested-action.flat:focus:active:disabled, .suggested-action.flat:focus:checked:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    .suggested-action:focus:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:focus:hover, .suggested-action.flat:focus:hover {
      background-color: #5cb860;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
      .suggested-action:focus:hover:focus, .suggested-action:focus:hover:hover, .suggested-action.flat:focus:hover:focus, .suggested-action.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      .suggested-action:focus:hover:active, .suggested-action:focus:hover:active:hover, .suggested-action:focus:hover:active:focus, .suggested-action:focus:hover:active:hover:focus, .suggested-action:focus:hover:checked, .suggested-action:focus:hover:checked:hover, .suggested-action:focus:hover:checked:focus, .suggested-action:focus:hover:checked:hover:focus, .suggested-action.flat:focus:hover:active, .suggested-action.flat:focus:hover:active:hover, .suggested-action.flat:focus:hover:active:focus, .suggested-action.flat:focus:hover:active:hover:focus, .suggested-action.flat:focus:hover:checked, .suggested-action.flat:focus:hover:checked:hover, .suggested-action.flat:focus:hover:checked:focus, .suggested-action.flat:focus:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      .suggested-action:focus:hover:disabled, .suggested-action.flat:focus:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      .suggested-action:focus:hover:active:disabled, .suggested-action:focus:hover:checked:disabled, .suggested-action.flat:focus:hover:active:disabled, .suggested-action.flat:focus:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
  .suggested-action:checked, headerbar.selection-mode button.suggested-action:checked,
  .titlebar:not(headerbar).selection-mode button.suggested-action:checked, .suggested-action:active, headerbar.selection-mode button.suggested-action:active,
  .titlebar:not(headerbar).selection-mode button.suggested-action:active, .suggested-action.flat:checked,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action:checked, .suggested-action.flat:active,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
    border-color: rgba(0, 0, 0, 0.32); }
    .suggested-action:checked:focus,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:focus, .suggested-action:checked:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:hover, .suggested-action:active:focus,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:focus, .suggested-action:active:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:hover, .suggested-action.flat:checked:focus, .suggested-action.flat:checked:hover, .suggested-action.flat:active:focus, .suggested-action.flat:active:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    .suggested-action:checked:active,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:active, .suggested-action:checked:active:hover, .suggested-action:checked:active:focus, .suggested-action:checked:active:hover:focus, .suggested-action:checked:checked,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:checked, .suggested-action:checked:checked:hover, .suggested-action:checked:checked:focus, .suggested-action:checked:checked:hover:focus, .suggested-action:active:active,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:active, .suggested-action:active:active:hover, .suggested-action:active:active:focus, .suggested-action:active:active:hover:focus, .suggested-action:active:checked,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:checked, .suggested-action:active:checked:hover, .suggested-action:active:checked:focus, .suggested-action:active:checked:hover:focus, .suggested-action.flat:checked:active, .suggested-action.flat:checked:active:hover, .suggested-action.flat:checked:active:focus, .suggested-action.flat:checked:active:hover:focus, .suggested-action.flat:checked:checked, .suggested-action.flat:checked:checked:hover, .suggested-action.flat:checked:checked:focus, .suggested-action.flat:checked:checked:hover:focus, .suggested-action.flat:active:active, .suggested-action.flat:active:active:hover, .suggested-action.flat:active:active:focus, .suggested-action.flat:active:active:hover:focus, .suggested-action.flat:active:checked, .suggested-action.flat:active:checked:hover, .suggested-action.flat:active:checked:focus, .suggested-action.flat:active:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    .suggested-action:checked:disabled,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:disabled, .suggested-action:active:disabled,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:disabled, .suggested-action.flat:checked:disabled, .suggested-action.flat:active:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    .suggested-action:checked:active:disabled, .suggested-action:checked:checked:disabled, .suggested-action:active:active:disabled, .suggested-action:active:checked:disabled, .suggested-action.flat:checked:active:disabled, .suggested-action.flat:checked:checked:disabled, .suggested-action.flat:active:active:disabled, .suggested-action.flat:active:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    .suggested-action:checked:focus,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:focus, .suggested-action:checked:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:checked:hover, .suggested-action:active:focus,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:focus, .suggested-action:active:hover,
    .titlebar:not(headerbar).selection-mode button.suggested-action:active:hover, .suggested-action.flat:checked:focus, .suggested-action.flat:checked:hover, .suggested-action.flat:active:focus, .suggested-action.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  .suggested-action:focus, headerbar.selection-mode button.suggested-action:focus,
  .titlebar:not(headerbar).selection-mode button.suggested-action:focus, .suggested-action:hover, headerbar.selection-mode button.suggested-action:hover,
  .titlebar:not(headerbar).selection-mode button.suggested-action:hover, .suggested-action.flat:focus,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action:focus, .suggested-action.flat:hover,
  .titlebar:not(headerbar).selection-mode button.flat.suggested-action:hover {
    color: #000000; }
  .suggested-action:disabled:disabled,
  .titlebar:not(headerbar).selection-mode button.suggested-action:disabled:disabled, .suggested-action.flat:disabled:disabled {
    background-color: alpha(mix(#4caf50,#000000,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(#4caf50,#000000,0.5);
    box-shadow: none; }
  .suggested-action:active:disabled,
  .titlebar:not(headerbar).selection-mode button.suggested-action:active:disabled, .suggested-action:checked:disabled,
  .titlebar:not(headerbar).selection-mode button.suggested-action:checked:disabled, .suggested-action.flat:active:disabled, .suggested-action.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  .suggested-action.separator, headerbar.selection-mode button.separator.suggested-action,
  .titlebar:not(headerbar).selection-mode button.separator.suggested-action, .suggested-action .separator, headerbar.selection-mode button.suggested-action .separator,
  .titlebar:not(headerbar).selection-mode button.suggested-action .separator {
    border: 1px solid currentColor;
    color: rgba(76, 175, 80, 0.9); }
    .suggested-action.separator:disabled,
    .titlebar:not(headerbar).selection-mode button.separator.suggested-action:disabled, .suggested-action .separator:disabled,
    .titlebar:not(headerbar).selection-mode button.suggested-action .separator:disabled {
      color: rgba(76, 175, 80, 0.85); }

.destructive-action {
  background-color: #f44336;
  background-image: none;
  border-color: rgba(0, 0, 0, 0.32);
  color: #000000;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
  .destructive-action:focus, .destructive-action:hover {
    border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
  .destructive-action:active, .destructive-action:active:hover, .destructive-action:active:focus, .destructive-action:active:hover:focus, .destructive-action:checked, .destructive-action:checked:hover, .destructive-action:checked:focus, .destructive-action:checked:hover:focus {
    border-color: rgba(0, 0, 0, 0.32); }
  .destructive-action:disabled {
    border-color: rgba(0, 0, 0, 0.32); }
  .destructive-action:active:disabled, .destructive-action:checked:disabled {
    border-color: rgba(0, 0, 0, 0.32); }
  .destructive-action.flat {
    color: #000000;
    border-color: rgba(244, 67, 54, 0);
    background-color: rgba(244, 67, 54, 0);
    background-image: none;
    box-shadow: none; }
  .destructive-action:hover, .destructive-action.flat:hover {
    background-color: #f55044;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.4);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .destructive-action:hover:focus, .destructive-action:hover:hover, .destructive-action.flat:hover:focus, .destructive-action.flat:hover:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
    .destructive-action:hover:active, .destructive-action:hover:active:hover, .destructive-action:hover:active:focus, .destructive-action:hover:active:hover:focus, .destructive-action:hover:checked, .destructive-action:hover:checked:hover, .destructive-action:hover:checked:focus, .destructive-action:hover:checked:hover:focus, .destructive-action.flat:hover:active, .destructive-action.flat:hover:active:hover, .destructive-action.flat:hover:active:focus, .destructive-action.flat:hover:active:hover:focus, .destructive-action.flat:hover:checked, .destructive-action.flat:hover:checked:hover, .destructive-action.flat:hover:checked:focus, .destructive-action.flat:hover:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.4); }
    .destructive-action:hover:disabled, .destructive-action.flat:hover:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    .destructive-action:hover:active:disabled, .destructive-action:hover:checked:disabled, .destructive-action.flat:hover:active:disabled, .destructive-action.flat:hover:checked:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
  .destructive-action:focus, .destructive-action.flat:focus {
    background-color: #f55044;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.4);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    .destructive-action:focus:focus, .destructive-action:focus:hover, .destructive-action.flat:focus:focus, .destructive-action.flat:focus:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
    .destructive-action:focus:active, .destructive-action:focus:active:hover, .destructive-action:focus:active:focus, .destructive-action:focus:active:hover:focus, .destructive-action:focus:checked, .destructive-action:focus:checked:hover, .destructive-action:focus:checked:focus, .destructive-action:focus:checked:hover:focus, .destructive-action.flat:focus:active, .destructive-action.flat:focus:active:hover, .destructive-action.flat:focus:active:focus, .destructive-action.flat:focus:active:hover:focus, .destructive-action.flat:focus:checked, .destructive-action.flat:focus:checked:hover, .destructive-action.flat:focus:checked:focus, .destructive-action.flat:focus:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.4); }
    .destructive-action:focus:disabled, .destructive-action.flat:focus:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    .destructive-action:focus:active:disabled, .destructive-action:focus:checked:disabled, .destructive-action.flat:focus:active:disabled, .destructive-action.flat:focus:checked:disabled {
      border-color: rgba(0, 0, 0, 0.4); }
    .destructive-action:focus:hover, .destructive-action.flat:focus:hover {
      background-color: #f65d52;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
      .destructive-action:focus:hover:focus, .destructive-action:focus:hover:hover, .destructive-action.flat:focus:hover:focus, .destructive-action.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      .destructive-action:focus:hover:active, .destructive-action:focus:hover:active:hover, .destructive-action:focus:hover:active:focus, .destructive-action:focus:hover:active:hover:focus, .destructive-action:focus:hover:checked, .destructive-action:focus:hover:checked:hover, .destructive-action:focus:hover:checked:focus, .destructive-action:focus:hover:checked:hover:focus, .destructive-action.flat:focus:hover:active, .destructive-action.flat:focus:hover:active:hover, .destructive-action.flat:focus:hover:active:focus, .destructive-action.flat:focus:hover:active:hover:focus, .destructive-action.flat:focus:hover:checked, .destructive-action.flat:focus:hover:checked:hover, .destructive-action.flat:focus:hover:checked:focus, .destructive-action.flat:focus:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      .destructive-action:focus:hover:disabled, .destructive-action.flat:focus:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      .destructive-action:focus:hover:active:disabled, .destructive-action:focus:hover:checked:disabled, .destructive-action.flat:focus:hover:active:disabled, .destructive-action.flat:focus:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
  .destructive-action:checked, .destructive-action:active, .destructive-action.flat:checked, .destructive-action.flat:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
    border-color: rgba(0, 0, 0, 0.32); }
    .destructive-action:checked:focus, .destructive-action:checked:hover, .destructive-action:active:focus, .destructive-action:active:hover, .destructive-action.flat:checked:focus, .destructive-action.flat:checked:hover, .destructive-action.flat:active:focus, .destructive-action.flat:active:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    .destructive-action:checked:active, .destructive-action:checked:active:hover, .destructive-action:checked:active:focus, .destructive-action:checked:active:hover:focus, .destructive-action:checked:checked, .destructive-action:checked:checked:hover, .destructive-action:checked:checked:focus, .destructive-action:checked:checked:hover:focus, .destructive-action:active:active, .destructive-action:active:active:hover, .destructive-action:active:active:focus, .destructive-action:active:active:hover:focus, .destructive-action:active:checked, .destructive-action:active:checked:hover, .destructive-action:active:checked:focus, .destructive-action:active:checked:hover:focus, .destructive-action.flat:checked:active, .destructive-action.flat:checked:active:hover, .destructive-action.flat:checked:active:focus, .destructive-action.flat:checked:active:hover:focus, .destructive-action.flat:checked:checked, .destructive-action.flat:checked:checked:hover, .destructive-action.flat:checked:checked:focus, .destructive-action.flat:checked:checked:hover:focus, .destructive-action.flat:active:active, .destructive-action.flat:active:active:hover, .destructive-action.flat:active:active:focus, .destructive-action.flat:active:active:hover:focus, .destructive-action.flat:active:checked, .destructive-action.flat:active:checked:hover, .destructive-action.flat:active:checked:focus, .destructive-action.flat:active:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    .destructive-action:checked:disabled, .destructive-action:active:disabled, .destructive-action.flat:checked:disabled, .destructive-action.flat:active:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    .destructive-action:checked:active:disabled, .destructive-action:checked:checked:disabled, .destructive-action:active:active:disabled, .destructive-action:active:checked:disabled, .destructive-action.flat:checked:active:disabled, .destructive-action.flat:checked:checked:disabled, .destructive-action.flat:active:active:disabled, .destructive-action.flat:active:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    .destructive-action:checked:focus, .destructive-action:checked:hover, .destructive-action:active:focus, .destructive-action:active:hover, .destructive-action.flat:checked:focus, .destructive-action.flat:checked:hover, .destructive-action.flat:active:focus, .destructive-action.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  .destructive-action:focus, .destructive-action:hover, .destructive-action.flat:focus, .destructive-action.flat:hover {
    color: #000000; }
  .destructive-action:disabled:disabled, .destructive-action.flat:disabled:disabled {
    background-color: alpha(mix(#f44336,#000000,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(#f44336,#000000,0.5);
    box-shadow: none; }
  .destructive-action:active:disabled, .destructive-action:checked:disabled, .destructive-action.flat:active:disabled, .destructive-action.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  .destructive-action.separator, .destructive-action .separator {
    border: 1px solid currentColor;
    color: rgba(244, 67, 54, 0.9); }
    .destructive-action.separator:disabled, .destructive-action .separator:disabled {
      color: rgba(244, 67, 54, 0.85); }

/******************
 ! Selection mode *
*******************/
headerbar.selection-mode,
.titlebar:not(headerbar).selection-mode {
  background-color: #B36A87;
  background-image: none;
  border-color: #974d6a;
  color: #000000; }
  headerbar.selection-mode:focus, headerbar.selection-mode:hover,
  .titlebar:not(headerbar).selection-mode:focus,
  .titlebar:not(headerbar).selection-mode:hover {
    border-color: mix(#B36A87,#B36A87,0.3); }
  headerbar.selection-mode:active, headerbar.selection-mode:active:hover, headerbar.selection-mode:active:focus, headerbar.selection-mode:active:hover:focus, headerbar.selection-mode:checked, headerbar.selection-mode:checked:hover, headerbar.selection-mode:checked:focus, headerbar.selection-mode:checked:hover:focus,
  .titlebar:not(headerbar).selection-mode:active,
  .titlebar:not(headerbar).selection-mode:active:hover,
  .titlebar:not(headerbar).selection-mode:active:focus,
  .titlebar:not(headerbar).selection-mode:active:hover:focus,
  .titlebar:not(headerbar).selection-mode:checked,
  .titlebar:not(headerbar).selection-mode:checked:hover,
  .titlebar:not(headerbar).selection-mode:checked:focus,
  .titlebar:not(headerbar).selection-mode:checked:hover:focus {
    border-color: #84435d; }
  headerbar.selection-mode:disabled,
  .titlebar:not(headerbar).selection-mode:disabled {
    border-color: #a05271; }
  headerbar.selection-mode:active:disabled, headerbar.selection-mode:checked:disabled,
  .titlebar:not(headerbar).selection-mode:active:disabled,
  .titlebar:not(headerbar).selection-mode:checked:disabled {
    border-color: #974d6a; }
  headerbar.selection-mode:disabled,
  .titlebar:not(headerbar).selection-mode:disabled {
    background-color: #a95778;
    background-image: none;
    color: mix(#000000,#B36A87,0.5); }
  headerbar.selection-mode .title,
  .titlebar:not(headerbar).selection-mode .title {
    font-weight: bold;
    padding: 0 6px; }
  headerbar.selection-mode .subtitle,
  .titlebar:not(headerbar).selection-mode .subtitle {
    font-size: smaller;
    padding: 0 6px; }
  headerbar.selection-mode button,
  .titlebar:not(headerbar).selection-mode button {
    background-color: #B36A87;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.32);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    headerbar.selection-mode button:focus, headerbar.selection-mode button:hover,
    .titlebar:not(headerbar).selection-mode button:focus,
    .titlebar:not(headerbar).selection-mode button:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    headerbar.selection-mode button:active, headerbar.selection-mode button:active:hover, headerbar.selection-mode button:active:focus, headerbar.selection-mode button:active:hover:focus, headerbar.selection-mode button:checked, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button:checked:focus, headerbar.selection-mode button:checked:hover:focus,
    .titlebar:not(headerbar).selection-mode button:active,
    .titlebar:not(headerbar).selection-mode button:active:hover,
    .titlebar:not(headerbar).selection-mode button:active:focus,
    .titlebar:not(headerbar).selection-mode button:active:hover:focus,
    .titlebar:not(headerbar).selection-mode button:checked,
    .titlebar:not(headerbar).selection-mode button:checked:hover,
    .titlebar:not(headerbar).selection-mode button:checked:focus,
    .titlebar:not(headerbar).selection-mode button:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode button:disabled,
    .titlebar:not(headerbar).selection-mode button:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode button:active:disabled, headerbar.selection-mode button:checked:disabled,
    .titlebar:not(headerbar).selection-mode button:active:disabled,
    .titlebar:not(headerbar).selection-mode button:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode button.flat,
    .titlebar:not(headerbar).selection-mode button.flat {
      color: #000000;
      border-color: rgba(179, 106, 135, 0);
      background-color: rgba(179, 106, 135, 0);
      background-image: none;
      box-shadow: none; }
    headerbar.selection-mode button:hover, headerbar.selection-mode button.flat:hover,
    .titlebar:not(headerbar).selection-mode button:hover,
    .titlebar:not(headerbar).selection-mode button.flat:hover {
      background-color: #b8738f;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
      headerbar.selection-mode button:hover:focus, headerbar.selection-mode button:hover:hover, headerbar.selection-mode button.flat:hover:focus, headerbar.selection-mode button.flat:hover:hover,
      .titlebar:not(headerbar).selection-mode button:hover:focus,
      .titlebar:not(headerbar).selection-mode button:hover:hover,
      .titlebar:not(headerbar).selection-mode button.flat:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      headerbar.selection-mode button:hover:active, headerbar.selection-mode button:hover:active:hover, headerbar.selection-mode button:hover:active:focus, headerbar.selection-mode button:hover:active:hover:focus, headerbar.selection-mode button:hover:checked, headerbar.selection-mode button:hover:checked:hover, headerbar.selection-mode button:hover:checked:focus, headerbar.selection-mode button:hover:checked:hover:focus, headerbar.selection-mode button.flat:hover:active, headerbar.selection-mode button.flat:hover:active:hover, headerbar.selection-mode button.flat:hover:active:focus, headerbar.selection-mode button.flat:hover:active:hover:focus, headerbar.selection-mode button.flat:hover:checked, headerbar.selection-mode button.flat:hover:checked:hover, headerbar.selection-mode button.flat:hover:checked:focus, headerbar.selection-mode button.flat:hover:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:hover:active,
      .titlebar:not(headerbar).selection-mode button:hover:active:hover,
      .titlebar:not(headerbar).selection-mode button:hover:active:focus,
      .titlebar:not(headerbar).selection-mode button:hover:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:hover:checked,
      .titlebar:not(headerbar).selection-mode button:hover:checked:hover,
      .titlebar:not(headerbar).selection-mode button:hover:checked:focus,
      .titlebar:not(headerbar).selection-mode button:hover:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:hover:disabled, headerbar.selection-mode button.flat:hover:disabled,
      .titlebar:not(headerbar).selection-mode button:hover:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:hover:active:disabled, headerbar.selection-mode button:hover:checked:disabled, headerbar.selection-mode button.flat:hover:active:disabled, headerbar.selection-mode button.flat:hover:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:hover:active:disabled,
      .titlebar:not(headerbar).selection-mode button:hover:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
    headerbar.selection-mode button:focus, headerbar.selection-mode button.flat:focus,
    .titlebar:not(headerbar).selection-mode button:focus,
    .titlebar:not(headerbar).selection-mode button.flat:focus {
      background-color: #b8738f;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:focus:focus, headerbar.selection-mode button:focus:hover, headerbar.selection-mode button.flat:focus:focus, headerbar.selection-mode button.flat:focus:hover,
      .titlebar:not(headerbar).selection-mode button:focus:focus,
      .titlebar:not(headerbar).selection-mode button:focus:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      headerbar.selection-mode button:focus:active, headerbar.selection-mode button:focus:active:hover, headerbar.selection-mode button:focus:active:focus, headerbar.selection-mode button:focus:active:hover:focus, headerbar.selection-mode button:focus:checked, headerbar.selection-mode button:focus:checked:hover, headerbar.selection-mode button:focus:checked:focus, headerbar.selection-mode button:focus:checked:hover:focus, headerbar.selection-mode button.flat:focus:active, headerbar.selection-mode button.flat:focus:active:hover, headerbar.selection-mode button.flat:focus:active:focus, headerbar.selection-mode button.flat:focus:active:hover:focus, headerbar.selection-mode button.flat:focus:checked, headerbar.selection-mode button.flat:focus:checked:hover, headerbar.selection-mode button.flat:focus:checked:focus, headerbar.selection-mode button.flat:focus:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:focus:active,
      .titlebar:not(headerbar).selection-mode button:focus:active:hover,
      .titlebar:not(headerbar).selection-mode button:focus:active:focus,
      .titlebar:not(headerbar).selection-mode button:focus:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:focus:checked,
      .titlebar:not(headerbar).selection-mode button:focus:checked:hover,
      .titlebar:not(headerbar).selection-mode button:focus:checked:focus,
      .titlebar:not(headerbar).selection-mode button:focus:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:focus:disabled, headerbar.selection-mode button.flat:focus:disabled,
      .titlebar:not(headerbar).selection-mode button:focus:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:focus:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:focus:active:disabled, headerbar.selection-mode button:focus:checked:disabled, headerbar.selection-mode button.flat:focus:active:disabled, headerbar.selection-mode button.flat:focus:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:focus:active:disabled,
      .titlebar:not(headerbar).selection-mode button:focus:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:focus:hover, headerbar.selection-mode button.flat:focus:hover,
      .titlebar:not(headerbar).selection-mode button:focus:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:hover {
        background-color: #bd7d96;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
        headerbar.selection-mode button:focus:hover:focus, headerbar.selection-mode button:focus:hover:hover, headerbar.selection-mode button.flat:focus:hover:focus, headerbar.selection-mode button.flat:focus:hover:hover,
        .titlebar:not(headerbar).selection-mode button:focus:hover:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:hover,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        headerbar.selection-mode button:focus:hover:active, headerbar.selection-mode button:focus:hover:active:hover, headerbar.selection-mode button:focus:hover:active:focus, headerbar.selection-mode button:focus:hover:active:hover:focus, headerbar.selection-mode button:focus:hover:checked, headerbar.selection-mode button:focus:hover:checked:hover, headerbar.selection-mode button:focus:hover:checked:focus, headerbar.selection-mode button:focus:hover:checked:hover:focus, headerbar.selection-mode button.flat:focus:hover:active, headerbar.selection-mode button.flat:focus:hover:active:hover, headerbar.selection-mode button.flat:focus:hover:active:focus, headerbar.selection-mode button.flat:focus:hover:active:hover:focus, headerbar.selection-mode button.flat:focus:hover:checked, headerbar.selection-mode button.flat:focus:hover:checked:hover, headerbar.selection-mode button.flat:focus:hover:checked:focus, headerbar.selection-mode button.flat:focus:hover:checked:hover:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:hover,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:hover:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:hover,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:hover:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:hover,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:hover:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:hover,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        headerbar.selection-mode button:focus:hover:disabled, headerbar.selection-mode button.flat:focus:hover:disabled,
        .titlebar:not(headerbar).selection-mode button:focus:hover:disabled,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        headerbar.selection-mode button:focus:hover:active:disabled, headerbar.selection-mode button:focus:hover:checked:disabled, headerbar.selection-mode button.flat:focus:hover:active:disabled, headerbar.selection-mode button.flat:focus:hover:checked:disabled,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:disabled,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:disabled,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:disabled,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
    headerbar.selection-mode button:checked, headerbar.selection-mode button:active, headerbar.selection-mode button.flat:checked, headerbar.selection-mode button.flat:active,
    .titlebar:not(headerbar).selection-mode button:checked,
    .titlebar:not(headerbar).selection-mode button:active,
    .titlebar:not(headerbar).selection-mode button.flat:checked,
    .titlebar:not(headerbar).selection-mode button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
      border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:focus, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button:active:focus, headerbar.selection-mode button:active:hover, headerbar.selection-mode button.flat:checked:focus, headerbar.selection-mode button.flat:checked:hover, headerbar.selection-mode button.flat:active:focus, headerbar.selection-mode button.flat:active:hover,
      .titlebar:not(headerbar).selection-mode button:checked:focus,
      .titlebar:not(headerbar).selection-mode button:checked:hover,
      .titlebar:not(headerbar).selection-mode button:active:focus,
      .titlebar:not(headerbar).selection-mode button:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      headerbar.selection-mode button:checked:active, headerbar.selection-mode button:checked:active:hover, headerbar.selection-mode button:checked:active:focus, headerbar.selection-mode button:checked:active:hover:focus, headerbar.selection-mode button:checked:checked, headerbar.selection-mode button:checked:checked:hover, headerbar.selection-mode button:checked:checked:focus, headerbar.selection-mode button:checked:checked:hover:focus, headerbar.selection-mode button:active:active, headerbar.selection-mode button:active:active:hover, headerbar.selection-mode button:active:active:focus, headerbar.selection-mode button:active:active:hover:focus, headerbar.selection-mode button:active:checked, headerbar.selection-mode button:active:checked:hover, headerbar.selection-mode button:active:checked:focus, headerbar.selection-mode button:active:checked:hover:focus, headerbar.selection-mode button.flat:checked:active, headerbar.selection-mode button.flat:checked:active:hover, headerbar.selection-mode button.flat:checked:active:focus, headerbar.selection-mode button.flat:checked:active:hover:focus, headerbar.selection-mode button.flat:checked:checked, headerbar.selection-mode button.flat:checked:checked:hover, headerbar.selection-mode button.flat:checked:checked:focus, headerbar.selection-mode button.flat:checked:checked:hover:focus, headerbar.selection-mode button.flat:active:active, headerbar.selection-mode button.flat:active:active:hover, headerbar.selection-mode button.flat:active:active:focus, headerbar.selection-mode button.flat:active:active:hover:focus, headerbar.selection-mode button.flat:active:checked, headerbar.selection-mode button.flat:active:checked:hover, headerbar.selection-mode button.flat:active:checked:focus, headerbar.selection-mode button.flat:active:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:checked:active,
      .titlebar:not(headerbar).selection-mode button:checked:active:hover,
      .titlebar:not(headerbar).selection-mode button:checked:active:focus,
      .titlebar:not(headerbar).selection-mode button:checked:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:checked:checked,
      .titlebar:not(headerbar).selection-mode button:checked:checked:hover,
      .titlebar:not(headerbar).selection-mode button:checked:checked:focus,
      .titlebar:not(headerbar).selection-mode button:checked:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:active:active,
      .titlebar:not(headerbar).selection-mode button:active:active:hover,
      .titlebar:not(headerbar).selection-mode button:active:active:focus,
      .titlebar:not(headerbar).selection-mode button:active:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:active:checked,
      .titlebar:not(headerbar).selection-mode button:active:checked:hover,
      .titlebar:not(headerbar).selection-mode button:active:checked:focus,
      .titlebar:not(headerbar).selection-mode button:active:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:active,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:disabled, headerbar.selection-mode button:active:disabled, headerbar.selection-mode button.flat:checked:disabled, headerbar.selection-mode button.flat:active:disabled,
      .titlebar:not(headerbar).selection-mode button:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:active:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:active:disabled, headerbar.selection-mode button:checked:checked:disabled, headerbar.selection-mode button:active:active:disabled, headerbar.selection-mode button:active:checked:disabled, headerbar.selection-mode button.flat:checked:active:disabled, headerbar.selection-mode button.flat:checked:checked:disabled, headerbar.selection-mode button.flat:active:active:disabled, headerbar.selection-mode button.flat:active:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:checked:active:disabled,
      .titlebar:not(headerbar).selection-mode button:checked:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:active:active:disabled,
      .titlebar:not(headerbar).selection-mode button:active:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:focus, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button:active:focus, headerbar.selection-mode button:active:hover, headerbar.selection-mode button.flat:checked:focus, headerbar.selection-mode button.flat:checked:hover, headerbar.selection-mode button.flat:active:focus, headerbar.selection-mode button.flat:active:hover,
      .titlebar:not(headerbar).selection-mode button:checked:focus,
      .titlebar:not(headerbar).selection-mode button:checked:hover,
      .titlebar:not(headerbar).selection-mode button:active:focus,
      .titlebar:not(headerbar).selection-mode button:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    headerbar.selection-mode button:focus, headerbar.selection-mode button:hover, headerbar.selection-mode button.flat:focus, headerbar.selection-mode button.flat:hover,
    .titlebar:not(headerbar).selection-mode button:focus,
    .titlebar:not(headerbar).selection-mode button:hover,
    .titlebar:not(headerbar).selection-mode button.flat:focus,
    .titlebar:not(headerbar).selection-mode button.flat:hover {
      color: #000000; }
    headerbar.selection-mode button:disabled:disabled, headerbar.selection-mode button.flat:disabled:disabled,
    .titlebar:not(headerbar).selection-mode button:disabled:disabled,
    .titlebar:not(headerbar).selection-mode button.flat:disabled:disabled {
      background-color: alpha(mix(#B36A87,#000000,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#B36A87,#000000,0.5);
      box-shadow: none; }
    headerbar.selection-mode button:active:disabled, headerbar.selection-mode button:checked:disabled, headerbar.selection-mode button.flat:active:disabled, headerbar.selection-mode button.flat:checked:disabled,
    .titlebar:not(headerbar).selection-mode button:active:disabled,
    .titlebar:not(headerbar).selection-mode button:checked:disabled,
    .titlebar:not(headerbar).selection-mode button.flat:active:disabled,
    .titlebar:not(headerbar).selection-mode button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    headerbar.selection-mode button.separator, headerbar.selection-mode button .separator,
    .titlebar:not(headerbar).selection-mode button.separator,
    .titlebar:not(headerbar).selection-mode button .separator {
      border: 1px solid currentColor;
      color: rgba(179, 106, 135, 0.9); }
      headerbar.selection-mode button.separator:disabled, headerbar.selection-mode button .separator:disabled,
      .titlebar:not(headerbar).selection-mode button.separator:disabled,
      .titlebar:not(headerbar).selection-mode button .separator:disabled {
        color: rgba(179, 106, 135, 0.85); }
  headerbar.selection-mode .linked > button,
  .titlebar:not(headerbar).selection-mode .linked > button {
    box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    headerbar.selection-mode .linked > button:focus, headerbar.selection-mode .linked > button:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:hover {
      box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.12),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode .linked > button:active, headerbar.selection-mode .linked > button:active:hover, headerbar.selection-mode .linked > button:active:focus, headerbar.selection-mode .linked > button:active:hover:focus, headerbar.selection-mode .linked > button:checked, headerbar.selection-mode .linked > button:checked:hover, headerbar.selection-mode .linked > button:checked:focus, headerbar.selection-mode .linked > button:checked:hover:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:active,
    .titlebar:not(headerbar).selection-mode .linked > button:active:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:active:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:active:hover:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:checked,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:hover:focus {
      box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
    headerbar.selection-mode .linked > button:disabled,
    .titlebar:not(headerbar).selection-mode .linked > button:disabled {
      box-shadow: inset -1px 0 #974d6a; }
    headerbar.selection-mode .linked > button:last-child, headerbar.selection-mode .linked > button:only-child,
    .titlebar:not(headerbar).selection-mode .linked > button:last-child,
    .titlebar:not(headerbar).selection-mode .linked > button:only-child {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    headerbar.selection-mode .linked > button:last-child:hover, headerbar.selection-mode .linked > button:only-child:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:last-child:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:only-child:hover {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode .linked > button:disabled:last-child, headerbar.selection-mode .linked > button:disabled:only-child, headerbar.selection-mode .linked > button:active:disabled:last-child, headerbar.selection-mode .linked > button:active:disabled:only-child, headerbar.selection-mode .linked > button:checked:disabled:last-child, headerbar.selection-mode .linked > button:checked:disabled:only-child,
    .titlebar:not(headerbar).selection-mode .linked > button:disabled:last-child,
    .titlebar:not(headerbar).selection-mode .linked > button:disabled:only-child,
    .titlebar:not(headerbar).selection-mode .linked > button:active:disabled:last-child,
    .titlebar:not(headerbar).selection-mode .linked > button:active:disabled:only-child,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:disabled:last-child,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:disabled:only-child {
      box-shadow: none; }
    headerbar.selection-mode .linked > button:active:last-child, headerbar.selection-mode .linked > button:active:last-child:focus, headerbar.selection-mode .linked > button:active:last-child:hover, headerbar.selection-mode .linked > button:active:last-child:hover:focus, headerbar.selection-mode .linked > button:checked:last-child, headerbar.selection-mode .linked > button:checked:last-child:focus, headerbar.selection-mode .linked > button:checked:last-child:hover, headerbar.selection-mode .linked > button:checked:last-child:hover:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:active:last-child,
    .titlebar:not(headerbar).selection-mode .linked > button:active:last-child:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:active:last-child:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:active:last-child:hover:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:last-child,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:last-child:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:last-child:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:last-child:hover:focus {
      box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    headerbar.selection-mode .linked > button:active:only-child, headerbar.selection-mode .linked > button:active:only-child:focus, headerbar.selection-mode .linked > button:active:only-child:hover, headerbar.selection-mode .linked > button:active:only-child:hover:focus, headerbar.selection-mode .linked > button:checked:only-child, headerbar.selection-mode .linked > button:checked:only-child:focus, headerbar.selection-mode .linked > button:checked:only-child:hover, headerbar.selection-mode .linked > button:checked:only-child:hover:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:active:only-child,
    .titlebar:not(headerbar).selection-mode .linked > button:active:only-child:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:active:only-child:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:active:only-child:hover:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:only-child,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:only-child:focus,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:only-child:hover,
    .titlebar:not(headerbar).selection-mode .linked > button:checked:only-child:hover:focus {
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  headerbar.selection-mode combobox, headerbar.selection-mode button,
  .titlebar:not(headerbar).selection-mode combobox,
  .titlebar:not(headerbar).selection-mode button {
    padding: 3px; }
    headerbar.selection-mode combobox.text-button, headerbar.selection-mode button.text-button,
    .titlebar:not(headerbar).selection-mode combobox.text-button,
    .titlebar:not(headerbar).selection-mode button.text-button {
      padding: 3px; }
    headerbar.selection-mode combobox.image-button, headerbar.selection-mode button.image-button,
    .titlebar:not(headerbar).selection-mode combobox.image-button,
    .titlebar:not(headerbar).selection-mode button.image-button {
      padding: 3px; }
  headerbar.selection-mode separator, headerbar.selection-mode separator:disabled,
  .titlebar:not(headerbar).selection-mode separator,
  .titlebar:not(headerbar).selection-mode separator:disabled {
    color: #713a50;
    border-color: currentColor;
    -GtkWidget-window-dragging: true; }
  headerbar.selection-mode button,
  .titlebar:not(headerbar).selection-mode button {
    background-color: #B36A87;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.32);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    headerbar.selection-mode button:focus, headerbar.selection-mode button:hover,
    .titlebar:not(headerbar).selection-mode button:focus,
    .titlebar:not(headerbar).selection-mode button:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    headerbar.selection-mode button:active, headerbar.selection-mode button:active:hover, headerbar.selection-mode button:active:focus, headerbar.selection-mode button:active:hover:focus, headerbar.selection-mode button:checked, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button:checked:focus, headerbar.selection-mode button:checked:hover:focus,
    .titlebar:not(headerbar).selection-mode button:active,
    .titlebar:not(headerbar).selection-mode button:active:hover,
    .titlebar:not(headerbar).selection-mode button:active:focus,
    .titlebar:not(headerbar).selection-mode button:active:hover:focus,
    .titlebar:not(headerbar).selection-mode button:checked,
    .titlebar:not(headerbar).selection-mode button:checked:hover,
    .titlebar:not(headerbar).selection-mode button:checked:focus,
    .titlebar:not(headerbar).selection-mode button:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode button:disabled,
    .titlebar:not(headerbar).selection-mode button:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode button:active:disabled, headerbar.selection-mode button:checked:disabled,
    .titlebar:not(headerbar).selection-mode button:active:disabled,
    .titlebar:not(headerbar).selection-mode button:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    headerbar.selection-mode button.flat,
    .titlebar:not(headerbar).selection-mode button.flat {
      color: #000000;
      border-color: rgba(179, 106, 135, 0);
      background-color: rgba(179, 106, 135, 0);
      background-image: none;
      box-shadow: none; }
    headerbar.selection-mode button:hover, headerbar.selection-mode button.flat:hover,
    .titlebar:not(headerbar).selection-mode button:hover,
    .titlebar:not(headerbar).selection-mode button.flat:hover {
      background-color: #b8738f;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
      headerbar.selection-mode button:hover:focus, headerbar.selection-mode button:hover:hover, headerbar.selection-mode button.flat:hover:focus, headerbar.selection-mode button.flat:hover:hover,
      .titlebar:not(headerbar).selection-mode button:hover:focus,
      .titlebar:not(headerbar).selection-mode button:hover:hover,
      .titlebar:not(headerbar).selection-mode button.flat:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      headerbar.selection-mode button:hover:active, headerbar.selection-mode button:hover:active:hover, headerbar.selection-mode button:hover:active:focus, headerbar.selection-mode button:hover:active:hover:focus, headerbar.selection-mode button:hover:checked, headerbar.selection-mode button:hover:checked:hover, headerbar.selection-mode button:hover:checked:focus, headerbar.selection-mode button:hover:checked:hover:focus, headerbar.selection-mode button.flat:hover:active, headerbar.selection-mode button.flat:hover:active:hover, headerbar.selection-mode button.flat:hover:active:focus, headerbar.selection-mode button.flat:hover:active:hover:focus, headerbar.selection-mode button.flat:hover:checked, headerbar.selection-mode button.flat:hover:checked:hover, headerbar.selection-mode button.flat:hover:checked:focus, headerbar.selection-mode button.flat:hover:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:hover:active,
      .titlebar:not(headerbar).selection-mode button:hover:active:hover,
      .titlebar:not(headerbar).selection-mode button:hover:active:focus,
      .titlebar:not(headerbar).selection-mode button:hover:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:hover:checked,
      .titlebar:not(headerbar).selection-mode button:hover:checked:hover,
      .titlebar:not(headerbar).selection-mode button:hover:checked:focus,
      .titlebar:not(headerbar).selection-mode button:hover:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:hover:disabled, headerbar.selection-mode button.flat:hover:disabled,
      .titlebar:not(headerbar).selection-mode button:hover:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:hover:active:disabled, headerbar.selection-mode button:hover:checked:disabled, headerbar.selection-mode button.flat:hover:active:disabled, headerbar.selection-mode button.flat:hover:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:hover:active:disabled,
      .titlebar:not(headerbar).selection-mode button:hover:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:hover:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
    headerbar.selection-mode button:focus, headerbar.selection-mode button.flat:focus,
    .titlebar:not(headerbar).selection-mode button:focus,
    .titlebar:not(headerbar).selection-mode button.flat:focus {
      background-color: #b8738f;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:focus:focus, headerbar.selection-mode button:focus:hover, headerbar.selection-mode button.flat:focus:focus, headerbar.selection-mode button.flat:focus:hover,
      .titlebar:not(headerbar).selection-mode button:focus:focus,
      .titlebar:not(headerbar).selection-mode button:focus:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      headerbar.selection-mode button:focus:active, headerbar.selection-mode button:focus:active:hover, headerbar.selection-mode button:focus:active:focus, headerbar.selection-mode button:focus:active:hover:focus, headerbar.selection-mode button:focus:checked, headerbar.selection-mode button:focus:checked:hover, headerbar.selection-mode button:focus:checked:focus, headerbar.selection-mode button:focus:checked:hover:focus, headerbar.selection-mode button.flat:focus:active, headerbar.selection-mode button.flat:focus:active:hover, headerbar.selection-mode button.flat:focus:active:focus, headerbar.selection-mode button.flat:focus:active:hover:focus, headerbar.selection-mode button.flat:focus:checked, headerbar.selection-mode button.flat:focus:checked:hover, headerbar.selection-mode button.flat:focus:checked:focus, headerbar.selection-mode button.flat:focus:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:focus:active,
      .titlebar:not(headerbar).selection-mode button:focus:active:hover,
      .titlebar:not(headerbar).selection-mode button:focus:active:focus,
      .titlebar:not(headerbar).selection-mode button:focus:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:focus:checked,
      .titlebar:not(headerbar).selection-mode button:focus:checked:hover,
      .titlebar:not(headerbar).selection-mode button:focus:checked:focus,
      .titlebar:not(headerbar).selection-mode button:focus:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:focus:disabled, headerbar.selection-mode button.flat:focus:disabled,
      .titlebar:not(headerbar).selection-mode button:focus:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:focus:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:focus:active:disabled, headerbar.selection-mode button:focus:checked:disabled, headerbar.selection-mode button.flat:focus:active:disabled, headerbar.selection-mode button.flat:focus:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:focus:active:disabled,
      .titlebar:not(headerbar).selection-mode button:focus:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:focus:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:focus:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      headerbar.selection-mode button:focus:hover, headerbar.selection-mode button.flat:focus:hover,
      .titlebar:not(headerbar).selection-mode button:focus:hover,
      .titlebar:not(headerbar).selection-mode button.flat:focus:hover {
        background-color: #bd7d96;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
        headerbar.selection-mode button:focus:hover:focus, headerbar.selection-mode button:focus:hover:hover, headerbar.selection-mode button.flat:focus:hover:focus, headerbar.selection-mode button.flat:focus:hover:hover,
        .titlebar:not(headerbar).selection-mode button:focus:hover:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:hover,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        headerbar.selection-mode button:focus:hover:active, headerbar.selection-mode button:focus:hover:active:hover, headerbar.selection-mode button:focus:hover:active:focus, headerbar.selection-mode button:focus:hover:active:hover:focus, headerbar.selection-mode button:focus:hover:checked, headerbar.selection-mode button:focus:hover:checked:hover, headerbar.selection-mode button:focus:hover:checked:focus, headerbar.selection-mode button:focus:hover:checked:hover:focus, headerbar.selection-mode button.flat:focus:hover:active, headerbar.selection-mode button.flat:focus:hover:active:hover, headerbar.selection-mode button.flat:focus:hover:active:focus, headerbar.selection-mode button.flat:focus:hover:active:hover:focus, headerbar.selection-mode button.flat:focus:hover:checked, headerbar.selection-mode button.flat:focus:hover:checked:hover, headerbar.selection-mode button.flat:focus:hover:checked:focus, headerbar.selection-mode button.flat:focus:hover:checked:hover:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:hover,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:hover:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:hover,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:focus,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:hover:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:hover,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:hover:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:hover,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:focus,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        headerbar.selection-mode button:focus:hover:disabled, headerbar.selection-mode button.flat:focus:hover:disabled,
        .titlebar:not(headerbar).selection-mode button:focus:hover:disabled,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        headerbar.selection-mode button:focus:hover:active:disabled, headerbar.selection-mode button:focus:hover:checked:disabled, headerbar.selection-mode button.flat:focus:hover:active:disabled, headerbar.selection-mode button.flat:focus:hover:checked:disabled,
        .titlebar:not(headerbar).selection-mode button:focus:hover:active:disabled,
        .titlebar:not(headerbar).selection-mode button:focus:hover:checked:disabled,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:active:disabled,
        .titlebar:not(headerbar).selection-mode button.flat:focus:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
    headerbar.selection-mode button:checked, headerbar.selection-mode button:active, headerbar.selection-mode button.flat:checked, headerbar.selection-mode button.flat:active,
    .titlebar:not(headerbar).selection-mode button:checked,
    .titlebar:not(headerbar).selection-mode button:active,
    .titlebar:not(headerbar).selection-mode button.flat:checked,
    .titlebar:not(headerbar).selection-mode button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
      border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:focus, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button:active:focus, headerbar.selection-mode button:active:hover, headerbar.selection-mode button.flat:checked:focus, headerbar.selection-mode button.flat:checked:hover, headerbar.selection-mode button.flat:active:focus, headerbar.selection-mode button.flat:active:hover,
      .titlebar:not(headerbar).selection-mode button:checked:focus,
      .titlebar:not(headerbar).selection-mode button:checked:hover,
      .titlebar:not(headerbar).selection-mode button:active:focus,
      .titlebar:not(headerbar).selection-mode button:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      headerbar.selection-mode button:checked:active, headerbar.selection-mode button:checked:active:hover, headerbar.selection-mode button:checked:active:focus, headerbar.selection-mode button:checked:active:hover:focus, headerbar.selection-mode button:checked:checked, headerbar.selection-mode button:checked:checked:hover, headerbar.selection-mode button:checked:checked:focus, headerbar.selection-mode button:checked:checked:hover:focus, headerbar.selection-mode button:active:active, headerbar.selection-mode button:active:active:hover, headerbar.selection-mode button:active:active:focus, headerbar.selection-mode button:active:active:hover:focus, headerbar.selection-mode button:active:checked, headerbar.selection-mode button:active:checked:hover, headerbar.selection-mode button:active:checked:focus, headerbar.selection-mode button:active:checked:hover:focus, headerbar.selection-mode button.flat:checked:active, headerbar.selection-mode button.flat:checked:active:hover, headerbar.selection-mode button.flat:checked:active:focus, headerbar.selection-mode button.flat:checked:active:hover:focus, headerbar.selection-mode button.flat:checked:checked, headerbar.selection-mode button.flat:checked:checked:hover, headerbar.selection-mode button.flat:checked:checked:focus, headerbar.selection-mode button.flat:checked:checked:hover:focus, headerbar.selection-mode button.flat:active:active, headerbar.selection-mode button.flat:active:active:hover, headerbar.selection-mode button.flat:active:active:focus, headerbar.selection-mode button.flat:active:active:hover:focus, headerbar.selection-mode button.flat:active:checked, headerbar.selection-mode button.flat:active:checked:hover, headerbar.selection-mode button.flat:active:checked:focus, headerbar.selection-mode button.flat:active:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:checked:active,
      .titlebar:not(headerbar).selection-mode button:checked:active:hover,
      .titlebar:not(headerbar).selection-mode button:checked:active:focus,
      .titlebar:not(headerbar).selection-mode button:checked:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:checked:checked,
      .titlebar:not(headerbar).selection-mode button:checked:checked:hover,
      .titlebar:not(headerbar).selection-mode button:checked:checked:focus,
      .titlebar:not(headerbar).selection-mode button:checked:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button:active:active,
      .titlebar:not(headerbar).selection-mode button:active:active:hover,
      .titlebar:not(headerbar).selection-mode button:active:active:focus,
      .titlebar:not(headerbar).selection-mode button:active:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button:active:checked,
      .titlebar:not(headerbar).selection-mode button:active:checked:hover,
      .titlebar:not(headerbar).selection-mode button:active:checked:focus,
      .titlebar:not(headerbar).selection-mode button:active:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:active,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:hover:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:disabled, headerbar.selection-mode button:active:disabled, headerbar.selection-mode button.flat:checked:disabled, headerbar.selection-mode button.flat:active:disabled,
      .titlebar:not(headerbar).selection-mode button:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:active:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:active:disabled, headerbar.selection-mode button:checked:checked:disabled, headerbar.selection-mode button:active:active:disabled, headerbar.selection-mode button:active:checked:disabled, headerbar.selection-mode button.flat:checked:active:disabled, headerbar.selection-mode button.flat:checked:checked:disabled, headerbar.selection-mode button.flat:active:active:disabled, headerbar.selection-mode button.flat:active:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:checked:active:disabled,
      .titlebar:not(headerbar).selection-mode button:checked:checked:disabled,
      .titlebar:not(headerbar).selection-mode button:active:active:disabled,
      .titlebar:not(headerbar).selection-mode button:active:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:checked:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:checked:checked:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:active:active:disabled,
      .titlebar:not(headerbar).selection-mode button.flat:active:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      headerbar.selection-mode button:checked:focus, headerbar.selection-mode button:checked:hover, headerbar.selection-mode button:active:focus, headerbar.selection-mode button:active:hover, headerbar.selection-mode button.flat:checked:focus, headerbar.selection-mode button.flat:checked:hover, headerbar.selection-mode button.flat:active:focus, headerbar.selection-mode button.flat:active:hover,
      .titlebar:not(headerbar).selection-mode button:checked:focus,
      .titlebar:not(headerbar).selection-mode button:checked:hover,
      .titlebar:not(headerbar).selection-mode button:active:focus,
      .titlebar:not(headerbar).selection-mode button:active:hover,
      .titlebar:not(headerbar).selection-mode button.flat:checked:focus,
      .titlebar:not(headerbar).selection-mode button.flat:checked:hover,
      .titlebar:not(headerbar).selection-mode button.flat:active:focus,
      .titlebar:not(headerbar).selection-mode button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    headerbar.selection-mode button:focus, headerbar.selection-mode button:hover, headerbar.selection-mode button.flat:focus, headerbar.selection-mode button.flat:hover,
    .titlebar:not(headerbar).selection-mode button:focus,
    .titlebar:not(headerbar).selection-mode button:hover,
    .titlebar:not(headerbar).selection-mode button.flat:focus,
    .titlebar:not(headerbar).selection-mode button.flat:hover {
      color: #000000; }
    headerbar.selection-mode button:disabled:disabled, headerbar.selection-mode button.flat:disabled:disabled,
    .titlebar:not(headerbar).selection-mode button:disabled:disabled,
    .titlebar:not(headerbar).selection-mode button.flat:disabled:disabled {
      background-color: alpha(mix(#B36A87,#000000,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#B36A87,#000000,0.5);
      box-shadow: none; }
    headerbar.selection-mode button:active:disabled, headerbar.selection-mode button:checked:disabled, headerbar.selection-mode button.flat:active:disabled, headerbar.selection-mode button.flat:checked:disabled,
    .titlebar:not(headerbar).selection-mode button:active:disabled,
    .titlebar:not(headerbar).selection-mode button:checked:disabled,
    .titlebar:not(headerbar).selection-mode button.flat:active:disabled,
    .titlebar:not(headerbar).selection-mode button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    headerbar.selection-mode button.separator, headerbar.selection-mode button .separator,
    .titlebar:not(headerbar).selection-mode button.separator,
    .titlebar:not(headerbar).selection-mode button .separator {
      border: 1px solid currentColor;
      color: rgba(179, 106, 135, 0.9); }
      headerbar.selection-mode button.separator:disabled, headerbar.selection-mode button .separator:disabled,
      .titlebar:not(headerbar).selection-mode button.separator:disabled,
      .titlebar:not(headerbar).selection-mode button .separator:disabled {
        color: rgba(179, 106, 135, 0.85); }
  headerbar.selection-mode:backdrop,
  .titlebar:not(headerbar).selection-mode:backdrop {
    background-color: #B36A87;
    background-image: none; }
  headerbar.selection-mode .selection-menu:backdrop, headerbar.selection-mode .selection-menu,
  .titlebar:not(headerbar).selection-mode .selection-menu:backdrop,
  .titlebar:not(headerbar).selection-mode .selection-menu {
    color: #974d6a;
    background-color: transparent;
    background-image: none;
    box-shadow: none;
    border: 0; }
    headerbar.selection-mode .selection-menu:backdrop:hover, headerbar.selection-mode .selection-menu:hover,
    .titlebar:not(headerbar).selection-mode .selection-menu:backdrop:hover,
    .titlebar:not(headerbar).selection-mode .selection-menu:hover {
      color: #84435d; }
    headerbar.selection-mode .selection-menu:backdrop:active, headerbar.selection-mode .selection-menu:active,
    .titlebar:not(headerbar).selection-mode .selection-menu:backdrop:active,
    .titlebar:not(headerbar).selection-mode .selection-menu:active {
      color: #8e4864; }
    headerbar.selection-mode .selection-menu:backdrop .arrow, headerbar.selection-mode .selection-menu .arrow,
    .titlebar:not(headerbar).selection-mode .selection-menu:backdrop .arrow,
    .titlebar:not(headerbar).selection-mode .selection-menu .arrow {
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
      color: rgba(0, 0, 0, 0.5);
      -gtk-icon-shadow: none; }
  headerbar.selection-mode .dim-label, headerbar.selection-mode label.separator, .selection-menu headerbar.selection-mode .dim-label, .selection-menu headerbar.selection-mode label.separator,
  .titlebar:not(headerbar).selection-mode .dim-label,
  .titlebar:not(headerbar).selection-mode label.separator, .selection-menu
  .titlebar:not(headerbar).selection-mode .dim-label, .selection-menu
  .titlebar:not(headerbar).selection-mode label.separator {
    color: #84435d; }

/**********
 ! Calendar
***********/
calendar {
  padding: 1px 3px;
  outline-offset: -1px;
  color: #000000; }
  calendar:selected {
    border-radius: 3px; }
  calendar.header {
    border-bottom: 1px solid rgba(0, 0, 0, 0.1);
    border-radius: 0; }
    calendar.header:backdrop {
      border-color: rgba(0, 0, 0, 0.1); }
  calendar.button {
    color: rgba(0, 0, 0, 0.55); }
    calendar.button:hover {
      color: #000000; }
    calendar.button:backdrop {
      color: alpha(mix(#000000,#a5a5a5,0.5),0.55); }
    calendar.button:disabled {
      color: alpha(mix(#000000,#a5a5a5,0.5),0.55); }
  calendar:indeterminate, calendar:indeterminate:backdrop {
    color: mix(#000000,#a5a5a5,0.5); }
  calendar.highlight, calendar.highlight:backdrop {
    font-size: smaller;
    color: mix(#B36A87,#000000,0.5); }
  calendar:backdrop {
    color: mix(#d0d0d0,#000000,0.8); }

/* gnome-calendar */
.calendar-view {
  background-color: #d0d0d0;
  color: #000000; }

/***************
 ! Color chooser
****************/
colorswatch:drop(active), colorswatch {
  border-style: none; }

colorswatch.top {
  border-top-left-radius: 3.5px;
  border-top-right-radius: 3.5px; }
  colorswatch.top overlay {
    border-top-left-radius: 3px;
    border-top-right-radius: 3px; }

colorswatch.bottom {
  border-bottom-left-radius: 3.5px;
  border-bottom-right-radius: 3.5px; }
  colorswatch.bottom overlay {
    border-bottom-left-radius: 3px;
    border-bottom-right-radius: 3px; }

colorswatch.left, colorswatch:first-child:not(.top) {
  border-top-left-radius: 3.5px;
  border-bottom-left-radius: 3.5px; }
  colorswatch.left overlay, colorswatch:first-child:not(.top) overlay {
    border-top-left-radius: 3px;
    border-bottom-left-radius: 3px; }

colorswatch.right, colorswatch:last-child:not(.bottom) {
  border-top-right-radius: 3.5px;
  border-bottom-right-radius: 3.5px; }
  colorswatch.right overlay, colorswatch:last-child:not(.bottom) overlay {
    border-top-right-radius: 3px;
    border-bottom-right-radius: 3px; }

colorswatch.dark overlay {
  color: #000000; }
  colorswatch.dark overlay:hover {
    border-color: rgba(0, 0, 0, 0.8); }
  colorswatch.dark overlay:backdrop {
    color: mix(#B36A87,#000000,0.66); }

colorswatch.light overlay {
  color: #000000; }
  colorswatch.light overlay:hover {
    border-color: rgba(0, 0, 0, 0.5); }
  colorswatch.light overlay:backdrop {
    color: mix(#d0d0d0,#000000,0.8); }

colorswatch:drop(active) {
  box-shadow: none; }
  colorswatch:drop(active).light overlay {
    border-color: #4e9a06;
    box-shadow: inset 0 0 0 2px #3d7805, inset 0 0 0 1px #4e9a06; }
  colorswatch:drop(active).dark overlay {
    border-color: #4e9a06;
    box-shadow: inset 0 0 0 2px rgba(0, 0, 0, 0.3), inset 0 0 0 1px #4e9a06; }

colorswatch overlay {
  border: 1px solid rgba(0, 0, 0, 0.3); }
  colorswatch overlay:hover {
    box-shadow: inset 0 1px rgba(255, 255, 255, 0.4), inset 0 -1px rgba(0, 0, 0, 0.2); }
  colorswatch overlay:backdrop, colorswatch overlay:backdrop:hover {
    border-color: rgba(0, 0, 0, 0.3);
    box-shadow: none; }

colorswatch:disabled {
  opacity: .5; }
  colorswatch:disabled overlay {
    border-color: rgba(0, 0, 0, 0.6);
    box-shadow: none; }

row:selected colorswatch {
  box-shadow: 0 0 0 2px #000000; }

colorswatch#add-color-button {
  border-radius: 3px 3px 0 0; }
  colorswatch#add-color-button:only-child {
    border-radius: 3px; }
  colorswatch#add-color-button overlay {
    background-color: #9d9d9d;
    color: #000000; }
    colorswatch#add-color-button overlay:hover {
      background-color: #959595; }
    colorswatch#add-color-button overlay:backdrop {
      background-color: #9d9d9d; }

colorswatch#editor-color-sample {
  border-radius: 3px; }
  colorswatch#editor-color-sample overlay {
    border-radius: 3.5px; }

button.color {
  padding: 3px; }
  button.color colorswatch:only-child {
    box-shadow: 0 1px rgba(255, 255, 255, 0.769231); }
    button.color colorswatch:only-child, button.color colorswatch:only-child overlay {
      border-radius: 0; }
  button.color:disabled colorswatch:only-child, button.color:backdrop colorswatch:only-child, button.color:active colorswatch:only-child, button.color:checked colorswatch:only-child {
    box-shadow: none; }

/***********************
! Font and file choosers
************************/
filechooser {
  /* for fallback when header bar not used */ }
  filechooser .dialog-action-box {
    border-top: 1px solid mix(#a5a5a5,#000000,0.08); }
    filechooser .dialog-action-box:backdrop {
      border-top-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9); }
  filechooser #pathbarbox {
    border-bottom: 1px solid #a5a5a5; }

filechooserbutton:drop(active) {
  box-shadow: none;
  border-color: transparent; }

/******************
 ! Grid and flowbox
*******************/
list {
  color: #000000;
  background-color: #d0d0d0;
  border-color: mix(#a5a5a5,#000000,0.08); }
  list:backdrop {
    background-color: #d0d0d0;
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9); }
  list row {
    padding: 3px; }

row {
  transition: all 150ms cubic-bezier(0.25, 0.46, 0.45, 0.94); }
  row:hover {
    transition: none; }
  row:backdrop {
    transition: 200ms ease-out; }
  row.activatable.has-open-popup, row.activatable:hover {
    background-color: rgba(0, 0, 0, 0.05); }
  row.activatable:active {
    box-shadow: inset 0 2px 2px -2px rgba(0, 0, 0, 0.2); }
  row.activatable:backdrop:hover {
    background-color: transparent; }
  row.activatable:selected:active {
    box-shadow: inset 0 2px 3px -1px rgba(0, 0, 0, 0.5); }
  row.activatable:selected.has-open-popup, row.activatable:selected:hover {
    background-color: mix(#B36A87,#000000,0.1); }
  row.activatable:selected:backdrop {
    background-color: #B36A87; }

flowbox flowboxchild {
  padding: 3px;
  border-radius: 3px; }
  flowbox flowboxchild:selected {
    outline-offset: -2px; }

/*********
 ! Buttons
**********/
/******************
! ComboBoxes *
*******************/
/*********
 ! Infobar
**********/
infobar {
  border: 0; }
  infobar.info, infobar.info:backdrop {
    background-color: #03a9f4;
    background-image: none;
    border: 1px solid #0287c3;
    caret-color: currentColor; }
    infobar.info label, infobar.info, infobar.info:backdrop label, infobar.info:backdrop {
      color: #fff; }
  infobar.info button {
    background-color: #03a9f4;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.22);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    infobar.info button:focus, infobar.info button:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
    infobar.info button:active, infobar.info button:active:hover, infobar.info button:active:focus, infobar.info button:active:hover:focus, infobar.info button:checked, infobar.info button:checked:hover, infobar.info button:checked:focus, infobar.info button:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.22); }
    infobar.info button:disabled {
      border-color: rgba(217, 217, 217, 0.22); }
    infobar.info button:active:disabled, infobar.info button:checked:disabled {
      border-color: rgba(204, 204, 204, 0.22); }
    infobar.info button.flat {
      color: #fff;
      border-color: rgba(3, 169, 244, 0);
      background-color: rgba(3, 169, 244, 0);
      background-image: none;
      box-shadow: none; }
    infobar.info button:hover, infobar.info button.flat:hover {
      background-color: #07b0fc;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      infobar.info button:hover:focus, infobar.info button:hover:hover, infobar.info button.flat:hover:focus, infobar.info button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.info button:hover:active, infobar.info button:hover:active:hover, infobar.info button:hover:active:focus, infobar.info button:hover:active:hover:focus, infobar.info button:hover:checked, infobar.info button:hover:checked:hover, infobar.info button:hover:checked:focus, infobar.info button:hover:checked:hover:focus, infobar.info button.flat:hover:active, infobar.info button.flat:hover:active:hover, infobar.info button.flat:hover:active:focus, infobar.info button.flat:hover:active:hover:focus, infobar.info button.flat:hover:checked, infobar.info button.flat:hover:checked:hover, infobar.info button.flat:hover:checked:focus, infobar.info button.flat:hover:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.info button:hover:disabled, infobar.info button.flat:hover:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.info button:hover:active:disabled, infobar.info button:hover:checked:disabled, infobar.info button.flat:hover:active:disabled, infobar.info button.flat:hover:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
    infobar.info button:focus, infobar.info button.flat:focus {
      background-color: #07b0fc;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      infobar.info button:focus:focus, infobar.info button:focus:hover, infobar.info button.flat:focus:focus, infobar.info button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.info button:focus:active, infobar.info button:focus:active:hover, infobar.info button:focus:active:focus, infobar.info button:focus:active:hover:focus, infobar.info button:focus:checked, infobar.info button:focus:checked:hover, infobar.info button:focus:checked:focus, infobar.info button:focus:checked:hover:focus, infobar.info button.flat:focus:active, infobar.info button.flat:focus:active:hover, infobar.info button.flat:focus:active:focus, infobar.info button.flat:focus:active:hover:focus, infobar.info button.flat:focus:checked, infobar.info button.flat:focus:checked:hover, infobar.info button.flat:focus:checked:focus, infobar.info button.flat:focus:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.info button:focus:disabled, infobar.info button.flat:focus:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.info button:focus:active:disabled, infobar.info button:focus:checked:disabled, infobar.info button.flat:focus:active:disabled, infobar.info button.flat:focus:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
      infobar.info button:focus:hover, infobar.info button.flat:focus:hover {
        background-color: #14b4fc;
        background-image: none;
        border-color: rgba(204, 204, 204, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        infobar.info button:focus:hover:focus, infobar.info button:focus:hover:hover, infobar.info button.flat:focus:hover:focus, infobar.info button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
        infobar.info button:focus:hover:active, infobar.info button:focus:hover:active:hover, infobar.info button:focus:hover:active:focus, infobar.info button:focus:hover:active:hover:focus, infobar.info button:focus:hover:checked, infobar.info button:focus:hover:checked:hover, infobar.info button:focus:hover:checked:focus, infobar.info button:focus:hover:checked:hover:focus, infobar.info button.flat:focus:hover:active, infobar.info button.flat:focus:hover:active:hover, infobar.info button.flat:focus:hover:active:focus, infobar.info button.flat:focus:hover:active:hover:focus, infobar.info button.flat:focus:hover:checked, infobar.info button.flat:focus:hover:checked:hover, infobar.info button.flat:focus:hover:checked:focus, infobar.info button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(179, 179, 179, 0.3); }
        infobar.info button:focus:hover:disabled, infobar.info button.flat:focus:hover:disabled {
          border-color: rgba(217, 217, 217, 0.3); }
        infobar.info button:focus:hover:active:disabled, infobar.info button:focus:hover:checked:disabled, infobar.info button.flat:focus:hover:active:disabled, infobar.info button.flat:focus:hover:checked:disabled {
          border-color: rgba(204, 204, 204, 0.3); }
    infobar.info button:checked, infobar.info button:active, infobar.info button.flat:checked, infobar.info button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(255, 255, 255, 0.06), inset 0 1px rgba(255, 255, 255, 0.07), inset -1px 0 rgba(255, 255, 255, 0.06), inset 0 -1px rgba(255, 255, 255, 0.05);
      border-color: rgba(204, 204, 204, 0.22); }
      infobar.info button:checked:focus, infobar.info button:checked:hover, infobar.info button:active:focus, infobar.info button:active:hover, infobar.info button.flat:checked:focus, infobar.info button.flat:checked:hover, infobar.info button.flat:active:focus, infobar.info button.flat:active:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
      infobar.info button:checked:active, infobar.info button:checked:active:hover, infobar.info button:checked:active:focus, infobar.info button:checked:active:hover:focus, infobar.info button:checked:checked, infobar.info button:checked:checked:hover, infobar.info button:checked:checked:focus, infobar.info button:checked:checked:hover:focus, infobar.info button:active:active, infobar.info button:active:active:hover, infobar.info button:active:active:focus, infobar.info button:active:active:hover:focus, infobar.info button:active:checked, infobar.info button:active:checked:hover, infobar.info button:active:checked:focus, infobar.info button:active:checked:hover:focus, infobar.info button.flat:checked:active, infobar.info button.flat:checked:active:hover, infobar.info button.flat:checked:active:focus, infobar.info button.flat:checked:active:hover:focus, infobar.info button.flat:checked:checked, infobar.info button.flat:checked:checked:hover, infobar.info button.flat:checked:checked:focus, infobar.info button.flat:checked:checked:hover:focus, infobar.info button.flat:active:active, infobar.info button.flat:active:active:hover, infobar.info button.flat:active:active:focus, infobar.info button.flat:active:active:hover:focus, infobar.info button.flat:active:checked, infobar.info button.flat:active:checked:hover, infobar.info button.flat:active:checked:focus, infobar.info button.flat:active:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.22); }
      infobar.info button:checked:disabled, infobar.info button:active:disabled, infobar.info button.flat:checked:disabled, infobar.info button.flat:active:disabled {
        border-color: rgba(217, 217, 217, 0.22); }
      infobar.info button:checked:active:disabled, infobar.info button:checked:checked:disabled, infobar.info button:active:active:disabled, infobar.info button:active:checked:disabled, infobar.info button.flat:checked:active:disabled, infobar.info button.flat:checked:checked:disabled, infobar.info button.flat:active:active:disabled, infobar.info button.flat:active:checked:disabled {
        border-color: rgba(204, 204, 204, 0.22); }
      infobar.info button:checked:focus, infobar.info button:checked:hover, infobar.info button:active:focus, infobar.info button:active:hover, infobar.info button.flat:checked:focus, infobar.info button.flat:checked:hover, infobar.info button.flat:active:focus, infobar.info button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    infobar.info button:focus, infobar.info button:hover, infobar.info button.flat:focus, infobar.info button.flat:hover {
      color: #fff; }
    infobar.info button:disabled:disabled, infobar.info button.flat:disabled:disabled {
      background-color: alpha(mix(#03a9f4,#fff,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#03a9f4,#fff,0.5);
      box-shadow: none; }
    infobar.info button:active:disabled, infobar.info button:checked:disabled, infobar.info button.flat:active:disabled, infobar.info button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    infobar.info button.separator, infobar.info button .separator {
      border: 1px solid currentColor;
      color: rgba(3, 169, 244, 0.9); }
      infobar.info button.separator:disabled, infobar.info button .separator:disabled {
        color: rgba(3, 169, 244, 0.85); }
  infobar.warning, infobar.warning:backdrop {
    background-color: #ef6c00;
    background-image: none;
    border: 1px solid #bf5600;
    caret-color: currentColor; }
    infobar.warning label, infobar.warning, infobar.warning:backdrop label, infobar.warning:backdrop {
      color: #fff; }
  infobar.warning button {
    background-color: #ef6c00;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.22);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    infobar.warning button:focus, infobar.warning button:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
    infobar.warning button:active, infobar.warning button:active:hover, infobar.warning button:active:focus, infobar.warning button:active:hover:focus, infobar.warning button:checked, infobar.warning button:checked:hover, infobar.warning button:checked:focus, infobar.warning button:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.22); }
    infobar.warning button:disabled {
      border-color: rgba(217, 217, 217, 0.22); }
    infobar.warning button:active:disabled, infobar.warning button:checked:disabled {
      border-color: rgba(204, 204, 204, 0.22); }
    infobar.warning button.flat {
      color: #fff;
      border-color: rgba(239, 108, 0, 0);
      background-color: rgba(239, 108, 0, 0);
      background-image: none;
      box-shadow: none; }
    infobar.warning button:hover, infobar.warning button.flat:hover {
      background-color: #fb7100;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      infobar.warning button:hover:focus, infobar.warning button:hover:hover, infobar.warning button.flat:hover:focus, infobar.warning button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.warning button:hover:active, infobar.warning button:hover:active:hover, infobar.warning button:hover:active:focus, infobar.warning button:hover:active:hover:focus, infobar.warning button:hover:checked, infobar.warning button:hover:checked:hover, infobar.warning button:hover:checked:focus, infobar.warning button:hover:checked:hover:focus, infobar.warning button.flat:hover:active, infobar.warning button.flat:hover:active:hover, infobar.warning button.flat:hover:active:focus, infobar.warning button.flat:hover:active:hover:focus, infobar.warning button.flat:hover:checked, infobar.warning button.flat:hover:checked:hover, infobar.warning button.flat:hover:checked:focus, infobar.warning button.flat:hover:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.warning button:hover:disabled, infobar.warning button.flat:hover:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.warning button:hover:active:disabled, infobar.warning button:hover:checked:disabled, infobar.warning button.flat:hover:active:disabled, infobar.warning button.flat:hover:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
    infobar.warning button:focus, infobar.warning button.flat:focus {
      background-color: #fb7100;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      infobar.warning button:focus:focus, infobar.warning button:focus:hover, infobar.warning button.flat:focus:focus, infobar.warning button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.warning button:focus:active, infobar.warning button:focus:active:hover, infobar.warning button:focus:active:focus, infobar.warning button:focus:active:hover:focus, infobar.warning button:focus:checked, infobar.warning button:focus:checked:hover, infobar.warning button:focus:checked:focus, infobar.warning button:focus:checked:hover:focus, infobar.warning button.flat:focus:active, infobar.warning button.flat:focus:active:hover, infobar.warning button.flat:focus:active:focus, infobar.warning button.flat:focus:active:hover:focus, infobar.warning button.flat:focus:checked, infobar.warning button.flat:focus:checked:hover, infobar.warning button.flat:focus:checked:focus, infobar.warning button.flat:focus:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.warning button:focus:disabled, infobar.warning button.flat:focus:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.warning button:focus:active:disabled, infobar.warning button:focus:checked:disabled, infobar.warning button.flat:focus:active:disabled, infobar.warning button.flat:focus:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
      infobar.warning button:focus:hover, infobar.warning button.flat:focus:hover {
        background-color: #ff7808;
        background-image: none;
        border-color: rgba(204, 204, 204, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        infobar.warning button:focus:hover:focus, infobar.warning button:focus:hover:hover, infobar.warning button.flat:focus:hover:focus, infobar.warning button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
        infobar.warning button:focus:hover:active, infobar.warning button:focus:hover:active:hover, infobar.warning button:focus:hover:active:focus, infobar.warning button:focus:hover:active:hover:focus, infobar.warning button:focus:hover:checked, infobar.warning button:focus:hover:checked:hover, infobar.warning button:focus:hover:checked:focus, infobar.warning button:focus:hover:checked:hover:focus, infobar.warning button.flat:focus:hover:active, infobar.warning button.flat:focus:hover:active:hover, infobar.warning button.flat:focus:hover:active:focus, infobar.warning button.flat:focus:hover:active:hover:focus, infobar.warning button.flat:focus:hover:checked, infobar.warning button.flat:focus:hover:checked:hover, infobar.warning button.flat:focus:hover:checked:focus, infobar.warning button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(179, 179, 179, 0.3); }
        infobar.warning button:focus:hover:disabled, infobar.warning button.flat:focus:hover:disabled {
          border-color: rgba(217, 217, 217, 0.3); }
        infobar.warning button:focus:hover:active:disabled, infobar.warning button:focus:hover:checked:disabled, infobar.warning button.flat:focus:hover:active:disabled, infobar.warning button.flat:focus:hover:checked:disabled {
          border-color: rgba(204, 204, 204, 0.3); }
    infobar.warning button:checked, infobar.warning button:active, infobar.warning button.flat:checked, infobar.warning button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(255, 255, 255, 0.06), inset 0 1px rgba(255, 255, 255, 0.07), inset -1px 0 rgba(255, 255, 255, 0.06), inset 0 -1px rgba(255, 255, 255, 0.05);
      border-color: rgba(204, 204, 204, 0.22); }
      infobar.warning button:checked:focus, infobar.warning button:checked:hover, infobar.warning button:active:focus, infobar.warning button:active:hover, infobar.warning button.flat:checked:focus, infobar.warning button.flat:checked:hover, infobar.warning button.flat:active:focus, infobar.warning button.flat:active:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
      infobar.warning button:checked:active, infobar.warning button:checked:active:hover, infobar.warning button:checked:active:focus, infobar.warning button:checked:active:hover:focus, infobar.warning button:checked:checked, infobar.warning button:checked:checked:hover, infobar.warning button:checked:checked:focus, infobar.warning button:checked:checked:hover:focus, infobar.warning button:active:active, infobar.warning button:active:active:hover, infobar.warning button:active:active:focus, infobar.warning button:active:active:hover:focus, infobar.warning button:active:checked, infobar.warning button:active:checked:hover, infobar.warning button:active:checked:focus, infobar.warning button:active:checked:hover:focus, infobar.warning button.flat:checked:active, infobar.warning button.flat:checked:active:hover, infobar.warning button.flat:checked:active:focus, infobar.warning button.flat:checked:active:hover:focus, infobar.warning button.flat:checked:checked, infobar.warning button.flat:checked:checked:hover, infobar.warning button.flat:checked:checked:focus, infobar.warning button.flat:checked:checked:hover:focus, infobar.warning button.flat:active:active, infobar.warning button.flat:active:active:hover, infobar.warning button.flat:active:active:focus, infobar.warning button.flat:active:active:hover:focus, infobar.warning button.flat:active:checked, infobar.warning button.flat:active:checked:hover, infobar.warning button.flat:active:checked:focus, infobar.warning button.flat:active:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.22); }
      infobar.warning button:checked:disabled, infobar.warning button:active:disabled, infobar.warning button.flat:checked:disabled, infobar.warning button.flat:active:disabled {
        border-color: rgba(217, 217, 217, 0.22); }
      infobar.warning button:checked:active:disabled, infobar.warning button:checked:checked:disabled, infobar.warning button:active:active:disabled, infobar.warning button:active:checked:disabled, infobar.warning button.flat:checked:active:disabled, infobar.warning button.flat:checked:checked:disabled, infobar.warning button.flat:active:active:disabled, infobar.warning button.flat:active:checked:disabled {
        border-color: rgba(204, 204, 204, 0.22); }
      infobar.warning button:checked:focus, infobar.warning button:checked:hover, infobar.warning button:active:focus, infobar.warning button:active:hover, infobar.warning button.flat:checked:focus, infobar.warning button.flat:checked:hover, infobar.warning button.flat:active:focus, infobar.warning button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    infobar.warning button:focus, infobar.warning button:hover, infobar.warning button.flat:focus, infobar.warning button.flat:hover {
      color: #fff; }
    infobar.warning button:disabled:disabled, infobar.warning button.flat:disabled:disabled {
      background-color: alpha(mix(#ef6c00,#fff,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#ef6c00,#fff,0.5);
      box-shadow: none; }
    infobar.warning button:active:disabled, infobar.warning button:checked:disabled, infobar.warning button.flat:active:disabled, infobar.warning button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    infobar.warning button.separator, infobar.warning button .separator {
      border: 1px solid currentColor;
      color: rgba(239, 108, 0, 0.9); }
      infobar.warning button.separator:disabled, infobar.warning button .separator:disabled {
        color: rgba(239, 108, 0, 0.85); }
  infobar.question, infobar.question:backdrop {
    background-color: #673ab7;
    background-image: none;
    border: 1px solid #522e92;
    caret-color: currentColor; }
    infobar.question label, infobar.question, infobar.question:backdrop label, infobar.question:backdrop {
      color: #fff; }
  infobar.question button {
    background-color: #673ab7;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.22);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    infobar.question button:focus, infobar.question button:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
    infobar.question button:active, infobar.question button:active:hover, infobar.question button:active:focus, infobar.question button:active:hover:focus, infobar.question button:checked, infobar.question button:checked:hover, infobar.question button:checked:focus, infobar.question button:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.22); }
    infobar.question button:disabled {
      border-color: rgba(217, 217, 217, 0.22); }
    infobar.question button:active:disabled, infobar.question button:checked:disabled {
      border-color: rgba(204, 204, 204, 0.22); }
    infobar.question button.flat {
      color: #fff;
      border-color: rgba(103, 58, 183, 0);
      background-color: rgba(103, 58, 183, 0);
      background-image: none;
      box-shadow: none; }
    infobar.question button:hover, infobar.question button.flat:hover {
      background-color: #6c3dc0;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      infobar.question button:hover:focus, infobar.question button:hover:hover, infobar.question button.flat:hover:focus, infobar.question button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.question button:hover:active, infobar.question button:hover:active:hover, infobar.question button:hover:active:focus, infobar.question button:hover:active:hover:focus, infobar.question button:hover:checked, infobar.question button:hover:checked:hover, infobar.question button:hover:checked:focus, infobar.question button:hover:checked:hover:focus, infobar.question button.flat:hover:active, infobar.question button.flat:hover:active:hover, infobar.question button.flat:hover:active:focus, infobar.question button.flat:hover:active:hover:focus, infobar.question button.flat:hover:checked, infobar.question button.flat:hover:checked:hover, infobar.question button.flat:hover:checked:focus, infobar.question button.flat:hover:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.question button:hover:disabled, infobar.question button.flat:hover:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.question button:hover:active:disabled, infobar.question button:hover:checked:disabled, infobar.question button.flat:hover:active:disabled, infobar.question button.flat:hover:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
    infobar.question button:focus, infobar.question button.flat:focus {
      background-color: #6c3dc0;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      infobar.question button:focus:focus, infobar.question button:focus:hover, infobar.question button.flat:focus:focus, infobar.question button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.question button:focus:active, infobar.question button:focus:active:hover, infobar.question button:focus:active:focus, infobar.question button:focus:active:hover:focus, infobar.question button:focus:checked, infobar.question button:focus:checked:hover, infobar.question button:focus:checked:focus, infobar.question button:focus:checked:hover:focus, infobar.question button.flat:focus:active, infobar.question button.flat:focus:active:hover, infobar.question button.flat:focus:active:focus, infobar.question button.flat:focus:active:hover:focus, infobar.question button.flat:focus:checked, infobar.question button.flat:focus:checked:hover, infobar.question button.flat:focus:checked:focus, infobar.question button.flat:focus:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.question button:focus:disabled, infobar.question button.flat:focus:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.question button:focus:active:disabled, infobar.question button:focus:checked:disabled, infobar.question button.flat:focus:active:disabled, infobar.question button.flat:focus:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
      infobar.question button:focus:hover, infobar.question button.flat:focus:hover {
        background-color: #7345c4;
        background-image: none;
        border-color: rgba(204, 204, 204, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        infobar.question button:focus:hover:focus, infobar.question button:focus:hover:hover, infobar.question button.flat:focus:hover:focus, infobar.question button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
        infobar.question button:focus:hover:active, infobar.question button:focus:hover:active:hover, infobar.question button:focus:hover:active:focus, infobar.question button:focus:hover:active:hover:focus, infobar.question button:focus:hover:checked, infobar.question button:focus:hover:checked:hover, infobar.question button:focus:hover:checked:focus, infobar.question button:focus:hover:checked:hover:focus, infobar.question button.flat:focus:hover:active, infobar.question button.flat:focus:hover:active:hover, infobar.question button.flat:focus:hover:active:focus, infobar.question button.flat:focus:hover:active:hover:focus, infobar.question button.flat:focus:hover:checked, infobar.question button.flat:focus:hover:checked:hover, infobar.question button.flat:focus:hover:checked:focus, infobar.question button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(179, 179, 179, 0.3); }
        infobar.question button:focus:hover:disabled, infobar.question button.flat:focus:hover:disabled {
          border-color: rgba(217, 217, 217, 0.3); }
        infobar.question button:focus:hover:active:disabled, infobar.question button:focus:hover:checked:disabled, infobar.question button.flat:focus:hover:active:disabled, infobar.question button.flat:focus:hover:checked:disabled {
          border-color: rgba(204, 204, 204, 0.3); }
    infobar.question button:checked, infobar.question button:active, infobar.question button.flat:checked, infobar.question button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(255, 255, 255, 0.06), inset 0 1px rgba(255, 255, 255, 0.07), inset -1px 0 rgba(255, 255, 255, 0.06), inset 0 -1px rgba(255, 255, 255, 0.05);
      border-color: rgba(204, 204, 204, 0.22); }
      infobar.question button:checked:focus, infobar.question button:checked:hover, infobar.question button:active:focus, infobar.question button:active:hover, infobar.question button.flat:checked:focus, infobar.question button.flat:checked:hover, infobar.question button.flat:active:focus, infobar.question button.flat:active:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
      infobar.question button:checked:active, infobar.question button:checked:active:hover, infobar.question button:checked:active:focus, infobar.question button:checked:active:hover:focus, infobar.question button:checked:checked, infobar.question button:checked:checked:hover, infobar.question button:checked:checked:focus, infobar.question button:checked:checked:hover:focus, infobar.question button:active:active, infobar.question button:active:active:hover, infobar.question button:active:active:focus, infobar.question button:active:active:hover:focus, infobar.question button:active:checked, infobar.question button:active:checked:hover, infobar.question button:active:checked:focus, infobar.question button:active:checked:hover:focus, infobar.question button.flat:checked:active, infobar.question button.flat:checked:active:hover, infobar.question button.flat:checked:active:focus, infobar.question button.flat:checked:active:hover:focus, infobar.question button.flat:checked:checked, infobar.question button.flat:checked:checked:hover, infobar.question button.flat:checked:checked:focus, infobar.question button.flat:checked:checked:hover:focus, infobar.question button.flat:active:active, infobar.question button.flat:active:active:hover, infobar.question button.flat:active:active:focus, infobar.question button.flat:active:active:hover:focus, infobar.question button.flat:active:checked, infobar.question button.flat:active:checked:hover, infobar.question button.flat:active:checked:focus, infobar.question button.flat:active:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.22); }
      infobar.question button:checked:disabled, infobar.question button:active:disabled, infobar.question button.flat:checked:disabled, infobar.question button.flat:active:disabled {
        border-color: rgba(217, 217, 217, 0.22); }
      infobar.question button:checked:active:disabled, infobar.question button:checked:checked:disabled, infobar.question button:active:active:disabled, infobar.question button:active:checked:disabled, infobar.question button.flat:checked:active:disabled, infobar.question button.flat:checked:checked:disabled, infobar.question button.flat:active:active:disabled, infobar.question button.flat:active:checked:disabled {
        border-color: rgba(204, 204, 204, 0.22); }
      infobar.question button:checked:focus, infobar.question button:checked:hover, infobar.question button:active:focus, infobar.question button:active:hover, infobar.question button.flat:checked:focus, infobar.question button.flat:checked:hover, infobar.question button.flat:active:focus, infobar.question button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    infobar.question button:focus, infobar.question button:hover, infobar.question button.flat:focus, infobar.question button.flat:hover {
      color: #fff; }
    infobar.question button:disabled:disabled, infobar.question button.flat:disabled:disabled {
      background-color: alpha(mix(#673ab7,#fff,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#673ab7,#fff,0.5);
      box-shadow: none; }
    infobar.question button:active:disabled, infobar.question button:checked:disabled, infobar.question button.flat:active:disabled, infobar.question button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    infobar.question button.separator, infobar.question button .separator {
      border: 1px solid currentColor;
      color: rgba(103, 58, 183, 0.9); }
      infobar.question button.separator:disabled, infobar.question button .separator:disabled {
        color: rgba(103, 58, 183, 0.85); }
  infobar.error, infobar.error:backdrop {
    background-color: #f44336;
    background-image: none;
    border: 1px solid #e21b0c;
    caret-color: currentColor; }
    infobar.error label, infobar.error, infobar.error:backdrop label, infobar.error:backdrop {
      color: #fff; }
  infobar.error button {
    background-color: #f44336;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.22);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
    infobar.error button:focus, infobar.error button:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
    infobar.error button:active, infobar.error button:active:hover, infobar.error button:active:focus, infobar.error button:active:hover:focus, infobar.error button:checked, infobar.error button:checked:hover, infobar.error button:checked:focus, infobar.error button:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.22); }
    infobar.error button:disabled {
      border-color: rgba(217, 217, 217, 0.22); }
    infobar.error button:active:disabled, infobar.error button:checked:disabled {
      border-color: rgba(204, 204, 204, 0.22); }
    infobar.error button.flat {
      color: #fff;
      border-color: rgba(244, 67, 54, 0);
      background-color: rgba(244, 67, 54, 0);
      background-image: none;
      box-shadow: none; }
    infobar.error button:hover, infobar.error button.flat:hover {
      background-color: #f55044;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
      infobar.error button:hover:focus, infobar.error button:hover:hover, infobar.error button.flat:hover:focus, infobar.error button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.error button:hover:active, infobar.error button:hover:active:hover, infobar.error button:hover:active:focus, infobar.error button:hover:active:hover:focus, infobar.error button:hover:checked, infobar.error button:hover:checked:hover, infobar.error button:hover:checked:focus, infobar.error button:hover:checked:hover:focus, infobar.error button.flat:hover:active, infobar.error button.flat:hover:active:hover, infobar.error button.flat:hover:active:focus, infobar.error button.flat:hover:active:hover:focus, infobar.error button.flat:hover:checked, infobar.error button.flat:hover:checked:hover, infobar.error button.flat:hover:checked:focus, infobar.error button.flat:hover:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.error button:hover:disabled, infobar.error button.flat:hover:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.error button:hover:active:disabled, infobar.error button:hover:checked:disabled, infobar.error button.flat:hover:active:disabled, infobar.error button.flat:hover:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
    infobar.error button:focus, infobar.error button.flat:focus {
      background-color: #f55044;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      color: #fff;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      infobar.error button:focus:focus, infobar.error button:focus:hover, infobar.error button.flat:focus:focus, infobar.error button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      infobar.error button:focus:active, infobar.error button:focus:active:hover, infobar.error button:focus:active:focus, infobar.error button:focus:active:hover:focus, infobar.error button:focus:checked, infobar.error button:focus:checked:hover, infobar.error button:focus:checked:focus, infobar.error button:focus:checked:hover:focus, infobar.error button.flat:focus:active, infobar.error button.flat:focus:active:hover, infobar.error button.flat:focus:active:focus, infobar.error button.flat:focus:active:hover:focus, infobar.error button.flat:focus:checked, infobar.error button.flat:focus:checked:hover, infobar.error button.flat:focus:checked:focus, infobar.error button.flat:focus:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      infobar.error button:focus:disabled, infobar.error button.flat:focus:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      infobar.error button:focus:active:disabled, infobar.error button:focus:checked:disabled, infobar.error button.flat:focus:active:disabled, infobar.error button.flat:focus:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
      infobar.error button:focus:hover, infobar.error button.flat:focus:hover {
        background-color: #f65d52;
        background-image: none;
        border-color: rgba(204, 204, 204, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
        infobar.error button:focus:hover:focus, infobar.error button:focus:hover:hover, infobar.error button.flat:focus:hover:focus, infobar.error button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
        infobar.error button:focus:hover:active, infobar.error button:focus:hover:active:hover, infobar.error button:focus:hover:active:focus, infobar.error button:focus:hover:active:hover:focus, infobar.error button:focus:hover:checked, infobar.error button:focus:hover:checked:hover, infobar.error button:focus:hover:checked:focus, infobar.error button:focus:hover:checked:hover:focus, infobar.error button.flat:focus:hover:active, infobar.error button.flat:focus:hover:active:hover, infobar.error button.flat:focus:hover:active:focus, infobar.error button.flat:focus:hover:active:hover:focus, infobar.error button.flat:focus:hover:checked, infobar.error button.flat:focus:hover:checked:hover, infobar.error button.flat:focus:hover:checked:focus, infobar.error button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(179, 179, 179, 0.3); }
        infobar.error button:focus:hover:disabled, infobar.error button.flat:focus:hover:disabled {
          border-color: rgba(217, 217, 217, 0.3); }
        infobar.error button:focus:hover:active:disabled, infobar.error button:focus:hover:checked:disabled, infobar.error button.flat:focus:hover:active:disabled, infobar.error button.flat:focus:hover:checked:disabled {
          border-color: rgba(204, 204, 204, 0.3); }
    infobar.error button:checked, infobar.error button:active, infobar.error button.flat:checked, infobar.error button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(255, 255, 255, 0.06), inset 0 1px rgba(255, 255, 255, 0.07), inset -1px 0 rgba(255, 255, 255, 0.06), inset 0 -1px rgba(255, 255, 255, 0.05);
      border-color: rgba(204, 204, 204, 0.22); }
      infobar.error button:checked:focus, infobar.error button:checked:hover, infobar.error button:active:focus, infobar.error button:active:hover, infobar.error button.flat:checked:focus, infobar.error button.flat:checked:hover, infobar.error button.flat:active:focus, infobar.error button.flat:active:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
      infobar.error button:checked:active, infobar.error button:checked:active:hover, infobar.error button:checked:active:focus, infobar.error button:checked:active:hover:focus, infobar.error button:checked:checked, infobar.error button:checked:checked:hover, infobar.error button:checked:checked:focus, infobar.error button:checked:checked:hover:focus, infobar.error button:active:active, infobar.error button:active:active:hover, infobar.error button:active:active:focus, infobar.error button:active:active:hover:focus, infobar.error button:active:checked, infobar.error button:active:checked:hover, infobar.error button:active:checked:focus, infobar.error button:active:checked:hover:focus, infobar.error button.flat:checked:active, infobar.error button.flat:checked:active:hover, infobar.error button.flat:checked:active:focus, infobar.error button.flat:checked:active:hover:focus, infobar.error button.flat:checked:checked, infobar.error button.flat:checked:checked:hover, infobar.error button.flat:checked:checked:focus, infobar.error button.flat:checked:checked:hover:focus, infobar.error button.flat:active:active, infobar.error button.flat:active:active:hover, infobar.error button.flat:active:active:focus, infobar.error button.flat:active:active:hover:focus, infobar.error button.flat:active:checked, infobar.error button.flat:active:checked:hover, infobar.error button.flat:active:checked:focus, infobar.error button.flat:active:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.22); }
      infobar.error button:checked:disabled, infobar.error button:active:disabled, infobar.error button.flat:checked:disabled, infobar.error button.flat:active:disabled {
        border-color: rgba(217, 217, 217, 0.22); }
      infobar.error button:checked:active:disabled, infobar.error button:checked:checked:disabled, infobar.error button:active:active:disabled, infobar.error button:active:checked:disabled, infobar.error button.flat:checked:active:disabled, infobar.error button.flat:checked:checked:disabled, infobar.error button.flat:active:active:disabled, infobar.error button.flat:active:checked:disabled {
        border-color: rgba(204, 204, 204, 0.22); }
      infobar.error button:checked:focus, infobar.error button:checked:hover, infobar.error button:active:focus, infobar.error button:active:hover, infobar.error button.flat:checked:focus, infobar.error button.flat:checked:hover, infobar.error button.flat:active:focus, infobar.error button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    infobar.error button:focus, infobar.error button:hover, infobar.error button.flat:focus, infobar.error button.flat:hover {
      color: #fff; }
    infobar.error button:disabled:disabled, infobar.error button.flat:disabled:disabled {
      background-color: alpha(mix(#f44336,#fff,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#f44336,#fff,0.5);
      box-shadow: none; }
    infobar.error button:active:disabled, infobar.error button:checked:disabled, infobar.error button.flat:active:disabled, infobar.error button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    infobar.error button.separator, infobar.error button .separator {
      border: 1px solid currentColor;
      color: rgba(244, 67, 54, 0.9); }
      infobar.error button.separator:disabled, infobar.error button .separator:disabled {
        color: rgba(244, 67, 54, 0.85); }

/*********
 ! Entry *
**********/
.linked:not(.vertical) > entry {
  border-width: 1px;
  border-radius: 0;
  border-right-width: 0;
  border-left-width: 0; }
  .linked:not(.vertical) > entry:first-child {
    border-width: 1px;
    border-radius: 3px;
    border-right-width: 0;
    border-bottom-right-radius: 0;
    border-top-right-radius: 0; }
    .linked:not(.vertical) > entry:first-child:dir(rtl) {
      border-left-width: 0;
      border-right-width: 1px; }
  .linked:not(.vertical) > entry:last-child {
    border-width: 1px;
    border-radius: 3px;
    border-left-width: 0;
    border-bottom-left-radius: 0;
    border-top-left-radius: 0; }
    .linked:not(.vertical) > entry:last-child:dir(rtl) {
      border-left-width: 1px;
      border-right-width: 0; }
  .linked:not(.vertical) > entry:only-child, .linked:not(.vertical) > entry:first-child:only-child {
    border-width: 1px; }
  .linked:not(.vertical) > entry:only-child {
    border-radius: 3px; }

.linked.vertical > entry {
  border-width: 1px;
  border-radius: 0;
  border-top-width: 0;
  border-bottom-width: 0; }
  .linked.vertical > entry:first-child {
    border-width: 1px;
    border-radius: 3px;
    border-top-width: 1px;
    border-bottom-width: 0;
    border-bottom-left-radius: 0;
    border-bottom-right-radius: 0; }
    .linked.vertical > entry:first-child:dir(rtl) {
      border-top-width: 0;
      border-bottom-width: 1px; }
  .linked.vertical > entry:last-child {
    border-width: 1px;
    border-radius: 3px;
    border-top-width: 0;
    border-bottom-width: 1px;
    border-top-left-radius: 0;
    border-top-right-radius: 0; }
    .linked.vertical > entry:last-child:dir(rtl) {
      border-top-width: 1px;
      border-bottom-width: 0; }
  .linked.vertical > entry:only-child, .linked.vertical > entry:first-child:only-child {
    border-width: 1px; }
  .linked.vertical > entry:only-child {
    border-radius: 3px; }

entry, menuitem entry, popover.background entry, .osd entry,
#XfceNotifyWindow entry, #login_window entry {
  border-width: 1px;
  border-style: solid;
  border-radius: 3px;
  border-color: #848484;
  transition: border 100ms ease-out;
  box-shadow: inset 1px 0 rgba(0, 0, 0, 0.1), inset 0 1px rgba(0, 0, 0, 0.12), inset -1px 0 rgba(0, 0, 0, 0.1), inset 0 -1px rgba(0, 0, 0, 0.05); }
  entry:focus, popover.background entry:focus,
  #XfceNotifyWindow entry:focus, #login_window entry:focus, entry:hover, popover.background entry:hover,
  #XfceNotifyWindow entry:hover, #login_window entry:hover, entry:active, popover.background entry:active,
  #XfceNotifyWindow entry:active, #login_window entry:active {
    transition: border 100ms ease-in; }
  entry:selected, popover.background entry:selected,
  #XfceNotifyWindow entry:selected, #login_window entry:selected, entry:selected:selected:focus,
  #XfceNotifyWindow entry:selected:selected:focus, #login_window entry:selected:selected:focus {
    background-color: #B36A87;
    color: #000000; }
  entry:disabled, popover.background entry:disabled,
  #XfceNotifyWindow entry:disabled, #login_window entry:disabled {
    box-shadow: none; }
  entry progress, popover.background entry progress, .osd entry progress,
  #XfceNotifyWindow entry progress, #login_window entry progress {
    background-color: #B36A87;
    background-image: none;
    border-width: 0;
    border-radius: 3px;
    color: #000000; }
  entry image.left,
  #XfceNotifyWindow entry image.left, #login_window entry image.left {
    padding-right: 3px; }
  entry image.right,
  #XfceNotifyWindow entry image.right, #login_window entry image.right {
    padding-left: 3px; }
  entry.warning, popover.background entry.warning,
  #XfceNotifyWindow entry.warning, #login_window entry.warning {
    color: #fff;
    border-color: #bf5600;
    background-color: mix(#d0d0d0,#ef6c00,0.6); }
    entry.warning image,
    #XfceNotifyWindow entry.warning image, #login_window entry.warning image {
      color: #fff; }
    entry.warning:focus,
    #XfceNotifyWindow entry.warning:focus, #login_window entry.warning:focus {
      color: #fff;
      border-color: mix(#B36A87,#ef6c00,0.3);
      background-color: #ef6c00;
      box-shadow: none; }
    entry.warning selection,
    #XfceNotifyWindow entry.warning selection, #login_window entry.warning selection {
      background-color: #fff;
      color: #ef6c00; }
  entry.error, popover.background entry.error,
  #XfceNotifyWindow entry.error, #login_window entry.error {
    color: #fff;
    border-color: #e21b0c;
    background-color: mix(#d0d0d0,#f44336,0.6); }
    entry.error image,
    #XfceNotifyWindow entry.error image, #login_window entry.error image {
      color: #fff; }
    entry.error:focus,
    #XfceNotifyWindow entry.error:focus, #login_window entry.error:focus {
      color: #fff;
      border-color: mix(#B36A87,#f44336,0.3);
      background-color: #f44336;
      box-shadow: none; }
    entry.error selection,
    #XfceNotifyWindow entry.error selection, #login_window entry.error selection {
      background-color: #fff;
      color: #f44336; }
  entry.search-missing, popover.background entry.search-missing,
  #XfceNotifyWindow entry.search-missing, #login_window entry.search-missing {
    color: #fff;
    border-color: #e21b0c;
    background-color: mix(#d0d0d0,#f44336,0.6); }
    entry.search-missing image,
    #XfceNotifyWindow entry.search-missing image, #login_window entry.search-missing image {
      color: #fff; }
    entry.search-missing:focus,
    #XfceNotifyWindow entry.search-missing:focus, #login_window entry.search-missing:focus {
      color: #fff;
      border-color: mix(#B36A87,#f44336,0.3);
      background-color: #f44336;
      box-shadow: none; }
    entry.search-missing selection,
    #XfceNotifyWindow entry.search-missing selection, #login_window entry.search-missing selection {
      background-color: #fff;
      color: #f44336; }

/*********
 ! Menubar
**********/
menubar, .menubar {
  -GtkWidget-window-dragging: true;
  padding: 0;
  border: 0;
  background-color: #0f0f0f;
  background-image: none;
  color: #a5a5a5; }
  menubar > menuitem, .menubar > menuitem {
    min-height: 16px;
    padding: 4.5px 7.5px;
    border: 1px solid transparent;
    background-color: transparent;
    background-image: none;
    color: #a5a5a5; }
    menubar > menuitem:hover, .menubar > menuitem:hover {
      border-color: mix(#0f0f0f,#a5a5a5,0.21);
      background-color: mix(#0f0f0f,#a5a5a5,0.21);
      background-image: none;
      color: #b2b2b2; }
    menubar > menuitem *:hover, .menubar > menuitem *:hover {
      color: #b2b2b2; }

/******
 ! Menu
*******/
menu,
.menu,
.context-menu {
  border: 0;
  border-radius: 0;
  padding: 3px;
  background-color: #0f0f0f;
  color: #a5a5a5; }
  .csd menu, .csd
  .menu, .csd
  .context-menu {
    border: 0; }
  menu:selected,
  .menu:selected,
  .context-menu:selected {
    background-color: #B36A87; }
  menu separator,
  .csd menu separator,
  .menu separator,
  .csd
  .menu separator,
  .context-menu separator,
  .csd
  .context-menu separator {
    background-color: #0e0e0e;
    margin: 1px 0; }
  menu .separator,
  .csd menu .separator,
  .menu .separator,
  .csd
  .menu .separator,
  .context-menu .separator,
  .csd
  .context-menu .separator {
    color: #0e0e0e; }
  menu menuitem,
  .menu menuitem,
  .context-menu menuitem {
    min-height: 16px;
    min-width: 40px;
    padding: 3px;
    border-radius: 0; }
    menu menuitem:active, menu menuitem:hover,
    .menu menuitem:active,
    .menu menuitem:hover,
    .context-menu menuitem:active,
    .context-menu menuitem:hover {
      border: 0;
      background-color: #B36A87;
      background-image: none;
      color: #000000; }
    menu menuitem *:active, menu menuitem *:hover,
    .menu menuitem *:active,
    .menu menuitem *:hover,
    .context-menu menuitem *:active,
    .context-menu menuitem *:hover {
      color: #000000; }
    menu menuitem:disabled, menu menuitem *:disabled,
    .menu menuitem:disabled,
    .menu menuitem *:disabled,
    .context-menu menuitem:disabled,
    .context-menu menuitem *:disabled {
      color: mix(#a5a5a5,#0f0f0f,0.5); }
    menu menuitem arrow,
    .menu menuitem arrow,
    .context-menu menuitem arrow {
      min-height: 16px;
      min-width: 16px; }
      menu menuitem arrow:dir(ltr),
      .menu menuitem arrow:dir(ltr),
      .context-menu menuitem arrow:dir(ltr) {
        -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
        margin-left: 10px; }
      menu menuitem arrow:dir(rtl),
      .menu menuitem arrow:dir(rtl),
      .context-menu menuitem arrow:dir(rtl) {
        -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl");
        margin-right: 10px; }
    menu menuitem.button, menu menuitem.button:focus, menu menuitem.button:active, menu menuitem.button:disabled, menu menuitem.button:active:disabled, menu menuitem.button.flat, menu menuitem.button.flat:focus, menu menuitem.button.flat:active, menu menuitem.button.flat:disabled, menu menuitem.button.flat:active:disabled,
    .menu menuitem.button,
    .menu menuitem.button:focus,
    .menu menuitem.button:active,
    .menu menuitem.button:disabled,
    .menu menuitem.button:active:disabled,
    .menu menuitem.button.flat,
    .menu menuitem.button.flat:focus,
    .menu menuitem.button.flat:active,
    .menu menuitem.button.flat:disabled,
    .menu menuitem.button.flat:active:disabled,
    .context-menu menuitem.button,
    .context-menu menuitem.button:focus,
    .context-menu menuitem.button:active,
    .context-menu menuitem.button:disabled,
    .context-menu menuitem.button:active:disabled,
    .context-menu menuitem.button.flat,
    .context-menu menuitem.button.flat:focus,
    .context-menu menuitem.button.flat:active,
    .context-menu menuitem.button.flat:disabled,
    .context-menu menuitem.button.flat:active:disabled {
      background-color: transparent;
      background-image: none;
      border: 0;
      box-shadow: none;
      color: currentColor; }
    menu menuitem.button:hover, menu menuitem.button:focus:hover, menu menuitem.button:active:hover, menu menuitem.button:selected, menu menuitem.button.flat:hover, menu menuitem.button.flat:focus:hover, menu menuitem.button.flat:active:hover, menu menuitem.button.flat:selected,
    .menu menuitem.button:hover,
    .menu menuitem.button:focus:hover,
    .menu menuitem.button:active:hover,
    .menu menuitem.button:selected,
    .menu menuitem.button.flat:hover,
    .menu menuitem.button.flat:focus:hover,
    .menu menuitem.button.flat:active:hover,
    .menu menuitem.button.flat:selected,
    .context-menu menuitem.button:hover,
    .context-menu menuitem.button:focus:hover,
    .context-menu menuitem.button:active:hover,
    .context-menu menuitem.button:selected,
    .context-menu menuitem.button.flat:hover,
    .context-menu menuitem.button.flat:focus:hover,
    .context-menu menuitem.button.flat:active:hover,
    .context-menu menuitem.button.flat:selected {
      background-image: none;
      background-color: #B36A87;
      color: #000000; }
    menu menuitem calendar,
    .menu menuitem calendar,
    .context-menu menuitem calendar {
      color: #a5a5a5; }
      menu menuitem calendar.header,
      .menu menuitem calendar.header,
      .context-menu menuitem calendar.header {
        border-bottom: 1px solid #0e0e0e;
        border-radius: 0; }
        menu menuitem calendar.header:backdrop,
        .menu menuitem calendar.header:backdrop,
        .context-menu menuitem calendar.header:backdrop {
          border-color: #0e0e0e; }
      menu menuitem calendar.button,
      .menu menuitem calendar.button,
      .context-menu menuitem calendar.button {
        color: rgba(165, 165, 165, 0.55); }
        menu menuitem calendar.button:hover,
        .menu menuitem calendar.button:hover,
        .context-menu menuitem calendar.button:hover {
          color: #a5a5a5; }
      menu menuitem calendar:indeterminate, menu menuitem calendar:indeterminate:backdrop,
      .menu menuitem calendar:indeterminate,
      .menu menuitem calendar:indeterminate:backdrop,
      .context-menu menuitem calendar:indeterminate,
      .context-menu menuitem calendar:indeterminate:backdrop {
        color: mix(#a5a5a5,#0f0f0f,0.5); }
    menu menuitem label:dir(rtl), menu menuitem label:dir(ltr),
    .menu menuitem label:dir(rtl),
    .menu menuitem label:dir(ltr),
    .context-menu menuitem label:dir(rtl),
    .context-menu menuitem label:dir(ltr) {
      color: inherit; }
  menu > arrow,
  .menu > arrow,
  .context-menu > arrow {
    min-height: 16px;
    min-width: 16px;
    padding: 3px;
    background-color: #0f0f0f;
    border-radius: 0; }
    menu > arrow.top,
    .menu > arrow.top,
    .context-menu > arrow.top {
      margin-top: -6px;
      border-bottom: 1px solid mix(#000000,#d0d0d0,0.1);
      -gtk-icon-source: -gtk-icontheme("pan-up-symbolic"); }
    menu > arrow.bottom,
    .menu > arrow.bottom,
    .context-menu > arrow.bottom {
      margin-bottom: -6px;
      border-top: 1px solid mix(#000000,#d0d0d0,0.1);
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }
    menu > arrow:hover,
    .menu > arrow:hover,
    .context-menu > arrow:hover {
      background-color: mix(#000000,#d0d0d0,0.1); }
    menu > arrow:backdrop,
    .menu > arrow:backdrop,
    .context-menu > arrow:backdrop {
      background-color: #d0d0d0; }
    menu > arrow:disabled,
    .menu > arrow:disabled,
    .context-menu > arrow:disabled {
      color: transparent;
      background-color: transparent;
      border-color: transparent; }

.context-menu {
  font: initial; }

.monospace {
  font-family: monospace; }

menuitem accelerator {
  color: rgba(165, 165, 165, 0.6); }
  menuitem accelerator:hover {
    color: rgba(0, 0, 0, 0.8); }
  menuitem accelerator:disabled {
    color: alpha(mix(#a5a5a5,#0f0f0f,0.5),0.4); }

menuitem check, menuitem radio {
  min-height: 16px;
  min-width: 16px; }
  menuitem check:dir(ltr), menuitem radio:dir(ltr) {
    margin-right: 7px; }
  menuitem check:dir(rtl), menuitem radio:dir(rtl) {
    margin-left: 7px; }

menuitem window decoration {
  box-shadow: 0 2px 3px rgba(0, 0, 0, 0.2); }

menuitem entry {
  background-color: #0f0f0f;
  background-image: none;
  border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.8);
  padding: 3px;
  color: #a5a5a5;
  caret-color: #000000;
  -gtk-secondary-caret-color: #000000; }
  menuitem entry:focus, menuitem entry:hover {
    border-color: mix(#B36A87,mix(#0f0f0f,#a5a5a5,0.2),0.3); }
  menuitem entry:active, menuitem entry:active:hover, menuitem entry:active:focus, menuitem entry:active:hover:focus, menuitem entry:checked, menuitem entry:checked:hover, menuitem entry:checked:focus, menuitem entry:checked:hover:focus {
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.7); }
  menuitem entry:disabled {
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.85); }
  menuitem entry:active:disabled, menuitem entry:checked:disabled {
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.8); }
  menuitem entry:focus, menuitem entry:active {
    border-color: mix(#B36A87,mix(#a5a5a5,#000000,0.08),0.3); }
  menuitem entry:disabled {
    background-color: #0e0e0e;
    background-image: none;
    border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.8);
    /*@include border(mix($bg, $fg, .20));*/
    /*border-color: alpha(mix($bg, $fg, .15), .8);*/
    /*opacity: .8;*/
    color: mix(#0f0f0f,#a5a5a5,0.5); }
    menuitem entry:disabled:focus, menuitem entry:disabled:hover {
      border-color: mix(#B36A87,alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.3); }
    menuitem entry:disabled:active, menuitem entry:disabled:active:hover, menuitem entry:disabled:active:focus, menuitem entry:disabled:active:hover:focus, menuitem entry:disabled:checked, menuitem entry:disabled:checked:hover, menuitem entry:disabled:checked:focus, menuitem entry:disabled:checked:hover:focus {
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.7); }
    menuitem entry:disabled:disabled {
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.85); }
    menuitem entry:disabled:active:disabled, menuitem entry:disabled:checked:disabled {
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.8); }

/*********
 ! Popover
**********/
popover.background {
  padding: 0px;
  border-radius: 3px;
  background-clip: border-box;
  background-color: #0f0f0f;
  background-image: none;
  color: #a5a5a5;
  box-shadow: 0 3px 6px rgba(0, 0, 0, 0.16); }
  .csd popover.background, popover.background {
    /*@include border($menu_bg_color);*/
    border-color: rgba(132, 132, 132, 0.5);
    border-width: 1px;
    border-style: solid; }
    .csd popover.background:focus, .csd popover.background:hover, popover.background:focus, popover.background:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.5),0.3); }
    .csd popover.background:active, .csd popover.background:active:hover, .csd popover.background:active:focus, .csd popover.background:active:hover:focus, .csd popover.background:checked, .csd popover.background:checked:hover, .csd popover.background:checked:focus, .csd popover.background:checked:hover:focus, popover.background:active, popover.background:active:hover, popover.background:active:focus, popover.background:active:hover:focus, popover.background:checked, popover.background:checked:hover, popover.background:checked:focus, popover.background:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.5); }
    .csd popover.background:disabled, popover.background:disabled {
      border-color: rgba(140, 140, 140, 0.5); }
    .csd popover.background:active:disabled, .csd popover.background:checked:disabled, popover.background:active:disabled, popover.background:checked:disabled {
      border-color: rgba(132, 132, 132, 0.5); }
  popover.background:backdrop {
    box-shadow: none; }
  popover.background treeview.view:hover, popover.background treeview.view:selected, popover.background treeview.view:selected:focus, popover.background treeview.view:backdrop:selected, popover.background treeview.view:backdrop:selected:focus {
    border-top-color: #B36A87; }
  popover.background treeview.view, popover.background treeview.view:backdrop {
    border-top-color: #121212; }
  popover.background view:hover, popover.background .view:hover, popover.background iconview:hover, popover.background list:hover {
    background-image: none;
    background-color: #B36A87;
    color: #000000; }
  popover.background view, popover.background view:backdrop, popover.background .view, popover.background iconview, popover.background .view:backdrop, popover.background iconview:backdrop, popover.background list, popover.background list:backdrop {
    background-color: #141414;
    background-image: none;
    color: #a5a5a5;
    border-color: #0c0c0c; }
  popover.background list row, popover.background list row .button {
    background-color: transparent;
    background-image: none;
    color: #a5a5a5; }
    popover.background list row:focus, popover.background list row:hover, popover.background list row:active, popover.background list row .button:focus, popover.background list row .button:hover, popover.background list row .button:active {
      background-image: none;
      background-color: #B36A87;
      color: #000000; }
  popover.background .frame {
    border-color: #0c0c0c;
    border-radius: 3px; }
  popover.background entry {
    background-color: #d0d0d0;
    background-image: none;
    border-color: shade(mix(#d0d0d0,#000000,0.2),0.8);
    padding: 3px;
    color: #000000;
    caret-color: #000000;
    -gtk-secondary-caret-color: #000000; }
    popover.background entry:focus, popover.background entry:hover {
      border-color: mix(#B36A87,mix(#d0d0d0,#000000,0.2),0.3); }
    popover.background entry:active, popover.background entry:active:hover, popover.background entry:active:focus, popover.background entry:active:hover:focus, popover.background entry:checked, popover.background entry:checked:hover, popover.background entry:checked:focus, popover.background entry:checked:hover:focus {
      border-color: shade(mix(#d0d0d0,#000000,0.2),0.7); }
    popover.background entry:disabled {
      border-color: shade(mix(#d0d0d0,#000000,0.2),0.85); }
    popover.background entry:active:disabled, popover.background entry:checked:disabled {
      border-color: shade(mix(#d0d0d0,#000000,0.2),0.8); }
    popover.background entry:focus, popover.background entry:active {
      border-color: mix(#B36A87,mix(#a5a5a5,#000000,0.08),0.3); }
    popover.background entry:disabled {
      background-color: #bbbbbb;
      background-image: none;
      border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.8);
      /*@include border(mix($bg, $fg, .20));*/
      /*border-color: alpha(mix($bg, $fg, .15), .8);*/
      /*opacity: .8;*/
      color: mix(#d0d0d0,#000000,0.5); }
      popover.background entry:disabled:focus, popover.background entry:disabled:hover {
        border-color: mix(#B36A87,alpha(mix(#d0d0d0,#000000,0.2),0.4),0.3); }
      popover.background entry:disabled:active, popover.background entry:disabled:active:hover, popover.background entry:disabled:active:focus, popover.background entry:disabled:active:hover:focus, popover.background entry:disabled:checked, popover.background entry:disabled:checked:hover, popover.background entry:disabled:checked:focus, popover.background entry:disabled:checked:hover:focus {
        border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.7); }
      popover.background entry:disabled:disabled {
        border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.85); }
      popover.background entry:disabled:active:disabled, popover.background entry:disabled:checked:disabled {
        border-color: shade(alpha(mix(#d0d0d0,#000000,0.2),0.4),0.8); }
  popover.background button {
    background-color: #8b639b;
    background-image: none;
    border-color: rgba(0, 0, 0, 0.32);
    color: #000000;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    popover.background button:focus, popover.background button:hover {
      border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
    popover.background button:active, popover.background button:active:hover, popover.background button:active:focus, popover.background button:active:hover:focus, popover.background button:checked, popover.background button:checked:hover, popover.background button:checked:focus, popover.background button:checked:hover:focus {
      border-color: rgba(0, 0, 0, 0.32); }
    popover.background button:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    popover.background button:active:disabled, popover.background button:checked:disabled {
      border-color: rgba(0, 0, 0, 0.32); }
    popover.background button.flat {
      color: #000000;
      border-color: rgba(139, 99, 155, 0);
      background-color: rgba(139, 99, 155, 0);
      background-image: none;
      box-shadow: none; }
    popover.background button:hover, popover.background button.flat:hover {
      background-color: #916ba0;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      popover.background button:hover:focus, popover.background button:hover:hover, popover.background button.flat:hover:focus, popover.background button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      popover.background button:hover:active, popover.background button:hover:active:hover, popover.background button:hover:active:focus, popover.background button:hover:active:hover:focus, popover.background button:hover:checked, popover.background button:hover:checked:hover, popover.background button:hover:checked:focus, popover.background button:hover:checked:hover:focus, popover.background button.flat:hover:active, popover.background button.flat:hover:active:hover, popover.background button.flat:hover:active:focus, popover.background button.flat:hover:active:hover:focus, popover.background button.flat:hover:checked, popover.background button.flat:hover:checked:hover, popover.background button.flat:hover:checked:focus, popover.background button.flat:hover:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      popover.background button:hover:disabled, popover.background button.flat:hover:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      popover.background button:hover:active:disabled, popover.background button:hover:checked:disabled, popover.background button.flat:hover:active:disabled, popover.background button.flat:hover:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
    popover.background button:focus, popover.background button.flat:focus {
      background-color: #916ba0;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.4);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      popover.background button:focus:focus, popover.background button:focus:hover, popover.background button.flat:focus:focus, popover.background button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
      popover.background button:focus:active, popover.background button:focus:active:hover, popover.background button:focus:active:focus, popover.background button:focus:active:hover:focus, popover.background button:focus:checked, popover.background button:focus:checked:hover, popover.background button:focus:checked:focus, popover.background button:focus:checked:hover:focus, popover.background button.flat:focus:active, popover.background button.flat:focus:active:hover, popover.background button.flat:focus:active:focus, popover.background button.flat:focus:active:hover:focus, popover.background button.flat:focus:checked, popover.background button.flat:focus:checked:hover, popover.background button.flat:focus:checked:focus, popover.background button.flat:focus:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.4); }
      popover.background button:focus:disabled, popover.background button.flat:focus:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      popover.background button:focus:active:disabled, popover.background button:focus:checked:disabled, popover.background button.flat:focus:active:disabled, popover.background button.flat:focus:checked:disabled {
        border-color: rgba(0, 0, 0, 0.4); }
      popover.background button:focus:hover, popover.background button.flat:focus:hover {
        background-color: #9772a5;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        popover.background button:focus:hover:focus, popover.background button:focus:hover:hover, popover.background button.flat:focus:hover:focus, popover.background button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        popover.background button:focus:hover:active, popover.background button:focus:hover:active:hover, popover.background button:focus:hover:active:focus, popover.background button:focus:hover:active:hover:focus, popover.background button:focus:hover:checked, popover.background button:focus:hover:checked:hover, popover.background button:focus:hover:checked:focus, popover.background button:focus:hover:checked:hover:focus, popover.background button.flat:focus:hover:active, popover.background button.flat:focus:hover:active:hover, popover.background button.flat:focus:hover:active:focus, popover.background button.flat:focus:hover:active:hover:focus, popover.background button.flat:focus:hover:checked, popover.background button.flat:focus:hover:checked:hover, popover.background button.flat:focus:hover:checked:focus, popover.background button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        popover.background button:focus:hover:disabled, popover.background button.flat:focus:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        popover.background button:focus:hover:active:disabled, popover.background button:focus:hover:checked:disabled, popover.background button.flat:focus:hover:active:disabled, popover.background button.flat:focus:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
    popover.background button:checked, popover.background button:active, popover.background button.flat:checked, popover.background button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
      border-color: rgba(0, 0, 0, 0.32); }
      popover.background button:checked:focus, popover.background button:checked:hover, popover.background button:active:focus, popover.background button:active:hover, popover.background button.flat:checked:focus, popover.background button.flat:checked:hover, popover.background button.flat:active:focus, popover.background button.flat:active:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      popover.background button:checked:active, popover.background button:checked:active:hover, popover.background button:checked:active:focus, popover.background button:checked:active:hover:focus, popover.background button:checked:checked, popover.background button:checked:checked:hover, popover.background button:checked:checked:focus, popover.background button:checked:checked:hover:focus, popover.background button:active:active, popover.background button:active:active:hover, popover.background button:active:active:focus, popover.background button:active:active:hover:focus, popover.background button:active:checked, popover.background button:active:checked:hover, popover.background button:active:checked:focus, popover.background button:active:checked:hover:focus, popover.background button.flat:checked:active, popover.background button.flat:checked:active:hover, popover.background button.flat:checked:active:focus, popover.background button.flat:checked:active:hover:focus, popover.background button.flat:checked:checked, popover.background button.flat:checked:checked:hover, popover.background button.flat:checked:checked:focus, popover.background button.flat:checked:checked:hover:focus, popover.background button.flat:active:active, popover.background button.flat:active:active:hover, popover.background button.flat:active:active:focus, popover.background button.flat:active:active:hover:focus, popover.background button.flat:active:checked, popover.background button.flat:active:checked:hover, popover.background button.flat:active:checked:focus, popover.background button.flat:active:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      popover.background button:checked:disabled, popover.background button:active:disabled, popover.background button.flat:checked:disabled, popover.background button.flat:active:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      popover.background button:checked:active:disabled, popover.background button:checked:checked:disabled, popover.background button:active:active:disabled, popover.background button:active:checked:disabled, popover.background button.flat:checked:active:disabled, popover.background button.flat:checked:checked:disabled, popover.background button.flat:active:active:disabled, popover.background button.flat:active:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      popover.background button:checked:focus, popover.background button:checked:hover, popover.background button:active:focus, popover.background button:active:hover, popover.background button.flat:checked:focus, popover.background button.flat:checked:hover, popover.background button.flat:active:focus, popover.background button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    popover.background button:focus, popover.background button:hover, popover.background button.flat:focus, popover.background button.flat:hover {
      color: #000000; }
    popover.background button:disabled:disabled, popover.background button.flat:disabled:disabled {
      background-color: alpha(mix(#8b639b,#000000,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(#8b639b,#000000,0.5);
      box-shadow: none; }
    popover.background button:active:disabled, popover.background button:checked:disabled, popover.background button.flat:active:disabled, popover.background button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    popover.background button.separator, popover.background button .separator {
      border: 1px solid currentColor;
      color: rgba(139, 99, 155, 0.9); }
      popover.background button.separator:disabled, popover.background button .separator:disabled {
        color: rgba(139, 99, 155, 0.85); }
  popover.background .linked > button {
    box-shadow: inset -1px 0 rgba(0, 0, 0, 0.22), 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    popover.background .linked > button:focus, popover.background .linked > button:hover {
      box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.22),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    popover.background .linked > button:active, popover.background .linked > button:active:hover, popover.background .linked > button:active:focus, popover.background .linked > button:active:hover:focus, popover.background .linked > button:checked, popover.background .linked > button:checked:hover, popover.background .linked > button:checked:focus, popover.background .linked > button:checked:hover:focus {
      box-shadow: inset -1px 0 rgba(0, 0, 0, 0.22), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
    popover.background .linked > button:disabled {
      box-shadow: inset -1px 0 #6f4f7c; }
    popover.background .linked > button:last-child, popover.background .linked > button:only-child {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    popover.background .linked > button:last-child:hover, popover.background .linked > button:only-child:hover {
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    popover.background .linked > button:disabled:last-child, popover.background .linked > button:disabled:only-child, popover.background .linked > button:active:disabled:last-child, popover.background .linked > button:active:disabled:only-child, popover.background .linked > button:checked:disabled:last-child, popover.background .linked > button:checked:disabled:only-child {
      box-shadow: none; }
    popover.background .linked > button:active:last-child, popover.background .linked > button:active:last-child:focus, popover.background .linked > button:active:last-child:hover, popover.background .linked > button:active:last-child:hover:focus, popover.background .linked > button:checked:last-child, popover.background .linked > button:checked:last-child:focus, popover.background .linked > button:checked:last-child:hover, popover.background .linked > button:checked:last-child:hover:focus {
      box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    popover.background .linked > button:active:only-child, popover.background .linked > button:active:only-child:focus, popover.background .linked > button:active:only-child:hover, popover.background .linked > button:active:only-child:hover:focus, popover.background .linked > button:checked:only-child, popover.background .linked > button:checked:only-child:focus, popover.background .linked > button:checked:only-child:hover, popover.background .linked > button:checked:only-child:hover:focus {
      box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  popover.background > list, popover.background > .view, popover.background > iconview, popover.background > toolbar {
    border-style: none;
    background-color: transparent; }

modelbutton.flat,
menuitem.button.flat {
  padding: 3px 5px;
  outline-color: transparent;
  transition: none; }
  modelbutton.flat:hover,
  menuitem.button.flat:hover {
    background-color: #B36A87;
    color: #000000; }
  modelbutton.flat:checked,
  menuitem.button.flat:checked {
    color: #000000; }
  modelbutton.flat arrow.left,
  menuitem.button.flat arrow.left {
    -gtk-icon-source: -gtk-icontheme("pan-start-symbolic"); }
  modelbutton.flat arrow.right,
  menuitem.button.flat arrow.right {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic"); }
  modelbutton.flat check:last-child,
  modelbutton.flat radio:last-child,
  menuitem.button.flat check:last-child,
  menuitem.button.flat radio:last-child {
    margin-left: 8px; }
  modelbutton.flat check:first-child,
  modelbutton.flat radio:first-child,
  menuitem.button.flat check:first-child,
  menuitem.button.flat radio:first-child {
    margin-right: 8px; }

/***************
! Dimmed label *
****************/
.dim-label, label.separator {
  opacity: .5;
  text-shadow: none; }

/***********
 ! Tooltip *
************/
.tooltip.background, .tooltip.background.csd,
tooltip.background,
tooltip.background.csd {
  background-color: #a5a5a5;
  background-clip: padding-box;
  border: 1px solid #848484;
  border-radius: 3px;
  color: #000000; }

.tooltip *,
tooltip * {
  background-color: transparent;
  color: inherit; }

/***********
 ! Dialogs *
************/
messagedialog, .message-dialog, .prompt {
  -GtkDialog-content-area-border: 0;
  -GtkDialog-action-area-border: 0;
  -GtkDialog-button-spacing: 3px;
  margin: 0;
  padding: 0; }

printdialog paper {
  color: #000000;
  border: 1px solid mix(#a5a5a5,#000000,0.08);
  background: #fff;
  padding: 0; }
  printdialog paper:backdrop {
    color: mix(#000000,#a5a5a5,0.5);
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9); }

printdialog .dialog-action-box {
  margin: 6px; }

/*********************
 ! App notifications *
**********************/
frame.app-notification {
  border-style: solid;
  border-color: rgba(12, 12, 12, 0.8);
  border-width: 0 1px 1px;
  border-radius: 0 0 3px 3px;
  padding: 6px;
  background-color: rgba(15, 15, 15, 0.8);
  background-image: none;
  color: #a5a5a5; }
  frame.app-notification button {
    background-color: rgba(15, 15, 15, 0.8);
    background-image: none;
    border-color: rgba(132, 132, 132, 0.22);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    frame.app-notification button:focus, frame.app-notification button:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
    frame.app-notification button:active, frame.app-notification button:active:hover, frame.app-notification button:active:focus, frame.app-notification button:active:hover:focus, frame.app-notification button:checked, frame.app-notification button:checked:hover, frame.app-notification button:checked:focus, frame.app-notification button:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.22); }
    frame.app-notification button:disabled {
      border-color: rgba(140, 140, 140, 0.22); }
    frame.app-notification button:active:disabled, frame.app-notification button:checked:disabled {
      border-color: rgba(132, 132, 132, 0.22); }
    frame.app-notification button.flat {
      color: #a5a5a5;
      border-color: rgba(15, 15, 15, 0);
      background-color: rgba(15, 15, 15, 0);
      background-image: none;
      box-shadow: none; }
    frame.app-notification button:hover, frame.app-notification button.flat:hover {
      background-color: rgba(16, 16, 16, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      frame.app-notification button:hover:focus, frame.app-notification button:hover:hover, frame.app-notification button.flat:hover:focus, frame.app-notification button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      frame.app-notification button:hover:active, frame.app-notification button:hover:active:hover, frame.app-notification button:hover:active:focus, frame.app-notification button:hover:active:hover:focus, frame.app-notification button:hover:checked, frame.app-notification button:hover:checked:hover, frame.app-notification button:hover:checked:focus, frame.app-notification button:hover:checked:hover:focus, frame.app-notification button.flat:hover:active, frame.app-notification button.flat:hover:active:hover, frame.app-notification button.flat:hover:active:focus, frame.app-notification button.flat:hover:active:hover:focus, frame.app-notification button.flat:hover:checked, frame.app-notification button.flat:hover:checked:hover, frame.app-notification button.flat:hover:checked:focus, frame.app-notification button.flat:hover:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      frame.app-notification button:hover:disabled, frame.app-notification button.flat:hover:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      frame.app-notification button:hover:active:disabled, frame.app-notification button:hover:checked:disabled, frame.app-notification button.flat:hover:active:disabled, frame.app-notification button.flat:hover:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
    frame.app-notification button:focus, frame.app-notification button.flat:focus {
      background-color: rgba(16, 16, 16, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      frame.app-notification button:focus:focus, frame.app-notification button:focus:hover, frame.app-notification button.flat:focus:focus, frame.app-notification button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      frame.app-notification button:focus:active, frame.app-notification button:focus:active:hover, frame.app-notification button:focus:active:focus, frame.app-notification button:focus:active:hover:focus, frame.app-notification button:focus:checked, frame.app-notification button:focus:checked:hover, frame.app-notification button:focus:checked:focus, frame.app-notification button:focus:checked:hover:focus, frame.app-notification button.flat:focus:active, frame.app-notification button.flat:focus:active:hover, frame.app-notification button.flat:focus:active:focus, frame.app-notification button.flat:focus:active:hover:focus, frame.app-notification button.flat:focus:checked, frame.app-notification button.flat:focus:checked:hover, frame.app-notification button.flat:focus:checked:focus, frame.app-notification button.flat:focus:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      frame.app-notification button:focus:disabled, frame.app-notification button.flat:focus:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      frame.app-notification button:focus:active:disabled, frame.app-notification button:focus:checked:disabled, frame.app-notification button.flat:focus:active:disabled, frame.app-notification button.flat:focus:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
      frame.app-notification button:focus:hover, frame.app-notification button.flat:focus:hover {
        background-color: rgba(17, 17, 17, 0.8);
        background-image: none;
        border-color: rgba(132, 132, 132, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        frame.app-notification button:focus:hover:focus, frame.app-notification button:focus:hover:hover, frame.app-notification button.flat:focus:hover:focus, frame.app-notification button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
        frame.app-notification button:focus:hover:active, frame.app-notification button:focus:hover:active:hover, frame.app-notification button:focus:hover:active:focus, frame.app-notification button:focus:hover:active:hover:focus, frame.app-notification button:focus:hover:checked, frame.app-notification button:focus:hover:checked:hover, frame.app-notification button:focus:hover:checked:focus, frame.app-notification button:focus:hover:checked:hover:focus, frame.app-notification button.flat:focus:hover:active, frame.app-notification button.flat:focus:hover:active:hover, frame.app-notification button.flat:focus:hover:active:focus, frame.app-notification button.flat:focus:hover:active:hover:focus, frame.app-notification button.flat:focus:hover:checked, frame.app-notification button.flat:focus:hover:checked:hover, frame.app-notification button.flat:focus:hover:checked:focus, frame.app-notification button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.3); }
        frame.app-notification button:focus:hover:disabled, frame.app-notification button.flat:focus:hover:disabled {
          border-color: rgba(140, 140, 140, 0.3); }
        frame.app-notification button:focus:hover:active:disabled, frame.app-notification button:focus:hover:checked:disabled, frame.app-notification button.flat:focus:hover:active:disabled, frame.app-notification button.flat:focus:hover:checked:disabled {
          border-color: rgba(132, 132, 132, 0.3); }
    frame.app-notification button:checked, frame.app-notification button:active, frame.app-notification button.flat:checked, frame.app-notification button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
      border-color: rgba(132, 132, 132, 0.22); }
      frame.app-notification button:checked:focus, frame.app-notification button:checked:hover, frame.app-notification button:active:focus, frame.app-notification button:active:hover, frame.app-notification button.flat:checked:focus, frame.app-notification button.flat:checked:hover, frame.app-notification button.flat:active:focus, frame.app-notification button.flat:active:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
      frame.app-notification button:checked:active, frame.app-notification button:checked:active:hover, frame.app-notification button:checked:active:focus, frame.app-notification button:checked:active:hover:focus, frame.app-notification button:checked:checked, frame.app-notification button:checked:checked:hover, frame.app-notification button:checked:checked:focus, frame.app-notification button:checked:checked:hover:focus, frame.app-notification button:active:active, frame.app-notification button:active:active:hover, frame.app-notification button:active:active:focus, frame.app-notification button:active:active:hover:focus, frame.app-notification button:active:checked, frame.app-notification button:active:checked:hover, frame.app-notification button:active:checked:focus, frame.app-notification button:active:checked:hover:focus, frame.app-notification button.flat:checked:active, frame.app-notification button.flat:checked:active:hover, frame.app-notification button.flat:checked:active:focus, frame.app-notification button.flat:checked:active:hover:focus, frame.app-notification button.flat:checked:checked, frame.app-notification button.flat:checked:checked:hover, frame.app-notification button.flat:checked:checked:focus, frame.app-notification button.flat:checked:checked:hover:focus, frame.app-notification button.flat:active:active, frame.app-notification button.flat:active:active:hover, frame.app-notification button.flat:active:active:focus, frame.app-notification button.flat:active:active:hover:focus, frame.app-notification button.flat:active:checked, frame.app-notification button.flat:active:checked:hover, frame.app-notification button.flat:active:checked:focus, frame.app-notification button.flat:active:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.22); }
      frame.app-notification button:checked:disabled, frame.app-notification button:active:disabled, frame.app-notification button.flat:checked:disabled, frame.app-notification button.flat:active:disabled {
        border-color: rgba(140, 140, 140, 0.22); }
      frame.app-notification button:checked:active:disabled, frame.app-notification button:checked:checked:disabled, frame.app-notification button:active:active:disabled, frame.app-notification button:active:checked:disabled, frame.app-notification button.flat:checked:active:disabled, frame.app-notification button.flat:checked:checked:disabled, frame.app-notification button.flat:active:active:disabled, frame.app-notification button.flat:active:checked:disabled {
        border-color: rgba(132, 132, 132, 0.22); }
      frame.app-notification button:checked:focus, frame.app-notification button:checked:hover, frame.app-notification button:active:focus, frame.app-notification button:active:hover, frame.app-notification button.flat:checked:focus, frame.app-notification button.flat:checked:hover, frame.app-notification button.flat:active:focus, frame.app-notification button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    frame.app-notification button:focus, frame.app-notification button:hover, frame.app-notification button.flat:focus, frame.app-notification button.flat:hover {
      color: #a5a5a5; }
    frame.app-notification button:disabled:disabled, frame.app-notification button.flat:disabled:disabled {
      background-color: alpha(mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.5);
      box-shadow: none; }
    frame.app-notification button:active:disabled, frame.app-notification button:checked:disabled, frame.app-notification button.flat:active:disabled, frame.app-notification button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    frame.app-notification button.separator, frame.app-notification button .separator {
      border: 1px solid currentColor;
      color: rgba(15, 15, 15, 0.7); }
      frame.app-notification button.separator:disabled, frame.app-notification button .separator:disabled {
        color: rgba(15, 15, 15, 0.65); }
  frame.app-notification border {
    border: 0; }

/*************
 ! Expanders *
**************/
expander arrow {
  min-width: 16px;
  min-height: 16px;
  -gtk-icon-source: -gtk-icontheme("pan-end-symbolic"); }
  expander arrow:dir(rtl) {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl"); }
  expander arrow:hover {
    color: alpha(currentColor,0.8); }
  expander arrow:checked {
    -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }

/*******************
 ! Symbolic images *
********************/
.image {
  color: alpha(currentColor,0.5); }
  .image:hover {
    color: alpha(currentColor,0.9); }
  .image:selected, .image:selected:hover {
    color: #000000; }

/****************
 ! Floating bar *
*****************/
.floating-bar {
  background-color: #a5a5a5;
  background-image: none;
  border: 1px solid #848484;
  border-radius: 3px;
  color: #000000; }
  .floating-bar.top {
    border-top-width: 0;
    border-top-right-radius: 0;
    border-top-left-radius: 0; }
  .floating-bar.right {
    border-right-width: 0;
    border-top-right-radius: 0;
    border-bottom-right-radius: 0; }
  .floating-bar.bottom {
    border-bottom-width: 0;
    border-bottom-right-radius: 0;
    border-bottom-left-radius: 0; }
  .floating-bar.left {
    border-left-width: 0;
    border-bottom-left-radius: 0;
    border-top-left-radius: 0; }
  .floating-bar button {
    border: 0;
    background-color: transparent;
    background-image: none; }

/*************************
 ! Touch text selections *
**************************/
GtkBubbleWindow {
  border-radius: 3px;
  background-clip: border-box; }
  GtkBubbleWindow.osd.background {
    background-color: rgba(15, 15, 15, 0.8); }
  GtkBubbleWindow .toolbar {
    background-color: transparent; }

/***************
 ! Font-viewer *
****************/
SushiFontWidget {
  padding: 3px 6px; }

/*************
 ! Gucharmap *
**************/
GucharmapChartable {
  background-color: #d0d0d0;
  color: #000000; }

/*************
 ! Evolution *
**************/
EPreviewPane .entry {
  background-color: #d0d0d0;
  color: #000000; }

/*******************
 ! Gnome Bluetooth *
********************/
entry.entry.pin-entry {
  font-style: normal;
  font-size: 50px;
  padding-left: 15px;
  padding-right: 15px; }

label.pin-label {
  font-style: normal;
  font-size: 50px; }

/************************
 ! Shortcut window keys *
*************************/
.keycap {
  min-width: 20px;
  min-height: 24px;
  margin-top: 2px;
  padding-bottom: 1.5px;
  padding-left: 3px;
  padding-right: 3px;
  color: #000000;
  background-color: #d0d0d0;
  border: 1px solid;
  border-color: mix(mix(#a5a5a5,#000000,0.08),#a5a5a5,0.5);
  border-radius: 3px;
  box-shadow: inset 0 -3px mix(#d0d0d0,#a5a5a5,0.2);
  font-size: smaller; }
  .keycap:backdrop {
    background-color: #d0d0d0;
    color: mix(#000000,#a5a5a5,0.5);
    transition: 200ms ease-out; }

/*****************
 ! Stackswitcher *
******************/
stackswitcher button.text-button {
  min-width: 80px; }

stackswitcher button.circular, stackswitcher button.nautilus-circular-button.image-button {
  min-width: 28px;
  min-height: 28px;
  padding: 0; }

/*******************
 ! Selected Items  *
********************/
entry selection, menuitem entry selection, popover.background entry selection, .osd entry selection,
#XfceNotifyWindow entry selection, #login_window entry selection, calendar:selected, row:selected, flowbox flowboxchild:selected, modelbutton.flat:active, modelbutton.flat:active arrow, modelbutton.flat:selected, modelbutton.flat:selected arrow,
menuitem.button.flat:active,
menuitem.button.flat:active arrow,
menuitem.button.flat:selected,
menuitem.button.flat:selected arrow, .view:selected:focus, iconview:selected:focus, .view:selected, iconview:selected, .view text:selected:focus, iconview text:selected:focus,
textview text:selected:focus, .view text:selected, iconview text:selected,
textview text:selected, .view text selection:focus, iconview text selection:focus, .view text selection, iconview text selection,
textview text selection:focus,
textview text selection, treeview.view:selected:focus, treeview.view:selected, .cs-category-view:selected:focus, .cs-category-view:selected, .cs-category-view .view:selected:focus, .cs-category-view iconview:selected:focus, .cs-category-view .view:selected, .cs-category-view iconview:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview.view:selected:focus, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:selected:focus, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview.view:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:selected, .nemo-window .sidebar .nemo-places-sidebar .view:selected:focus, .nemo-window .sidebar .nemo-places-sidebar iconview:selected:focus, .nemo-window .sidebar .nemo-places-sidebar .view:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:selected {
  background-color: #B36A87; }
  row:selected label, label:selected, entry selection, menuitem entry selection, popover.background entry selection, .osd entry selection,
  #XfceNotifyWindow entry selection, #login_window entry selection, calendar:selected, row:selected, flowbox flowboxchild:selected, modelbutton.flat:active, modelbutton.flat:active arrow, modelbutton.flat:selected, modelbutton.flat:selected arrow,
  menuitem.button.flat:active,
  menuitem.button.flat:active arrow,
  menuitem.button.flat:selected,
  menuitem.button.flat:selected arrow, .view:selected:focus, iconview:selected:focus, .view:selected, iconview:selected, .view text:selected:focus, iconview text:selected:focus,
  textview text:selected:focus, .view text:selected, iconview text:selected,
  textview text:selected, .view text selection:focus, iconview text selection:focus, .view text selection, iconview text selection,
  textview text selection:focus,
  textview text selection, treeview.view:selected:focus, treeview.view:selected, .cs-category-view:selected:focus, .cs-category-view:selected, .cs-category-view .view:selected:focus, .cs-category-view iconview:selected:focus, .cs-category-view .view:selected, .cs-category-view iconview:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview.view:selected:focus, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:selected:focus, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview.view:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:selected, .nemo-window .sidebar .nemo-places-sidebar .view:selected:focus, .nemo-window .sidebar .nemo-places-sidebar iconview:selected:focus, .nemo-window .sidebar .nemo-places-sidebar .view:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:selected {
    color: #000000;
    outline-color: rgba(0, 0, 0, 0.3); }
    row:selected label:disabled, label:disabled:selected, label:disabled selection, entry selection:disabled,
    #XfceNotifyWindow entry selection:disabled, #login_window entry selection:disabled, calendar:disabled:selected, row:disabled:selected, flowbox flowboxchild:disabled:selected, modelbutton.flat:disabled:active, modelbutton.flat:active arrow:disabled, modelbutton.flat:disabled:selected, modelbutton.flat:selected arrow:disabled,
    menuitem.button.flat:disabled:active,
    menuitem.button.flat:active arrow:disabled,
    menuitem.button.flat:disabled:selected,
    menuitem.button.flat:selected arrow:disabled, iconview:disabled:selected:focus, .view:disabled:selected, iconview:disabled:selected, iconview text:disabled:selected:focus,
    textview text:disabled:selected:focus, .view text:disabled:selected, iconview text:disabled:selected,
    textview text:disabled:selected, iconview text selection:disabled:focus, .view text selection:disabled, iconview text selection:disabled,
    textview text selection:disabled, .cs-category-view:disabled:selected, .cs-category-view iconview:disabled:selected:focus, .cs-category-view iconview:disabled:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:disabled:selected:focus, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:disabled:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:disabled:selected:focus, .nemo-window .sidebar .nemo-places-sidebar .view:disabled:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:disabled:selected {
      color: mix(#000000,#B36A87,0.5); }
    row:selected label:backdrop, label:backdrop:selected, label:backdrop selection, entry selection:backdrop,
    #XfceNotifyWindow entry selection:backdrop, #login_window entry selection:backdrop, calendar:backdrop:selected, row:backdrop:selected, flowbox flowboxchild:backdrop:selected, modelbutton.flat:backdrop:active, modelbutton.flat:active arrow:backdrop, modelbutton.flat:backdrop:selected, modelbutton.flat:selected arrow:backdrop,
    menuitem.button.flat:backdrop:active,
    menuitem.button.flat:active arrow:backdrop,
    menuitem.button.flat:backdrop:selected,
    menuitem.button.flat:selected arrow:backdrop, iconview:backdrop:selected:focus, .view:backdrop:selected, iconview:backdrop:selected, iconview text:backdrop:selected:focus,
    textview text:backdrop:selected:focus, .view text:backdrop:selected, iconview text:backdrop:selected,
    textview text:backdrop:selected, iconview text selection:backdrop:focus, .view text selection:backdrop, iconview text selection:backdrop,
    textview text selection:backdrop, .cs-category-view:backdrop:selected, .cs-category-view iconview:backdrop:selected:focus, .cs-category-view iconview:backdrop:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:backdrop:selected:focus, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:backdrop:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:backdrop:selected:focus, .nemo-window .sidebar .nemo-places-sidebar .view:backdrop:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:backdrop:selected {
      background-color: #B36A87;
      color: mix(#B36A87,#000000,0.66); }
      row:selected label:backdrop:disabled, label:backdrop:disabled:selected, label:disabled selection:backdrop, label:backdrop selection:disabled, entry selection:backdrop:disabled,
      #XfceNotifyWindow entry selection:backdrop:disabled, #login_window entry selection:backdrop:disabled, calendar:backdrop:disabled:selected, row:backdrop:disabled:selected, flowbox flowboxchild:backdrop:disabled:selected, modelbutton.flat:backdrop:disabled:active, modelbutton.flat:active arrow:backdrop:disabled, modelbutton.flat:backdrop:disabled:selected, modelbutton.flat:selected arrow:backdrop:disabled,
      menuitem.button.flat:backdrop:disabled:active,
      menuitem.button.flat:active arrow:backdrop:disabled,
      menuitem.button.flat:backdrop:disabled:selected,
      menuitem.button.flat:selected arrow:backdrop:disabled, .view:backdrop:disabled:selected, iconview:backdrop:disabled:selected, .view text:backdrop:disabled:selected, iconview text:backdrop:disabled:selected,
      textview text:backdrop:disabled:selected, .view text selection:backdrop:disabled, iconview text selection:backdrop:disabled,
      textview text selection:backdrop:disabled, .cs-category-view:backdrop:disabled:selected, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:backdrop:disabled:selected, .nemo-window .sidebar .nemo-places-sidebar .view:backdrop:disabled:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:backdrop:disabled:selected {
        color: mix(mix(#B36A87,#000000,0.66),#B36A87,0.3); }

/*********
 ! Buttons
**********/
/******************
! ComboBoxes *
*******************/
/**********
 ! Notebook
***********/
notebook {
  padding: 0; }
  notebook.frame {
    border: 1px solid mix(#a5a5a5,#000000,0.08); }
    notebook.frame > header {
      margin: -1px; }
      notebook.frame > header.top {
        margin-bottom: 0; }
      notebook.frame > header.bottom {
        margin-top: 0; }
      notebook.frame > header.left {
        margin-right: 0; }
      notebook.frame > header.right {
        margin-left: 0; }
      notebook.frame > header.top, notebook.frame > header.bottom {
        padding-left: 0;
        padding-right: 0; }
      notebook.frame > header.left, notebook.frame > header.right {
        padding-top: 0;
        padding-bottom: 0; }
  notebook > stack:not(:only-child) {
    background-color: #a5a5a5; }
  notebook > header {
    padding: 3px;
    background-color: #a5a5a5; }
    notebook > header.top {
      box-shadow: inset 0 -1px mix(#a5a5a5,#000000,0.08); }
    notebook > header.bottom {
      box-shadow: inset 0 1px mix(#a5a5a5,#000000,0.08); }
    notebook > header.right {
      box-shadow: inset 1px 0 mix(#a5a5a5,#000000,0.08); }
    notebook > header.left {
      box-shadow: inset -1px 0 mix(#a5a5a5,#000000,0.08); }
    notebook > header.top {
      padding-bottom: 0; }
      notebook > header.top > tabs > tab {
        padding: 3px 11px;
        min-width: 20px;
        min-height: 20px;
        outline-offset: -4px;
        border: 1px solid transparent;
        border-bottom: none;
        border-top-width: 3px;
        border-radius: 1px 1px 0 0; }
        notebook > header.top > tabs > tab:checked {
          border-top-color: #B36A87; }
        notebook > header.top > tabs > tab + tab {
          margin-left: -1px; }
    notebook > header.bottom {
      padding-top: 0; }
      notebook > header.bottom > tabs > tab {
        padding: 3px 11px;
        min-width: 20px;
        min-height: 20px;
        outline-offset: -4px;
        border: 1px solid transparent;
        border-top: none;
        border-bottom-width: 3px;
        border-radius: 0 0 1px 1px; }
        notebook > header.bottom > tabs > tab:checked {
          border-bottom-color: #B36A87; }
        notebook > header.bottom > tabs > tab + tab {
          margin-left: -1px; }
    notebook > header.right {
      padding-left: 0; }
      notebook > header.right > tabs > tab {
        padding: 3px 11px;
        min-width: 20px;
        min-height: 20px;
        outline-offset: -4px;
        border: 1px solid transparent;
        border-left: none;
        border-right-width: 3px;
        border-radius: 0 1px 1px 0; }
        notebook > header.right > tabs > tab:checked {
          border-right-color: #B36A87; }
        notebook > header.right > tabs > tab + tab {
          margin-top: -1px; }
    notebook > header.left {
      padding-right: 0; }
      notebook > header.left > tabs > tab {
        padding: 3px 11px;
        min-width: 20px;
        min-height: 20px;
        outline-offset: -4px;
        border: 1px solid transparent;
        border-right: none;
        border-left-width: 3px;
        border-radius: 1px 0 0 1px; }
        notebook > header.left > tabs > tab:checked {
          border-left-color: #B36A87; }
        notebook > header.left > tabs > tab + tab {
          margin-top: -1px; }
    notebook > header.top > tabs > arrow.up, notebook > header.bottom > tabs > arrow.up {
      -gtk-icon-source: -gtk-icontheme("pan-end-symbolic"); }
      notebook > header.top > tabs > arrow.up:last-child, notebook > header.bottom > tabs > arrow.up:last-child {
        margin-left: 2px; }
    notebook > header.top > tabs > arrow.down, notebook > header.bottom > tabs > arrow.down {
      -gtk-icon-source: -gtk-icontheme("pan-start-symbolic"); }
      notebook > header.top > tabs > arrow.down:first-child, notebook > header.bottom > tabs > arrow.down:first-child {
        margin-right: 2px; }
    notebook > header.left > tabs > arrow.up, notebook > header.right > tabs > arrow.up {
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }
      notebook > header.left > tabs > arrow.up:last-child, notebook > header.right > tabs > arrow.up:last-child {
        margin-top: 2px; }
    notebook > header.left > tabs > arrow.down, notebook > header.right > tabs > arrow.down {
      -gtk-icon-source: -gtk-icontheme("pan-up-symbolic"); }
      notebook > header.left > tabs > arrow.down:first-child, notebook > header.right > tabs > arrow.down:first-child {
        margin-bottom: 2px; }
    notebook > header > tabs > arrow {
      color: mix(#000000,#a5a5a5,0.5); }
      notebook > header > tabs > arrow:hover {
        color: mix(#000000,mix(#000000,#a5a5a5,0.5),0.5); }
      notebook > header > tabs > arrow:active {
        color: #000000; }
      notebook > header > tabs > arrow:disabled {
        color: alpha(mix(#000000,#a5a5a5,0.5),0.3); }
    notebook > header.top > tabs > tab:hover:not(:checked) {
      box-shadow: inset 0 -1px mix(#a5a5a5,#000000,0.08); }
    notebook > header.bottom > tabs > tab:hover:not(:checked) {
      box-shadow: inset 0 1px mix(#a5a5a5,#000000,0.08); }
    notebook > header.left > tabs > tab:hover:not(:checked) {
      box-shadow: inset -1px 0 mix(#a5a5a5,#000000,0.08); }
    notebook > header.right > tabs > tab:hover:not(:checked) {
      box-shadow: inset 1px 0 mix(#a5a5a5,#000000,0.08); }
    notebook > header > tabs > tab {
      color: rgba(0, 0, 0, 0.8);
      background-color: alpha(mix(#a5a5a5,#000000,0.1),0.7); }
      notebook > header > tabs > tab:hover:not(:checked) {
        color: mix(#000000,mix(#000000,#a5a5a5,0.5),0.5);
        background-color: rgba(208, 208, 208, 0.5);
        border-color: mix(#a5a5a5,#000000,0.08); }
      notebook > header > tabs > tab:checked {
        border-color: mix(#a5a5a5,#000000,0.08);
        color: #000000;
        background-color: #a5a5a5; }
      notebook > header > tabs > tab button.flat {
        min-height: 22px;
        min-width: 16px;
        padding: 0;
        color: mix(#a5a5a5,#000000,0.35); }
        notebook > header > tabs > tab button.flat:hover {
          color: #ff4d4d; }
        notebook > header > tabs > tab button.flat:active, notebook > header > tabs > tab button.flat:active:hover {
          color: #B36A87; }

/*********
 ! Buttons
**********/
/******************
! ComboBoxes *
*******************/
/*******
 ! OSD *
********/
overlay.osd {
  background-color: transparent; }

colorchooser .popover.osd {
  border-radius: 3px; }

button.color .osd colorswatch:only-child {
  box-shadow: none; }

.osd button.color:disabled colorswatch:only-child, .osd button.color:backdrop colorswatch:only-child, .osd button.color:active colorswatch:only-child, .osd button.color:checked colorswatch:only-child {
  box-shadow: none; }

button.osd,
#XfceNotifyWindow button {
  background-color: rgba(15, 15, 15, 0.8);
  background-image: none;
  border-color: rgba(132, 132, 132, 0.22);
  color: #a5a5a5;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
  button.osd:focus, button.osd:hover,
  #XfceNotifyWindow button:focus,
  #XfceNotifyWindow button:hover {
    border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
  button.osd:active, button.osd:active:hover, button.osd:active:focus, button.osd:active:hover:focus, button.osd:checked, button.osd:checked:hover, button.osd:checked:focus, button.osd:checked:hover:focus,
  #XfceNotifyWindow button:active,
  #XfceNotifyWindow button:active:hover,
  #XfceNotifyWindow button:active:focus,
  #XfceNotifyWindow button:active:hover:focus,
  #XfceNotifyWindow button:checked,
  #XfceNotifyWindow button:checked:hover,
  #XfceNotifyWindow button:checked:focus,
  #XfceNotifyWindow button:checked:hover:focus {
    border-color: rgba(116, 116, 116, 0.22); }
  button.osd:disabled,
  #XfceNotifyWindow button:disabled {
    border-color: rgba(140, 140, 140, 0.22); }
  button.osd:active:disabled, button.osd:checked:disabled,
  #XfceNotifyWindow button:active:disabled,
  #XfceNotifyWindow button:checked:disabled {
    border-color: rgba(132, 132, 132, 0.22); }
  button.osd.flat,
  #XfceNotifyWindow button.flat {
    color: #a5a5a5;
    border-color: rgba(15, 15, 15, 0);
    background-color: rgba(15, 15, 15, 0);
    background-image: none;
    box-shadow: none; }
  button.osd:hover, button.osd.flat:hover,
  #XfceNotifyWindow button:hover,
  #XfceNotifyWindow button.flat:hover {
    background-color: rgba(16, 16, 16, 0.8);
    background-image: none;
    border-color: rgba(132, 132, 132, 0.3);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    button.osd:hover:focus, button.osd:hover:hover, button.osd.flat:hover:focus, button.osd.flat:hover:hover,
    #XfceNotifyWindow button:hover:focus,
    #XfceNotifyWindow button:hover:hover,
    #XfceNotifyWindow button.flat:hover:focus,
    #XfceNotifyWindow button.flat:hover:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
    button.osd:hover:active, button.osd:hover:active:hover, button.osd:hover:active:focus, button.osd:hover:active:hover:focus, button.osd:hover:checked, button.osd:hover:checked:hover, button.osd:hover:checked:focus, button.osd:hover:checked:hover:focus, button.osd.flat:hover:active, button.osd.flat:hover:active:hover, button.osd.flat:hover:active:focus, button.osd.flat:hover:active:hover:focus, button.osd.flat:hover:checked, button.osd.flat:hover:checked:hover, button.osd.flat:hover:checked:focus, button.osd.flat:hover:checked:hover:focus,
    #XfceNotifyWindow button:hover:active,
    #XfceNotifyWindow button:hover:active:hover,
    #XfceNotifyWindow button:hover:active:focus,
    #XfceNotifyWindow button:hover:active:hover:focus,
    #XfceNotifyWindow button:hover:checked,
    #XfceNotifyWindow button:hover:checked:hover,
    #XfceNotifyWindow button:hover:checked:focus,
    #XfceNotifyWindow button:hover:checked:hover:focus,
    #XfceNotifyWindow button.flat:hover:active,
    #XfceNotifyWindow button.flat:hover:active:hover,
    #XfceNotifyWindow button.flat:hover:active:focus,
    #XfceNotifyWindow button.flat:hover:active:hover:focus,
    #XfceNotifyWindow button.flat:hover:checked,
    #XfceNotifyWindow button.flat:hover:checked:hover,
    #XfceNotifyWindow button.flat:hover:checked:focus,
    #XfceNotifyWindow button.flat:hover:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.3); }
    button.osd:hover:disabled, button.osd.flat:hover:disabled,
    #XfceNotifyWindow button:hover:disabled,
    #XfceNotifyWindow button.flat:hover:disabled {
      border-color: rgba(140, 140, 140, 0.3); }
    button.osd:hover:active:disabled, button.osd:hover:checked:disabled, button.osd.flat:hover:active:disabled, button.osd.flat:hover:checked:disabled,
    #XfceNotifyWindow button:hover:active:disabled,
    #XfceNotifyWindow button:hover:checked:disabled,
    #XfceNotifyWindow button.flat:hover:active:disabled,
    #XfceNotifyWindow button.flat:hover:checked:disabled {
      border-color: rgba(132, 132, 132, 0.3); }
  button.osd:focus, button.osd.flat:focus,
  #XfceNotifyWindow button:focus,
  #XfceNotifyWindow button.flat:focus {
    background-color: rgba(16, 16, 16, 0.8);
    background-image: none;
    border-color: rgba(132, 132, 132, 0.3);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    button.osd:focus:focus, button.osd:focus:hover, button.osd.flat:focus:focus, button.osd.flat:focus:hover,
    #XfceNotifyWindow button:focus:focus,
    #XfceNotifyWindow button:focus:hover,
    #XfceNotifyWindow button.flat:focus:focus,
    #XfceNotifyWindow button.flat:focus:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
    button.osd:focus:active, button.osd:focus:active:hover, button.osd:focus:active:focus, button.osd:focus:active:hover:focus, button.osd:focus:checked, button.osd:focus:checked:hover, button.osd:focus:checked:focus, button.osd:focus:checked:hover:focus, button.osd.flat:focus:active, button.osd.flat:focus:active:hover, button.osd.flat:focus:active:focus, button.osd.flat:focus:active:hover:focus, button.osd.flat:focus:checked, button.osd.flat:focus:checked:hover, button.osd.flat:focus:checked:focus, button.osd.flat:focus:checked:hover:focus,
    #XfceNotifyWindow button:focus:active,
    #XfceNotifyWindow button:focus:active:hover,
    #XfceNotifyWindow button:focus:active:focus,
    #XfceNotifyWindow button:focus:active:hover:focus,
    #XfceNotifyWindow button:focus:checked,
    #XfceNotifyWindow button:focus:checked:hover,
    #XfceNotifyWindow button:focus:checked:focus,
    #XfceNotifyWindow button:focus:checked:hover:focus,
    #XfceNotifyWindow button.flat:focus:active,
    #XfceNotifyWindow button.flat:focus:active:hover,
    #XfceNotifyWindow button.flat:focus:active:focus,
    #XfceNotifyWindow button.flat:focus:active:hover:focus,
    #XfceNotifyWindow button.flat:focus:checked,
    #XfceNotifyWindow button.flat:focus:checked:hover,
    #XfceNotifyWindow button.flat:focus:checked:focus,
    #XfceNotifyWindow button.flat:focus:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.3); }
    button.osd:focus:disabled, button.osd.flat:focus:disabled,
    #XfceNotifyWindow button:focus:disabled,
    #XfceNotifyWindow button.flat:focus:disabled {
      border-color: rgba(140, 140, 140, 0.3); }
    button.osd:focus:active:disabled, button.osd:focus:checked:disabled, button.osd.flat:focus:active:disabled, button.osd.flat:focus:checked:disabled,
    #XfceNotifyWindow button:focus:active:disabled,
    #XfceNotifyWindow button:focus:checked:disabled,
    #XfceNotifyWindow button.flat:focus:active:disabled,
    #XfceNotifyWindow button.flat:focus:checked:disabled {
      border-color: rgba(132, 132, 132, 0.3); }
    button.osd:focus:hover, button.osd.flat:focus:hover,
    #XfceNotifyWindow button:focus:hover,
    #XfceNotifyWindow button.flat:focus:hover {
      background-color: rgba(17, 17, 17, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
      button.osd:focus:hover:focus, button.osd:focus:hover:hover, button.osd.flat:focus:hover:focus, button.osd.flat:focus:hover:hover,
      #XfceNotifyWindow button:focus:hover:focus,
      #XfceNotifyWindow button:focus:hover:hover,
      #XfceNotifyWindow button.flat:focus:hover:focus,
      #XfceNotifyWindow button.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      button.osd:focus:hover:active, button.osd:focus:hover:active:hover, button.osd:focus:hover:active:focus, button.osd:focus:hover:active:hover:focus, button.osd:focus:hover:checked, button.osd:focus:hover:checked:hover, button.osd:focus:hover:checked:focus, button.osd:focus:hover:checked:hover:focus, button.osd.flat:focus:hover:active, button.osd.flat:focus:hover:active:hover, button.osd.flat:focus:hover:active:focus, button.osd.flat:focus:hover:active:hover:focus, button.osd.flat:focus:hover:checked, button.osd.flat:focus:hover:checked:hover, button.osd.flat:focus:hover:checked:focus, button.osd.flat:focus:hover:checked:hover:focus,
      #XfceNotifyWindow button:focus:hover:active,
      #XfceNotifyWindow button:focus:hover:active:hover,
      #XfceNotifyWindow button:focus:hover:active:focus,
      #XfceNotifyWindow button:focus:hover:active:hover:focus,
      #XfceNotifyWindow button:focus:hover:checked,
      #XfceNotifyWindow button:focus:hover:checked:hover,
      #XfceNotifyWindow button:focus:hover:checked:focus,
      #XfceNotifyWindow button:focus:hover:checked:hover:focus,
      #XfceNotifyWindow button.flat:focus:hover:active,
      #XfceNotifyWindow button.flat:focus:hover:active:hover,
      #XfceNotifyWindow button.flat:focus:hover:active:focus,
      #XfceNotifyWindow button.flat:focus:hover:active:hover:focus,
      #XfceNotifyWindow button.flat:focus:hover:checked,
      #XfceNotifyWindow button.flat:focus:hover:checked:hover,
      #XfceNotifyWindow button.flat:focus:hover:checked:focus,
      #XfceNotifyWindow button.flat:focus:hover:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      button.osd:focus:hover:disabled, button.osd.flat:focus:hover:disabled,
      #XfceNotifyWindow button:focus:hover:disabled,
      #XfceNotifyWindow button.flat:focus:hover:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      button.osd:focus:hover:active:disabled, button.osd:focus:hover:checked:disabled, button.osd.flat:focus:hover:active:disabled, button.osd.flat:focus:hover:checked:disabled,
      #XfceNotifyWindow button:focus:hover:active:disabled,
      #XfceNotifyWindow button:focus:hover:checked:disabled,
      #XfceNotifyWindow button.flat:focus:hover:active:disabled,
      #XfceNotifyWindow button.flat:focus:hover:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
  button.osd:checked, button.osd:active, button.osd.flat:checked, button.osd.flat:active,
  #XfceNotifyWindow button:checked,
  #XfceNotifyWindow button:active,
  #XfceNotifyWindow button.flat:checked,
  #XfceNotifyWindow button.flat:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
    border-color: rgba(132, 132, 132, 0.22); }
    button.osd:checked:focus, button.osd:checked:hover, button.osd:active:focus, button.osd:active:hover, button.osd.flat:checked:focus, button.osd.flat:checked:hover, button.osd.flat:active:focus, button.osd.flat:active:hover,
    #XfceNotifyWindow button:checked:focus,
    #XfceNotifyWindow button:checked:hover,
    #XfceNotifyWindow button:active:focus,
    #XfceNotifyWindow button:active:hover,
    #XfceNotifyWindow button.flat:checked:focus,
    #XfceNotifyWindow button.flat:checked:hover,
    #XfceNotifyWindow button.flat:active:focus,
    #XfceNotifyWindow button.flat:active:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
    button.osd:checked:active, button.osd:checked:active:hover, button.osd:checked:active:focus, button.osd:checked:active:hover:focus, button.osd:checked:checked, button.osd:checked:checked:hover, button.osd:checked:checked:focus, button.osd:checked:checked:hover:focus, button.osd:active:active, button.osd:active:active:hover, button.osd:active:active:focus, button.osd:active:active:hover:focus, button.osd:active:checked, button.osd:active:checked:hover, button.osd:active:checked:focus, button.osd:active:checked:hover:focus, button.osd.flat:checked:active, button.osd.flat:checked:active:hover, button.osd.flat:checked:active:focus, button.osd.flat:checked:active:hover:focus, button.osd.flat:checked:checked, button.osd.flat:checked:checked:hover, button.osd.flat:checked:checked:focus, button.osd.flat:checked:checked:hover:focus, button.osd.flat:active:active, button.osd.flat:active:active:hover, button.osd.flat:active:active:focus, button.osd.flat:active:active:hover:focus, button.osd.flat:active:checked, button.osd.flat:active:checked:hover, button.osd.flat:active:checked:focus, button.osd.flat:active:checked:hover:focus,
    #XfceNotifyWindow button:checked:active,
    #XfceNotifyWindow button:checked:active:hover,
    #XfceNotifyWindow button:checked:active:focus,
    #XfceNotifyWindow button:checked:active:hover:focus,
    #XfceNotifyWindow button:checked:checked,
    #XfceNotifyWindow button:checked:checked:hover,
    #XfceNotifyWindow button:checked:checked:focus,
    #XfceNotifyWindow button:checked:checked:hover:focus,
    #XfceNotifyWindow button:active:active,
    #XfceNotifyWindow button:active:active:hover,
    #XfceNotifyWindow button:active:active:focus,
    #XfceNotifyWindow button:active:active:hover:focus,
    #XfceNotifyWindow button:active:checked,
    #XfceNotifyWindow button:active:checked:hover,
    #XfceNotifyWindow button:active:checked:focus,
    #XfceNotifyWindow button:active:checked:hover:focus,
    #XfceNotifyWindow button.flat:checked:active,
    #XfceNotifyWindow button.flat:checked:active:hover,
    #XfceNotifyWindow button.flat:checked:active:focus,
    #XfceNotifyWindow button.flat:checked:active:hover:focus,
    #XfceNotifyWindow button.flat:checked:checked,
    #XfceNotifyWindow button.flat:checked:checked:hover,
    #XfceNotifyWindow button.flat:checked:checked:focus,
    #XfceNotifyWindow button.flat:checked:checked:hover:focus,
    #XfceNotifyWindow button.flat:active:active,
    #XfceNotifyWindow button.flat:active:active:hover,
    #XfceNotifyWindow button.flat:active:active:focus,
    #XfceNotifyWindow button.flat:active:active:hover:focus,
    #XfceNotifyWindow button.flat:active:checked,
    #XfceNotifyWindow button.flat:active:checked:hover,
    #XfceNotifyWindow button.flat:active:checked:focus,
    #XfceNotifyWindow button.flat:active:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.22); }
    button.osd:checked:disabled, button.osd:active:disabled, button.osd.flat:checked:disabled, button.osd.flat:active:disabled,
    #XfceNotifyWindow button:checked:disabled,
    #XfceNotifyWindow button:active:disabled,
    #XfceNotifyWindow button.flat:checked:disabled,
    #XfceNotifyWindow button.flat:active:disabled {
      border-color: rgba(140, 140, 140, 0.22); }
    button.osd:checked:active:disabled, button.osd:checked:checked:disabled, button.osd:active:active:disabled, button.osd:active:checked:disabled, button.osd.flat:checked:active:disabled, button.osd.flat:checked:checked:disabled, button.osd.flat:active:active:disabled, button.osd.flat:active:checked:disabled,
    #XfceNotifyWindow button:checked:active:disabled,
    #XfceNotifyWindow button:checked:checked:disabled,
    #XfceNotifyWindow button:active:active:disabled,
    #XfceNotifyWindow button:active:checked:disabled,
    #XfceNotifyWindow button.flat:checked:active:disabled,
    #XfceNotifyWindow button.flat:checked:checked:disabled,
    #XfceNotifyWindow button.flat:active:active:disabled,
    #XfceNotifyWindow button.flat:active:checked:disabled {
      border-color: rgba(132, 132, 132, 0.22); }
    button.osd:checked:focus, button.osd:checked:hover, button.osd:active:focus, button.osd:active:hover, button.osd.flat:checked:focus, button.osd.flat:checked:hover, button.osd.flat:active:focus, button.osd.flat:active:hover,
    #XfceNotifyWindow button:checked:focus,
    #XfceNotifyWindow button:checked:hover,
    #XfceNotifyWindow button:active:focus,
    #XfceNotifyWindow button:active:hover,
    #XfceNotifyWindow button.flat:checked:focus,
    #XfceNotifyWindow button.flat:checked:hover,
    #XfceNotifyWindow button.flat:active:focus,
    #XfceNotifyWindow button.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  button.osd:focus, button.osd:hover, button.osd.flat:focus, button.osd.flat:hover,
  #XfceNotifyWindow button:focus,
  #XfceNotifyWindow button:hover,
  #XfceNotifyWindow button.flat:focus,
  #XfceNotifyWindow button.flat:hover {
    color: #a5a5a5; }
  button.osd:disabled:disabled, button.osd.flat:disabled:disabled,
  #XfceNotifyWindow button:disabled:disabled,
  #XfceNotifyWindow button.flat:disabled:disabled {
    background-color: alpha(mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.5);
    box-shadow: none; }
  button.osd:active:disabled, button.osd:checked:disabled, button.osd.flat:active:disabled, button.osd.flat:checked:disabled,
  #XfceNotifyWindow button:active:disabled,
  #XfceNotifyWindow button:checked:disabled,
  #XfceNotifyWindow button.flat:active:disabled,
  #XfceNotifyWindow button.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  button.osd.separator, button.osd .separator,
  #XfceNotifyWindow button.separator,
  #XfceNotifyWindow button .separator {
    border: 1px solid currentColor;
    color: rgba(15, 15, 15, 0.7); }
    button.osd.separator:disabled, button.osd .separator:disabled,
    #XfceNotifyWindow button.separator:disabled,
    #XfceNotifyWindow button .separator:disabled {
      color: rgba(15, 15, 15, 0.65); }
  button.osd.image-button,
  #XfceNotifyWindow button.image-button {
    padding: 0;
    min-height: 36px;
    min-width: 36px; }

toolbar.osd {
  -GtkToolbar-button-relief: normal;
  padding: 3px;
  border: 1px solid rgba(12, 12, 12, 0.8);
  border-radius: 3px;
  background-color: rgba(15, 15, 15, 0.8);
  background-image: none;
  color: #a5a5a5; }
  toolbar.osd separator {
    color: rgba(14, 14, 14, 0.8); }
  toolbar.osd.left, toolbar.osd.right, toolbar.osd.top, toolbar.osd.bottom {
    border-radius: 0; }

progressbar.osd {
  margin: 2px;
  min-height: 2px;
  min-width: 2px; }
  progressbar.osd trough {
    border-style: none;
    border-radius: 0;
    background-image: none;
    background-color: transparent; }
  progressbar.osd progress {
    border-style: none;
    border-radius: 0;
    background-color: #B36A87;
    background-image: none; }

.osd,
#XfceNotifyWindow {
  background-color: rgba(15, 15, 15, 0.8);
  color: #a5a5a5;
  /* used by gnome-settings-daemon's media-keys OSD */
  /* used by Documents */ }
  .osd.background,
  #XfceNotifyWindow.background {
    background-color: rgba(15, 15, 15, 0.6);
    color: #a5a5a5; }
  .osd .frame,
  #XfceNotifyWindow .frame {
    background-clip: border-box;
    background-origin: border-box; }
  .osd button,
  #XfceNotifyWindow button {
    background-color: rgba(15, 15, 15, 0.8);
    background-image: none;
    border-color: rgba(132, 132, 132, 0.22);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .osd button:focus, .osd button:hover,
    #XfceNotifyWindow button:focus,
    #XfceNotifyWindow button:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
    .osd button:active, .osd button:active:hover, .osd button:active:focus, .osd button:active:hover:focus, .osd button:checked, .osd button:checked:hover, .osd button:checked:focus, .osd button:checked:hover:focus,
    #XfceNotifyWindow button:active,
    #XfceNotifyWindow button:active:hover,
    #XfceNotifyWindow button:active:focus,
    #XfceNotifyWindow button:active:hover:focus,
    #XfceNotifyWindow button:checked,
    #XfceNotifyWindow button:checked:hover,
    #XfceNotifyWindow button:checked:focus,
    #XfceNotifyWindow button:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.22); }
    .osd button:disabled,
    #XfceNotifyWindow button:disabled {
      border-color: rgba(140, 140, 140, 0.22); }
    .osd button:active:disabled, .osd button:checked:disabled,
    #XfceNotifyWindow button:active:disabled,
    #XfceNotifyWindow button:checked:disabled {
      border-color: rgba(132, 132, 132, 0.22); }
    .osd button.flat,
    #XfceNotifyWindow button.flat {
      color: #a5a5a5;
      border-color: rgba(15, 15, 15, 0);
      background-color: rgba(15, 15, 15, 0);
      background-image: none;
      box-shadow: none; }
    .osd button:hover, .osd button.flat:hover,
    #XfceNotifyWindow button:hover,
    #XfceNotifyWindow button.flat:hover {
      background-color: rgba(16, 16, 16, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      .osd button:hover:focus, .osd button:hover:hover, .osd button.flat:hover:focus, .osd button.flat:hover:hover,
      #XfceNotifyWindow button:hover:focus,
      #XfceNotifyWindow button:hover:hover,
      #XfceNotifyWindow button.flat:hover:focus,
      #XfceNotifyWindow button.flat:hover:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      .osd button:hover:active, .osd button:hover:active:hover, .osd button:hover:active:focus, .osd button:hover:active:hover:focus, .osd button:hover:checked, .osd button:hover:checked:hover, .osd button:hover:checked:focus, .osd button:hover:checked:hover:focus, .osd button.flat:hover:active, .osd button.flat:hover:active:hover, .osd button.flat:hover:active:focus, .osd button.flat:hover:active:hover:focus, .osd button.flat:hover:checked, .osd button.flat:hover:checked:hover, .osd button.flat:hover:checked:focus, .osd button.flat:hover:checked:hover:focus,
      #XfceNotifyWindow button:hover:active,
      #XfceNotifyWindow button:hover:active:hover,
      #XfceNotifyWindow button:hover:active:focus,
      #XfceNotifyWindow button:hover:active:hover:focus,
      #XfceNotifyWindow button:hover:checked,
      #XfceNotifyWindow button:hover:checked:hover,
      #XfceNotifyWindow button:hover:checked:focus,
      #XfceNotifyWindow button:hover:checked:hover:focus,
      #XfceNotifyWindow button.flat:hover:active,
      #XfceNotifyWindow button.flat:hover:active:hover,
      #XfceNotifyWindow button.flat:hover:active:focus,
      #XfceNotifyWindow button.flat:hover:active:hover:focus,
      #XfceNotifyWindow button.flat:hover:checked,
      #XfceNotifyWindow button.flat:hover:checked:hover,
      #XfceNotifyWindow button.flat:hover:checked:focus,
      #XfceNotifyWindow button.flat:hover:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      .osd button:hover:disabled, .osd button.flat:hover:disabled,
      #XfceNotifyWindow button:hover:disabled,
      #XfceNotifyWindow button.flat:hover:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      .osd button:hover:active:disabled, .osd button:hover:checked:disabled, .osd button.flat:hover:active:disabled, .osd button.flat:hover:checked:disabled,
      #XfceNotifyWindow button:hover:active:disabled,
      #XfceNotifyWindow button:hover:checked:disabled,
      #XfceNotifyWindow button.flat:hover:active:disabled,
      #XfceNotifyWindow button.flat:hover:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
    .osd button:focus, .osd button.flat:focus,
    #XfceNotifyWindow button:focus,
    #XfceNotifyWindow button.flat:focus {
      background-color: rgba(16, 16, 16, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      .osd button:focus:focus, .osd button:focus:hover, .osd button.flat:focus:focus, .osd button.flat:focus:hover,
      #XfceNotifyWindow button:focus:focus,
      #XfceNotifyWindow button:focus:hover,
      #XfceNotifyWindow button.flat:focus:focus,
      #XfceNotifyWindow button.flat:focus:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      .osd button:focus:active, .osd button:focus:active:hover, .osd button:focus:active:focus, .osd button:focus:active:hover:focus, .osd button:focus:checked, .osd button:focus:checked:hover, .osd button:focus:checked:focus, .osd button:focus:checked:hover:focus, .osd button.flat:focus:active, .osd button.flat:focus:active:hover, .osd button.flat:focus:active:focus, .osd button.flat:focus:active:hover:focus, .osd button.flat:focus:checked, .osd button.flat:focus:checked:hover, .osd button.flat:focus:checked:focus, .osd button.flat:focus:checked:hover:focus,
      #XfceNotifyWindow button:focus:active,
      #XfceNotifyWindow button:focus:active:hover,
      #XfceNotifyWindow button:focus:active:focus,
      #XfceNotifyWindow button:focus:active:hover:focus,
      #XfceNotifyWindow button:focus:checked,
      #XfceNotifyWindow button:focus:checked:hover,
      #XfceNotifyWindow button:focus:checked:focus,
      #XfceNotifyWindow button:focus:checked:hover:focus,
      #XfceNotifyWindow button.flat:focus:active,
      #XfceNotifyWindow button.flat:focus:active:hover,
      #XfceNotifyWindow button.flat:focus:active:focus,
      #XfceNotifyWindow button.flat:focus:active:hover:focus,
      #XfceNotifyWindow button.flat:focus:checked,
      #XfceNotifyWindow button.flat:focus:checked:hover,
      #XfceNotifyWindow button.flat:focus:checked:focus,
      #XfceNotifyWindow button.flat:focus:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      .osd button:focus:disabled, .osd button.flat:focus:disabled,
      #XfceNotifyWindow button:focus:disabled,
      #XfceNotifyWindow button.flat:focus:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      .osd button:focus:active:disabled, .osd button:focus:checked:disabled, .osd button.flat:focus:active:disabled, .osd button.flat:focus:checked:disabled,
      #XfceNotifyWindow button:focus:active:disabled,
      #XfceNotifyWindow button:focus:checked:disabled,
      #XfceNotifyWindow button.flat:focus:active:disabled,
      #XfceNotifyWindow button.flat:focus:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
      .osd button:focus:hover, .osd button.flat:focus:hover,
      #XfceNotifyWindow button:focus:hover,
      #XfceNotifyWindow button.flat:focus:hover {
        background-color: rgba(17, 17, 17, 0.8);
        background-image: none;
        border-color: rgba(132, 132, 132, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        .osd button:focus:hover:focus, .osd button:focus:hover:hover, .osd button.flat:focus:hover:focus, .osd button.flat:focus:hover:hover,
        #XfceNotifyWindow button:focus:hover:focus,
        #XfceNotifyWindow button:focus:hover:hover,
        #XfceNotifyWindow button.flat:focus:hover:focus,
        #XfceNotifyWindow button.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
        .osd button:focus:hover:active, .osd button:focus:hover:active:hover, .osd button:focus:hover:active:focus, .osd button:focus:hover:active:hover:focus, .osd button:focus:hover:checked, .osd button:focus:hover:checked:hover, .osd button:focus:hover:checked:focus, .osd button:focus:hover:checked:hover:focus, .osd button.flat:focus:hover:active, .osd button.flat:focus:hover:active:hover, .osd button.flat:focus:hover:active:focus, .osd button.flat:focus:hover:active:hover:focus, .osd button.flat:focus:hover:checked, .osd button.flat:focus:hover:checked:hover, .osd button.flat:focus:hover:checked:focus, .osd button.flat:focus:hover:checked:hover:focus,
        #XfceNotifyWindow button:focus:hover:active,
        #XfceNotifyWindow button:focus:hover:active:hover,
        #XfceNotifyWindow button:focus:hover:active:focus,
        #XfceNotifyWindow button:focus:hover:active:hover:focus,
        #XfceNotifyWindow button:focus:hover:checked,
        #XfceNotifyWindow button:focus:hover:checked:hover,
        #XfceNotifyWindow button:focus:hover:checked:focus,
        #XfceNotifyWindow button:focus:hover:checked:hover:focus,
        #XfceNotifyWindow button.flat:focus:hover:active,
        #XfceNotifyWindow button.flat:focus:hover:active:hover,
        #XfceNotifyWindow button.flat:focus:hover:active:focus,
        #XfceNotifyWindow button.flat:focus:hover:active:hover:focus,
        #XfceNotifyWindow button.flat:focus:hover:checked,
        #XfceNotifyWindow button.flat:focus:hover:checked:hover,
        #XfceNotifyWindow button.flat:focus:hover:checked:focus,
        #XfceNotifyWindow button.flat:focus:hover:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.3); }
        .osd button:focus:hover:disabled, .osd button.flat:focus:hover:disabled,
        #XfceNotifyWindow button:focus:hover:disabled,
        #XfceNotifyWindow button.flat:focus:hover:disabled {
          border-color: rgba(140, 140, 140, 0.3); }
        .osd button:focus:hover:active:disabled, .osd button:focus:hover:checked:disabled, .osd button.flat:focus:hover:active:disabled, .osd button.flat:focus:hover:checked:disabled,
        #XfceNotifyWindow button:focus:hover:active:disabled,
        #XfceNotifyWindow button:focus:hover:checked:disabled,
        #XfceNotifyWindow button.flat:focus:hover:active:disabled,
        #XfceNotifyWindow button.flat:focus:hover:checked:disabled {
          border-color: rgba(132, 132, 132, 0.3); }
    .osd button:checked, .osd button:active, .osd button.flat:checked, .osd button.flat:active,
    #XfceNotifyWindow button:checked,
    #XfceNotifyWindow button:active,
    #XfceNotifyWindow button.flat:checked,
    #XfceNotifyWindow button.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
      border-color: rgba(132, 132, 132, 0.22); }
      .osd button:checked:focus, .osd button:checked:hover, .osd button:active:focus, .osd button:active:hover, .osd button.flat:checked:focus, .osd button.flat:checked:hover, .osd button.flat:active:focus, .osd button.flat:active:hover,
      #XfceNotifyWindow button:checked:focus,
      #XfceNotifyWindow button:checked:hover,
      #XfceNotifyWindow button:active:focus,
      #XfceNotifyWindow button:active:hover,
      #XfceNotifyWindow button.flat:checked:focus,
      #XfceNotifyWindow button.flat:checked:hover,
      #XfceNotifyWindow button.flat:active:focus,
      #XfceNotifyWindow button.flat:active:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
      .osd button:checked:active, .osd button:checked:active:hover, .osd button:checked:active:focus, .osd button:checked:active:hover:focus, .osd button:checked:checked, .osd button:checked:checked:hover, .osd button:checked:checked:focus, .osd button:checked:checked:hover:focus, .osd button:active:active, .osd button:active:active:hover, .osd button:active:active:focus, .osd button:active:active:hover:focus, .osd button:active:checked, .osd button:active:checked:hover, .osd button:active:checked:focus, .osd button:active:checked:hover:focus, .osd button.flat:checked:active, .osd button.flat:checked:active:hover, .osd button.flat:checked:active:focus, .osd button.flat:checked:active:hover:focus, .osd button.flat:checked:checked, .osd button.flat:checked:checked:hover, .osd button.flat:checked:checked:focus, .osd button.flat:checked:checked:hover:focus, .osd button.flat:active:active, .osd button.flat:active:active:hover, .osd button.flat:active:active:focus, .osd button.flat:active:active:hover:focus, .osd button.flat:active:checked, .osd button.flat:active:checked:hover, .osd button.flat:active:checked:focus, .osd button.flat:active:checked:hover:focus,
      #XfceNotifyWindow button:checked:active,
      #XfceNotifyWindow button:checked:active:hover,
      #XfceNotifyWindow button:checked:active:focus,
      #XfceNotifyWindow button:checked:active:hover:focus,
      #XfceNotifyWindow button:checked:checked,
      #XfceNotifyWindow button:checked:checked:hover,
      #XfceNotifyWindow button:checked:checked:focus,
      #XfceNotifyWindow button:checked:checked:hover:focus,
      #XfceNotifyWindow button:active:active,
      #XfceNotifyWindow button:active:active:hover,
      #XfceNotifyWindow button:active:active:focus,
      #XfceNotifyWindow button:active:active:hover:focus,
      #XfceNotifyWindow button:active:checked,
      #XfceNotifyWindow button:active:checked:hover,
      #XfceNotifyWindow button:active:checked:focus,
      #XfceNotifyWindow button:active:checked:hover:focus,
      #XfceNotifyWindow button.flat:checked:active,
      #XfceNotifyWindow button.flat:checked:active:hover,
      #XfceNotifyWindow button.flat:checked:active:focus,
      #XfceNotifyWindow button.flat:checked:active:hover:focus,
      #XfceNotifyWindow button.flat:checked:checked,
      #XfceNotifyWindow button.flat:checked:checked:hover,
      #XfceNotifyWindow button.flat:checked:checked:focus,
      #XfceNotifyWindow button.flat:checked:checked:hover:focus,
      #XfceNotifyWindow button.flat:active:active,
      #XfceNotifyWindow button.flat:active:active:hover,
      #XfceNotifyWindow button.flat:active:active:focus,
      #XfceNotifyWindow button.flat:active:active:hover:focus,
      #XfceNotifyWindow button.flat:active:checked,
      #XfceNotifyWindow button.flat:active:checked:hover,
      #XfceNotifyWindow button.flat:active:checked:focus,
      #XfceNotifyWindow button.flat:active:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.22); }
      .osd button:checked:disabled, .osd button:active:disabled, .osd button.flat:checked:disabled, .osd button.flat:active:disabled,
      #XfceNotifyWindow button:checked:disabled,
      #XfceNotifyWindow button:active:disabled,
      #XfceNotifyWindow button.flat:checked:disabled,
      #XfceNotifyWindow button.flat:active:disabled {
        border-color: rgba(140, 140, 140, 0.22); }
      .osd button:checked:active:disabled, .osd button:checked:checked:disabled, .osd button:active:active:disabled, .osd button:active:checked:disabled, .osd button.flat:checked:active:disabled, .osd button.flat:checked:checked:disabled, .osd button.flat:active:active:disabled, .osd button.flat:active:checked:disabled,
      #XfceNotifyWindow button:checked:active:disabled,
      #XfceNotifyWindow button:checked:checked:disabled,
      #XfceNotifyWindow button:active:active:disabled,
      #XfceNotifyWindow button:active:checked:disabled,
      #XfceNotifyWindow button.flat:checked:active:disabled,
      #XfceNotifyWindow button.flat:checked:checked:disabled,
      #XfceNotifyWindow button.flat:active:active:disabled,
      #XfceNotifyWindow button.flat:active:checked:disabled {
        border-color: rgba(132, 132, 132, 0.22); }
      .osd button:checked:focus, .osd button:checked:hover, .osd button:active:focus, .osd button:active:hover, .osd button.flat:checked:focus, .osd button.flat:checked:hover, .osd button.flat:active:focus, .osd button.flat:active:hover,
      #XfceNotifyWindow button:checked:focus,
      #XfceNotifyWindow button:checked:hover,
      #XfceNotifyWindow button:active:focus,
      #XfceNotifyWindow button:active:hover,
      #XfceNotifyWindow button.flat:checked:focus,
      #XfceNotifyWindow button.flat:checked:hover,
      #XfceNotifyWindow button.flat:active:focus,
      #XfceNotifyWindow button.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    .osd button:focus, .osd button:hover, .osd button.flat:focus, .osd button.flat:hover,
    #XfceNotifyWindow button:focus,
    #XfceNotifyWindow button:hover,
    #XfceNotifyWindow button.flat:focus,
    #XfceNotifyWindow button.flat:hover {
      color: #a5a5a5; }
    .osd button:disabled:disabled, .osd button.flat:disabled:disabled,
    #XfceNotifyWindow button:disabled:disabled,
    #XfceNotifyWindow button.flat:disabled:disabled {
      background-color: alpha(mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.5);
      box-shadow: none; }
    .osd button:active:disabled, .osd button:checked:disabled, .osd button.flat:active:disabled, .osd button.flat:checked:disabled,
    #XfceNotifyWindow button:active:disabled,
    #XfceNotifyWindow button:checked:disabled,
    #XfceNotifyWindow button.flat:active:disabled,
    #XfceNotifyWindow button.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    .osd button.separator, .osd button .separator,
    #XfceNotifyWindow button.separator,
    #XfceNotifyWindow button .separator {
      border: 1px solid currentColor;
      color: rgba(15, 15, 15, 0.7); }
      .osd button.separator:disabled, .osd button .separator:disabled,
      #XfceNotifyWindow button.separator:disabled,
      #XfceNotifyWindow button .separator:disabled {
        color: rgba(15, 15, 15, 0.65); }
  .osd entry,
  #XfceNotifyWindow entry {
    background-color: #0f0f0f;
    background-image: none;
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.8);
    padding: 3px;
    color: #a5a5a5;
    caret-color: #000000;
    -gtk-secondary-caret-color: #000000; }
    .osd entry:focus, .osd entry:hover,
    #XfceNotifyWindow entry:focus,
    #XfceNotifyWindow entry:hover {
      border-color: mix(#B36A87,mix(#0f0f0f,#a5a5a5,0.2),0.3); }
    .osd entry:active, .osd entry:active:hover, .osd entry:active:focus, .osd entry:active:hover:focus, .osd entry:checked, .osd entry:checked:hover, .osd entry:checked:focus, .osd entry:checked:hover:focus,
    #XfceNotifyWindow entry:active,
    #XfceNotifyWindow entry:active:hover,
    #XfceNotifyWindow entry:active:focus,
    #XfceNotifyWindow entry:active:hover:focus,
    #XfceNotifyWindow entry:checked,
    #XfceNotifyWindow entry:checked:hover,
    #XfceNotifyWindow entry:checked:focus,
    #XfceNotifyWindow entry:checked:hover:focus {
      border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.7); }
    .osd entry:disabled,
    #XfceNotifyWindow entry:disabled {
      border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.85); }
    .osd entry:active:disabled, .osd entry:checked:disabled,
    #XfceNotifyWindow entry:active:disabled,
    #XfceNotifyWindow entry:checked:disabled {
      border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.8); }
    .osd entry:focus, .osd entry:active,
    #XfceNotifyWindow entry:focus,
    #XfceNotifyWindow entry:active {
      border-color: mix(#B36A87,rgba(13, 13, 13, 0.8),0.3); }
    .osd entry:disabled,
    #XfceNotifyWindow entry:disabled {
      background-color: #0e0e0e;
      background-image: none;
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.8);
      /*@include border(mix($bg, $fg, .20));*/
      /*border-color: alpha(mix($bg, $fg, .15), .8);*/
      /*opacity: .8;*/
      color: mix(#0f0f0f,#a5a5a5,0.5); }
      .osd entry:disabled:focus, .osd entry:disabled:hover,
      #XfceNotifyWindow entry:disabled:focus,
      #XfceNotifyWindow entry:disabled:hover {
        border-color: mix(#B36A87,alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.3); }
      .osd entry:disabled:active, .osd entry:disabled:active:hover, .osd entry:disabled:active:focus, .osd entry:disabled:active:hover:focus, .osd entry:disabled:checked, .osd entry:disabled:checked:hover, .osd entry:disabled:checked:focus, .osd entry:disabled:checked:hover:focus,
      #XfceNotifyWindow entry:disabled:active,
      #XfceNotifyWindow entry:disabled:active:hover,
      #XfceNotifyWindow entry:disabled:active:focus,
      #XfceNotifyWindow entry:disabled:active:hover:focus,
      #XfceNotifyWindow entry:disabled:checked,
      #XfceNotifyWindow entry:disabled:checked:hover,
      #XfceNotifyWindow entry:disabled:checked:focus,
      #XfceNotifyWindow entry:disabled:checked:hover:focus {
        border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.7); }
      .osd entry:disabled:disabled,
      #XfceNotifyWindow entry:disabled:disabled {
        border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.85); }
      .osd entry:disabled:active:disabled, .osd entry:disabled:checked:disabled,
      #XfceNotifyWindow entry:disabled:active:disabled,
      #XfceNotifyWindow entry:disabled:checked:disabled {
        border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.8); }
  .osd trough, .osd.trough,
  #XfceNotifyWindow trough,
  #XfceNotifyWindow.trough {
    background-color: rgba(165, 165, 165, 0.3); }
  .osd progressbar, .osd.progressbar,
  #XfceNotifyWindow progressbar,
  #XfceNotifyWindow.progressbar {
    background-color: #a5a5a5; }
  .osd scale slider,
  #XfceNotifyWindow scale slider {
    background-color: rgba(16, 16, 16, 0.8);
    background-image: none;
    border-color: rgba(12, 12, 12, 0.8); }
    .osd scale slider:focus, .osd scale slider:hover,
    #XfceNotifyWindow scale slider:focus,
    #XfceNotifyWindow scale slider:hover {
      border-color: mix(#B36A87,rgba(15, 15, 15, 0.8),0.3); }
    .osd scale slider:active, .osd scale slider:active:hover, .osd scale slider:active:focus, .osd scale slider:active:hover:focus, .osd scale slider:checked, .osd scale slider:checked:hover, .osd scale slider:checked:focus, .osd scale slider:checked:hover:focus,
    #XfceNotifyWindow scale slider:active,
    #XfceNotifyWindow scale slider:active:hover,
    #XfceNotifyWindow scale slider:active:focus,
    #XfceNotifyWindow scale slider:active:hover:focus,
    #XfceNotifyWindow scale slider:checked,
    #XfceNotifyWindow scale slider:checked:hover,
    #XfceNotifyWindow scale slider:checked:focus,
    #XfceNotifyWindow scale slider:checked:hover:focus {
      border-color: rgba(11, 11, 11, 0.8); }
    .osd scale slider:disabled,
    #XfceNotifyWindow scale slider:disabled {
      border-color: rgba(13, 13, 13, 0.8); }
    .osd scale slider:active:disabled, .osd scale slider:checked:disabled,
    #XfceNotifyWindow scale slider:active:disabled,
    #XfceNotifyWindow scale slider:checked:disabled {
      border-color: rgba(12, 12, 12, 0.8); }
    .osd scale slider:disabled,
    #XfceNotifyWindow scale slider:disabled {
      background-color: rgba(14, 14, 14, 0.8);
      background-image: none; }
  .osd scale trough,
  #XfceNotifyWindow scale trough {
    border-color: rgba(12, 12, 12, 0.8);
    background-color: rgba(16, 16, 16, 0.8);
    background-image: none; }
    .osd scale trough.highlight,
    #XfceNotifyWindow scale trough.highlight {
      border-color: #B36A87;
      background-color: #B36A87;
      background-image: none; }
    .osd scale trough:disabled, .osd scale trough.highlight:disabled,
    #XfceNotifyWindow scale trough:disabled,
    #XfceNotifyWindow scale trough.highlight:disabled {
      border-color: rgba(13, 13, 13, 0.8);
      background-color: rgba(14, 14, 14, 0.8);
      background-image: none; }
  .osd scale trough,
  #XfceNotifyWindow scale trough {
    background-color: rgba(33, 33, 33, 0.8); }
    .osd scale trough highlight,
    #XfceNotifyWindow scale trough highlight {
      background-color: #B36A87; }
  .osd scale slider,
  #XfceNotifyWindow scale slider {
    background-clip: border-box;
    background-color: #B36A87;
    border-color: #B36A87; }
    .osd scale slider:hover,
    #XfceNotifyWindow scale slider:hover {
      background-color: #c48ca2;
      border-color: #c48ca2; }
    .osd scale slider:active,
    #XfceNotifyWindow scale slider:active {
      background-color: #9b4f6d;
      border-color: #9b4f6d; }
  .osd.view, iconview.osd, .osd .view, .osd iconview, .osd view,
  #XfceNotifyWindow.view,
  iconview#XfceNotifyWindow,
  #XfceNotifyWindow .view,
  #XfceNotifyWindow iconview,
  #XfceNotifyWindow view {
    background-color: rgba(15, 15, 15, 0.8); }
  .osd scrollbar trough,
  #XfceNotifyWindow scrollbar trough {
    background-color: rgba(15, 15, 15, 0.8); }
  .osd scrollbar slider,
  #XfceNotifyWindow scrollbar slider {
    border: 1px solid mix(rgba(13, 13, 13, 0.8),#a5a5a5,0.21);
    border-radius: 0;
    background-color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.21); }
    .osd scrollbar slider:hover,
    #XfceNotifyWindow scrollbar slider:hover {
      border-color: mix(rgba(13, 13, 13, 0.8),#a5a5a5,0.31);
      background-color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.31); }
    .osd scrollbar slider:active,
    #XfceNotifyWindow scrollbar slider:active {
      border-color: #a95778;
      background-color: #B36A87; }
  .osd iconview.cell:selected, .osd iconview.cell:selected:focus,
  #XfceNotifyWindow iconview.cell:selected,
  #XfceNotifyWindow iconview.cell:selected:focus {
    background-color: transparent;
    border: 3px solid mix(rgba(13, 13, 13, 0.8),#a5a5a5,0.21);
    border-radius: 3px;
    outline-color: transparent; }
  .osd .page-thumbnail,
  #XfceNotifyWindow .page-thumbnail {
    border: 1px solid rgba(14, 14, 14, 0.8);
    /* when there's no pixbuf yet */
    background-color: rgba(15, 15, 15, 0.8); }
  .osd popover.background,
  #XfceNotifyWindow popover.background {
    box-shadow: 0 2px 7px 3px rgba(0, 0, 0, 0.5); }
    .osd popover.background > toolbar button,
    #XfceNotifyWindow popover.background > toolbar button {
      border-radius: 0;
      border-width: 0;
      background-color: transparent;
      background-image: none; }
  .osd spinbutton:not(.vertical),
  #XfceNotifyWindow spinbutton:not(.vertical) {
    background-color: #0f0f0f;
    background-image: none;
    border-color: #0c0c0c;
    padding: 0;
    color: #a5a5a5;
    caret-color: #a5a5a5; }
    .osd spinbutton:not(.vertical):focus, .osd spinbutton:not(.vertical):hover,
    #XfceNotifyWindow spinbutton:not(.vertical):focus,
    #XfceNotifyWindow spinbutton:not(.vertical):hover {
      border-color: mix(#B36A87,#0f0f0f,0.3); }
    .osd spinbutton:not(.vertical):active, .osd spinbutton:not(.vertical):active:hover, .osd spinbutton:not(.vertical):active:focus, .osd spinbutton:not(.vertical):active:hover:focus, .osd spinbutton:not(.vertical):checked, .osd spinbutton:not(.vertical):checked:hover, .osd spinbutton:not(.vertical):checked:focus, .osd spinbutton:not(.vertical):checked:hover:focus,
    #XfceNotifyWindow spinbutton:not(.vertical):active,
    #XfceNotifyWindow spinbutton:not(.vertical):active:hover,
    #XfceNotifyWindow spinbutton:not(.vertical):active:focus,
    #XfceNotifyWindow spinbutton:not(.vertical):active:hover:focus,
    #XfceNotifyWindow spinbutton:not(.vertical):checked,
    #XfceNotifyWindow spinbutton:not(.vertical):checked:hover,
    #XfceNotifyWindow spinbutton:not(.vertical):checked:focus,
    #XfceNotifyWindow spinbutton:not(.vertical):checked:hover:focus {
      border-color: #0b0b0b; }
    .osd spinbutton:not(.vertical):disabled,
    #XfceNotifyWindow spinbutton:not(.vertical):disabled {
      border-color: #0d0d0d; }
    .osd spinbutton:not(.vertical):active:disabled, .osd spinbutton:not(.vertical):checked:disabled,
    #XfceNotifyWindow spinbutton:not(.vertical):active:disabled,
    #XfceNotifyWindow spinbutton:not(.vertical):checked:disabled {
      border-color: #0c0c0c; }
    .osd spinbutton:not(.vertical):focus, .osd spinbutton:not(.vertical):active,
    #XfceNotifyWindow spinbutton:not(.vertical):focus,
    #XfceNotifyWindow spinbutton:not(.vertical):active {
      border-color: mix(#B36A87,rgba(13, 13, 13, 0.8),0.3); }
    .osd spinbutton:not(.vertical):disabled,
    #XfceNotifyWindow spinbutton:not(.vertical):disabled {
      background-color: #0e0e0e;
      background-image: none;
      color: mix(#0f0f0f,#a5a5a5,0.5); }
    .osd spinbutton:not(.vertical) button,
    #XfceNotifyWindow spinbutton:not(.vertical) button {
      background-color: rgba(15, 15, 15, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.22);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22);
      border-radius: 0;
      border-color: rgba(13, 13, 13, 0.5);
      border-style: none none none solid;
      background-image: none;
      box-shadow: none; }
      .osd spinbutton:not(.vertical) button:focus, .osd spinbutton:not(.vertical) button:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
      .osd spinbutton:not(.vertical) button:active, .osd spinbutton:not(.vertical) button:active:hover, .osd spinbutton:not(.vertical) button:active:focus, .osd spinbutton:not(.vertical) button:active:hover:focus, .osd spinbutton:not(.vertical) button:checked, .osd spinbutton:not(.vertical) button:checked:hover, .osd spinbutton:not(.vertical) button:checked:focus, .osd spinbutton:not(.vertical) button:checked:hover:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active:hover:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.22); }
      .osd spinbutton:not(.vertical) button:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:disabled {
        border-color: rgba(140, 140, 140, 0.22); }
      .osd spinbutton:not(.vertical) button:active:disabled, .osd spinbutton:not(.vertical) button:checked:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked:disabled {
        border-color: rgba(132, 132, 132, 0.22); }
      .osd spinbutton:not(.vertical) button.flat,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat {
        color: #a5a5a5;
        border-color: rgba(15, 15, 15, 0);
        background-color: rgba(15, 15, 15, 0);
        background-image: none;
        box-shadow: none; }
      .osd spinbutton:not(.vertical) button:hover, .osd spinbutton:not(.vertical) button.flat:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover {
        background-color: rgba(16, 16, 16, 0.8);
        background-image: none;
        border-color: rgba(132, 132, 132, 0.3);
        color: #a5a5a5;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
        .osd spinbutton:not(.vertical) button:hover:focus, .osd spinbutton:not(.vertical) button:hover:hover, .osd spinbutton:not(.vertical) button.flat:hover:focus, .osd spinbutton:not(.vertical) button.flat:hover:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
        .osd spinbutton:not(.vertical) button:hover:active, .osd spinbutton:not(.vertical) button:hover:active:hover, .osd spinbutton:not(.vertical) button:hover:active:focus, .osd spinbutton:not(.vertical) button:hover:active:hover:focus, .osd spinbutton:not(.vertical) button:hover:checked, .osd spinbutton:not(.vertical) button:hover:checked:hover, .osd spinbutton:not(.vertical) button:hover:checked:focus, .osd spinbutton:not(.vertical) button:hover:checked:hover:focus, .osd spinbutton:not(.vertical) button.flat:hover:active, .osd spinbutton:not(.vertical) button.flat:hover:active:hover, .osd spinbutton:not(.vertical) button.flat:hover:active:focus, .osd spinbutton:not(.vertical) button.flat:hover:active:hover:focus, .osd spinbutton:not(.vertical) button.flat:hover:checked, .osd spinbutton:not(.vertical) button.flat:hover:checked:hover, .osd spinbutton:not(.vertical) button.flat:hover:checked:focus, .osd spinbutton:not(.vertical) button.flat:hover:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.3); }
        .osd spinbutton:not(.vertical) button:hover:disabled, .osd spinbutton:not(.vertical) button.flat:hover:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:disabled {
          border-color: rgba(140, 140, 140, 0.3); }
        .osd spinbutton:not(.vertical) button:hover:active:disabled, .osd spinbutton:not(.vertical) button:hover:checked:disabled, .osd spinbutton:not(.vertical) button.flat:hover:active:disabled, .osd spinbutton:not(.vertical) button.flat:hover:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:hover:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover:checked:disabled {
          border-color: rgba(132, 132, 132, 0.3); }
      .osd spinbutton:not(.vertical) button:focus, .osd spinbutton:not(.vertical) button.flat:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus {
        background-color: rgba(16, 16, 16, 0.8);
        background-image: none;
        border-color: rgba(132, 132, 132, 0.3);
        color: #a5a5a5;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
        .osd spinbutton:not(.vertical) button:focus:focus, .osd spinbutton:not(.vertical) button:focus:hover, .osd spinbutton:not(.vertical) button.flat:focus:focus, .osd spinbutton:not(.vertical) button.flat:focus:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
        .osd spinbutton:not(.vertical) button:focus:active, .osd spinbutton:not(.vertical) button:focus:active:hover, .osd spinbutton:not(.vertical) button:focus:active:focus, .osd spinbutton:not(.vertical) button:focus:active:hover:focus, .osd spinbutton:not(.vertical) button:focus:checked, .osd spinbutton:not(.vertical) button:focus:checked:hover, .osd spinbutton:not(.vertical) button:focus:checked:focus, .osd spinbutton:not(.vertical) button:focus:checked:hover:focus, .osd spinbutton:not(.vertical) button.flat:focus:active, .osd spinbutton:not(.vertical) button.flat:focus:active:hover, .osd spinbutton:not(.vertical) button.flat:focus:active:focus, .osd spinbutton:not(.vertical) button.flat:focus:active:hover:focus, .osd spinbutton:not(.vertical) button.flat:focus:checked, .osd spinbutton:not(.vertical) button.flat:focus:checked:hover, .osd spinbutton:not(.vertical) button.flat:focus:checked:focus, .osd spinbutton:not(.vertical) button.flat:focus:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.3); }
        .osd spinbutton:not(.vertical) button:focus:disabled, .osd spinbutton:not(.vertical) button.flat:focus:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:disabled {
          border-color: rgba(140, 140, 140, 0.3); }
        .osd spinbutton:not(.vertical) button:focus:active:disabled, .osd spinbutton:not(.vertical) button:focus:checked:disabled, .osd spinbutton:not(.vertical) button.flat:focus:active:disabled, .osd spinbutton:not(.vertical) button.flat:focus:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:checked:disabled {
          border-color: rgba(132, 132, 132, 0.3); }
        .osd spinbutton:not(.vertical) button:focus:hover, .osd spinbutton:not(.vertical) button.flat:focus:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover {
          background-color: rgba(17, 17, 17, 0.8);
          background-image: none;
          border-color: rgba(132, 132, 132, 0.3);
          box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
          .osd spinbutton:not(.vertical) button:focus:hover:focus, .osd spinbutton:not(.vertical) button:focus:hover:hover, .osd spinbutton:not(.vertical) button.flat:focus:hover:focus, .osd spinbutton:not(.vertical) button.flat:focus:hover:hover,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:hover,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:hover {
            border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
          .osd spinbutton:not(.vertical) button:focus:hover:active, .osd spinbutton:not(.vertical) button:focus:hover:active:hover, .osd spinbutton:not(.vertical) button:focus:hover:active:focus, .osd spinbutton:not(.vertical) button:focus:hover:active:hover:focus, .osd spinbutton:not(.vertical) button:focus:hover:checked, .osd spinbutton:not(.vertical) button:focus:hover:checked:hover, .osd spinbutton:not(.vertical) button:focus:hover:checked:focus, .osd spinbutton:not(.vertical) button:focus:hover:checked:hover:focus, .osd spinbutton:not(.vertical) button.flat:focus:hover:active, .osd spinbutton:not(.vertical) button.flat:focus:hover:active:hover, .osd spinbutton:not(.vertical) button.flat:focus:hover:active:focus, .osd spinbutton:not(.vertical) button.flat:focus:hover:active:hover:focus, .osd spinbutton:not(.vertical) button.flat:focus:hover:checked, .osd spinbutton:not(.vertical) button.flat:focus:hover:checked:hover, .osd spinbutton:not(.vertical) button.flat:focus:hover:checked:focus, .osd spinbutton:not(.vertical) button.flat:focus:hover:checked:hover:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:active,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:active:hover,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:active:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:active:hover:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:checked,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:checked:hover,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:checked:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:checked:hover:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:active,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:active:hover,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:active:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:active:hover:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:checked,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:checked:hover,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:checked:focus,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:checked:hover:focus {
            border-color: rgba(116, 116, 116, 0.3); }
          .osd spinbutton:not(.vertical) button:focus:hover:disabled, .osd spinbutton:not(.vertical) button.flat:focus:hover:disabled,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:disabled,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:disabled {
            border-color: rgba(140, 140, 140, 0.3); }
          .osd spinbutton:not(.vertical) button:focus:hover:active:disabled, .osd spinbutton:not(.vertical) button:focus:hover:checked:disabled, .osd spinbutton:not(.vertical) button.flat:focus:hover:active:disabled, .osd spinbutton:not(.vertical) button.flat:focus:hover:checked:disabled,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:active:disabled,
          #XfceNotifyWindow spinbutton:not(.vertical) button:focus:hover:checked:disabled,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:active:disabled,
          #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus:hover:checked:disabled {
            border-color: rgba(132, 132, 132, 0.3); }
      .osd spinbutton:not(.vertical) button:checked, .osd spinbutton:not(.vertical) button:active, .osd spinbutton:not(.vertical) button.flat:checked, .osd spinbutton:not(.vertical) button.flat:active,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active {
        /*@include linear-gradient(shade($bg, .7), to top);*/
        background-color: #B36A87;
        background-image: none;
        color: #000000;
        box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
        border-color: rgba(132, 132, 132, 0.22); }
        .osd spinbutton:not(.vertical) button:checked:focus, .osd spinbutton:not(.vertical) button:checked:hover, .osd spinbutton:not(.vertical) button:active:focus, .osd spinbutton:not(.vertical) button:active:hover, .osd spinbutton:not(.vertical) button.flat:checked:focus, .osd spinbutton:not(.vertical) button.flat:checked:hover, .osd spinbutton:not(.vertical) button.flat:active:focus, .osd spinbutton:not(.vertical) button.flat:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
        .osd spinbutton:not(.vertical) button:checked:active, .osd spinbutton:not(.vertical) button:checked:active:hover, .osd spinbutton:not(.vertical) button:checked:active:focus, .osd spinbutton:not(.vertical) button:checked:active:hover:focus, .osd spinbutton:not(.vertical) button:checked:checked, .osd spinbutton:not(.vertical) button:checked:checked:hover, .osd spinbutton:not(.vertical) button:checked:checked:focus, .osd spinbutton:not(.vertical) button:checked:checked:hover:focus, .osd spinbutton:not(.vertical) button:active:active, .osd spinbutton:not(.vertical) button:active:active:hover, .osd spinbutton:not(.vertical) button:active:active:focus, .osd spinbutton:not(.vertical) button:active:active:hover:focus, .osd spinbutton:not(.vertical) button:active:checked, .osd spinbutton:not(.vertical) button:active:checked:hover, .osd spinbutton:not(.vertical) button:active:checked:focus, .osd spinbutton:not(.vertical) button:active:checked:hover:focus, .osd spinbutton:not(.vertical) button.flat:checked:active, .osd spinbutton:not(.vertical) button.flat:checked:active:hover, .osd spinbutton:not(.vertical) button.flat:checked:active:focus, .osd spinbutton:not(.vertical) button.flat:checked:active:hover:focus, .osd spinbutton:not(.vertical) button.flat:checked:checked, .osd spinbutton:not(.vertical) button.flat:checked:checked:hover, .osd spinbutton:not(.vertical) button.flat:checked:checked:focus, .osd spinbutton:not(.vertical) button.flat:checked:checked:hover:focus, .osd spinbutton:not(.vertical) button.flat:active:active, .osd spinbutton:not(.vertical) button.flat:active:active:hover, .osd spinbutton:not(.vertical) button.flat:active:active:focus, .osd spinbutton:not(.vertical) button.flat:active:active:hover:focus, .osd spinbutton:not(.vertical) button.flat:active:checked, .osd spinbutton:not(.vertical) button.flat:active:checked:hover, .osd spinbutton:not(.vertical) button.flat:active:checked:focus, .osd spinbutton:not(.vertical) button.flat:active:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:checked:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:active,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:active:hover:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:checked,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.22); }
        .osd spinbutton:not(.vertical) button:checked:disabled, .osd spinbutton:not(.vertical) button:active:disabled, .osd spinbutton:not(.vertical) button.flat:checked:disabled, .osd spinbutton:not(.vertical) button.flat:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:disabled {
          border-color: rgba(140, 140, 140, 0.22); }
        .osd spinbutton:not(.vertical) button:checked:active:disabled, .osd spinbutton:not(.vertical) button:checked:checked:disabled, .osd spinbutton:not(.vertical) button:active:active:disabled, .osd spinbutton:not(.vertical) button:active:checked:disabled, .osd spinbutton:not(.vertical) button.flat:checked:active:disabled, .osd spinbutton:not(.vertical) button.flat:checked:checked:disabled, .osd spinbutton:not(.vertical) button.flat:active:active:disabled, .osd spinbutton:not(.vertical) button.flat:active:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:checked:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:active:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:checked:disabled {
          border-color: rgba(132, 132, 132, 0.22); }
        .osd spinbutton:not(.vertical) button:checked:focus, .osd spinbutton:not(.vertical) button:checked:hover, .osd spinbutton:not(.vertical) button:active:focus, .osd spinbutton:not(.vertical) button:active:hover, .osd spinbutton:not(.vertical) button.flat:checked:focus, .osd spinbutton:not(.vertical) button.flat:checked:hover, .osd spinbutton:not(.vertical) button.flat:active:focus, .osd spinbutton:not(.vertical) button.flat:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button:active:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:hover,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:focus,
        #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:hover {
          /*@include linear-gradient(shade($bg, .65), to top);*/
          background-color: #b8738f;
          background-image: none;
          color: #000000; }
      .osd spinbutton:not(.vertical) button:focus, .osd spinbutton:not(.vertical) button:hover, .osd spinbutton:not(.vertical) button.flat:focus, .osd spinbutton:not(.vertical) button.flat:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:focus,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:hover {
        color: #a5a5a5; }
      .osd spinbutton:not(.vertical) button:disabled:disabled, .osd spinbutton:not(.vertical) button.flat:disabled:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:disabled:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:disabled:disabled {
        background-color: alpha(mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.2),0.4);
        background-image: none;
        /*border: 1px solid alpha($bg, .2);*/
        opacity: .4;
        color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.5);
        box-shadow: none; }
      .osd spinbutton:not(.vertical) button:active:disabled, .osd spinbutton:not(.vertical) button:checked:disabled, .osd spinbutton:not(.vertical) button.flat:active:disabled, .osd spinbutton:not(.vertical) button.flat:checked:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:active:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button.flat:checked:disabled {
        background-color: rgba(179, 106, 135, 0.6);
        background-image: none;
        color: #000000;
        box-shadow: none; }
      .osd spinbutton:not(.vertical) button.separator, .osd spinbutton:not(.vertical) button .separator,
      #XfceNotifyWindow spinbutton:not(.vertical) button.separator,
      #XfceNotifyWindow spinbutton:not(.vertical) button .separator {
        border: 1px solid currentColor;
        color: rgba(15, 15, 15, 0.7); }
        .osd spinbutton:not(.vertical) button.separator:disabled, .osd spinbutton:not(.vertical) button .separator:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button.separator:disabled,
        #XfceNotifyWindow spinbutton:not(.vertical) button .separator:disabled {
          color: rgba(15, 15, 15, 0.65); }
      .osd spinbutton:not(.vertical) button:dir(rtl),
      #XfceNotifyWindow spinbutton:not(.vertical) button:dir(rtl) {
        border-style: none solid none none; }
      .osd spinbutton:not(.vertical) button:active, .osd spinbutton:not(.vertical) button:checked, .osd spinbutton:not(.vertical) button:hover,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active,
      #XfceNotifyWindow spinbutton:not(.vertical) button:checked,
      #XfceNotifyWindow spinbutton:not(.vertical) button:hover {
        color: #a5a5a5; }
      .osd spinbutton:not(.vertical) button:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:disabled {
        color: alpha(mix(#a5a5a5,#0f0f0f,0.6),0.8); }
      .osd spinbutton:not(.vertical) button:backdrop,
      #XfceNotifyWindow spinbutton:not(.vertical) button:backdrop {
        color: mix(#d0d0d0,mix(#000000,#a5a5a5,0.5),0.9); }
      .osd spinbutton:not(.vertical) button:active,
      #XfceNotifyWindow spinbutton:not(.vertical) button:active {
        box-shadow: inset 0 2px 3px -1px rgba(0, 0, 0, 0.2); }
      .osd spinbutton:not(.vertical) button:backdrop:disabled,
      #XfceNotifyWindow spinbutton:not(.vertical) button:backdrop:disabled {
        color: rgba(127, 127, 127, 0.8);
        border-style: none none none solid; }
        .osd spinbutton:not(.vertical) button:backdrop:disabled:dir(rtl),
        #XfceNotifyWindow spinbutton:not(.vertical) button:backdrop:disabled:dir(rtl) {
          border-style: none solid none none; }
      .osd spinbutton:not(.vertical) button:dir(rtl):first-child,
      #XfceNotifyWindow spinbutton:not(.vertical) button:dir(rtl):first-child {
        border-radius: 3px 0 0 3px; }
      .osd spinbutton:not(.vertical) button:dir(ltr):last-child,
      #XfceNotifyWindow spinbutton:not(.vertical) button:dir(ltr):last-child {
        border-radius: 0 3px 3px 0; }
  .osd spinbutton.vertical button:first-child,
  #XfceNotifyWindow spinbutton.vertical button:first-child {
    background-color: rgba(15, 15, 15, 0.8);
    background-image: none;
    border-color: rgba(132, 132, 132, 0.22);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    .osd spinbutton.vertical button:first-child:focus, .osd spinbutton.vertical button:first-child:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
    .osd spinbutton.vertical button:first-child:active, .osd spinbutton.vertical button:first-child:active:hover, .osd spinbutton.vertical button:first-child:active:focus, .osd spinbutton.vertical button:first-child:active:hover:focus, .osd spinbutton.vertical button:first-child:checked, .osd spinbutton.vertical button:first-child:checked:hover, .osd spinbutton.vertical button:first-child:checked:focus, .osd spinbutton.vertical button:first-child:checked:hover:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active:hover:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.22); }
    .osd spinbutton.vertical button:first-child:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child:disabled {
      border-color: rgba(140, 140, 140, 0.22); }
    .osd spinbutton.vertical button:first-child:active:disabled, .osd spinbutton.vertical button:first-child:checked:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked:disabled {
      border-color: rgba(132, 132, 132, 0.22); }
    .osd spinbutton.vertical button:first-child.flat,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat {
      color: #a5a5a5;
      border-color: rgba(15, 15, 15, 0);
      background-color: rgba(15, 15, 15, 0);
      background-image: none;
      box-shadow: none; }
    .osd spinbutton.vertical button:first-child:hover, .osd spinbutton.vertical button:first-child.flat:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover {
      background-color: rgba(16, 16, 16, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      .osd spinbutton.vertical button:first-child:hover:focus, .osd spinbutton.vertical button:first-child:hover:hover, .osd spinbutton.vertical button:first-child.flat:hover:focus, .osd spinbutton.vertical button:first-child.flat:hover:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      .osd spinbutton.vertical button:first-child:hover:active, .osd spinbutton.vertical button:first-child:hover:active:hover, .osd spinbutton.vertical button:first-child:hover:active:focus, .osd spinbutton.vertical button:first-child:hover:active:hover:focus, .osd spinbutton.vertical button:first-child:hover:checked, .osd spinbutton.vertical button:first-child:hover:checked:hover, .osd spinbutton.vertical button:first-child:hover:checked:focus, .osd spinbutton.vertical button:first-child:hover:checked:hover:focus, .osd spinbutton.vertical button:first-child.flat:hover:active, .osd spinbutton.vertical button:first-child.flat:hover:active:hover, .osd spinbutton.vertical button:first-child.flat:hover:active:focus, .osd spinbutton.vertical button:first-child.flat:hover:active:hover:focus, .osd spinbutton.vertical button:first-child.flat:hover:checked, .osd spinbutton.vertical button:first-child.flat:hover:checked:hover, .osd spinbutton.vertical button:first-child.flat:hover:checked:focus, .osd spinbutton.vertical button:first-child.flat:hover:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      .osd spinbutton.vertical button:first-child:hover:disabled, .osd spinbutton.vertical button:first-child.flat:hover:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      .osd spinbutton.vertical button:first-child:hover:active:disabled, .osd spinbutton.vertical button:first-child:hover:checked:disabled, .osd spinbutton.vertical button:first-child.flat:hover:active:disabled, .osd spinbutton.vertical button:first-child.flat:hover:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:hover:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
    .osd spinbutton.vertical button:first-child:focus, .osd spinbutton.vertical button:first-child.flat:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus {
      background-color: rgba(16, 16, 16, 0.8);
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      color: #a5a5a5;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
      .osd spinbutton.vertical button:first-child:focus:focus, .osd spinbutton.vertical button:first-child:focus:hover, .osd spinbutton.vertical button:first-child.flat:focus:focus, .osd spinbutton.vertical button:first-child.flat:focus:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      .osd spinbutton.vertical button:first-child:focus:active, .osd spinbutton.vertical button:first-child:focus:active:hover, .osd spinbutton.vertical button:first-child:focus:active:focus, .osd spinbutton.vertical button:first-child:focus:active:hover:focus, .osd spinbutton.vertical button:first-child:focus:checked, .osd spinbutton.vertical button:first-child:focus:checked:hover, .osd spinbutton.vertical button:first-child:focus:checked:focus, .osd spinbutton.vertical button:first-child:focus:checked:hover:focus, .osd spinbutton.vertical button:first-child.flat:focus:active, .osd spinbutton.vertical button:first-child.flat:focus:active:hover, .osd spinbutton.vertical button:first-child.flat:focus:active:focus, .osd spinbutton.vertical button:first-child.flat:focus:active:hover:focus, .osd spinbutton.vertical button:first-child.flat:focus:checked, .osd spinbutton.vertical button:first-child.flat:focus:checked:hover, .osd spinbutton.vertical button:first-child.flat:focus:checked:focus, .osd spinbutton.vertical button:first-child.flat:focus:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      .osd spinbutton.vertical button:first-child:focus:disabled, .osd spinbutton.vertical button:first-child.flat:focus:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      .osd spinbutton.vertical button:first-child:focus:active:disabled, .osd spinbutton.vertical button:first-child:focus:checked:disabled, .osd spinbutton.vertical button:first-child.flat:focus:active:disabled, .osd spinbutton.vertical button:first-child.flat:focus:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
      .osd spinbutton.vertical button:first-child:focus:hover, .osd spinbutton.vertical button:first-child.flat:focus:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover {
        background-color: rgba(17, 17, 17, 0.8);
        background-image: none;
        border-color: rgba(132, 132, 132, 0.3);
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
        .osd spinbutton.vertical button:first-child:focus:hover:focus, .osd spinbutton.vertical button:first-child:focus:hover:hover, .osd spinbutton.vertical button:first-child.flat:focus:hover:focus, .osd spinbutton.vertical button:first-child.flat:focus:hover:hover,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:hover,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:hover {
          border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
        .osd spinbutton.vertical button:first-child:focus:hover:active, .osd spinbutton.vertical button:first-child:focus:hover:active:hover, .osd spinbutton.vertical button:first-child:focus:hover:active:focus, .osd spinbutton.vertical button:first-child:focus:hover:active:hover:focus, .osd spinbutton.vertical button:first-child:focus:hover:checked, .osd spinbutton.vertical button:first-child:focus:hover:checked:hover, .osd spinbutton.vertical button:first-child:focus:hover:checked:focus, .osd spinbutton.vertical button:first-child:focus:hover:checked:hover:focus, .osd spinbutton.vertical button:first-child.flat:focus:hover:active, .osd spinbutton.vertical button:first-child.flat:focus:hover:active:hover, .osd spinbutton.vertical button:first-child.flat:focus:hover:active:focus, .osd spinbutton.vertical button:first-child.flat:focus:hover:active:hover:focus, .osd spinbutton.vertical button:first-child.flat:focus:hover:checked, .osd spinbutton.vertical button:first-child.flat:focus:hover:checked:hover, .osd spinbutton.vertical button:first-child.flat:focus:hover:checked:focus, .osd spinbutton.vertical button:first-child.flat:focus:hover:checked:hover:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:active,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:active:hover,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:active:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:active:hover:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:checked,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:checked:hover,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:checked:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:checked:hover:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:active,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:active:hover,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:active:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:active:hover:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:checked,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:checked:hover,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:checked:focus,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:checked:hover:focus {
          border-color: rgba(116, 116, 116, 0.3); }
        .osd spinbutton.vertical button:first-child:focus:hover:disabled, .osd spinbutton.vertical button:first-child.flat:focus:hover:disabled,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:disabled,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:disabled {
          border-color: rgba(140, 140, 140, 0.3); }
        .osd spinbutton.vertical button:first-child:focus:hover:active:disabled, .osd spinbutton.vertical button:first-child:focus:hover:checked:disabled, .osd spinbutton.vertical button:first-child.flat:focus:hover:active:disabled, .osd spinbutton.vertical button:first-child.flat:focus:hover:checked:disabled,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:active:disabled,
        #XfceNotifyWindow spinbutton.vertical button:first-child:focus:hover:checked:disabled,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:active:disabled,
        #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus:hover:checked:disabled {
          border-color: rgba(132, 132, 132, 0.3); }
    .osd spinbutton.vertical button:first-child:checked, .osd spinbutton.vertical button:first-child:active, .osd spinbutton.vertical button:first-child.flat:checked, .osd spinbutton.vertical button:first-child.flat:active,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active {
      /*@include linear-gradient(shade($bg, .7), to top);*/
      background-color: #B36A87;
      background-image: none;
      color: #000000;
      box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
      border-color: rgba(132, 132, 132, 0.22); }
      .osd spinbutton.vertical button:first-child:checked:focus, .osd spinbutton.vertical button:first-child:checked:hover, .osd spinbutton.vertical button:first-child:active:focus, .osd spinbutton.vertical button:first-child:active:hover, .osd spinbutton.vertical button:first-child.flat:checked:focus, .osd spinbutton.vertical button:first-child.flat:checked:hover, .osd spinbutton.vertical button:first-child.flat:active:focus, .osd spinbutton.vertical button:first-child.flat:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
      .osd spinbutton.vertical button:first-child:checked:active, .osd spinbutton.vertical button:first-child:checked:active:hover, .osd spinbutton.vertical button:first-child:checked:active:focus, .osd spinbutton.vertical button:first-child:checked:active:hover:focus, .osd spinbutton.vertical button:first-child:checked:checked, .osd spinbutton.vertical button:first-child:checked:checked:hover, .osd spinbutton.vertical button:first-child:checked:checked:focus, .osd spinbutton.vertical button:first-child:checked:checked:hover:focus, .osd spinbutton.vertical button:first-child:active:active, .osd spinbutton.vertical button:first-child:active:active:hover, .osd spinbutton.vertical button:first-child:active:active:focus, .osd spinbutton.vertical button:first-child:active:active:hover:focus, .osd spinbutton.vertical button:first-child:active:checked, .osd spinbutton.vertical button:first-child:active:checked:hover, .osd spinbutton.vertical button:first-child:active:checked:focus, .osd spinbutton.vertical button:first-child:active:checked:hover:focus, .osd spinbutton.vertical button:first-child.flat:checked:active, .osd spinbutton.vertical button:first-child.flat:checked:active:hover, .osd spinbutton.vertical button:first-child.flat:checked:active:focus, .osd spinbutton.vertical button:first-child.flat:checked:active:hover:focus, .osd spinbutton.vertical button:first-child.flat:checked:checked, .osd spinbutton.vertical button:first-child.flat:checked:checked:hover, .osd spinbutton.vertical button:first-child.flat:checked:checked:focus, .osd spinbutton.vertical button:first-child.flat:checked:checked:hover:focus, .osd spinbutton.vertical button:first-child.flat:active:active, .osd spinbutton.vertical button:first-child.flat:active:active:hover, .osd spinbutton.vertical button:first-child.flat:active:active:focus, .osd spinbutton.vertical button:first-child.flat:active:active:hover:focus, .osd spinbutton.vertical button:first-child.flat:active:checked, .osd spinbutton.vertical button:first-child.flat:active:checked:hover, .osd spinbutton.vertical button:first-child.flat:active:checked:focus, .osd spinbutton.vertical button:first-child.flat:active:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:checked:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:active,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:active:hover:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:checked,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.22); }
      .osd spinbutton.vertical button:first-child:checked:disabled, .osd spinbutton.vertical button:first-child:active:disabled, .osd spinbutton.vertical button:first-child.flat:checked:disabled, .osd spinbutton.vertical button:first-child.flat:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:disabled {
        border-color: rgba(140, 140, 140, 0.22); }
      .osd spinbutton.vertical button:first-child:checked:active:disabled, .osd spinbutton.vertical button:first-child:checked:checked:disabled, .osd spinbutton.vertical button:first-child:active:active:disabled, .osd spinbutton.vertical button:first-child:active:checked:disabled, .osd spinbutton.vertical button:first-child.flat:checked:active:disabled, .osd spinbutton.vertical button:first-child.flat:checked:checked:disabled, .osd spinbutton.vertical button:first-child.flat:active:active:disabled, .osd spinbutton.vertical button:first-child.flat:active:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:checked:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:active:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:checked:disabled {
        border-color: rgba(132, 132, 132, 0.22); }
      .osd spinbutton.vertical button:first-child:checked:focus, .osd spinbutton.vertical button:first-child:checked:hover, .osd spinbutton.vertical button:first-child:active:focus, .osd spinbutton.vertical button:first-child:active:hover, .osd spinbutton.vertical button:first-child.flat:checked:focus, .osd spinbutton.vertical button:first-child.flat:checked:hover, .osd spinbutton.vertical button:first-child.flat:active:focus, .osd spinbutton.vertical button:first-child.flat:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child:active:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:hover,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:focus,
      #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:hover {
        /*@include linear-gradient(shade($bg, .65), to top);*/
        background-color: #b8738f;
        background-image: none;
        color: #000000; }
    .osd spinbutton.vertical button:first-child:focus, .osd spinbutton.vertical button:first-child:hover, .osd spinbutton.vertical button:first-child.flat:focus, .osd spinbutton.vertical button:first-child.flat:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child:hover,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:focus,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:hover {
      color: #a5a5a5; }
    .osd spinbutton.vertical button:first-child:disabled:disabled, .osd spinbutton.vertical button:first-child.flat:disabled:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child:disabled:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:disabled:disabled {
      background-color: alpha(mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.2),0.4);
      background-image: none;
      /*border: 1px solid alpha($bg, .2);*/
      opacity: .4;
      color: mix(rgba(15, 15, 15, 0.8),#a5a5a5,0.5);
      box-shadow: none; }
    .osd spinbutton.vertical button:first-child:active:disabled, .osd spinbutton.vertical button:first-child:checked:disabled, .osd spinbutton.vertical button:first-child.flat:active:disabled, .osd spinbutton.vertical button:first-child.flat:checked:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child:active:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child:checked:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:active:disabled,
    #XfceNotifyWindow spinbutton.vertical button:first-child.flat:checked:disabled {
      background-color: rgba(179, 106, 135, 0.6);
      background-image: none;
      color: #000000;
      box-shadow: none; }
    .osd spinbutton.vertical button:first-child.separator, .osd spinbutton.vertical button:first-child .separator,
    #XfceNotifyWindow spinbutton.vertical button:first-child.separator,
    #XfceNotifyWindow spinbutton.vertical button:first-child .separator {
      border: 1px solid currentColor;
      color: rgba(15, 15, 15, 0.7); }
      .osd spinbutton.vertical button:first-child.separator:disabled, .osd spinbutton.vertical button:first-child .separator:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child.separator:disabled,
      #XfceNotifyWindow spinbutton.vertical button:first-child .separator:disabled {
        color: rgba(15, 15, 15, 0.65); }

scrolledwindow viewport.frame {
  border-style: none; }

scrolledwindow overshoot.top {
  background-image: -gtk-gradient(radial, center top, 0, center top, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25))), -gtk-gradient(radial, center top, 0, center top, 0.6, from(rgba(179, 106, 135, 0.2)), to(rgba(179, 106, 135, 0)));
  background-size: 100% 5%, 100% 100%;
  background-repeat: no-repeat;
  background-position: center top;
  background-color: transparent;
  border: 0;
  box-shadow: none; }
  scrolledwindow overshoot.top:backdrop {
    background-image: -gtk-gradient(radial, center top, 0, center top, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25)));
    background-size: 100% 5%;
    background-repeat: no-repeat;
    background-position: center top;
    background-color: transparent;
    border: 0;
    box-shadow: none; }

scrolledwindow overshoot.bottom {
  background-image: -gtk-gradient(radial, center bottom, 0, center bottom, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25))), -gtk-gradient(radial, center bottom, 0, center bottom, 0.6, from(rgba(179, 106, 135, 0.2)), to(rgba(179, 106, 135, 0)));
  background-size: 100% 5%, 100% 100%;
  background-repeat: no-repeat;
  background-position: center bottom;
  background-color: transparent;
  border: 0;
  box-shadow: none; }
  scrolledwindow overshoot.bottom:backdrop {
    background-image: -gtk-gradient(radial, center bottom, 0, center bottom, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25)));
    background-size: 100% 5%;
    background-repeat: no-repeat;
    background-position: center bottom;
    background-color: transparent;
    border: 0;
    box-shadow: none; }

scrolledwindow overshoot.left {
  background-image: -gtk-gradient(radial, left center, 0, left center, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25))), -gtk-gradient(radial, left center, 0, left center, 0.6, from(rgba(179, 106, 135, 0.2)), to(rgba(179, 106, 135, 0)));
  background-size: 5% 100%, 100% 100%;
  background-repeat: no-repeat;
  background-position: left center;
  background-color: transparent;
  border: 0;
  box-shadow: none; }
  scrolledwindow overshoot.left:backdrop {
    background-image: -gtk-gradient(radial, left center, 0, left center, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25)));
    background-size: 5% 100%;
    background-repeat: no-repeat;
    background-position: left center;
    background-color: transparent;
    border: 0;
    box-shadow: none; }

scrolledwindow overshoot.right {
  background-image: -gtk-gradient(radial, right center, 0, right center, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25))), -gtk-gradient(radial, right center, 0, right center, 0.6, from(rgba(179, 106, 135, 0.2)), to(rgba(179, 106, 135, 0)));
  background-size: 5% 100%, 100% 100%;
  background-repeat: no-repeat;
  background-position: right center;
  background-color: transparent;
  border: 0;
  box-shadow: none; }
  scrolledwindow overshoot.right:backdrop {
    background-image: -gtk-gradient(radial, right center, 0, right center, 0.5, to(rgba(179, 106, 135, 0.35)), to(rgba(179, 106, 135, 0.25)));
    background-size: 5% 100%;
    background-repeat: no-repeat;
    background-position: right center;
    background-color: transparent;
    border: 0;
    box-shadow: none; }

scrolledwindow undershoot.top {
  background-color: transparent;
  /*background-image: linear-gradient(to $_gradient_dir, // this is the dashed line
                                         $_undershoot_color_light 50%,
                                         $_undershoot_color_dark 50%);*/
  padding-top: 1px;
  background-size: 10px 1px;
  background-repeat: repeat-x;
  background-origin: content-box;
  background-position: center top;
  border: 0;
  box-shadow: none; }

scrolledwindow undershoot.bottom {
  background-color: transparent;
  /*background-image: linear-gradient(to $_gradient_dir, // this is the dashed line
                                         $_undershoot_color_light 50%,
                                         $_undershoot_color_dark 50%);*/
  padding-bottom: 1px;
  background-size: 10px 1px;
  background-repeat: repeat-x;
  background-origin: content-box;
  background-position: center bottom;
  border: 0;
  box-shadow: none; }

scrolledwindow undershoot.left {
  background-color: transparent;
  /*background-image: linear-gradient(to $_gradient_dir, // this is the dashed line
                                         $_undershoot_color_light 50%,
                                         $_undershoot_color_dark 50%);*/
  padding-left: 1px;
  background-size: 1px 10px;
  background-repeat: repeat-y;
  background-origin: content-box;
  background-position: left center;
  border: 0;
  box-shadow: none; }

scrolledwindow undershoot.right {
  background-color: transparent;
  /*background-image: linear-gradient(to $_gradient_dir, // this is the dashed line
                                         $_undershoot_color_light 50%,
                                         $_undershoot_color_dark 50%);*/
  padding-right: 1px;
  background-size: 1px 10px;
  background-repeat: repeat-y;
  background-origin: content-box;
  background-position: right center;
  border: 0;
  box-shadow: none; }

scrolledwindow junction {
  border-color: transparent;
  border-image: linear-gradient(to bottom, mix(#a5a5a5,#000000,0.08) 1px, transparent 1px) 0 0 0 1/0 1px stretch;
  background-color: #989898; }
  scrolledwindow junction:dir(rtl) {
    border-image-slice: 0 1 0 0; }
  scrolledwindow junction:backdrop {
    border-image-source: linear-gradient(to bottom, mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9) 1px, transparent 1px);
    background-color: #9d9d9d;
    transition: 200ms ease-out; }

/*****************
 ! Progress bars *
******************/
progressbar {
  padding: 0;
  border-radius: 3px;
  font-size: smaller;
  color: rgba(0, 0, 0, 0.6); }
  progressbar.horizontal trough,
  progressbar.horizontal progress {
    min-height: 6px; }
  progressbar.vertical trough,
  progressbar.vertical progress {
    min-width: 6px; }
  progressbar trough {
    border: 1px solid mix(#a5a5a5,#000000,0.17);
    background-color: #b2b2b2;
    background-image: none;
    border-radius: 3px; }
  progressbar progress {
    background-color: #B36A87;
    background-image: none;
    border-radius: 0; }
    progressbar progress.left {
      border-top-left-radius: 3px;
      border-bottom-left-radius: 3px; }
    progressbar progress.right {
      border-top-right-radius: 3px;
      border-bottom-right-radius: 3px; }
    progressbar progress.bottom {
      border-bottom-left-radius: 3px;
      border-bottom-right-radius: 3px; }
    progressbar progress.top {
      border-top-left-radius: 3px;
      border-top-right-radius: 3px; }

levelbar.horizontal block {
  min-width: 34px;
  min-height: 4px; }

levelbar.vertical block {
  min-width: 4px;
  min-height: 34px; }

levelbar:backdrop {
  transition: 200ms ease-out; }

levelbar trough {
  background-color: #b2b2b2;
  background-image: none;
  border: 1px solid mix(#a5a5a5,#000000,0.17);
  border-radius: 3px;
  padding: 2px; }

levelbar.horizontal.discrete block {
  margin: 0 1px; }
  levelbar.horizontal.discrete block:first-child {
    margin: 0; }

levelbar.vertical.discrete block {
  margin: 1px 0; }
  levelbar.vertical.discrete block:first-child {
    margin: 0; }

levelbar block {
  background-color: #B36A87;
  background-image: none;
  border-color: transparent;
  border-radius: 3px; }
  levelbar block.low {
    background-color: #ef6c00;
    border-color: transparent; }
  levelbar block.high, levelbar block:not(.empty) {
    background-color: #4caf50;
    border-color: transparent; }
  levelbar block.full {
    background-color: #974d6a;
    border-color: transparent; }
  levelbar block.empty {
    background-color: transparent;
    border-color: transparent;
    box-shadow: none; }

scale {
  min-height: 10px;
  min-width: 10px;
  padding: 3px; }
  scale.horizontal trough {
    padding: 0 3px; }
  scale.horizontal highlight, scale.horizontal fill {
    margin: 0 -4px; }
  scale.vertical trough {
    padding: 3px 0; }
  scale.vertical highlight, scale.vertical fill {
    margin: -4px 0; }
  scale slider {
    min-height: 15px;
    min-width: 15px;
    margin: -7px; }
  scale.fine-tune slider {
    margin: -7px; }
  scale.fine-tune highlight {
    background-color: #bd7d96; }
  scale.fine-tune fill,
  scale.fine-tune highlight,
  scale.fine-tune trough {
    border-radius: 5px;
    -gtk-outline-radius: 7px; }
  scale trough {
    outline-offset: 2px;
    -gtk-outline-radius: 4.5px;
    border-radius: 2.5px;
    background-color: mix(#a5a5a5,#000000,0.2); }
    scale trough:disabled {
      background-color: mix(#a5a5a5,#000000,0.1); }
    menuitem:hover scale trough,
    row:selected scale trough,
    infobar scale trough {
      background-color: rgba(0, 0, 0, 0.2); }
      menuitem:hover scale trough highlight,
      row:selected scale trough highlight,
      infobar scale trough highlight {
        background-color: #000000; }
        menuitem:hover scale trough highlight:disabled,
        row:selected scale trough highlight:disabled,
        infobar scale trough highlight:disabled {
          background-color: mix(#000000,#B36A87,0.55); }
      menuitem:hover scale trough:disabled,
      row:selected scale trough:disabled,
      infobar scale trough:disabled {
        background-color: rgba(0, 0, 0, 0.1); }
  scale highlight {
    border-radius: 2.5px;
    background-color: #B36A87; }
    scale highlight:disabled {
      background-color: rgba(179, 106, 135, 0.55); }
  scale fill {
    border-radius: 2.5px;
    background-color: rgba(179, 106, 135, 0.5); }
    scale fill:disabled {
      background-color: transparent; }
  scale slider {
    background-color: #8b639b;
    border: 1px solid rgba(0, 0, 0, 0.16);
    border-radius: 100%;
    transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
    transition-property: background, border; }
    scale slider:hover {
      background-color: #9772a5; }
    scale slider:active {
      background-clip: border-box;
      background-color: #B36A87;
      border-color: #B36A87; }
    scale slider:disabled {
      background-color: mix(#8b639b,#a5a5a5,0.55);
      border-color: transparent; }
    menuitem:hover scale slider,
    row:selected scale slider,
    infobar scale slider {
      background-clip: border-box;
      background-color: #000000;
      border-color: #000000; }
      menuitem:hover scale slider:hover,
      row:selected scale slider:hover,
      infobar scale slider:hover {
        background-color: mix(#000000,#B36A87,0.85);
        border-color: mix(#000000,#B36A87,0.85); }
      menuitem:hover scale slider:active,
      row:selected scale slider:active,
      infobar scale slider:active {
        background-color: mix(#000000,#B36A87,0.5);
        border-color: mix(#000000,#B36A87,0.5); }
      menuitem:hover scale slider:disabled,
      row:selected scale slider:disabled,
      infobar scale slider:disabled {
        background-color: mix(#000000,#B36A87,0.55);
        border-color: mix(#000000,#B36A87,0.55); }
  scale value {
    color: alpha(currentColor,0.4); }
  scale marks {
    color: alpha(currentColor,0.4); }
    scale marks.top {
      margin-bottom: 1px;
      margin-top: -4px; }
    scale marks.bottom {
      margin-top: 1px;
      margin-bottom: -4px; }
    scale marks.top {
      margin-right: 1px;
      margin-left: -4px; }
    scale marks.bottom {
      margin-left: 1px;
      margin-right: -4px; }
  scale.fine-tune marks.top {
    margin-bottom: 0px;
    margin-top: -2px; }
  scale.fine-tune marks.bottom {
    margin-top: 0px;
    margin-bottom: -2px; }
  scale.fine-tune marks.top {
    margin-right: 0px;
    margin-left: -2px; }
  scale.fine-tune marks.bottom {
    margin-left: 0px;
    margin-right: -2px; }
  scale.horizontal indicator {
    min-height: 3px;
    min-width: 1px; }
  scale.horizontal.fine-tune indicator {
    min-height: 2px; }
  scale.vertical indicator {
    min-height: 1px;
    min-width: 3px; }
  scale.vertical.fine-tune indicator {
    min-width: 2px; }
  scale.color trough {
    padding: 0;
    border: 0;
    background-image: none; }
  scale.color highlight, scale.color fill {
    margin: 0; }
  scale.color.horizontal {
    padding: 0 0 6px 0; }
    scale.color.horizontal trough {
      border-top-left-radius: 0;
      border-top-right-radius: 0; }
    scale.color.horizontal slider:hover, scale.color.horizontal slider:backdrop, scale.color.horizontal slider:disabled, scale.color.horizontal slider:backdrop:disabled, scale.color.horizontal slider {
      margin-bottom: 0;
      margin-top: 0; }
  scale.color.vertical:dir(ltr) {
    padding: 0 0 0 6px; }
    scale.color.vertical:dir(ltr) trough {
      border-bottom-right-radius: 0;
      border-top-right-radius: 0; }
    scale.color.vertical:dir(ltr) slider:hover, scale.color.vertical:dir(ltr) slider:backdrop, scale.color.vertical:dir(ltr) slider:disabled, scale.color.vertical:dir(ltr) slider:backdrop:disabled, scale.color.vertical:dir(ltr) slider {
      margin-left: 0;
      margin-right: 0; }
  scale.color.vertical:dir(rtl) {
    padding: 0 6px 0 0; }
    scale.color.vertical:dir(rtl) trough {
      border-bottom-left-radius: 0;
      border-top-left-radius: 0; }
    scale.color.vertical:dir(rtl) slider:hover, scale.color.vertical:dir(rtl) slider:backdrop, scale.color.vertical:dir(rtl) slider:disabled, scale.color.vertical:dir(rtl) slider:backdrop:disabled, scale.color.vertical:dir(rtl) slider {
      margin-right: 0;
      margin-left: 0; }

/***********
 ! Scrollbar
************/
scrollbar {
  background-color: #989898;
  transition: 300ms ease-out; }
  * {
    -GtkScrollbar-has-backward-stepper: false;
    -GtkScrollbar-has-forward-stepper: false; }
  scrollbar.top {
    border-bottom: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar.bottom {
    border-top: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar.left {
    border-right: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar.right {
    border-left: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar:backdrop {
    background-color: #9d9d9d;
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9);
    transition: 400ms ease-in; }
  scrollbar slider {
    min-width: 7px;
    min-height: 7px;
    border: 1px solid transparent;
    border-radius: 3px;
    background-clip: padding-box;
    background-color: mix(#a5a5a5,#000000,0.5); }
    scrollbar slider:hover {
      background-color: mix(#a5a5a5,#000000,0.7); }
    scrollbar slider:hover:active {
      background-color: #aa5979; }
    scrollbar slider:backdrop {
      background-color: mix(mix(#000000,#a5a5a5,0.5),#a5a5a5,0.4); }
    scrollbar slider:disabled {
      background-color: transparent; }
  scrollbar.horizontal slider {
    min-width: 40px; }
  scrollbar.vertical slider {
    min-height: 40px; }
  scrollbar.fine-tune slider:active {
    background-color: #bc7b95; }
  scrollbar.overlay-indicator {
    opacity: .8; }
    scrollbar.overlay-indicator:not(.dragging):not(.hovering) {
      border-color: transparent;
      opacity: .4;
      background-color: transparent; }
      scrollbar.overlay-indicator:not(.dragging):not(.hovering) slider {
        min-width: 4px;
        min-height: 4px;
        background-color: #000000;
        border: 1px solid #fff; }
      scrollbar.overlay-indicator:not(.dragging):not(.hovering) button {
        min-width: 4px;
        min-height: 4px;
        border-color: transparent;
        -gtk-icon-source: none; }
      scrollbar.overlay-indicator:not(.dragging):not(.hovering).horizontal slider {
        min-width: 40px; }
      scrollbar.overlay-indicator:not(.dragging):not(.hovering).horizontal button {
        min-width: 7px; }
      scrollbar.overlay-indicator:not(.dragging):not(.hovering).vertical slider {
        min-height: 40px; }
      scrollbar.overlay-indicator:not(.dragging):not(.hovering).vertical button {
        min-height: 7px; }
  scrollbar button {
    min-width: 7px;
    min-height: 7px;
    padding: 0;
    border: 0;
    border-radius: 0;
    border-color: mix(#a5a5a5,#000000,0.08);
    background-color: transparent;
    box-shadow: none;
    color: mix(#a5a5a5,#000000,0.5); }
    scrollbar button:hover {
      color: mix(#a5a5a5,#000000,0.7); }
    scrollbar button:active, scrollbar button:checked {
      color: #aa5979; }
    scrollbar button:backdrop {
      color: mix(mix(#000000,#a5a5a5,0.5),#a5a5a5,0.4); }
  scrollbar.vertical button.down {
    -gtk-icon-source: -gtk-icontheme("pan-down-symbolic");
    border-top: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar.vertical button.up {
    -gtk-icon-source: -gtk-icontheme("pan-up-symbolic");
    border-bottom: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar.horizontal button.down {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
    border-left: 1px solid mix(#a5a5a5,#000000,0.08); }
  scrollbar.horizontal button.up {
    -gtk-icon-source: -gtk-icontheme("pan-start-symbolic");
    border-right: 1px solid mix(#a5a5a5,#000000,0.08); }

/*********
 ! Sidebar
**********/
.sidebar {
  border-style: none;
  background-color: mix(#a5a5a5,#d0d0d0,0.5); }
  stacksidebar.sidebar:dir(ltr) list, stacksidebar.sidebar.left list, stacksidebar.sidebar.left:dir(rtl) list, .sidebar:dir(ltr), .sidebar.left, .sidebar.left:dir(rtl) {
    border-right: 1px solid mix(#a5a5a5,#000000,0.08);
    border-left-style: none; }
  stacksidebar.sidebar:dir(rtl) list .sidebar:dir(rtl), stacksidebar.sidebar.right list .sidebar:dir(rtl), .sidebar.right {
    border-left: 1px solid mix(#a5a5a5,#000000,0.08);
    border-right-style: none; }
  .sidebar:backdrop {
    background-color: mix(#a5a5a5,#d0d0d0,0.5);
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9);
    transition: 200ms ease-out; }
  .sidebar .frame, .sidebar frame {
    border-width: 0; }
  .sidebar list {
    background-color: transparent; }
  paned .sidebar.left, paned .sidebar.right, paned .sidebar.left:dir(rtl), paned .sidebar:dir(rtl), paned .sidebar:dir(ltr), paned .sidebar {
    border-style: none; }

stacksidebar row {
  padding: 6px 3px; }
  stacksidebar row > label {
    padding-left: 3px;
    padding-right: 3px; }
  stacksidebar row.needs-attention > label {
    background-size: 6px 6px, 0 0; }

placessidebar > viewport.frame {
  border-style: none; }

placessidebar row {
  min-height: 32px;
  padding: 0; }
  placessidebar row > revealer {
    padding: 0 6px; }
  placessidebar row:selected {
    color: #000000; }
  placessidebar row:disabled {
    color: mix(#000000,#a5a5a5,0.5); }
  placessidebar row:backdrop {
    color: mix(#000000,#a5a5a5,0.5); }
    placessidebar row:backdrop:selected {
      color: mix(#B36A87,#000000,0.66); }
    placessidebar row:backdrop:disabled {
      color: #7f7f7f; }
  placessidebar row image.sidebar-icon {
    opacity: 0.7; }
    placessidebar row image.sidebar-icon:dir(ltr) {
      padding-right: 8px;
      padding-left: 3px; }
    placessidebar row image.sidebar-icon:dir(rtl) {
      padding-left: 8px;
      padding-right: 3px; }
  placessidebar row label.sidebar-label:dir(ltr) {
    padding-right: 2px; }
  placessidebar row label.sidebar-label:dir(rtl) {
    padding-left: 2px; }
  button.sidebar-button {
    min-height: 20px;
    min-width: 20px;
    margin-top: 2px;
    margin-bottom: 2px;
    padding: 0;
    border-radius: 100%;
    -gtk-outline-radius: 100%; }
    button.sidebar-button:not(:hover):not(:active) > image, button.sidebar-button:backdrop > image {
      opacity: 0.7; }
  placessidebar row:selected:active {
    box-shadow: none; }
  placessidebar row.sidebar-placeholder-row {
    padding: 0 8px;
    min-height: 2px;
    background-image: image(#4e9a06);
    background-clip: content-box; }
  placessidebar row.sidebar-new-bookmark-row {
    color: #B36A87; }
  placessidebar row:drop(active):not(:disabled) {
    color: #4e9a06;
    box-shadow: inset 0 1px #4e9a06, inset 0 -1px #4e9a06; }
    placessidebar row:drop(active):not(:disabled):selected {
      color: #000000;
      background-color: #4e9a06; }

/******
! Paned
*******/
paned > separator {
  min-width: 1px;
  min-height: 1px;
  -gtk-icon-source: none;
  border-style: none;
  background-color: transparent;
  background-image: image(#959595);
  background-size: 1px 1px;
  background-position: center center; }
  paned > separator:selected {
    background-image: image(#B36A87); }
  paned > separator:backdrop {
    background-image: image(mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9)); }
  paned > separator.wide {
    min-width: 5px;
    min-height: 5px;
    background-color: #a5a5a5;
    background-image: image(#848484), image(#848484);
    background-size: 1px 1px, 1px 1px; }
    paned > separator.wide:backdrop {
      background-color: #a5a5a5;
      background-image: image(mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9)), image(mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9)); }

paned.horizontal > separator {
  background-repeat: repeat-y;
  padding: 0 2px;
  margin: 0 -2px; }
  paned.horizontal > separator.wide {
    margin: 0;
    padding: 0;
    background-repeat: repeat-y, repeat-y;
    background-position: left, right; }

paned.vertical > separator {
  background-repeat: repeat-x;
  padding: 2px 0;
  margin: -2px 0; }
  paned.vertical > separator.wide {
    margin: 0;
    padding: 0;
    background-repeat: repeat-x, repeat-x;
    background-position: bottom, top; }

paned.titlebar > separator {
  background-image: image(#0e0e0e); }

/*******************
 ! Spinner animation
********************/
@keyframes spin {
  to {
    -gtk-icon-transform: rotate(1turn); } }

spinner {
  background-image: none;
  color: #B36A87;
  opacity: 0;
  -gtk-icon-source: -gtk-icontheme("process-working-symbolic"); }
  spinner:checked {
    opacity: 1;
    animation: spin 1s linear infinite; }
    spinner:checked:disabled {
      opacity: .5; }

/***********************
 ! Check and Radio items
************************/
radio {
  background-image: none;
  -gtk-icon-source: url("../assets/radio-unchecked.svg");
  min-width: 16px;
  min-height: 16px;
  margin-right: 3px; }
  radio:disabled {
    -gtk-icon-source: url("../assets/radio-unchecked-disabled.svg"); }
  radio:checked, radio:active {
    -gtk-icon-source: url("../assets/radio-checked.svg"); }
    radio:checked:disabled, radio:active:disabled {
      -gtk-icon-source: url("../assets/radio-checked-disabled.svg"); }
  radio:indeterminate {
    -gtk-icon-source: url("../assets/radio-mixed.svg"); }
    radio:indeterminate:disabled {
      -gtk-icon-source: url("../assets/radio-mixed-disabled.svg"); }
  menuitem radio, modelbutton radio {
    -gtk-icon-source: url("../assets/menuitem-radio-unchecked.svg"); }
    menuitem radio:disabled, modelbutton radio:disabled {
      -gtk-icon-source: url("../assets/menuitem-radio-checked-disabled.svg"); }
    menuitem radio:checked, menuitem radio:active, modelbutton radio:checked, modelbutton radio:active {
      -gtk-icon-source: url("../assets/menuitem-radio-checked.svg"); }
      menuitem radio:checked:hover, menuitem radio:active:hover, modelbutton radio:checked:hover, modelbutton radio:active:hover {
        -gtk-icon-source: url("../assets/menuitem-radio-checked-hover.svg"); }
      menuitem radio:checked:disabled, menuitem radio:active:disabled, modelbutton radio:checked:disabled, modelbutton radio:active:disabled {
        -gtk-icon-source: url("../assets/menuitem-radio-checked-disabled.svg"); }
    menuitem radio:indeterminate, modelbutton radio:indeterminate {
      -gtk-icon-source: url("../assets/menuitem-radio-mixed.svg"); }
      menuitem radio:indeterminate:hover, modelbutton radio:indeterminate:hover {
        -gtk-icon-source: url("../assets/menuitem-radio-mixed-hover.svg"); }
      menuitem radio:indeterminate:disabled, modelbutton radio:indeterminate:disabled {
        -gtk-icon-source: url("../assets/menuitem-radio-mixed-disabled.svg"); }

check {
  background-image: none;
  -gtk-icon-source: url("../assets/checkbox-unchecked.svg");
  min-width: 16px;
  min-height: 16px;
  margin-right: 3px; }
  check:disabled {
    -gtk-icon-source: url("../assets/checkbox-unchecked-disabled.svg"); }
  check:checked, check:active {
    -gtk-icon-source: url("../assets/checkbox-checked.svg"); }
    check:checked:disabled, check:active:disabled {
      -gtk-icon-source: url("../assets/checkbox-checked-disabled.svg"); }
  check:indeterminate {
    -gtk-icon-source: url("../assets/checkbox-mixed.svg"); }
    check:indeterminate:disabled {
      -gtk-icon-source: url("../assets/checkbox-mixed-disabled.svg"); }
  menuitem check, modelbutton check {
    -gtk-icon-source: url("../assets/menuitem-checkbox-unchecked.svg"); }
    menuitem check:disabled, modelbutton check:disabled {
      -gtk-icon-source: url("../assets/menuitem-checkbox-checked-disabled.svg"); }
    menuitem check:checked, menuitem check:active, modelbutton check:checked, modelbutton check:active {
      -gtk-icon-source: url("../assets/menuitem-checkbox-checked.svg"); }
      menuitem check:checked:hover, menuitem check:active:hover, modelbutton check:checked:hover, modelbutton check:active:hover {
        -gtk-icon-source: url("../assets/menuitem-checkbox-checked-hover.svg"); }
      menuitem check:checked:disabled, menuitem check:active:disabled, modelbutton check:checked:disabled, modelbutton check:active:disabled {
        -gtk-icon-source: url("../assets/menuitem-checkbox-checked-disabled.svg"); }
    menuitem check:indeterminate, modelbutton check:indeterminate {
      -gtk-icon-source: url("../assets/menuitem-checkbox-mixed.svg"); }
      menuitem check:indeterminate:hover, modelbutton check:indeterminate:hover {
        -gtk-icon-source: url("../assets/menuitem-checkbox-mixed-hover.svg"); }
      menuitem check:indeterminate:disabled, modelbutton check:indeterminate:disabled {
        -gtk-icon-source: url("../assets/menuitem-checkbox-mixed-disabled.svg"); }

radio:dir(rtl), check:dir(rtl) {
  margin-right: 0;
  margin-left: 3px; }

.view.content-view.check:not(list), iconview.content-view.check:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-unchecked.svg");
  background-color: transparent; }

.view.content-view.check:hover:not(list), iconview.content-view.check:hover:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-unchecked.svg");
  background-color: transparent; }

.view.content-view.check:active:not(list), iconview.content-view.check:active:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-checked.svg");
  background-color: transparent; }

.view.content-view.check:backdrop:not(list), iconview.content-view.check:backdrop:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-unchecked.svg");
  background-color: transparent; }

.view.content-view.check:checked:not(list), iconview.content-view.check:checked:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-checked.svg");
  background-color: transparent; }

.view.content-view.check:checked:hover:not(list), iconview.content-view.check:checked:hover:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-checked.svg");
  background-color: transparent; }

.view.content-view.check:checked:active:not(list), iconview.content-view.check:checked:active:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-checked.svg");
  background-color: transparent; }

.view.content-view.check:backdrop:checked:not(list), iconview.content-view.check:backdrop:checked:not(list) {
  -gtk-icon-shadow: none;
  -gtk-icon-source: url("../assets/grid-selection-checked.svg");
  background-color: transparent; }

/********
 ! Switch
*********/
switch {
  border-radius: 3px;
  padding: 2px;
  border: none;
  outline: none;
  transition: background-color .3s linear;
  min-width: 88px;
  min-height: 24px;
  background-color: mix(#a5a5a5,#d0d0d0,0.3);
  color: #000000;
  box-shadow: inset 1px -1px 0 rgba(0, 0, 0, 0.06), inset -1px 1px 0 rgba(0, 0, 0, 0.06); }
  switch slider {
    background-color: mix(#000000,#a5a5a5,0.5);
    transition: all 0.3s ease-in;
    box-shadow: 0 1px 2px 0 rgba(0, 0, 0, 0.07), 1px 0 2px 0 rgba(0, 0, 0, 0.07);
    border-radius: 3px; }
  switch:checked {
    background-color: #B36A87;
    background-image: none;
    border-color: #B36A87;
    color: #d0d0d0; }
    switch:checked slider {
      background-color: #fff;
      box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.1); }
  switch:disabled {
    background-color: mix(#a5a5a5,#d0d0d0,0.5);
    background-image: none;
    border-color: #a5a5a5;
    color: #a5a5a5;
    box-shadow: none; }
    switch:disabled slider {
      background-color: #a5a5a5; }
  list row:selected switch {
    background-color: #a5a5a5;
    color: mix(#a5a5a5,#d0d0d0,0.5); }
    list row:selected switch slider {
      background-color: mix(mix(#a5a5a5,#d0d0d0,0.5),#d0d0d0,0.4); }
    list row:selected switch:checked {
      color: #B36A87;
      background-color: mix(#000000,#a5a5a5,0.5); }
      list row:selected switch:checked slider {
        background-color: #B36A87; }

/*********
 ! Buttons
**********/
/******************
! ComboBoxes *
*******************/
/*********
 ! Toolbar
**********/
/***************
 ! Generic views
****************/
.view, iconview,
.view text,
iconview text,
textview text {
  color: #000000;
  background-color: #d0d0d0;
  caret-color: #000000;
  -gtk-secondary-caret-color: #000000; }
  .view:backdrop, iconview:backdrop,
  .view text:backdrop,
  iconview text:backdrop,
  textview text:backdrop {
    color: mix(#d0d0d0,#000000,0.8);
    background-color: #d0d0d0; }
  .view:selected:focus, iconview:selected:focus, .view:selected, iconview:selected,
  .view text:selected:focus,
  iconview text:selected:focus,
  textview text:selected:focus,
  .view text:selected,
  iconview text:selected,
  textview text:selected {
    border-radius: 3px; }

textview border {
  background-color: mix(#a5a5a5,#d0d0d0,0.5); }

/************
! Treeview
*************/
.rubberband,
rubberband,
flowbox rubberband,
treeview.view rubberband {
  border: 1px solid #b36a87;
  background-color: rgba(179, 106, 135, 0.2); }

treeview entry:focus:dir(rtl), treeview entry:focus:dir(ltr) {
  background-color: #d0d0d0;
  transition-property: color, background; }

treeview entry.flat, treeview entry {
  border-radius: 0;
  background-image: none;
  background-color: #d0d0d0; }
  treeview entry.flat:focus, treeview entry:focus {
    border-color: #B36A87; }

treeview.view header button, treeview.view header button:hover, treeview.view header button:active {
  padding: 1px 4px;
  border-radius: 0;
  background-image: none;
  text-shadow: none;
  border-style: none solid solid none;
  border-color: #a5a5a5; }
  treeview.view header button:disabled {
    border-color: #a5a5a5;
    background-image: none; }
  treeview.view header button:backdrop {
    border-color: #a5a5a5;
    border-style: none solid solid none;
    color: mix(mix(#000000,#a5a5a5,0.5),#a5a5a5,0.5);
    background-image: none;
    background-color: #d0d0d0; }
    treeview.view header button:backdrop:disabled {
      border-color: #a5a5a5;
      background-image: none; }

treeview.view {
  -GtkTreeView-grid-line-width: 1;
  -GtkTreeView-grid-line-pattern: '';
  -GtkTreeView-tree-line-width: 1;
  -GtkTreeView-tree-line-pattern: '';
  border-left-color: mix(#000000,#d0d0d0,0.5);
  border-top-color: #a5a5a5; }
  treeview.view:selected:focus, treeview.view:selected {
    border-radius: 0; }
  treeview.view:selected:backdrop, treeview.view:selected {
    border-left-color: mix(#000000,#B36A87,0.5);
    border-top-color: rgba(0, 0, 0, 0.1); }
  treeview.view:disabled {
    color: mix(#000000,#a5a5a5,0.5); }
    treeview.view:disabled:selected {
      color: mix(#000000,#B36A87,0.4); }
      treeview.view:disabled:selected:backdrop {
        color: mix(mix(#B36A87,#000000,0.66),#B36A87,0.3); }
    treeview.view:disabled:backdrop {
      color: #7f7f7f; }
  treeview.view.separator {
    min-height: 2px;
    color: #a5a5a5; }
    treeview.view.separator:backdrop {
      color: rgba(165, 165, 165, 0.1); }
  treeview.view:backdrop {
    border-left-color: mix(mix(#000000,#a5a5a5,0.5),#a5a5a5,0.5);
    border-top: #a5a5a5; }
  treeview.view:drop(active) {
    border-style: solid none;
    border-width: 1px;
    border-color: mix(#B36A87,#000000,0.3); }
    treeview.view:drop(active).after {
      border-top-style: none; }
    treeview.view:drop(active).before {
      border-bottom-style: none; }
  treeview.view.expander {
    -gtk-icon-source: -gtk-icontheme("pan-end-symbolic");
    color: mix(#d0d0d0,#000000,0.7); }
    treeview.view.expander:dir(rtl) {
      -gtk-icon-source: -gtk-icontheme("pan-end-symbolic-rtl"); }
    treeview.view.expander:hover {
      color: #000000; }
    treeview.view.expander:selected {
      color: mix(#B36A87,#000000,0.7); }
      treeview.view.expander:selected:hover {
        color: #000000; }
      treeview.view.expander:selected:backdrop {
        color: mix(#B36A87,mix(#B36A87,#000000,0.66),0.7); }
    treeview.view.expander:checked {
      -gtk-icon-source: -gtk-icontheme("pan-down-symbolic"); }
    treeview.view.expander:backdrop {
      color: mix(#d0d0d0,mix(#000000,#a5a5a5,0.5),0.7); }
  treeview.view.progressbar {
    color: #d0d0d0;
    border-radius: 3px;
    border: 1px solid mix(#B36A87,#000000,0.3);
    background-color: #B36A87; }
    treeview.view.progressbar:selected {
      border: 1px solid mix(#B36A87,#000000,0.2); }
      treeview.view.progressbar:selected:focus, treeview.view.progressbar:selected {
        color: #000000;
        box-shadow: none;
        background-color: #B36A87;
        background-image: none;
        border-radius: 3px; }
        treeview.view.progressbar:selected:focus:backdrop, treeview.view.progressbar:selected:backdrop {
          color: mix(#B36A87,#000000,0.66);
          border-color: mix(#B36A87,#000000,0.3);
          background-color: mix(#d0d0d0,#B36A87,0.9); }
    treeview.view.progressbar:disabled {
      background-color: #a5a5a5;
      background-image: none;
      border-color: #8c8c8c; }
    treeview.view.progressbar:backdrop {
      color: #d0d0d0;
      background-image: none;
      box-shadow: none; }
  treeview.view.trough {
    background-color: rgba(0, 0, 0, 0.1);
    border-radius: 3px; }
    treeview.view.trough:selected:focus, treeview.view.trough:selected {
      background-color: rgba(0, 0, 0, 0.3);
      border-width: 1px 0;
      border-style: solid;
      border-color: #B36A87;
      border-radius: 3px; }
  treeview.view header button {
    color: mix(#000000,#d0d0d0,0.5);
    background-color: #d0d0d0;
    font-weight: bold;
    text-shadow: none;
    box-shadow: none; }
    treeview.view header button:hover {
      color: mix(mix(#000000,#d0d0d0,0.5),#000000,0.5);
      box-shadow: none;
      transition: none; }
    treeview.view header button:active {
      color: #000000;
      transition: none; }
  treeview.view header button:last-child:backdrop, treeview.view header button:last-child {
    border-right-style: none; }
  treeview.view button.dnd:active, treeview.view button.dnd:selected, treeview.view button.dnd:hover, treeview.view button.dnd,
  treeview.view header.button.dnd:active,
  treeview.view header.button.dnd:selected,
  treeview.view header.button.dnd:hover,
  treeview.view header.button.dnd {
    padding: 0 6px;
    transition: none;
    background-image: none;
    background-color: #B36A87;
    color: #d0d0d0;
    border-radius: 0;
    border-style: none;
    box-shadow: inset 0 0 0 1px #d0d0d0;
    text-shadow: none; }
  treeview.view acceleditor > label {
    background-color: #B36A87; }

/***********
 ! Separator
************/
separator {
  background: rgba(0, 0, 0, 0.1);
  min-width: 1px;
  min-height: 1px; }

/**********
 ! Frames *
***********/
frame > border, .frame {
  border: 1px solid mix(#a5a5a5,#000000,0.08); }
  frame > border.flat, .frame.flat {
    border-style: none; }
  frame > border:backdrop, .frame:backdrop {
    border-color: mix(#a5a5a5,mix(#a5a5a5,#000000,0.08),0.9); }

/* avoid double borders when a viewport is packed into a GtkScrolledWindow */
scrolledwindow viewport.frame {
  border: 0; }

/***************
 ! Places view *
****************/
placesview .server-list-button > image {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-transform: rotate(0turn); }

placesview .server-list-button:checked > image {
  transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
  -gtk-icon-transform: rotate(-0.5turn); }

placesview row.activatable:hover {
  background-color: transparent; }

placesview > actionbar > revealer > box > label {
  padding-left: 8px;
  padding-right: 8px; }

/**************
 ! Window frame
***************/
decoration {
  border-radius: 3px 3px 0 0;
  /* this is used for the resize cursor area */
  border-width: 1px;
  border-style: solid;
  border-color: #0f0f0f; }
  decoration:backdrop {
    border-color: #0f0f0f;
    transition: 200ms ease-out; }
  .maximized decoration, .fullscreen decoration, .tiled decoration {
    border-radius: 0; }
  .popup decoration {
    box-shadow: none; }
  .ssd decoration {
    box-shadow: 0 0 0 1px #B36A87; }
  .solid-csd decoration {
    border-radius: 0;
    box-shadow: none; }
  .csd.popup decoration {
    border-radius: 0;
    box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2), 0 0 0 1px rgba(0, 0, 0, 0.13); }
  tooltip.csd decoration {
    border-radius: 3px;
    box-shadow: none; }
  messagedialog.csd decoration {
    border-radius: 3px;
    box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2), 0 0 0 1px rgba(0, 0, 0, 0.13); }

/***********************
 ! Fallback mode panel *
************************/
panel-plug,
panel-toplevel.background, clock-box, clock-box:backdrop,
panel-applet > menubar,
panel-applet > menubar:backdrop,
panel-toplevel .gnome-panel-menu-bar,
panel-toplevel .gnome-panel-menu-bar:backdrop, clock-box menuitem, clock-box:backdrop menuitem,
panel-applet > menubar menuitem,
panel-applet > menubar:backdrop menuitem,
panel-toplevel .gnome-panel-menu-bar menuitem,
panel-toplevel .gnome-panel-menu-bar:backdrop menuitem, wnck-pager, wnck-tasklist, .mate-panel-menu-bar, .xfce4-panel, UnityPanelWidget, .unity-panel {
  background-color: #0f0f0f;
  background-image: none;
  color: #a5a5a5; }

clock-box menuitem label, clock-box:backdrop menuitem label,
panel-applet > menubar menuitem label,
panel-applet > menubar:backdrop menuitem label,
panel-toplevel .gnome-panel-menu-bar menuitem label,
panel-toplevel .gnome-panel-menu-bar:backdrop menuitem label, gp-calendar-window label,
#tasklist-button label,
#clock-applet-button label,
#showdesktop-button label {
  font-weight: normal;
  color: #a5a5a5; }

#clock-applet-button, #clock-applet-button:backdrop, panel-applet button, panel-applet button:backdrop, .xfce4-panel button, #login_window, #shutdown_dialog, #restart_dialog {
  border-width: 0 1px;
  border-radius: 0;
  border-color: transparent;
  background-color: transparent;
  background-image: none;
  color: #a5a5a5; }
  #clock-applet-button:hover, panel-applet button:hover, .xfce4-panel button:hover, #login_window:hover, #shutdown_dialog:hover, #restart_dialog:hover {
    background-color: mix(#0f0f0f,#a5a5a5,0.11);
    background-image: none;
    border-color: mix(#0f0f0f,#a5a5a5,0.11);
    color: #b2b2b2; }
  #clock-applet-button:active, panel-applet button:active, .xfce4-panel button:active, #login_window:active, #shutdown_dialog:active, #restart_dialog:active, #clock-applet-button:checked, panel-applet button:checked, .xfce4-panel button:checked, #login_window:checked, #shutdown_dialog:checked, #restart_dialog:checked {
    background-color: mix(#0f0f0f,#a5a5a5,0.21);
    background-image: none;
    border-color: mix(#0f0f0f,#a5a5a5,0.21);
    color: #b2b2b2; }
    #clock-applet-button:active:hover, panel-applet button:active:hover, .xfce4-panel button:active:hover, #login_window:active:hover, #shutdown_dialog:active:hover, #restart_dialog:active:hover, #clock-applet-button:checked:hover, panel-applet button:checked:hover, .xfce4-panel button:checked:hover, #login_window:checked:hover, #shutdown_dialog:checked:hover, #restart_dialog:checked:hover {
      background-color: mix(#0f0f0f,#a5a5a5,0.31);
      background-image: none;
      border-color: mix(#0f0f0f,#a5a5a5,0.31); }

panel-plug,
panel-toplevel.background {
  padding: 0; }

.gp-text-color {
  color: #000; }

panel-applet {
  border: 0; }

clock-box menuitem, clock-box:backdrop menuitem,
panel-applet > menubar menuitem,
panel-applet > menubar:backdrop menuitem,
panel-toplevel .gnome-panel-menu-bar menuitem,
panel-toplevel .gnome-panel-menu-bar:backdrop menuitem {
  border: 0; }

/****************
 ! MATE styles *
*****************/
.mate-panel-menu-bar {
  border: 0;
  padding: 0;
  text-shadow: none; }

#PanelApplet label,
.mate-panel-menu-bar menubar > menuitem {
  color: #a5a5a5; }

PanelSeparator, MatePanelAppletFrameDBus {
  border-width: 0;
  color: transparent;
  background-image: -gtk-scaled(url("../assets/pane-handle.png"), url("../assets/pane-handle@2.png"));
  background-color: transparent;
  background-repeat: no-repeat;
  background-position: left; }

#PanelApplet button,
#PanelApplet button.flat,
#PanelApplet button.toggle #PanelApplet button.flat.toggle {
  background-image: none;
  background-color: transparent;
  border-color: transparent;
  border-style: solid;
  border-radius: 0;
  border-width: 1px;
  color: #a5a5a5;
  text-shadow: none;
  box-shadow: none;
  padding: 2px; }

#PanelApplet button:hover:active,
#PanelApplet button:checked,
#PanelApplet button:checked:hover,
#PanelApplet button.flat:hover:active,
#PanelApplet button.flat:checked,
#PanelApplet button.flat:checked:hover,
#PanelApplet button.toggle:hover:active,
#PanelApplet button.toggle:checked,
#PanelApplet button.toggle:checked:hover,
#PanelApplet button.flat.toggle:hover:active,
#PanelApplet button.flat.toggle:checked,
#PanelApplet button.flat.toggle:checked:hover {
  background-image: none;
  background-color: darker(#0f0f0f);
  border-color: transparent;
  border-radius: 0;
  border-width: 1px;
  color: lighter(#a5a5a5);
  text-shadow: none;
  padding: 2px; }

#PanelApplet button:hover,
#PanelApplet button.flat:hover,
#PanelApplet button.toggle:hover,
#PanelApplet button.flat.toggle:hover {
  background-image: none;
  background-color: #141414;
  border-color: transparent;
  border-radius: 0;
  border-width: 1px;
  color: #000000;
  text-shadow: none;
  padding: 2px; }

.mate-panel-menu-bar menubar > menuitem {
  padding: 3px 7px; }

/*********************
 ! Cinnamon Settings *
**********************/
.cs-category-view, .cs-category-view:backdrop, .cs-category-view .view, .cs-category-view iconview, .cs-category-view .view:backdrop, .cs-category-view iconview:backdrop {
  background-color: transparent; }

/****************
 ! Gnome clocks *
*****************/
.clocks-analog-frame.trough {
  color: mix(#000000,#a5a5a5,0.85); }

.clocks-analog-frame.progress {
  color: mix(#a5a5a5,#B36A87,0.5); }

.clocks-analog-frame.progress-fast {
  color: #84435d; }

/*****************
 ! Gnome Builder *
******************/
workbench.csd > stack.titlebar:not(headerbar) {
  padding: 0;
  background: none;
  border: 0;
  box-shadow: none; }
  workbench.csd > stack.titlebar:not(headerbar) headerbar, workbench.csd > stack.titlebar:not(headerbar) headerbar:first-child, workbench.csd > stack.titlebar:not(headerbar) headerbar:last-child {
    border-radius: 3px 3px 0 0; }

/************************
 ! Unity-Control-Center *
*************************/
.background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame {
  border: 0 none transparent; }
  .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview.view, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview.view:backdrop, .background:not(.csd):not(.solid-csd) > box.vertical > notebook.frame > stack > scrolledwindow > viewport > box.vertical > frame > box.vertical iconview:backdrop {
    background-color: transparent; }

/***********************
 ! Unity Greeter *
 ***********************/
@keyframes dashentry_spinner {
  to {
    -gtk-icon-transform: rotate(1turn); } }

.lightdm.button, .lightdm-combo.combobox-entry .button, .lightdm-combo .cell, .lightdm-combo .button, .lightdm-combo .entry {
  background-image: none;
  background-color: rgba(0, 0, 0, 0.3);
  border-color: rgba(255, 255, 255, 0.9);
  border-radius: 7px;
  padding: 3px;
  color: #fff; }

.lightdm.menu {
  background-image: none;
  background-color: rgba(0, 0, 0, 0.6);
  border-color: rgba(255, 255, 255, 0.2);
  border-radius: 6px;
  padding: 1px;
  color: #fff; }
  .lightdm.menu .menuitem *, .lightdm.menu .menuitem.check:active, .lightdm.menu .menuitem.radio:active {
    color: #fff; }

.lightdm.menubar *, .lightdm.menubar.menuitem {
  padding: 0px; }

.lightdm.option-button {
  padding: 3px;
  background: none;
  border: 0; }

.lightdm.toggle-button {
  background: none;
  border-width: 0; }
  .lightdm.toggle-button.selected {
    background-color: rgba(0, 0, 0, 0.3);
    border-color: rgba(255, 255, 255, 0.3);
    border-width: 1px; }
    .lightdm.toggle-button.selected:hover {
      background-color: rgba(255, 255, 255, 0.3); }

.lightdm.button:hover {
  background-color: rgba(255, 255, 255, 0.3);
  border-color: rgba(255, 255, 255, 0.6);
  text-shadow: none; }

.lightdm.entry, .lightdm.button:active, .lightdm.button:active:focus, .lightdm.button:focus {
  background-image: none;
  background-color: rgba(0, 0, 0, 0.3);
  border-color: rgba(255, 255, 255, 0.6);
  border-radius: 7px;
  padding: 5px;
  color: #fff;
  text-shadow: none; }

.lightdm.entry:hover, .lightdm.entry:active, .lightdm.entry:active:focus {
  background-image: none;
  border-image: none; }

.lightdm.entry:active {
  -gtk-icon-source: -gtk-icontheme("process-working-symbolic");
  animation: dashentry_spinner 1s infinite linear; }

.lightdm.entry:focus {
  border-color: rgba(255, 255, 255, 0.6);
  border-width: 1px;
  border-style: solid;
  color: #fff; }

.lightdm.entry:selected {
  background-color: rgba(255, 255, 255, 0.2); }

.lightdm-combo.menu {
  background-color: #101010;
  border-radius: 0;
  padding: 0;
  color: #fff; }

/*********
 ! Gedit *
**********/
GeditWindow .pane-separator {
  border-width: 0 1px 0 0;
  border-style: solid; }
  GeditWindow .pane-separator, GeditWindow .pane-separator:hover {
    border-color: #959595;
    background-color: #a5a5a5; }

.gedit-document-panel {
  background-color: #a5a5a5;
  color: mix(#000000,#a5a5a5,0.1); }
  .gedit-document-panel list row {
    padding: 3px; }
    .gedit-document-panel list row button {
      padding: 1px;
      border-radius: 3px;
      border-style: solid;
      border-color: transparent;
      border-width: 1px;
      background-color: transparent;
      background-image: none;
      color: transparent;
      -gtk-icon-shadow: none; }
  .gedit-document-panel .prelight-row button {
    border-color: rgba(0, 0, 0, 0.1);
    color: rgba(255, 255, 255, 0.8); }
    .gedit-document-panel .prelight-row button:active {
      border-color: rgba(0, 0, 0, 0.2);
      background-color: rgba(0, 0, 0, 0.08);
      color: #fff; }
  .gedit-document-panel list row button:hover, .gedit-document-panel .prelight-row button:hover {
    border-color: rgba(0, 0, 0, 0.1);
    color: #fff; }

.gedit-document-panel-group-row, .gedit-document-panel-group-row:hover {
  border-top: 1px solid #959595;
  background-color: #a5a5a5; }

.gedit-document-panel-document-row:hover {
  background-color: #adadad; }

.gedit-document-panel-dragged-row {
  border: 1px solid rgba(0, 0, 0, 0.1);
  background-color: rgba(0, 0, 0, 0.5);
  color: #fff; }

.gedit-document-panel-placeholder-row {
  border: 0;
  background-color: rgba(0, 0, 0, 0.08);
  transition: all 200ms ease-in; }

statusbar GeditSmallButton, GeditStatusMenuButton {
  text-shadow: none; }
  statusbar GeditSmallButton button, GeditStatusMenuButton button {
    border-style: solid;
    border-width: 0 1px;
    border-color: transparent;
    border-radius: 0;
    padding: 1px 6px 2px 4px; }
    statusbar GeditSmallButton button:hover, statusbar GeditSmallButton button:active, statusbar GeditSmallButton button:active:hover, GeditStatusMenuButton button:hover, GeditStatusMenuButton button:active, GeditStatusMenuButton button:active:hover {
      border-color: #848484; }
    statusbar GeditSmallButton button:active, GeditStatusMenuButton button:active {
      background-color: #9d9d9d;
      color: #000000; }

GeditViewFrame .gedit-search-slider {
  padding: 3px;
  border-radius: 0 0 3px 3px;
  border-width: 0 1px 1px;
  border-style: solid;
  border-color: #a6a6a6;
  background-color: #d0d0d0; }
  GeditViewFrame .gedit-search-slider .not-found {
    background-color: #f44336;
    background-image: none;
    color: #fff; }

GeditFileBrowserWidget .toolbar {
  padding: 1.5px;
  border-top: 0;
  background-color: #a5a5a5;
  background-image: none; }

.gedit-search-entry-occurrences-tag {
  margin: 1.5px;
  padding: 1.5px;
  color: mix(#000000,#d0d0d0,0.5); }

.gedit-bottom-panel-paned,
.gedit-side-panel-paned,
paned.titlebar {
  margin-right: 0; }

.gedit-bottom-panel-paned notebook {
  border-top: none; }

/************
 ! Nautilus *
*************/
.nautilus-desktop, .nautilus-desktop:backdrop, .nautilus-desktop *, .nautilus-desktop *:backdrop {
  color: #fff;
  text-shadow: 1px 1px #000; }
  .nautilus-desktop:active, .nautilus-desktop:backdrop:active, .nautilus-desktop *:active, .nautilus-desktop *:backdrop:active {
    color: #000000; }
  .nautilus-desktop:selected, .nautilus-desktop:backdrop:selected, .nautilus-desktop *:selected, .nautilus-desktop *:backdrop:selected {
    color: #000000; }
  .nautilus-desktop:active, .nautilus-desktop:hover, .nautilus-desktop:selected, .nautilus-desktop:backdrop:active, .nautilus-desktop:backdrop:hover, .nautilus-desktop:backdrop:selected, .nautilus-desktop *:active, .nautilus-desktop *:hover, .nautilus-desktop *:selected, .nautilus-desktop *:backdrop:active, .nautilus-desktop *:backdrop:hover, .nautilus-desktop *:backdrop:selected {
    text-shadow: none; }

.nautilus-window toolbar {
  border-width: 0 0 1px;
  border-style: solid;
  border-color: #848484; }

.nautilus-window .sidebar {
  border: 0; }
  .nautilus-window .sidebar frame {
    border: 0; }

.nautilus-window notebook {
  background-color: #d0d0d0;
  border: 0; }
  .nautilus-window notebook frame {
    border: 0; }

.nautilus-window .searchbar-container {
  margin-top: -1px; }
  .nautilus-window .searchbar-container searchbar {
    padding-top: 0px;
    padding-bottom: 1px;
    border-bottom: 1px solid mix(#a5a5a5,#000000,0.08); }

.disk-space-display {
  border-style: solid;
  border-width: 1px; }
  .disk-space-display.unknown {
    background-color: rgba(0, 0, 0, 0.5);
    border-color: rgba(0, 0, 0, 0.5); }
  .disk-space-display.used {
    background-color: rgba(179, 106, 135, 0.8);
    border-color: rgba(169, 87, 120, 0.8); }
  .disk-space-display.free {
    background-color: #9d9d9d;
    border-color: #8d8d8d; }

.conflict-row.activatable, .conflict-row.activatable:active {
  color: #fff;
  background-color: #f44336; }

.conflict-row.activatable:hover {
  background-color: #f65d52; }

.conflict-row.activatable:selected {
  color: #000000;
  background-color: #B36A87; }

/********
 ! Nemo *
*********/
.nemo-desktop, .nemo-desktop:backdrop, .nemo-desktop *, .nemo-desktop *:backdrop {
  color: #fff;
  text-shadow: 1px 1px #000; }
  .nemo-desktop:active, .nemo-desktop:backdrop:active, .nemo-desktop *:active, .nemo-desktop *:backdrop:active {
    color: #000000; }
  .nemo-desktop:selected, .nemo-desktop:backdrop:selected, .nemo-desktop *:selected, .nemo-desktop *:backdrop:selected {
    color: #000000; }
  .nemo-desktop:active, .nemo-desktop:hover, .nemo-desktop:selected, .nemo-desktop:backdrop:active, .nemo-desktop:backdrop:hover, .nemo-desktop:backdrop:selected, .nemo-desktop *:active, .nemo-desktop *:hover, .nemo-desktop *:selected, .nemo-desktop *:backdrop:active, .nemo-desktop *:backdrop:hover, .nemo-desktop *:backdrop:selected {
    text-shadow: none; }

.nemo-window {
  /* Status Bar */ }
  .nemo-window toolbar {
    border-width: 0 0 1px;
    border-style: solid;
    border-color: #848484;
    /* Path Bar */ }
    .nemo-window toolbar button {
      background-color: #a5a5a5;
      background-image: none;
      border-color: rgba(0, 0, 0, 0.32);
      color: #000000;
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
      .nemo-window toolbar button:focus, .nemo-window toolbar button:hover {
        border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
      .nemo-window toolbar button:active, .nemo-window toolbar button:active:hover, .nemo-window toolbar button:active:focus, .nemo-window toolbar button:active:hover:focus, .nemo-window toolbar button:checked, .nemo-window toolbar button:checked:hover, .nemo-window toolbar button:checked:focus, .nemo-window toolbar button:checked:hover:focus {
        border-color: rgba(0, 0, 0, 0.32); }
      .nemo-window toolbar button:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      .nemo-window toolbar button:active:disabled, .nemo-window toolbar button:checked:disabled {
        border-color: rgba(0, 0, 0, 0.32); }
      .nemo-window toolbar button.flat {
        color: #000000;
        border-color: rgba(165, 165, 165, 0);
        background-color: rgba(165, 165, 165, 0);
        background-image: none;
        box-shadow: none; }
      .nemo-window toolbar button:hover, .nemo-window toolbar button.flat:hover {
        background-color: #adadad;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        color: #000000;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
        .nemo-window toolbar button:hover:focus, .nemo-window toolbar button:hover:hover, .nemo-window toolbar button.flat:hover:focus, .nemo-window toolbar button.flat:hover:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        .nemo-window toolbar button:hover:active, .nemo-window toolbar button:hover:active:hover, .nemo-window toolbar button:hover:active:focus, .nemo-window toolbar button:hover:active:hover:focus, .nemo-window toolbar button:hover:checked, .nemo-window toolbar button:hover:checked:hover, .nemo-window toolbar button:hover:checked:focus, .nemo-window toolbar button:hover:checked:hover:focus, .nemo-window toolbar button.flat:hover:active, .nemo-window toolbar button.flat:hover:active:hover, .nemo-window toolbar button.flat:hover:active:focus, .nemo-window toolbar button.flat:hover:active:hover:focus, .nemo-window toolbar button.flat:hover:checked, .nemo-window toolbar button.flat:hover:checked:hover, .nemo-window toolbar button.flat:hover:checked:focus, .nemo-window toolbar button.flat:hover:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        .nemo-window toolbar button:hover:disabled, .nemo-window toolbar button.flat:hover:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        .nemo-window toolbar button:hover:active:disabled, .nemo-window toolbar button:hover:checked:disabled, .nemo-window toolbar button.flat:hover:active:disabled, .nemo-window toolbar button.flat:hover:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
      .nemo-window toolbar button:focus, .nemo-window toolbar button.flat:focus {
        background-color: #adadad;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.4);
        color: #000000;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar button:focus:focus, .nemo-window toolbar button:focus:hover, .nemo-window toolbar button.flat:focus:focus, .nemo-window toolbar button.flat:focus:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
        .nemo-window toolbar button:focus:active, .nemo-window toolbar button:focus:active:hover, .nemo-window toolbar button:focus:active:focus, .nemo-window toolbar button:focus:active:hover:focus, .nemo-window toolbar button:focus:checked, .nemo-window toolbar button:focus:checked:hover, .nemo-window toolbar button:focus:checked:focus, .nemo-window toolbar button:focus:checked:hover:focus, .nemo-window toolbar button.flat:focus:active, .nemo-window toolbar button.flat:focus:active:hover, .nemo-window toolbar button.flat:focus:active:focus, .nemo-window toolbar button.flat:focus:active:hover:focus, .nemo-window toolbar button.flat:focus:checked, .nemo-window toolbar button.flat:focus:checked:hover, .nemo-window toolbar button.flat:focus:checked:focus, .nemo-window toolbar button.flat:focus:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.4); }
        .nemo-window toolbar button:focus:disabled, .nemo-window toolbar button.flat:focus:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        .nemo-window toolbar button:focus:active:disabled, .nemo-window toolbar button:focus:checked:disabled, .nemo-window toolbar button.flat:focus:active:disabled, .nemo-window toolbar button.flat:focus:checked:disabled {
          border-color: rgba(0, 0, 0, 0.4); }
        .nemo-window toolbar button:focus:hover, .nemo-window toolbar button.flat:focus:hover {
          background-color: #b6b6b6;
          background-image: none;
          border-color: rgba(0, 0, 0, 0.4);
          box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
          .nemo-window toolbar button:focus:hover:focus, .nemo-window toolbar button:focus:hover:hover, .nemo-window toolbar button.flat:focus:hover:focus, .nemo-window toolbar button.flat:focus:hover:hover {
            border-color: mix(#B36A87,rgba(0, 0, 0, 0.4),0.3); }
          .nemo-window toolbar button:focus:hover:active, .nemo-window toolbar button:focus:hover:active:hover, .nemo-window toolbar button:focus:hover:active:focus, .nemo-window toolbar button:focus:hover:active:hover:focus, .nemo-window toolbar button:focus:hover:checked, .nemo-window toolbar button:focus:hover:checked:hover, .nemo-window toolbar button:focus:hover:checked:focus, .nemo-window toolbar button:focus:hover:checked:hover:focus, .nemo-window toolbar button.flat:focus:hover:active, .nemo-window toolbar button.flat:focus:hover:active:hover, .nemo-window toolbar button.flat:focus:hover:active:focus, .nemo-window toolbar button.flat:focus:hover:active:hover:focus, .nemo-window toolbar button.flat:focus:hover:checked, .nemo-window toolbar button.flat:focus:hover:checked:hover, .nemo-window toolbar button.flat:focus:hover:checked:focus, .nemo-window toolbar button.flat:focus:hover:checked:hover:focus {
            border-color: rgba(0, 0, 0, 0.4); }
          .nemo-window toolbar button:focus:hover:disabled, .nemo-window toolbar button.flat:focus:hover:disabled {
            border-color: rgba(0, 0, 0, 0.4); }
          .nemo-window toolbar button:focus:hover:active:disabled, .nemo-window toolbar button:focus:hover:checked:disabled, .nemo-window toolbar button.flat:focus:hover:active:disabled, .nemo-window toolbar button.flat:focus:hover:checked:disabled {
            border-color: rgba(0, 0, 0, 0.4); }
      .nemo-window toolbar button:checked, .nemo-window toolbar button:active, .nemo-window toolbar button.flat:checked, .nemo-window toolbar button.flat:active {
        /*@include linear-gradient(shade($bg, .7), to top);*/
        background-color: #B36A87;
        background-image: none;
        color: #000000;
        box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06), inset 0 -1px rgba(0, 0, 0, 0.05);
        border-color: rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar button:checked:focus, .nemo-window toolbar button:checked:hover, .nemo-window toolbar button:active:focus, .nemo-window toolbar button:active:hover, .nemo-window toolbar button.flat:checked:focus, .nemo-window toolbar button.flat:checked:hover, .nemo-window toolbar button.flat:active:focus, .nemo-window toolbar button.flat:active:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.32),0.3); }
        .nemo-window toolbar button:checked:active, .nemo-window toolbar button:checked:active:hover, .nemo-window toolbar button:checked:active:focus, .nemo-window toolbar button:checked:active:hover:focus, .nemo-window toolbar button:checked:checked, .nemo-window toolbar button:checked:checked:hover, .nemo-window toolbar button:checked:checked:focus, .nemo-window toolbar button:checked:checked:hover:focus, .nemo-window toolbar button:active:active, .nemo-window toolbar button:active:active:hover, .nemo-window toolbar button:active:active:focus, .nemo-window toolbar button:active:active:hover:focus, .nemo-window toolbar button:active:checked, .nemo-window toolbar button:active:checked:hover, .nemo-window toolbar button:active:checked:focus, .nemo-window toolbar button:active:checked:hover:focus, .nemo-window toolbar button.flat:checked:active, .nemo-window toolbar button.flat:checked:active:hover, .nemo-window toolbar button.flat:checked:active:focus, .nemo-window toolbar button.flat:checked:active:hover:focus, .nemo-window toolbar button.flat:checked:checked, .nemo-window toolbar button.flat:checked:checked:hover, .nemo-window toolbar button.flat:checked:checked:focus, .nemo-window toolbar button.flat:checked:checked:hover:focus, .nemo-window toolbar button.flat:active:active, .nemo-window toolbar button.flat:active:active:hover, .nemo-window toolbar button.flat:active:active:focus, .nemo-window toolbar button.flat:active:active:hover:focus, .nemo-window toolbar button.flat:active:checked, .nemo-window toolbar button.flat:active:checked:hover, .nemo-window toolbar button.flat:active:checked:focus, .nemo-window toolbar button.flat:active:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar button:checked:disabled, .nemo-window toolbar button:active:disabled, .nemo-window toolbar button.flat:checked:disabled, .nemo-window toolbar button.flat:active:disabled {
          border-color: rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar button:checked:active:disabled, .nemo-window toolbar button:checked:checked:disabled, .nemo-window toolbar button:active:active:disabled, .nemo-window toolbar button:active:checked:disabled, .nemo-window toolbar button.flat:checked:active:disabled, .nemo-window toolbar button.flat:checked:checked:disabled, .nemo-window toolbar button.flat:active:active:disabled, .nemo-window toolbar button.flat:active:checked:disabled {
          border-color: rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar button:checked:focus, .nemo-window toolbar button:checked:hover, .nemo-window toolbar button:active:focus, .nemo-window toolbar button:active:hover, .nemo-window toolbar button.flat:checked:focus, .nemo-window toolbar button.flat:checked:hover, .nemo-window toolbar button.flat:active:focus, .nemo-window toolbar button.flat:active:hover {
          /*@include linear-gradient(shade($bg, .65), to top);*/
          background-color: #b8738f;
          background-image: none;
          color: #000000; }
      .nemo-window toolbar button:focus, .nemo-window toolbar button:hover, .nemo-window toolbar button.flat:focus, .nemo-window toolbar button.flat:hover {
        color: #000000; }
      .nemo-window toolbar button:disabled:disabled, .nemo-window toolbar button.flat:disabled:disabled {
        background-color: alpha(mix(#a5a5a5,#000000,0.2),0.4);
        background-image: none;
        /*border: 1px solid alpha($bg, .2);*/
        opacity: .4;
        color: mix(#a5a5a5,#000000,0.5);
        box-shadow: none; }
      .nemo-window toolbar button:active:disabled, .nemo-window toolbar button:checked:disabled, .nemo-window toolbar button.flat:active:disabled, .nemo-window toolbar button.flat:checked:disabled {
        background-color: rgba(179, 106, 135, 0.6);
        background-image: none;
        color: #000000;
        box-shadow: none; }
      .nemo-window toolbar button.separator, .nemo-window toolbar button .separator {
        border: 1px solid currentColor;
        color: rgba(165, 165, 165, 0.9); }
        .nemo-window toolbar button.separator:disabled, .nemo-window toolbar button .separator:disabled {
          color: rgba(165, 165, 165, 0.85); }
    .nemo-window toolbar .linked > button {
      box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
      .nemo-window toolbar .linked > button:focus, .nemo-window toolbar .linked > button:hover {
        box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.12),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      .nemo-window toolbar .linked > button:active, .nemo-window toolbar .linked > button:active:hover, .nemo-window toolbar .linked > button:active:focus, .nemo-window toolbar .linked > button:active:hover:focus, .nemo-window toolbar .linked > button:checked, .nemo-window toolbar .linked > button:checked:hover, .nemo-window toolbar .linked > button:checked:focus, .nemo-window toolbar .linked > button:checked:hover:focus {
        box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
      .nemo-window toolbar .linked > button:disabled {
        box-shadow: inset -1px 0 #848484; }
      .nemo-window toolbar .linked > button:last-child, .nemo-window toolbar .linked > button:only-child {
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
      .nemo-window toolbar .linked > button:last-child:hover, .nemo-window toolbar .linked > button:only-child:hover {
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
      .nemo-window toolbar .linked > button:disabled:last-child, .nemo-window toolbar .linked > button:disabled:only-child, .nemo-window toolbar .linked > button:active:disabled:last-child, .nemo-window toolbar .linked > button:active:disabled:only-child, .nemo-window toolbar .linked > button:checked:disabled:last-child, .nemo-window toolbar .linked > button:checked:disabled:only-child {
        box-shadow: none; }
      .nemo-window toolbar .linked > button:active:last-child, .nemo-window toolbar .linked > button:active:last-child:focus, .nemo-window toolbar .linked > button:active:last-child:hover, .nemo-window toolbar .linked > button:active:last-child:hover:focus, .nemo-window toolbar .linked > button:checked:last-child, .nemo-window toolbar .linked > button:checked:last-child:focus, .nemo-window toolbar .linked > button:checked:last-child:hover, .nemo-window toolbar .linked > button:checked:last-child:hover:focus {
        box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
      .nemo-window toolbar .linked > button:active:only-child, .nemo-window toolbar .linked > button:active:only-child:focus, .nemo-window toolbar .linked > button:active:only-child:hover, .nemo-window toolbar .linked > button:active:only-child:hover:focus, .nemo-window toolbar .linked > button:checked:only-child, .nemo-window toolbar .linked > button:checked:only-child:focus, .nemo-window toolbar .linked > button:checked:only-child:hover, .nemo-window toolbar .linked > button:checked:only-child:hover:focus {
        box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
    .nemo-window toolbar separator, .nemo-window toolbar separator:disabled {
      color: #959595;
      border-color: currentColor;
      -GtkWidget-window-dragging: true; }
    .nemo-window toolbar.primary-toolbar button.image-button {
      padding: 0 8px; }
    .nemo-window toolbar combobox, .nemo-window toolbar button {
      padding: 3px; }
      .nemo-window toolbar combobox.text-button, .nemo-window toolbar button.text-button {
        padding: 3px; }
      .nemo-window toolbar combobox.image-button, .nemo-window toolbar button.image-button {
        padding: 3px; }
    .nemo-window toolbar toolitem stack {
      margin-left: 15px; }
      .nemo-window toolbar toolitem stack widget button {
        box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), 0 1px 2px -1px rgba(0, 0, 0, 0.12);
        -NemoPathbarButton-border-radius: 3px; }
        .nemo-window toolbar toolitem stack widget button:focus, .nemo-window toolbar toolitem stack widget button:hover {
          box-shadow: inset -1px 0 mix(#B36A87,rgba(0, 0, 0, 0.12),0.3), 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar toolitem stack widget button:active, .nemo-window toolbar toolitem stack widget button:active:hover, .nemo-window toolbar toolitem stack widget button:active:focus, .nemo-window toolbar toolitem stack widget button:active:hover:focus, .nemo-window toolbar toolitem stack widget button:checked, .nemo-window toolbar toolitem stack widget button:checked:hover, .nemo-window toolbar toolitem stack widget button:checked:focus, .nemo-window toolbar toolitem stack widget button:checked:hover:focus {
          box-shadow: inset -1px 0 rgba(0, 0, 0, 0.12), inset 0 1px rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }
        .nemo-window toolbar toolitem stack widget button:disabled {
          box-shadow: inset -1px 0 #848484; }
        .nemo-window toolbar toolitem stack widget button:last-child, .nemo-window toolbar toolitem stack widget button:only-child {
          box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
        .nemo-window toolbar toolitem stack widget button:last-child:hover, .nemo-window toolbar toolitem stack widget button:only-child:hover {
          box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
        .nemo-window toolbar toolitem stack widget button:disabled:last-child, .nemo-window toolbar toolitem stack widget button:disabled:only-child, .nemo-window toolbar toolitem stack widget button:active:disabled:last-child, .nemo-window toolbar toolitem stack widget button:active:disabled:only-child, .nemo-window toolbar toolitem stack widget button:checked:disabled:last-child, .nemo-window toolbar toolitem stack widget button:checked:disabled:only-child {
          box-shadow: none; }
        .nemo-window toolbar toolitem stack widget button:active:last-child, .nemo-window toolbar toolitem stack widget button:active:last-child:focus, .nemo-window toolbar toolitem stack widget button:active:last-child:hover, .nemo-window toolbar toolitem stack widget button:active:last-child:hover:focus, .nemo-window toolbar toolitem stack widget button:checked:last-child, .nemo-window toolbar toolitem stack widget button:checked:last-child:focus, .nemo-window toolbar toolitem stack widget button:checked:last-child:hover, .nemo-window toolbar toolitem stack widget button:checked:last-child:hover:focus {
          box-shadow: inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
        .nemo-window toolbar toolitem stack widget button:active:only-child, .nemo-window toolbar toolitem stack widget button:active:only-child:focus, .nemo-window toolbar toolitem stack widget button:active:only-child:hover, .nemo-window toolbar toolitem stack widget button:active:only-child:hover:focus, .nemo-window toolbar toolitem stack widget button:checked:only-child, .nemo-window toolbar toolitem stack widget button:checked:only-child:focus, .nemo-window toolbar toolitem stack widget button:checked:only-child:hover, .nemo-window toolbar toolitem stack widget button:checked:only-child:hover:focus {
          box-shadow: inset 1px 0 rgba(0, 0, 0, 0.06), inset 0 1px rgba(0, 0, 0, 0.07), inset -1px 0 rgba(0, 0, 0, 0.06); }
  .nemo-window grid > widget:last-child button {
    min-height: 16px;
    min-width: 16px;
    padding: 2px 4px; }
  .nemo-window grid > widget:last-child button:first-child {
    margin-left: 20px; }
  .nemo-window grid > widget:last-child button:first-child + button {
    margin-right: 15px; }
  .nemo-window grid > widget:last-child button:first-child + button + separator + button {
    margin-left: 15px; }
  .nemo-window grid > widget:last-child > box > scale {
    margin-right: 12px; }
  .nemo-window grid > widget:last-child statusbar {
    border: 0; }
  .nemo-window .sidebar {
    /* Nemo Query Editor (File Search Bar) */ }
    .nemo-window .sidebar .frame {
      border: 0; }
    .nemo-window .sidebar image {
      padding-left: 3px;
      padding-right: 3px; }
    .nemo-window .sidebar .nemo-places-sidebar, .nemo-window .sidebar .nemo-places-sidebar .view, .nemo-window .sidebar .nemo-places-sidebar iconview {
      background-color: mix(#a5a5a5,#d0d0d0,0.5); }
    .nemo-window .sidebar .nemo-places-sidebar .view, .nemo-window .sidebar .nemo-places-sidebar iconview {
      -NemoPlacesTreeView-disk-full-bg-color: #848484;
      -NemoPlacesTreeView-disk-full-fg-color: #B36A87;
      -NemoPlacesTreeView-disk-full-bar-width: 2px;
      -NemoPlacesTreeView-disk-full-bar-radius: 1px;
      -NemoPlacesTreeView-disk-full-bottom-padding: 0;
      -NemoPlacesTreeView-disk-full-max-length: 75px; }
      .nemo-window .sidebar .nemo-places-sidebar .view:selected, .nemo-window .sidebar .nemo-places-sidebar iconview:selected {
        -NemoPlacesTreeView-disk-full-bg-color: #000000;
        -NemoPlacesTreeView-disk-full-fg-color: #c690a5; }
    .nemo-window .sidebar + separator + box .primary-toolbar {
      background-color: #a2a2a2;
      background-image: none;
      padding-top: 0px;
      padding-bottom: 0px;
      border-bottom: 1px solid mix(#a5a5a5,#000000,0.08); }
      .nemo-window .sidebar + separator + box .primary-toolbar button:nth-child(2) {
        border-right: none;
        border-top-right-radius: 0;
        border-bottom-right-radius: 0; }
      .nemo-window .sidebar + separator + box .primary-toolbar button:nth-child(3) {
        margin-left: -6px;
        border-left: none;
        border-top-left-radius: 0;
        border-bottom-left-radius: 0; }
      .nemo-window .sidebar + separator + box .primary-toolbar button.flat {
        background-color: #c6c6c6;
        background-image: none;
        border-color: rgba(0, 0, 0, 0.12);
        color: #000000;
        box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12);
        padding: 5px 6px; }
        .nemo-window .sidebar + separator + box .primary-toolbar button.flat:focus, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:hover {
          border-color: mix(#B36A87,rgba(0, 0, 0, 0.12),0.3); }
        .nemo-window .sidebar + separator + box .primary-toolbar button.flat:active, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:active:hover, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:active:focus, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:active:hover:focus, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:checked, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:checked:hover, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:checked:focus, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:checked:hover:focus {
          border-color: rgba(0, 0, 0, 0.12); }
        .nemo-window .sidebar + separator + box .primary-toolbar button.flat:disabled {
          border-color: rgba(0, 0, 0, 0.12); }
        .nemo-window .sidebar + separator + box .primary-toolbar button.flat:active:disabled, .nemo-window .sidebar + separator + box .primary-toolbar button.flat:checked:disabled {
          border-color: rgba(0, 0, 0, 0.12); }
  .nemo-window notebook {
    background-color: #d0d0d0;
    border-width: 0; }
    .nemo-window notebook tabs {
      border: 0; }

/***********************
 ! Fallback mode panel *
************************/
/************
 ! Synaptic *
*************/
GtkWindow > GtkVBox > .dock, GtkWindow > GtkVBox > .dock > GtkHBox > GtkToolbar {
  background-color: #a5a5a5;
  background-image: none;
  padding: 3px;
  border: 0;
  color: #000000; }

/***********************
 ! Fallback mode panel *
************************/
/***************
 ! Xfce styles *
****************/
.XfceHeading {
  margin: 0;
  padding: 0;
  border: 0;
  background-image: none;
  background-color: #d0d0d0;
  color: #000000; }

.xfce4-panel {
  font: inherit; }
  .xfce4-panel menu {
    -gtk-icon-effect: none;
    text-shadow: none; }

/***********************
 ! Fallback mode panel *
************************/
/****************
 ! Unity styles *
*****************/
UnityDecoration {
  -UnityDecoration-extents: 24px 1px 1px 1px;
  -UnityDecoration-input-extents: 10px;
  -UnityDecoration-shadow-offset-x: 1px;
  -UnityDecoration-shadow-offset-y: 1px;
  -UnityDecoration-active-shadow-color: rgba(0, 0, 0, 0.7);
  -UnityDecoration-active-shadow-radius: 8px;
  -UnityDecoration-inactive-shadow-color: rgba(0, 0, 0, 0.5);
  -UnityDecoration-inactive-shadow-radius: 5px;
  -UnityDecoration-glow-size: 10px;
  -UnityDecoration-glow-color: #B36A87;
  -UnityDecoration-title-indent: 10px;
  -UnityDecoration-title-fade: 35px;
  -UnityDecoration-title-alignment: 0; }
  UnityDecoration .top {
    border: 1px solid #B36A87;
    border-bottom: 0;
    border-radius: 3px 3px 0 0;
    padding: 1px 6px 0 6px;
    background-color: #0f0f0f;
    color: mix(#a5a5a5,#0f0f0f,0.1);
    text-shadow: none; }
    UnityDecoration .top:hover {
      border-radius: 0;
      border-color: mix(#0f0f0f,#a5a5a5,0.21);
      background-color: mix(#0f0f0f,#a5a5a5,0.21);
      background-image: none;
      color: #b2b2b2; }
    UnityDecoration .top:backdrop {
      border: 1px solid #0f0f0f;
      color: mix(#a5a5a5,#0f0f0f,0.4); }
  UnityDecoration .left, UnityDecoration .right, UnityDecoration .bottom {
    background-color: #0f0f0f; }
    UnityDecoration .left:backdrop, UnityDecoration .right:backdrop, UnityDecoration .bottom:backdrop {
      background-color: mix(#0b0b0b,#a5a5a5,0.21); }

UnityPanelWidget, .unity-panel {
  border: 0; }

.unity-panel.menuitem, .unity-panel .menuitem {
  border-width: 0 1px;
  color: #a5a5a5; }
  .unity-panel.menuitem:hover, .unity-panel.menuitem *:hover, .unity-panel .menuitem:hover, .unity-panel .menuitem *:hover {
    border-color: mix(#0f0f0f,#a5a5a5,0.21);
    background-color: mix(#0f0f0f,#a5a5a5,0.21);
    background-image: none;
    color: #b2b2b2; }

SheetStyleDialog.unity-force-quit {
  background-color: #a5a5a5; }

/***********************
 ! LightDM GTK Greeter *
 ***********************/
#panel_window {
  background-color: #0f0f0f;
  background-image: none;
  color: #fff;
  font-weight: bold;
  text-shadow: 0 1px rgba(0, 0, 0, 0.5);
  -gtk-icon-shadow: 0 1px rgba(0, 0, 0, 0.5); }
  #panel_window menubar {
    padding-left: 3px; }
    #panel_window menubar, #panel_window menubar > menuitem {
      background-color: transparent;
      background-image: none;
      border-style: none;
      color: #fff;
      text-shadow: 0 1px rgba(0, 0, 0, 0.5);
      -gtk-icon-shadow: 0 1px rgba(0, 0, 0, 0.5); }
      #panel_window menubar:hover, #panel_window menubar > menuitem:hover {
        background-color: rgba(255, 255, 255, 0.2);
        background-image: none;
        color: #fff; }
      #panel_window menubar *:hover, #panel_window menubar > menuitem *:hover {
        color: #fff; }
      #panel_window menubar:disabled, #panel_window menubar > menuitem:disabled {
        color: rgba(255, 255, 255, 0.7); }
    #panel_window menubar menu > menuitem {
      font-weight: normal; }

#content_frame {
  padding-bottom: 9px; }

#login_window, #shutdown_dialog, #restart_dialog {
  border-style: none;
  border-radius: 3px;
  background-color: #0f0f0f;
  color: #a5a5a5;
  /* draw border using box-shadow */
  box-shadow: inset 1px 0 mix(#0b0b0b,#a5a5a5,0.21), inset -1px 0 mix(#0b0b0b,#a5a5a5,0.21), inset 0 1px mix(#0b0b0b,#a5a5a5,0.21), inset 0 -1px mix(#0b0b0b,#a5a5a5,0.21); }

#login_window menu {
  border-radius: 0; }

#login_window button {
  background-color: #0f0f0f;
  background-image: none;
  border-color: rgba(132, 132, 132, 0.22);
  color: #a5a5a5;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
  #login_window button:focus, #login_window button:hover {
    border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
  #login_window button:active, #login_window button:active:hover, #login_window button:active:focus, #login_window button:active:hover:focus, #login_window button:checked, #login_window button:checked:hover, #login_window button:checked:focus, #login_window button:checked:hover:focus {
    border-color: rgba(116, 116, 116, 0.22); }
  #login_window button:disabled {
    border-color: rgba(140, 140, 140, 0.22); }
  #login_window button:active:disabled, #login_window button:checked:disabled {
    border-color: rgba(132, 132, 132, 0.22); }
  #login_window button.flat {
    color: #a5a5a5;
    border-color: rgba(15, 15, 15, 0);
    background-color: rgba(15, 15, 15, 0);
    background-image: none;
    box-shadow: none; }
  #login_window button:hover, #login_window button.flat:hover {
    background-color: #101010;
    background-image: none;
    border-color: rgba(132, 132, 132, 0.3);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    #login_window button:hover:focus, #login_window button:hover:hover, #login_window button.flat:hover:focus, #login_window button.flat:hover:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
    #login_window button:hover:active, #login_window button:hover:active:hover, #login_window button:hover:active:focus, #login_window button:hover:active:hover:focus, #login_window button:hover:checked, #login_window button:hover:checked:hover, #login_window button:hover:checked:focus, #login_window button:hover:checked:hover:focus, #login_window button.flat:hover:active, #login_window button.flat:hover:active:hover, #login_window button.flat:hover:active:focus, #login_window button.flat:hover:active:hover:focus, #login_window button.flat:hover:checked, #login_window button.flat:hover:checked:hover, #login_window button.flat:hover:checked:focus, #login_window button.flat:hover:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.3); }
    #login_window button:hover:disabled, #login_window button.flat:hover:disabled {
      border-color: rgba(140, 140, 140, 0.3); }
    #login_window button:hover:active:disabled, #login_window button:hover:checked:disabled, #login_window button.flat:hover:active:disabled, #login_window button.flat:hover:checked:disabled {
      border-color: rgba(132, 132, 132, 0.3); }
  #login_window button:focus, #login_window button.flat:focus {
    background-color: #101010;
    background-image: none;
    border-color: rgba(132, 132, 132, 0.3);
    color: #a5a5a5;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    #login_window button:focus:focus, #login_window button:focus:hover, #login_window button.flat:focus:focus, #login_window button.flat:focus:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
    #login_window button:focus:active, #login_window button:focus:active:hover, #login_window button:focus:active:focus, #login_window button:focus:active:hover:focus, #login_window button:focus:checked, #login_window button:focus:checked:hover, #login_window button:focus:checked:focus, #login_window button:focus:checked:hover:focus, #login_window button.flat:focus:active, #login_window button.flat:focus:active:hover, #login_window button.flat:focus:active:focus, #login_window button.flat:focus:active:hover:focus, #login_window button.flat:focus:checked, #login_window button.flat:focus:checked:hover, #login_window button.flat:focus:checked:focus, #login_window button.flat:focus:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.3); }
    #login_window button:focus:disabled, #login_window button.flat:focus:disabled {
      border-color: rgba(140, 140, 140, 0.3); }
    #login_window button:focus:active:disabled, #login_window button:focus:checked:disabled, #login_window button.flat:focus:active:disabled, #login_window button.flat:focus:checked:disabled {
      border-color: rgba(132, 132, 132, 0.3); }
    #login_window button:focus:hover, #login_window button.flat:focus:hover {
      background-color: #111111;
      background-image: none;
      border-color: rgba(132, 132, 132, 0.3);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
      #login_window button:focus:hover:focus, #login_window button:focus:hover:hover, #login_window button.flat:focus:hover:focus, #login_window button.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(165, 165, 165, 0.3),0.3); }
      #login_window button:focus:hover:active, #login_window button:focus:hover:active:hover, #login_window button:focus:hover:active:focus, #login_window button:focus:hover:active:hover:focus, #login_window button:focus:hover:checked, #login_window button:focus:hover:checked:hover, #login_window button:focus:hover:checked:focus, #login_window button:focus:hover:checked:hover:focus, #login_window button.flat:focus:hover:active, #login_window button.flat:focus:hover:active:hover, #login_window button.flat:focus:hover:active:focus, #login_window button.flat:focus:hover:active:hover:focus, #login_window button.flat:focus:hover:checked, #login_window button.flat:focus:hover:checked:hover, #login_window button.flat:focus:hover:checked:focus, #login_window button.flat:focus:hover:checked:hover:focus {
        border-color: rgba(116, 116, 116, 0.3); }
      #login_window button:focus:hover:disabled, #login_window button.flat:focus:hover:disabled {
        border-color: rgba(140, 140, 140, 0.3); }
      #login_window button:focus:hover:active:disabled, #login_window button:focus:hover:checked:disabled, #login_window button.flat:focus:hover:active:disabled, #login_window button.flat:focus:hover:checked:disabled {
        border-color: rgba(132, 132, 132, 0.3); }
  #login_window button:checked, #login_window button:active, #login_window button.flat:checked, #login_window button.flat:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(165, 165, 165, 0.06), inset 0 1px rgba(165, 165, 165, 0.07), inset -1px 0 rgba(165, 165, 165, 0.06), inset 0 -1px rgba(165, 165, 165, 0.05);
    border-color: rgba(132, 132, 132, 0.22); }
    #login_window button:checked:focus, #login_window button:checked:hover, #login_window button:active:focus, #login_window button:active:hover, #login_window button.flat:checked:focus, #login_window button.flat:checked:hover, #login_window button.flat:active:focus, #login_window button.flat:active:hover {
      border-color: mix(#B36A87,rgba(165, 165, 165, 0.22),0.3); }
    #login_window button:checked:active, #login_window button:checked:active:hover, #login_window button:checked:active:focus, #login_window button:checked:active:hover:focus, #login_window button:checked:checked, #login_window button:checked:checked:hover, #login_window button:checked:checked:focus, #login_window button:checked:checked:hover:focus, #login_window button:active:active, #login_window button:active:active:hover, #login_window button:active:active:focus, #login_window button:active:active:hover:focus, #login_window button:active:checked, #login_window button:active:checked:hover, #login_window button:active:checked:focus, #login_window button:active:checked:hover:focus, #login_window button.flat:checked:active, #login_window button.flat:checked:active:hover, #login_window button.flat:checked:active:focus, #login_window button.flat:checked:active:hover:focus, #login_window button.flat:checked:checked, #login_window button.flat:checked:checked:hover, #login_window button.flat:checked:checked:focus, #login_window button.flat:checked:checked:hover:focus, #login_window button.flat:active:active, #login_window button.flat:active:active:hover, #login_window button.flat:active:active:focus, #login_window button.flat:active:active:hover:focus, #login_window button.flat:active:checked, #login_window button.flat:active:checked:hover, #login_window button.flat:active:checked:focus, #login_window button.flat:active:checked:hover:focus {
      border-color: rgba(116, 116, 116, 0.22); }
    #login_window button:checked:disabled, #login_window button:active:disabled, #login_window button.flat:checked:disabled, #login_window button.flat:active:disabled {
      border-color: rgba(140, 140, 140, 0.22); }
    #login_window button:checked:active:disabled, #login_window button:checked:checked:disabled, #login_window button:active:active:disabled, #login_window button:active:checked:disabled, #login_window button.flat:checked:active:disabled, #login_window button.flat:checked:checked:disabled, #login_window button.flat:active:active:disabled, #login_window button.flat:active:checked:disabled {
      border-color: rgba(132, 132, 132, 0.22); }
    #login_window button:checked:focus, #login_window button:checked:hover, #login_window button:active:focus, #login_window button:active:hover, #login_window button.flat:checked:focus, #login_window button.flat:checked:hover, #login_window button.flat:active:focus, #login_window button.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  #login_window button:focus, #login_window button:hover, #login_window button.flat:focus, #login_window button.flat:hover {
    color: #a5a5a5; }
  #login_window button:disabled:disabled, #login_window button.flat:disabled:disabled {
    background-color: alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(#0f0f0f,#a5a5a5,0.5);
    box-shadow: none; }
  #login_window button:active:disabled, #login_window button:checked:disabled, #login_window button.flat:active:disabled, #login_window button.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  #login_window button.separator, #login_window button .separator {
    border: 1px solid currentColor;
    color: rgba(15, 15, 15, 0.9); }
    #login_window button.separator:disabled, #login_window button .separator:disabled {
      color: rgba(15, 15, 15, 0.85); }

#login_window entry {
  background-color: #0f0f0f;
  background-image: none;
  border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.8);
  padding: 3px;
  color: #a5a5a5;
  caret-color: #000000;
  -gtk-secondary-caret-color: #000000; }
  #login_window entry:focus, #login_window entry:hover {
    border-color: mix(#B36A87,mix(#0f0f0f,#a5a5a5,0.2),0.3); }
  #login_window entry:active, #login_window entry:active:hover, #login_window entry:active:focus, #login_window entry:active:hover:focus, #login_window entry:checked, #login_window entry:checked:hover, #login_window entry:checked:focus, #login_window entry:checked:hover:focus {
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.7); }
  #login_window entry:disabled {
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.85); }
  #login_window entry:active:disabled, #login_window entry:checked:disabled {
    border-color: shade(mix(#0f0f0f,#a5a5a5,0.2),0.8); }
  #login_window entry:focus, #login_window entry:active {
    border-color: mix(#B36A87,mix(#a5a5a5,#000000,0.08),0.3); }
  #login_window entry:disabled {
    background-color: #0e0e0e;
    background-image: none;
    border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.8);
    /*@include border(mix($bg, $fg, .20));*/
    /*border-color: alpha(mix($bg, $fg, .15), .8);*/
    /*opacity: .8;*/
    color: mix(#0f0f0f,#a5a5a5,0.5); }
    #login_window entry:disabled:focus, #login_window entry:disabled:hover {
      border-color: mix(#B36A87,alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.3); }
    #login_window entry:disabled:active, #login_window entry:disabled:active:hover, #login_window entry:disabled:active:focus, #login_window entry:disabled:active:hover:focus, #login_window entry:disabled:checked, #login_window entry:disabled:checked:hover, #login_window entry:disabled:checked:focus, #login_window entry:disabled:checked:hover:focus {
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.7); }
    #login_window entry:disabled:disabled {
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.85); }
    #login_window entry:disabled:active:disabled, #login_window entry:disabled:checked:disabled {
      border-color: shade(alpha(mix(#0f0f0f,#a5a5a5,0.2),0.4),0.8); }

#user_combobox {
  color: #a5a5a5;
  font-size: 18px; }
  #user_combobox menu {
    font-weight: normal; }
  #user_combobox arrow {
    color: mix(#a5a5a5,#0f0f0f,0.5); }

#user_image {
  border-radius: 3px;
  /* draw border using box-shadow */
  box-shadow: inset 1px 0 #0b0b0b, inset -1px 0 #0b0b0b, inset 0 1px #0b0b0b, inset 0 -1px #0b0b0b; }

#user_image_border {
  border-radius: 3px;
  background-color: #0e0e0e;
  background-image: none;
  box-shadow: inset 1px 0 rgba(0, 0, 0, 0.07), inset 0 1px rgba(0, 0, 0, 0.08), inset -1px 0 rgba(0, 0, 0, 0.07), inset 0 -1px rgba(0, 0, 0, 0.05); }

#buttonbox_frame {
  padding-top: 6px;
  padding-bottom: 0;
  border-style: none;
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px;
  background-color: transparent;
  background-image: none;
  box-shadow: none; }

/* shutdown button */
#shutdown_button button {
  background-color: #f44336;
  background-image: none;
  border-color: rgba(204, 204, 204, 0.22);
  color: #fff;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.12); }
  #shutdown_button button:focus, #shutdown_button button:hover {
    border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
  #shutdown_button button:active, #shutdown_button button:active:hover, #shutdown_button button:active:focus, #shutdown_button button:active:hover:focus, #shutdown_button button:checked, #shutdown_button button:checked:hover, #shutdown_button button:checked:focus, #shutdown_button button:checked:hover:focus {
    border-color: rgba(179, 179, 179, 0.22); }
  #shutdown_button button:disabled {
    border-color: rgba(217, 217, 217, 0.22); }
  #shutdown_button button:active:disabled, #shutdown_button button:checked:disabled {
    border-color: rgba(204, 204, 204, 0.22); }
  #shutdown_button button.flat {
    color: #fff;
    border-color: rgba(244, 67, 54, 0);
    background-color: rgba(244, 67, 54, 0);
    background-image: none;
    box-shadow: none; }
  #shutdown_button button:hover, #shutdown_button button.flat:hover {
    background-color: #f55044;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.3);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
    #shutdown_button button:hover:focus, #shutdown_button button:hover:hover, #shutdown_button button.flat:hover:focus, #shutdown_button button.flat:hover:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
    #shutdown_button button:hover:active, #shutdown_button button:hover:active:hover, #shutdown_button button:hover:active:focus, #shutdown_button button:hover:active:hover:focus, #shutdown_button button:hover:checked, #shutdown_button button:hover:checked:hover, #shutdown_button button:hover:checked:focus, #shutdown_button button:hover:checked:hover:focus, #shutdown_button button.flat:hover:active, #shutdown_button button.flat:hover:active:hover, #shutdown_button button.flat:hover:active:focus, #shutdown_button button.flat:hover:active:hover:focus, #shutdown_button button.flat:hover:checked, #shutdown_button button.flat:hover:checked:hover, #shutdown_button button.flat:hover:checked:focus, #shutdown_button button.flat:hover:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.3); }
    #shutdown_button button:hover:disabled, #shutdown_button button.flat:hover:disabled {
      border-color: rgba(217, 217, 217, 0.3); }
    #shutdown_button button:hover:active:disabled, #shutdown_button button:hover:checked:disabled, #shutdown_button button.flat:hover:active:disabled, #shutdown_button button.flat:hover:checked:disabled {
      border-color: rgba(204, 204, 204, 0.3); }
  #shutdown_button button:focus, #shutdown_button button.flat:focus {
    background-color: #f55044;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.3);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    #shutdown_button button:focus:focus, #shutdown_button button:focus:hover, #shutdown_button button.flat:focus:focus, #shutdown_button button.flat:focus:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
    #shutdown_button button:focus:active, #shutdown_button button:focus:active:hover, #shutdown_button button:focus:active:focus, #shutdown_button button:focus:active:hover:focus, #shutdown_button button:focus:checked, #shutdown_button button:focus:checked:hover, #shutdown_button button:focus:checked:focus, #shutdown_button button:focus:checked:hover:focus, #shutdown_button button.flat:focus:active, #shutdown_button button.flat:focus:active:hover, #shutdown_button button.flat:focus:active:focus, #shutdown_button button.flat:focus:active:hover:focus, #shutdown_button button.flat:focus:checked, #shutdown_button button.flat:focus:checked:hover, #shutdown_button button.flat:focus:checked:focus, #shutdown_button button.flat:focus:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.3); }
    #shutdown_button button:focus:disabled, #shutdown_button button.flat:focus:disabled {
      border-color: rgba(217, 217, 217, 0.3); }
    #shutdown_button button:focus:active:disabled, #shutdown_button button:focus:checked:disabled, #shutdown_button button.flat:focus:active:disabled, #shutdown_button button.flat:focus:checked:disabled {
      border-color: rgba(204, 204, 204, 0.3); }
    #shutdown_button button:focus:hover, #shutdown_button button.flat:focus:hover {
      background-color: #f65d52;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.38); }
      #shutdown_button button:focus:hover:focus, #shutdown_button button:focus:hover:hover, #shutdown_button button.flat:focus:hover:focus, #shutdown_button button.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      #shutdown_button button:focus:hover:active, #shutdown_button button:focus:hover:active:hover, #shutdown_button button:focus:hover:active:focus, #shutdown_button button:focus:hover:active:hover:focus, #shutdown_button button:focus:hover:checked, #shutdown_button button:focus:hover:checked:hover, #shutdown_button button:focus:hover:checked:focus, #shutdown_button button:focus:hover:checked:hover:focus, #shutdown_button button.flat:focus:hover:active, #shutdown_button button.flat:focus:hover:active:hover, #shutdown_button button.flat:focus:hover:active:focus, #shutdown_button button.flat:focus:hover:active:hover:focus, #shutdown_button button.flat:focus:hover:checked, #shutdown_button button.flat:focus:hover:checked:hover, #shutdown_button button.flat:focus:hover:checked:focus, #shutdown_button button.flat:focus:hover:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      #shutdown_button button:focus:hover:disabled, #shutdown_button button.flat:focus:hover:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      #shutdown_button button:focus:hover:active:disabled, #shutdown_button button:focus:hover:checked:disabled, #shutdown_button button.flat:focus:hover:active:disabled, #shutdown_button button.flat:focus:hover:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
  #shutdown_button button:checked, #shutdown_button button:active, #shutdown_button button.flat:checked, #shutdown_button button.flat:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(255, 255, 255, 0.06), inset 0 1px rgba(255, 255, 255, 0.07), inset -1px 0 rgba(255, 255, 255, 0.06), inset 0 -1px rgba(255, 255, 255, 0.05);
    border-color: rgba(204, 204, 204, 0.22); }
    #shutdown_button button:checked:focus, #shutdown_button button:checked:hover, #shutdown_button button:active:focus, #shutdown_button button:active:hover, #shutdown_button button.flat:checked:focus, #shutdown_button button.flat:checked:hover, #shutdown_button button.flat:active:focus, #shutdown_button button.flat:active:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
    #shutdown_button button:checked:active, #shutdown_button button:checked:active:hover, #shutdown_button button:checked:active:focus, #shutdown_button button:checked:active:hover:focus, #shutdown_button button:checked:checked, #shutdown_button button:checked:checked:hover, #shutdown_button button:checked:checked:focus, #shutdown_button button:checked:checked:hover:focus, #shutdown_button button:active:active, #shutdown_button button:active:active:hover, #shutdown_button button:active:active:focus, #shutdown_button button:active:active:hover:focus, #shutdown_button button:active:checked, #shutdown_button button:active:checked:hover, #shutdown_button button:active:checked:focus, #shutdown_button button:active:checked:hover:focus, #shutdown_button button.flat:checked:active, #shutdown_button button.flat:checked:active:hover, #shutdown_button button.flat:checked:active:focus, #shutdown_button button.flat:checked:active:hover:focus, #shutdown_button button.flat:checked:checked, #shutdown_button button.flat:checked:checked:hover, #shutdown_button button.flat:checked:checked:focus, #shutdown_button button.flat:checked:checked:hover:focus, #shutdown_button button.flat:active:active, #shutdown_button button.flat:active:active:hover, #shutdown_button button.flat:active:active:focus, #shutdown_button button.flat:active:active:hover:focus, #shutdown_button button.flat:active:checked, #shutdown_button button.flat:active:checked:hover, #shutdown_button button.flat:active:checked:focus, #shutdown_button button.flat:active:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.22); }
    #shutdown_button button:checked:disabled, #shutdown_button button:active:disabled, #shutdown_button button.flat:checked:disabled, #shutdown_button button.flat:active:disabled {
      border-color: rgba(217, 217, 217, 0.22); }
    #shutdown_button button:checked:active:disabled, #shutdown_button button:checked:checked:disabled, #shutdown_button button:active:active:disabled, #shutdown_button button:active:checked:disabled, #shutdown_button button.flat:checked:active:disabled, #shutdown_button button.flat:checked:checked:disabled, #shutdown_button button.flat:active:active:disabled, #shutdown_button button.flat:active:checked:disabled {
      border-color: rgba(204, 204, 204, 0.22); }
    #shutdown_button button:checked:focus, #shutdown_button button:checked:hover, #shutdown_button button:active:focus, #shutdown_button button:active:hover, #shutdown_button button.flat:checked:focus, #shutdown_button button.flat:checked:hover, #shutdown_button button.flat:active:focus, #shutdown_button button.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  #shutdown_button button:focus, #shutdown_button button:hover, #shutdown_button button.flat:focus, #shutdown_button button.flat:hover {
    color: #fff; }
  #shutdown_button button:disabled:disabled, #shutdown_button button.flat:disabled:disabled {
    background-color: alpha(mix(#f44336,#fff,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(#f44336,#fff,0.5);
    box-shadow: none; }
  #shutdown_button button:active:disabled, #shutdown_button button:checked:disabled, #shutdown_button button.flat:active:disabled, #shutdown_button button.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  #shutdown_button button.separator, #shutdown_button button .separator {
    border: 1px solid currentColor;
    color: rgba(244, 67, 54, 0.9); }
    #shutdown_button button.separator:disabled, #shutdown_button button .separator:disabled {
      color: rgba(244, 67, 54, 0.85); }

/* restart button */
#restart_button button {
  background-color: #ef6c00;
  background-image: none;
  border-color: rgba(204, 204, 204, 0.22);
  color: #fff;
  box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.22); }
  #restart_button button:focus, #restart_button button:hover {
    border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
  #restart_button button:active, #restart_button button:active:hover, #restart_button button:active:focus, #restart_button button:active:hover:focus, #restart_button button:checked, #restart_button button:checked:hover, #restart_button button:checked:focus, #restart_button button:checked:hover:focus {
    border-color: rgba(179, 179, 179, 0.22); }
  #restart_button button:disabled {
    border-color: rgba(217, 217, 217, 0.22); }
  #restart_button button:active:disabled, #restart_button button:checked:disabled {
    border-color: rgba(204, 204, 204, 0.22); }
  #restart_button button.flat {
    color: #fff;
    border-color: rgba(239, 108, 0, 0);
    background-color: rgba(239, 108, 0, 0);
    background-image: none;
    box-shadow: none; }
  #restart_button button:hover, #restart_button button.flat:hover {
    background-color: #fb7100;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.3);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.32); }
    #restart_button button:hover:focus, #restart_button button:hover:hover, #restart_button button.flat:hover:focus, #restart_button button.flat:hover:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
    #restart_button button:hover:active, #restart_button button:hover:active:hover, #restart_button button:hover:active:focus, #restart_button button:hover:active:hover:focus, #restart_button button:hover:checked, #restart_button button:hover:checked:hover, #restart_button button:hover:checked:focus, #restart_button button:hover:checked:hover:focus, #restart_button button.flat:hover:active, #restart_button button.flat:hover:active:hover, #restart_button button.flat:hover:active:focus, #restart_button button.flat:hover:active:hover:focus, #restart_button button.flat:hover:checked, #restart_button button.flat:hover:checked:hover, #restart_button button.flat:hover:checked:focus, #restart_button button.flat:hover:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.3); }
    #restart_button button:hover:disabled, #restart_button button.flat:hover:disabled {
      border-color: rgba(217, 217, 217, 0.3); }
    #restart_button button:hover:active:disabled, #restart_button button:hover:checked:disabled, #restart_button button.flat:hover:active:disabled, #restart_button button.flat:hover:checked:disabled {
      border-color: rgba(204, 204, 204, 0.3); }
  #restart_button button:focus, #restart_button button.flat:focus {
    background-color: #fb7100;
    background-image: none;
    border-color: rgba(204, 204, 204, 0.3);
    color: #fff;
    box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.42); }
    #restart_button button:focus:focus, #restart_button button:focus:hover, #restart_button button.flat:focus:focus, #restart_button button.flat:focus:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
    #restart_button button:focus:active, #restart_button button:focus:active:hover, #restart_button button:focus:active:focus, #restart_button button:focus:active:hover:focus, #restart_button button:focus:checked, #restart_button button:focus:checked:hover, #restart_button button:focus:checked:focus, #restart_button button:focus:checked:hover:focus, #restart_button button.flat:focus:active, #restart_button button.flat:focus:active:hover, #restart_button button.flat:focus:active:focus, #restart_button button.flat:focus:active:hover:focus, #restart_button button.flat:focus:checked, #restart_button button.flat:focus:checked:hover, #restart_button button.flat:focus:checked:focus, #restart_button button.flat:focus:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.3); }
    #restart_button button:focus:disabled, #restart_button button.flat:focus:disabled {
      border-color: rgba(217, 217, 217, 0.3); }
    #restart_button button:focus:active:disabled, #restart_button button:focus:checked:disabled, #restart_button button.flat:focus:active:disabled, #restart_button button.flat:focus:checked:disabled {
      border-color: rgba(204, 204, 204, 0.3); }
    #restart_button button:focus:hover, #restart_button button.flat:focus:hover {
      background-color: #ff7808;
      background-image: none;
      border-color: rgba(204, 204, 204, 0.3);
      box-shadow: 0 1px 2px -1px rgba(0, 0, 0, 0.48); }
      #restart_button button:focus:hover:focus, #restart_button button:focus:hover:hover, #restart_button button.flat:focus:hover:focus, #restart_button button.flat:focus:hover:hover {
        border-color: mix(#B36A87,rgba(255, 255, 255, 0.3),0.3); }
      #restart_button button:focus:hover:active, #restart_button button:focus:hover:active:hover, #restart_button button:focus:hover:active:focus, #restart_button button:focus:hover:active:hover:focus, #restart_button button:focus:hover:checked, #restart_button button:focus:hover:checked:hover, #restart_button button:focus:hover:checked:focus, #restart_button button:focus:hover:checked:hover:focus, #restart_button button.flat:focus:hover:active, #restart_button button.flat:focus:hover:active:hover, #restart_button button.flat:focus:hover:active:focus, #restart_button button.flat:focus:hover:active:hover:focus, #restart_button button.flat:focus:hover:checked, #restart_button button.flat:focus:hover:checked:hover, #restart_button button.flat:focus:hover:checked:focus, #restart_button button.flat:focus:hover:checked:hover:focus {
        border-color: rgba(179, 179, 179, 0.3); }
      #restart_button button:focus:hover:disabled, #restart_button button.flat:focus:hover:disabled {
        border-color: rgba(217, 217, 217, 0.3); }
      #restart_button button:focus:hover:active:disabled, #restart_button button:focus:hover:checked:disabled, #restart_button button.flat:focus:hover:active:disabled, #restart_button button.flat:focus:hover:checked:disabled {
        border-color: rgba(204, 204, 204, 0.3); }
  #restart_button button:checked, #restart_button button:active, #restart_button button.flat:checked, #restart_button button.flat:active {
    /*@include linear-gradient(shade($bg, .7), to top);*/
    background-color: #B36A87;
    background-image: none;
    color: #000000;
    box-shadow: inset 1px 0 rgba(255, 255, 255, 0.06), inset 0 1px rgba(255, 255, 255, 0.07), inset -1px 0 rgba(255, 255, 255, 0.06), inset 0 -1px rgba(255, 255, 255, 0.05);
    border-color: rgba(204, 204, 204, 0.22); }
    #restart_button button:checked:focus, #restart_button button:checked:hover, #restart_button button:active:focus, #restart_button button:active:hover, #restart_button button.flat:checked:focus, #restart_button button.flat:checked:hover, #restart_button button.flat:active:focus, #restart_button button.flat:active:hover {
      border-color: mix(#B36A87,rgba(255, 255, 255, 0.22),0.3); }
    #restart_button button:checked:active, #restart_button button:checked:active:hover, #restart_button button:checked:active:focus, #restart_button button:checked:active:hover:focus, #restart_button button:checked:checked, #restart_button button:checked:checked:hover, #restart_button button:checked:checked:focus, #restart_button button:checked:checked:hover:focus, #restart_button button:active:active, #restart_button button:active:active:hover, #restart_button button:active:active:focus, #restart_button button:active:active:hover:focus, #restart_button button:active:checked, #restart_button button:active:checked:hover, #restart_button button:active:checked:focus, #restart_button button:active:checked:hover:focus, #restart_button button.flat:checked:active, #restart_button button.flat:checked:active:hover, #restart_button button.flat:checked:active:focus, #restart_button button.flat:checked:active:hover:focus, #restart_button button.flat:checked:checked, #restart_button button.flat:checked:checked:hover, #restart_button button.flat:checked:checked:focus, #restart_button button.flat:checked:checked:hover:focus, #restart_button button.flat:active:active, #restart_button button.flat:active:active:hover, #restart_button button.flat:active:active:focus, #restart_button button.flat:active:active:hover:focus, #restart_button button.flat:active:checked, #restart_button button.flat:active:checked:hover, #restart_button button.flat:active:checked:focus, #restart_button button.flat:active:checked:hover:focus {
      border-color: rgba(179, 179, 179, 0.22); }
    #restart_button button:checked:disabled, #restart_button button:active:disabled, #restart_button button.flat:checked:disabled, #restart_button button.flat:active:disabled {
      border-color: rgba(217, 217, 217, 0.22); }
    #restart_button button:checked:active:disabled, #restart_button button:checked:checked:disabled, #restart_button button:active:active:disabled, #restart_button button:active:checked:disabled, #restart_button button.flat:checked:active:disabled, #restart_button button.flat:checked:checked:disabled, #restart_button button.flat:active:active:disabled, #restart_button button.flat:active:checked:disabled {
      border-color: rgba(204, 204, 204, 0.22); }
    #restart_button button:checked:focus, #restart_button button:checked:hover, #restart_button button:active:focus, #restart_button button:active:hover, #restart_button button.flat:checked:focus, #restart_button button.flat:checked:hover, #restart_button button.flat:active:focus, #restart_button button.flat:active:hover {
      /*@include linear-gradient(shade($bg, .65), to top);*/
      background-color: #b8738f;
      background-image: none;
      color: #000000; }
  #restart_button button:focus, #restart_button button:hover, #restart_button button.flat:focus, #restart_button button.flat:hover {
    color: #fff; }
  #restart_button button:disabled:disabled, #restart_button button.flat:disabled:disabled {
    background-color: alpha(mix(#ef6c00,#fff,0.2),0.4);
    background-image: none;
    /*border: 1px solid alpha($bg, .2);*/
    opacity: .4;
    color: mix(#ef6c00,#fff,0.5);
    box-shadow: none; }
  #restart_button button:active:disabled, #restart_button button:checked:disabled, #restart_button button.flat:active:disabled, #restart_button button.flat:checked:disabled {
    background-color: rgba(179, 106, 135, 0.6);
    background-image: none;
    color: #000000;
    box-shadow: none; }
  #restart_button button.separator, #restart_button button .separator {
    border: 1px solid currentColor;
    color: rgba(239, 108, 0, 0.9); }
    #restart_button button.separator:disabled, #restart_button button .separator:disabled {
      color: rgba(239, 108, 0, 0.85); }

/* password warning */
#greeter_infobar {
  font-weight: bold; }

/**********************
 ! Genome Terminal *
***********************/
VteTerminal {
  background-color: #0f0f0f;
  color: #a5a5a5; }

terminal-window junction, terminal-window scrollbar trough {
  background-color: #0f0f0f;
  border-color: #0c0c0c; }

terminal-window scrollbar.vertical slider {
  background-color: mix(#0f0f0f,#a5a5a5,0.2); }
  terminal-window scrollbar.vertical slider:hover {
    background-color: mix(#0f0f0f,#a5a5a5,0.3); }
  terminal-window scrollbar.vertical slider:hover:active {
    background-color: #B36A87; }
  terminal-window scrollbar.vertical slider:disabled {
    background-color: transparent; }

/******************
 ! Budgie Desktop *
*******************/
.budgie-container {
  background-color: transparent; }

.raven {
  background-color: rgba(165, 165, 165, 0.93); }
  .raven .raven-header {
    background-color: #a5a5a5;
    border: solid mix(#a5a5a5,#000000,0.08);
    border-width: 1px 0; }
  .raven .raven-background {
    background-color: rgba(165, 165, 165, 0.93); }

.raven-mpris {
  background-color: rgba(165, 165, 165, 0.7); }
